#ifndef DBAMPDEVICE_H_
#define DBAMPDEVICE_H_

#include <drvIIC.h>
#include <bootlogo/DbAttr.h>

#define MAX_AMP_NUM                                     4
#define MAX_IIC_BUS_NUM                                 16
#define MAX_SECTION_LEN                                 64
#define MAX_ATTR_INFOS_LEN                              CONFIG_DB_AMP_ATTR_SIZE

#define CONVERT_SUCCESS                                 (0)
#define CONVERT_INVALID                                 (-1)
#define CONVERT_PROCESSING_FAILED                       (-2)

typedef enum
{
    E_DB_I2C_TYPE_MIN = 0,

    E_DB_I2C_TYPE_NONE = E_DB_I2C_TYPE_MIN,             /// default type
    E_DB_I2C_TYPE_SW,                                   /// SW I2C
    E_DB_I2C_TYPE_HW,                                   /// HW I2C

    E_DB_I2C_TYPE_MAX,
} db_iic_type_e;

///I2C port
typedef enum
{
    E_DB_IIC_HW_PORT_0 = 0,                             /// port 0_0 //disable port 0
    E_DB_IIC_HW_PORT0_1,                                /// port 0_1
    E_DB_IIC_HW_PORT0_2,                                /// port 0_2
    E_DB_IIC_HW_PORT0_3,                                /// port 0_3
    E_DB_IIC_HW_PORT0_4,                                /// port 0_4
    E_DB_IIC_HW_PORT0_5,                                /// port 0_5
    E_DB_IIC_HW_PORT0_6,                                /// port 0_6
    E_DB_IIC_HW_PORT0_7,                                /// port 0_7

    E_DB_IIC_HW_PORT_1 = 8,                             /// port 1_0 //disable port 1
    E_DB_IIC_HW_PORT1_1,                                /// port 1_1
    E_DB_IIC_HW_PORT1_2,                                /// port 1_2
    E_DB_IIC_HW_PORT1_3,                                /// port 1_3
    E_DB_IIC_HW_PORT1_4,                                /// port 1_4
    E_DB_IIC_HW_PORT1_5,                                /// port 1_5
    E_DB_IIC_HW_PORT1_6,                                /// port 1_6
    E_DB_IIC_HW_PORT1_7,                                /// port 1_7

    E_DB_IIC_HW_PORT_2 = 16,                            /// port 2_0 //disable port 2
    E_DB_IIC_HW_PORT2_1,                                /// port 2_1
    E_DB_IIC_HW_PORT2_2,                                /// port 2_2
    E_DB_IIC_HW_PORT2_3,                                /// port 2_3
    E_DB_IIC_HW_PORT2_4,                                /// port 2_4
    E_DB_IIC_HW_PORT2_5,                                /// port 2_5
    E_DB_IIC_HW_PORT2_6,                                /// port 2_6
    E_DB_IIC_HW_PORT2_7,                                /// port 2_7

    E_DB_IIC_HW_PORT_3 = 24,                            /// port 3_0 //disable port 3
    E_DB_IIC_HW_PORT3_1,                                /// port 3_1
    E_DB_IIC_HW_PORT3_2,                                /// port 3_2
    E_DB_IIC_HW_PORT3_3,                                /// port 3_3
    E_DB_IIC_HW_PORT3_4,                                /// port 3_4
    E_DB_IIC_HW_PORT3_5,                                /// port 3_5
    E_DB_IIC_HW_PORT3_6,                                /// port 3_6
    E_DB_IIC_HW_PORT3_7,                                /// port 3_7

    E_DB_IIC_SW_PORT_0 = 1024,                          /// swiic port0
    E_DB_IIC_SW_PORT_1,                                 /// swiic port1
    E_DB_IIC_SW_PORT_2,                                 /// swiic port2
    E_DB_IIC_SW_PORT_3,                                 /// swiic port3
    E_DB_IIC_SW_PORT_4,                                 /// swiic port4
    E_DB_IIC_SW_PORT_5,                                 /// swiic port5
    E_DB_IIC_SW_PORT_6,                                 /// swiic port6
    E_DB_IIC_SW_PORT_7,                                 /// swiic port7

    E_DB_IIC_PORT_NOSUP,                                /// non-support port

} db_iic_port_e;

///sw iic store pad and delay to config sw iic driver.
typedef struct db_sw_iic_bus_s
{
    db_iic_port_e m_ePort;
    MS_U16 m_u16ClkPad;
    MS_U16 m_u16DataPad;;
    MS_U16 m_u16Delay;
} db_sw_iic_bus_info_t;

/// iic master pin config
typedef struct hw_iic_pin_cfg_s
{
    MS_U32 m_u32Reg;                                    /// register
    MS_U8 m_u8BitPos;                                   /// bit position
    MS_BOOL m_bEnable;                                  /// enable or disable
} db_hw_iic_pin_cfg_t;

///hw iic store bus_ch mapped to driver layer.
typedef struct db_hw_iic_bus_s
{
    db_hw_iic_pin_cfg_t m_stPinCfg;                     /// pin info
    MS_U8 m_u8Speed;                                    /// speed
    db_iic_port_e m_ePort;                              /// port
} db_hw_iic_bus_info_t;

typedef struct db_iic_bus_s
{
    db_iic_type_e m_eBusType;

    union
    {
        db_hw_iic_bus_info_t m_stHw;
        db_sw_iic_bus_info_t m_stSw;
    };
} db_iic_bus_info_t;

// It should be consistent w/ amp_device_iic_info_t defined in drv_audio_amp.h.
typedef struct db_iic_info_s
{
    db_iic_port_e m_ePort;
    MS_U16 m_u16BusNumSlaveID;
} db_iic_info_t;

// It should be consistent w/ amp_device_gpio_info_t defined in drv_audio_amp.h.
typedef struct db_gpio_info_s
{
    MS_U16 m_u16GpioNum;
    MS_BOOL m_bInvert;
} db_gpio_info_t;

typedef struct db_amp_device_info_s
{
    MS_U32 m_u32DeviceId;                     ///< Get it from borad.ini, such as AMPLIFIER_I2S_RT9113B, AMPLIFIER_I2S_TI5707, etc.
    db_gpio_info_t m_stGpioReset;
    db_gpio_info_t m_stGpioStandby;
    db_iic_info_t m_stIicInfo;
} db_amp_device_info_t;

int parse_board_ini_audio_amp_attr_info(db_attr_info_t* pstDbAmpDeviceAttrInfo);
int parse_board_ini_amp_info(db_amp_device_info_t stAmpDeviceInfos[MAX_AMP_NUM], db_iic_bus_info_t stIicBusInfos[MAX_IIC_BUS_NUM]);
int parse_audio_amp_config_ini_info(db_amp_device_info_t stAmpDeviceInfos[MAX_AMP_NUM], db_attr_info_t stAttrInfos[MAX_AMP_NUM]);

int serialize_db_attr_infos(jsmn_serialize_params_t stSerializeParams, db_attr_info_t stAttrInfos[MAX_AMP_NUM]);
int deserialize_db_attr_infos(db_attr_info_t stAttrInfos[MAX_AMP_NUM], const char *psSerializedAttrInfos);

int convert_2_cus_iic_port_from_db(MS_IIC_PORT *pePortTo, db_iic_port_e ePortFrom, db_iic_bus_info_t stIicBusInfos[MAX_IIC_BUS_NUM]);
void print_db_iic_bus_infos(db_iic_bus_info_t stInfos[MAX_IIC_BUS_NUM]);

#endif /* DBAMPDEVICE_H_ */
