#ifndef DBATTR_H_
#define DBATTR_H_

#include <MsTypes.h>
#include <bootlogo/jsmn.h>

#define MAX_KEY_LEN                                     256
#define MAX_VALUE_LEN                                   2048 // It should be sync w/ the buffer of iniparser
#define MAX_NAME_LEN                                    256

#define PARSE_SUCCESS                                   (0)
#define PARSE_NOTHING_FOUND                             (-1)
#define PARSE_PROCESSING_FAILED                         (-2)

// It should be consistent w/ amp_device_attr_data_type_e defined in drv_audio_amp.h.
typedef enum
{
    E_DB_ATTR_DATA_TYPE_INVALID = -1,
    E_DB_ATTR_DATA_TYPE_MIN = 0,

    E_DB_ATTR_DATA_TYPE_INT = E_DB_ATTR_DATA_TYPE_MIN,
    E_DB_ATTR_DATA_TYPE_UINT,
    E_DB_ATTR_DATA_TYPE_STRING,
    E_DB_ATTR_DATA_TYPE_INT_ARRAY,
    E_DB_ATTR_DATA_TYPE_UINT_ARRAY,

    E_DB_ATTR_DATA_TYPE_MAX,
} db_attr_data_type_e;

// It should be consistent w/ amp_device_attr_array_t defined in drv_audio_amp.h.
typedef struct db_attr_array_s
{
    MS_U16 m_u16Size;
    union
    {
        MS_S32 *m_ps32Array;
        MS_U32 *m_pu32Array;
    } m_stArray;
} db_attr_array_t;

// It should be consistent w/ amp_device_attr_data_t defined in drv_audio_amp.h.
typedef union db_attr_data_s
{
    MS_S32 m_s32Value;
    MS_U32 m_u32Value;
    char *m_pszValue;
    db_attr_array_t m_stValue;
} db_attr_data_t;

// It should be consistent w/ amp_device_attr_t defined in drv_audio_amp.h.
typedef struct db_attr_s
{
    char *m_pszName;
    db_attr_data_type_e m_eDataType;
    db_attr_data_t m_stData;
} db_attr_t;

// It should be consistent w/ amp_device_attr_info_t defined in drv_audio_amp.h.
typedef struct db_attr_info_s
{
    MS_U8 m_u8AttrTotalNumber;
    db_attr_t *m_pstAttr;
} db_attr_info_t;

typedef enum
{
    E_DB_ATTR_KEY_TYPE_INVALID = -1,
    E_DB_ATTR_KEY_TYPE_MIN = 0,

    E_DB_ATTR_KEY_TYPE_ATTR_TOTAL_NUMBER = E_DB_ATTR_KEY_TYPE_MIN,
    E_DB_ATTR_KEY_TYPE_NAME,
    E_DB_ATTR_KEY_TYPE_DATA_TYPE,
    E_DB_ATTR_KEY_TYPE_DATA,

    E_DB_ATTR_KEY_TYPE_MAX,
} db_attr_key_type_e;

typedef struct db_attr_key_params_s
{
    const char *m_psSection;
    MS_U16 m_u16AttrNo;
} db_attr_key_params_t;

typedef struct db_buffer_s
{
    MS_U8 *m_pu8Buffer;
    MS_U16 m_u16BufferSize;
} db_buffer_t;

typedef struct db_get_attr_key_params_s
{
    db_buffer_t m_stAttrKey;

    db_attr_key_type_e m_eAttrKeyType;
    db_attr_key_params_t m_stAttrKeyParams;
} db_get_attr_key_params_t;

typedef MS_BOOL (*db_get_attr_key)(db_get_attr_key_params_t *pstParams);

void free_db_attr_infos(db_attr_info_t *pstAttrInfos);
void print_db_attr_info(db_attr_info_t *pstAttrInfo);

int parse_db_attr(db_attr_info_t *pstAttrInfo, db_get_attr_key pfGetAttrKey, db_attr_key_params_t *pstAttrKeyParams);
int serialize_db_attr_info(jsmn_serialize_params_t stSerializeParams, db_attr_info_t *pstAttrInfo);
int deserialize_db_attr_info(db_attr_info_t *pstAttrInfos, const char *js, jsmntok_t *t, int currentTokenPos);

#endif /* DBATTR_H_ */
