#ifndef _VECTOR_H_
#define _VECTOR_H_

#include "MsTypes.h"

#ifdef __cplusplus
extern "C" {
#endif

#define VECTOR_GET_STRING(vec, id)                      (const char *)vector_At((vec), (id))
#define VECTOR_PRINT_STRING(vec)                        vector_Print(vec, E_VECTOR_ITEM_TYPE_STRING)

typedef struct vector_Vector_s
{
    void **pItems;
    MS_U16 u16Capacity;
    MS_U16 u16Size;
} vector_Vector_t;

typedef enum
{
    E_VECTOR_ITEM_TYPE_STRING,
    E_VECTOR_ITEM_TYPE_MAX,
} vector_ItemType_e;

//------------------------------------------------------------------------------
/// @brief Init a vector.
/// @return MI_Vector_t *: The new created vector.
/// @return NULL: Process is failed.
//------------------------------------------------------------------------------
vector_Vector_t *vector_Init(void);

//------------------------------------------------------------------------------
/// @brief DeInit the vector.
/// @param[in] pstVector: The de-initialized vector.
/// @return None.
//------------------------------------------------------------------------------
void vector_DeInit(vector_Vector_t *pstVector);

//------------------------------------------------------------------------------
/// @brief Get the size of the vector.
/// @param[in] pstVector: The vector.
/// @return the size.
//------------------------------------------------------------------------------
MS_U16 vector_Size(vector_Vector_t *pstVector);

//------------------------------------------------------------------------------
/// @brief Add a item to the end of the vector.
/// @param[in] pstVector: The vector.
/// @param[in] pItem: The added item.
/// @param[in] u32MemorySize: The memory size of the added item.
/// @return TRUE: Process is successful.
/// @return FALSE: Process is failed.
//------------------------------------------------------------------------------
MS_BOOL vector_PushBack(vector_Vector_t *pstVector, void *pItem, MS_U32 u32MemorySize);

//------------------------------------------------------------------------------
/// @brief Replace a item of the vector.
/// @param[in] pstVector: The vector.
/// @param[in] u16Idx: The index of the replaced item.
/// @param[in] pItem: The new item.
/// @param[in] u32MemorySize: The memory size of the added item.
/// @return TRUE: Process is successful.
/// @return FALSE: Process is failed.
//------------------------------------------------------------------------------
MS_BOOL vector_Assign(vector_Vector_t *pstVector, MS_U16 u16Idx, void *pItem, MS_U32 u32MemorySize);

//------------------------------------------------------------------------------
/// @brief Get a item of the vector.
/// @param[in] pstVector: The vector.
/// @param[in] u16Idx: The index of the item.
/// @return The Item.
//------------------------------------------------------------------------------
void* vector_At(vector_Vector_t* pstVector, MS_U16 u16Idx);

//------------------------------------------------------------------------------
/// @brief Erase a item of the vector.
/// @param[in] pstVector: The vector.
/// @param[in] u16Idx: The index of the erased item.
/// @return TRUE: Process is successful.
/// @return FALSE: Process is failed.
//------------------------------------------------------------------------------
MS_BOOL vector_Erase(vector_Vector_t *pstVector, MS_U16 u16Idx);

//------------------------------------------------------------------------------
/// @brief print all items of the vector.
/// @param[in] pstVector: The vector.
/// @param[in] eItemType: The type contained in the memory that items pointer to.
/// @return None.
//------------------------------------------------------------------------------
void vector_Print(vector_Vector_t *pstVector, vector_ItemType_e eItemType);

#ifdef __cplusplus
}
#endif

#endif///_VECTOR_H_
