//<MStar Software>
//******************************************************************************
// MStar Software
// Copyright (c) 2010 - 2012 MStar Semiconductor, Inc. All rights reserved.
// All software, firmware and related documentation herein ("MStar Software") are
// intellectual property of MStar Semiconductor, Inc. ("MStar") and protected by
// law, including, but not limited to, copyright law and international treaties.
// Any use, modification, reproduction, retransmission, or republication of all 
// or part of MStar Software is expressly prohibited, unless prior written 
// permission has been granted by MStar. 
//
// By accessing, browsing and/or using MStar Software, you acknowledge that you
// have read, understood, and agree, to be bound by below terms ("Terms") and to
// comply with all applicable laws and regulations:
//
// 1. MStar shall retain any and all right, ownership and interest to MStar
//    Software and any modification/derivatives thereof.
//    No right, ownership, or interest to MStar Software and any
//    modification/derivatives thereof is transferred to you under Terms.
//
// 2. You understand that MStar Software might include, incorporate or be
//    supplied together with third party`s software and the use of MStar
//    Software may require additional licenses from third parties.  
//    Therefore, you hereby agree it is your sole responsibility to separately
//    obtain any and all third party right and license necessary for your use of
//    such third party`s software. 
//
// 3. MStar Software and any modification/derivatives thereof shall be deemed as
//    MStar`s confidential information and you agree to keep MStar`s 
//    confidential information in strictest confidence and not disclose to any
//    third party.  
//
// 4. MStar Software is provided on an "AS IS" basis without warranties of any
//    kind. Any warranties are hereby expressly disclaimed by MStar, including
//    without limitation, any warranties of merchantability, non-infringement of
//    intellectual property rights, fitness for a particular purpose, error free
//    and in conformity with any international standard.  You agree to waive any
//    claim against MStar for any loss, damage, cost or expense that you may
//    incur related to your use of MStar Software.
//    In no event shall MStar be liable for any direct, indirect, incidental or
//    consequential damages, including without limitation, lost of profit or
//    revenues, lost or damage of data, and unauthorized system use.
//    You agree that this Section 4 shall still apply without being affected
//    even if MStar Software has been modified by MStar in accordance with your
//    request or instruction for your use, except otherwise agreed by both
//    parties in writing.
//
// 5. If requested, MStar may from time to time provide technical supports or
//    services in relation with MStar Software to you for your use of
//    MStar Software in conjunction with your or your customer`s product
//    ("Services").
//    You understand and agree that, except otherwise agreed by both parties in
//    writing, Services are provided on an "AS IS" basis and the warranty
//    disclaimer set forth in Section 4 above shall apply.  
//
// 6. Nothing contained herein shall be construed as by implication, estoppels
//    or otherwise:
//    (a) conferring any license or right to use MStar name, trademark, service
//        mark, symbol or any other identification;
//    (b) obligating MStar or any of its affiliates to furnish any person,
//        including without limitation, you and your customers, any assistance
//        of any kind whatsoever, or any information; or 
//    (c) conferring any license or right under any intellectual property right.
//
// 7. These terms shall be governed by and construed in accordance with the laws
//    of Taiwan, R.O.C., excluding its conflict of law rules.
//    Any and all dispute arising out hereof or related hereto shall be finally
//    settled by arbitration referred to the Chinese Arbitration Association,
//    Taipei in accordance with the ROC Arbitration Law and the Arbitration
//    Rules of the Association by three (3) arbitrators appointed in accordance
//    with the said Rules.
//    The place of arbitration shall be in Taipei, Taiwan and the language shall
//    be English.  
//    The arbitration award shall be final and binding to both parties.
//
//******************************************************************************
//<MStar Software>

#include <common.h>
#include <watchdog.h>
#include <command.h>
#include <malloc.h>   /* for free() prototype */
#include <stdio.h>
#include <post.h>
#include <ShareType.h>
#include <MsDebug.h>
#include <MsVfs.h>
#include <MsUtility.h>
#include <CusConfig.h>
#include <MsTypes.h>

#ifndef BUFFER_SIZE
#define BUFFER_SIZE                 128
#endif

#if (ENABLE_MODULE_VFS == 1)
#define VOLUME_NAME_LEN     32

//-------------------------------------------------------------------------------------------------
// Includes
//-------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------
//  Debug Functions
//-------------------------------------------------------------------------------------------------

//-------------------------------------------------------------------------------------------------
//  Local Defines
//-------------------------------------------------------------------------------------------------
static char volName[VOLUME_NAME_LEN] = {0};

//-------------------------------------------------------------------------------------------------
//  Global Variables
//-------------------------------------------------------------------------------------------------

//-------------------------------------------------------------------------------------------------
//  Local Variables
//-------------------------------------------------------------------------------------------------
static BOOLEAN bVfsReady=FALSE;
//-------------------------------------------------------------------------------------------------
//  Extern Functions
//-------------------------------------------------------------------------------------------------
extern int snprintf(char *str, size_t size, const char *fmt, ...);
extern void ubi_get_part_name(char *name);
extern void ubifs_get_sb_name(char *name);

//-------------------------------------------------------------------------------------------------
//  Private Functions
//-------------------------------------------------------------------------------------------------

//-------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------

#if (defined(CONFIG_UBI) && !defined(CONFIG_NAND_FTL))
int vfs_mount(char *volume)
{
    char buffer[CMD_BUF]="\0";
    char last_ubi_partname[32];
    char last_sb_name[32];
    int ret=-1;
    UBOOT_TRACE("IN\n");

    if(volume==NULL)
    {
        bVfsReady=FALSE;
        UBOOT_ERROR("Input parameter 'volume' is a null pointer\n");
        return -1;
    }

        
	memset(last_ubi_partname, 0, 32);
	memset(last_sb_name,0,32);
	ubi_get_part_name(last_ubi_partname);
	ubifs_get_sb_name(last_sb_name);

	if( strcmp(last_sb_name,volume) == 0)
    {
		UBOOT_DEBUG("%s is already mount!!\n",volume);
		return 0;
	}
     
    ret=vfs_umount();
    if(ret==-1)
    {
        bVfsReady=FALSE;   
        UBOOT_ERROR("umount previous filesystem fail\n");
        return -1;
    }
    
    //Assume this volume is in UBI partition
    if(strcmp(last_ubi_partname, UBIPART) != 0)
	{
        memset(buffer,0,CMD_BUF);    
        snprintf(buffer,CMD_BUF,"ubi part %s",UBIPART);
        ret=run_command(buffer,0);
    }
    else
        ret = 0;

    if(ret!=-1)
    {
        memset(buffer,0,CMD_BUF);
        snprintf(buffer,CMD_BUF,"ubifsmount %s",volume);
        UBOOT_DEBUG("cmd=%s\n",buffer);
        ret=run_command(buffer,0);
        if(ret==-1)
        {
            UBOOT_ERROR("ubifsmount fail (Assume this volume is in UBI(or ubi) partition)\n");
        }
        else
        {
            memset(volName, 0, sizeof(volName));
            strncpy(volName, volume, sizeof(volName)-1);
            bVfsReady=TRUE;              
            UBOOT_TRACE("OK\n");
            return ret;
        }
    } 
    return ret;
}

int vfs_umount(void)
{
    int ret = 0;
    UBOOT_TRACE("IN\n");
    if(bVfsReady==TRUE)
    {
        ret = run_command("ubifsumount",0);
    }
    if(ret==0)
    {
        UBOOT_TRACE("OK\n");
    }
    else
    {
        UBOOT_ERROR("unmount fail\n");
    }
    memset(volName, 0, sizeof(volName));
    bVfsReady=FALSE;  
  	return ret;
}



U32 vfs_getsize(char *filedir)
{
    int ret = -1;
	U32 filesize = 0;
    char *buffer=NULL;
    UBOOT_TRACE("IN\n"); 
    if(filedir==NULL)
    {
        UBOOT_ERROR("filedir is a null pointer\n");
        return FALSE;
    }
    if(bVfsReady==FALSE)
    {
        UBOOT_ERROR("filesystem is not mounted\n");
        return FALSE;
    }
    buffer=malloc(CMD_BUF);
    if(buffer==NULL)
    {
        UBOOT_ERROR("malloc buffer fail\n");
        return FALSE;
    }
    memset(buffer,0,CMD_BUF);
    
    snprintf(buffer, CMD_BUF, "ubifsfilesize %s 0x%x",filedir,&filesize);
    UBOOT_DEBUG("cmd=%s\n",buffer);
    ret = run_command(buffer, 0);
    if(ret==-1)
    {
        UBOOT_ERROR("ubifsfilesize %s fail\n",filedir);
        filesize=0;
    }
    else
    {
        UBOOT_TRACE("OK\n");     
    }
    free(buffer);
	return filesize;
}
int vfs_read(void* addr,char* filedir,unsigned int offset,unsigned int size )
{
    int ret = -1;
    char *buffer=NULL;
    UBOOT_TRACE("IN\n");
    
    if(addr==NULL)
    {
        UBOOT_ERROR("addr is a null pointer\n");
        return -1;
    }
    
    if(filedir==NULL)
    {
        UBOOT_ERROR("filedir is a null pointer\n");
        return -1;
    }

    if(bVfsReady==FALSE)
    {
        UBOOT_ERROR("filesystem is not mounted\n");
        return -1;
    }
    buffer=malloc(CMD_BUF);
    if(buffer==NULL)
    {
        UBOOT_ERROR("malloc buffer fail\n");
        return -1;
    }
    memset(buffer,0,CMD_BUF);
    snprintf(buffer, CMD_BUF, "ubifspartload %x %s %x %x", (unsigned int)addr, filedir, offset,size);
    UBOOT_DEBUG("cmd=%s\n",buffer);
    ret = run_command(buffer, 0);
    if(ret==-1)
    {
        UBOOT_ERROR("ubifspartload %s fail\n",filedir);
    }
    else
    {
        UBOOT_TRACE("OK\n"); 
    }
    free(buffer);
	return ret;  
}

int vfs_write(void* addr,char* filedir,unsigned int offset,unsigned int size )
{
    UBOOT_TRACE("IN\n");
    
    if(addr==NULL)
    {
        UBOOT_ERROR("addr is a null pointer\n");
        return -1;
    }
    if(filedir==NULL)
    {
        UBOOT_ERROR("filedir is a null pointer\n");
        return -1;
    }

    if(bVfsReady==FALSE)
    {
        UBOOT_ERROR("filesystem is not mounted\n");
        return -1;
    }
    UBOOT_ERROR("Not implement now\n");
    return -1;
}


#else

#if defined(CONFIG_DUAL_SYSTEM)
#include <MsBoot.h>

int vfs_mount(char *volume)
{
    int ret = 0;
    char vfs_volName[32] = {0};
    int partnum;
    UBOOT_TRACE("IN\n");

    ret = get_mmc_part_number(volume, &partnum);
    if (ret < 0)
        return ret;

    memset(vfs_volName, 0, sizeof(vfs_volName));
    strncpy(vfs_volName, volume, 31);
    if (0 != read_bootflag())
    {
        strcat(vfs_volName, DUAL_SYSTEM_PARTITION_SUFFIX);
    }

    memset(volName, 0, sizeof(volName));
    strncpy(volName, vfs_volName, sizeof(volName)-1);
    UBOOT_INFO("mmc change mount : %s\n", volName);
    bVfsReady=TRUE;
    UBOOT_TRACE("OK\n");
    return ret;
}
#else
int vfs_mount(char *volume)
{
    int ret = 0;
    int partnum;
    UBOOT_TRACE("IN\n");

    ret = get_mmc_part_number(volume, &partnum);
    if (ret < 0)
        return ret;

    if (strlen(volume) >= VOLUME_NAME_LEN)
    {
        UBOOT_ERROR("invalid volume name : %s\n", volume);
        return -1;
    }
    memset(volName, 0, sizeof(volName));
    strncpy(volName, volume, sizeof(volName)-1);
    UBOOT_INFO("mmc change mount : %s\n",volName);
    bVfsReady=TRUE;
    UBOOT_TRACE("OK\n");
	return ret;
}
#endif

int vfs_umount(void)
{
    int ret = 0;
    UBOOT_TRACE("IN\n");
    memset(volName, 0, sizeof(volName));
    bVfsReady=FALSE;
    UBOOT_TRACE("OK\n");
	return ret;
}

U32 vfs_getsize(char *filedir)
{
	U32 filesize = 0;
    int ret = 0;
    char *buffer=NULL;
    
    UBOOT_TRACE("IN\n"); 

    if(filedir==NULL)
    {
        UBOOT_ERROR("filedir is a null pointer\n");
        return FALSE;
    }

    if(bVfsReady==FALSE)
    {
        UBOOT_ERROR("filesystem is not mounted\n");
        return FALSE;
    }
    
    buffer=malloc(CMD_BUF);
    if(buffer==NULL)
    {
        UBOOT_ERROR("malloc buffer fail\n");
        return FALSE;
    }
    memset(buffer,0,CMD_BUF);    
    //snprintf(cmd, CMD_BUF, "ext4filesize mmc 0:%s %s",mmc_partition,filedir);
    #if(defined(CONFIG_NAND_FTL))
    snprintf(buffer, CMD_BUF, "ext4filesize nand 0:%s %s %x",volName,filedir,(U32)&filesize);
    #else
    snprintf(buffer, CMD_BUF, "ext4filesize mmc 0:%s %s %x", volName,filedir,(U32)&filesize);
    #endif
    UBOOT_DEBUG("cmd=%s\n",buffer);    
    ret=run_command(buffer, 0);
    if(ret == -1)
    {
        UBOOT_ERROR("ext4filesize mmc fail\n");
        filesize=0;
    }
    else
    {
        UBOOT_TRACE("OK\n");
    }
    free(buffer);
	return filesize;
}

int vfs_read(void* addr,char* filedir,unsigned int offset,unsigned int size )
{
    int ret = -1;
    char *buffer=NULL;
    
    UBOOT_TRACE("IN\n");

    if(addr==NULL)
    {
        UBOOT_ERROR("addr is a null pointer\n");
        return -1;
    }
    
    if(filedir==NULL)
    {
        UBOOT_ERROR("filedir is a null pointer\n");
        return -1;
    }

    if(bVfsReady==FALSE)
    {
        UBOOT_ERROR("filesystem is not mounted\n");
        return -1;
    }
    
    buffer=malloc(CMD_BUF);
    if(buffer==NULL)
    {
        UBOOT_ERROR("malloc buffer fail\n");
        return -1;
    }
    memset(buffer,0,CMD_BUF);
    #if(defined(CONFIG_NAND_FTL))
    snprintf(buffer, CMD_BUF, "ext4partload nand 0:%s %x %s %x %x",volName, (unsigned int)addr, filedir, offset,size);
    #else
    snprintf(buffer, CMD_BUF, "ext4partload mmc 0:%s %x %s %x %x",volName, (unsigned int)addr, filedir, offset,size);
    #endif
    UBOOT_DEBUG("cmd=%s\n",buffer); 
    ret = run_command(buffer, 0);
    if(ret==-1)
    {
        UBOOT_ERROR("ext4load mmc fail, try again!\n");
        ret = run_command(buffer, 0);
        if(ret==-1)
        {
          UBOOT_ERROR("ext4load mmc fail\n");
        }
    }
    else
    {
      UBOOT_TRACE("OK\n");
    }
    free(buffer);
	return ret;  
}

int vfs_write(void* addr,char* filedir,unsigned int offset,unsigned int size )
{
    UBOOT_TRACE("IN\n");
    
    if(addr==NULL)
    {
        UBOOT_ERROR("addr is a null pointer\n");
        return -1;
    }
    
    if(filedir==NULL)
    {
        UBOOT_ERROR("filedir is a null pointer\n");
        return -1;
    }

    if(bVfsReady==FALSE)
    {
        UBOOT_ERROR("filesystem is not mounted\n");
        return -1;
    }
    UBOOT_ERROR("Not implement now\n");
    return -1;
}
#endif

int __get_useable_file_path(char *path)
{
#define PATH_DEPTH  4
    int filesize;
    char find_path[BUFFER_SIZE];
    char *str = find_path;
    int found_depth = PATH_DEPTH;
    int ret = -1;

    UBOOT_TRACE("IN\n");
    if (path == NULL){
        ret = -1;
        goto exit;
    }
    if(snprintf(str, sizeof(find_path), "%s", path)>=0)
    {
        do{
            filesize = vfs_getsize(str);
            if (filesize > 0){
                ret = 0;
                break;
            }
            found_depth --;
            if (found_depth == 0){
                ret = -1;
                break;
            }
            while (*str == '/') str++;
        }while((str = strstr(str, "/")));
        if ((ret == 0) && (found_depth != PATH_DEPTH)) {
            if(snprintf(path, sizeof(find_path), "%s", str)>=0)
            {
                UBOOT_INFO("the abspath is %s\n", path);
            }
        }
    }
exit:
    UBOOT_TRACE("OK\n");
    return ret;
}

#if defined(CONFIG_DATA_SEPARATION)
int vfs_mount_by_abspath(char *abspath, char *subdir_buf, int buf_size)
{
    int ret = 0;
    char p_abspath[BUFFER_SIZE];
    char *pos, *last_pos;
    char mount_name[BUFFER_SIZE];

    UBOOT_TRACE("IN\n");
    if (abspath == NULL)
        return -1;

    pos = abspath;
    last_pos = pos;
    do{
        memset(mount_name, 0, sizeof(mount_name));
        strncpy(mount_name, last_pos, pos - last_pos);
        ret = vfs_mount(mount_name);
        if (ret == 0){
            if (subdir_buf)
                snprintf(subdir_buf, BUFFER_SIZE, "%s", pos);
            break;
        }
        last_pos = ++pos;
    }while((pos = strstr(pos, "/")));

    if (pos == NULL){
        UBOOT_ERROR("find mount partition fail, mount default partition.\n");
        ret = vfs_mount(CUSDATA);
    }
    UBOOT_TRACE("OK\n");
    return ret;
}
#else
int vfs_mount_by_abspath(char *abspath, char *subdir_buf, int buf_size)
{
    int ret = 0, plen;
    int subdir_valid = 0;
    char *path, *partition, *partition_s;

    UBOOT_TRACE("IN\n");
    if (abspath == NULL)
    {
        return -1;
    }
    UBOOT_DEBUG("abspath   = %s\n", abspath);
    if ((subdir_buf == NULL) || (buf_size <= 0))
    {
        subdir_valid = 0;
    }
    else
    {
        subdir_valid = 1;
    }

    path = abspath;
    if (path[0] != '/')
    {
        UBOOT_ERROR("invalid abspath(%s), should start with '/'\n", abspath);
        return -1;
    }
    while (*path== '/')
    {
        path++;
    }
    partition_s = path;

    path = strstr(path,"/");
    if (path != NULL)
    {
        // Case : abspath --> /tvconfig/boot0.jpg
        plen = path - partition_s;
        partition = malloc(plen + 1);
        if (partition == NULL)
        {
            UBOOT_ERROR("can not get 0x%x byte from malloc\n", plen+1);
            return -1;
        }
        memset(partition, 0, plen+1);
        strncpy(partition, partition_s, plen);
        UBOOT_INFO("partition = %s\n",partition);
        ret = vfs_mount(partition);

        if (subdir_valid)
        {
            memset(subdir_buf, 0, buf_size);
            path += 1;  // skip '/'
            plen  = strlen(path);
            if (plen >= buf_size)
            {
                UBOOT_ERROR("subdir(%s) is too long, (len:%d >= buf_size:%d)\n", path, plen, buf_size);
                free(partition);
                return -1;
            }
            strncpy(subdir_buf, path, plen);
            UBOOT_DEBUG("subdir    = %s\n", subdir_buf);
        }
        free(partition);
    }
    else
    {
        // Case : abspath --> /tvconfig
        ret = vfs_mount(partition_s);
        UBOOT_DEBUG("partition = %s\n", partition_s);
    }

    UBOOT_TRACE("OK\n");
    return ret;
}
#endif

char *vfs_get_volname(void)
{
    return volName;
}
#endif

