///////////////////////////////////////////////////////////////////////////////////////////////////
/// @file   DbAmpDevice.c
/// @brief  Parse Amp device information from ini files.
/// @author MediaTek Inc.
///////////////////////////////////////////////////////////////////////////////////////////////////

//-------------------------------------------------------------------------------------------------
//  Include Files
//-------------------------------------------------------------------------------------------------
#include <stdio.h>
#include <stdlib.h>
#include <linux/string.h>

#include <MsTypes.h>

#include <common.h>
#include <command.h>
#include <malloc.h>
#include <MsDebug.h>
#include <drvIIC.h>
#include <CusI2C.h>

#include <bootlogo/iniparser.h>
#include <bootlogo/vector.h>
#include <bootlogo/DbAmpDevice.h>

//-------------------------------------------------------------------------------------------------
//  Local Compiler Options
//-------------------------------------------------------------------------------------------------
#define PARSER_DEBUG

//-------------------------------------------------------------------------------------------------
//  Debug Defines
//-------------------------------------------------------------------------------------------------
#ifdef IS_NULL
#undef IS_NULL
#endif //#ifdef IS_NULL
#ifdef IS_NOT_NULL
#undef IS_NOT_NULL
#endif //#ifdef IS_NOT_NULL
#define IS_NULL(pointer)                                (NULL == (pointer))
#define IS_NOT_NULL(pointer)                            (!(IS_NULL(pointer)))

#ifndef ASSERT_FAIL_RETURN_FALSE
#define ASSERT_FAIL_RETURN_FALSE(_x_)                                                            \
    do  {                                                                                   \
        if ( ! ( _x_ ) )                                                                    \
        {                                                                                   \
            UBOOT_DEBUG("ASSERT FAIL RETURN FALSE: %s %s %d\n", __FILE__, __PRETTY_FUNCTION__, __LINE__);     \
            return FALSE;                                                                   \
        }                                                                                   \
    } while (0)
#endif //#ifndef ASSERT_FAIL_RETURN_FALSE

#ifndef CHECK_PARAM_NULL_RETURN
#define CHECK_PARAM_NULL_RETURN(parameter, value) \
    do \
    { \
        if(IS_NULL(parameter)) \
        { \
            UBOOT_DEBUG("%s: %d: " #parameter " is NULL.\n", __func__, __LINE__); \
            return (value); \
        } \
    } while(0)
#endif //#ifndef CHECK_PARAM_NULL_RETURN

//-------------------------------------------------------------------------------------------------
//  Local Defines
//-------------------------------------------------------------------------------------------------
#ifndef AMPLIFIER_NONE
#define AMPLIFIER_NONE                                  (0)
#endif //#ifndef AMPLIFIER_NONE

#define E_MI_SYS_BOARD_INFO_ID_IIC_DEVICE_AUDIO_AMP     (0x465)
#define E_MI_SYS_BOARD_INFO_ID_IIC_DEVICE_AUDIO_AMP1    (0x467)
#define E_MI_SYS_BOARD_INFO_ID_IIC_DEVICE_AUDIO_AMP2    (0x468)
#define E_MI_SYS_BOARD_INFO_ID_IIC_DEVICE_AUDIO_AMP3    (0x469)

#define E_MI_SYS_BOARD_INFO_ID_GPIO_AUDIO_AMP_RESET     (0x31A)
#define E_MI_SYS_BOARD_INFO_ID_GPIO_AUDIO_AMP_STANDBY   (0x31B)
#define E_MI_SYS_BOARD_INFO_ID_GPIO_AUDIO_AMP_RESET1    (0x328)
#define E_MI_SYS_BOARD_INFO_ID_GPIO_AUDIO_AMP_STANDBY1  (0x329)
#define E_MI_SYS_BOARD_INFO_ID_GPIO_AUDIO_AMP_RESET2    (0x32A)
#define E_MI_SYS_BOARD_INFO_ID_GPIO_AUDIO_AMP_STANDBY2  (0x32B)
#define E_MI_SYS_BOARD_INFO_ID_GPIO_AUDIO_AMP_RESET3    (0x32C)
#define E_MI_SYS_BOARD_INFO_ID_GPIO_AUDIO_AMP_STANDBY3  (0x32D)

#define BOARD_INI_GPIO_DEVICE_SECTION                   "BOARD_GPIO_SETTING"
#define BOARD_INI_GPIO_DEVICE_TOTAL_NUMBER              "BOARD_GPIO_NUM"
#define BOARD_INI_GPIO_DEVICE_ID                        "BOARD_GPIO_%d_ID"
#define BOARD_INI_GPIO_DEVICE_IS_INVERTED               "Board_GPIO_%d_INVERTOR"
#define BOARD_INI_GPIO_DEVICE_PAD_ID                    "Board_GPIO_%d_PAD_NO"

#define BOARD_INI_I2C_DEVICE_SECTION                    "BOARD_I2C_DEVICE"
#define BOARD_INI_I2C_DEVICE_TOTAL_NUMBER               "BOARD_I2C_DEVICE_NUM"
#define BOARD_INI_I2C_DEVICE_ID                         "BOARD_I2C_DEVICE_%d_ID"
#define BOARD_INI_I2C_DEVICE_PORT                       "BOARD_I2C_DEVICE_%d_PORT"
#define BOARD_INI_I2C_DEVICE_SLAVE_ID                   "BOARD_I2C_DEVICE_%d_SLAVE_ID"

#define BOARD_INI_I2C_SWBUS_SECTION                     "BOARD_I2C_SWBUS"
#define BOARD_INI_I2C_SWBUS_TOTAL_NUMBER                "BOARD_I2C_SWBUS_NUM"
#define BOARD_INI_I2C_SWBUS_PORT                        "BOARD_I2C_SWBUS_%d_PORT"
#define BOARD_INI_I2C_SWBUS_SCL_PAD                     "BOARD_I2C_SWBUS_%d_SCL_PAD"
#define BOARD_INI_I2C_SWBUS_SDA_PAD                     "BOARD_I2C_SWBUS_%d_SDA_PAD"
#define BOARD_INI_I2C_SWBUS_DELAY                       "BOARD_I2C_SWBUS_%d_DELAY"

#define BOARD_INI_I2C_HWBUS_SECTION                     "BOARD_I2C_HWBUS"
#define BOARD_INI_I2C_HWBUS_TOTAL_NUMBER                "BOARD_I2C_HWBUS_NUM"
#define BOARD_INI_I2C_HWBUS_PIN_REG                     "BOARD_I2C_HWBUS_%d_PIN_REG"
#define BOARD_INI_I2C_HWBUS_PIN_BITS                    "BOARD_I2C_HWBUS_%d_PIN_BITS"
#define BOARD_INI_I2C_HWBUS_PIN_ENABLE                  "BOARD_I2C_HWBUS_%d_PIN_ENABLE"
#define BOARD_INI_I2C_HWBUS_SPEED                       "BOARD_I2C_HWBUS_%d_SPEED"
#define BOARD_INI_I2C_HWBUS_PORT                        "BOARD_I2C_HWBUS_%d_PORT"

#define AUDIO_AMP_CONFIG_INI_DEVICE_TOTAL_NUMBER        "DEVICE_TOTAL_NUMBER"
#define AUDIO_AMP_CONFIG_INI_DEVICE_SECTION             "AUDIO_AMP_DEVICE_%d"
#define AUDIO_AMP_CONFIG_INI_DEVICE_ID                  "DEVICE_ID"
#define AUDIO_AMP_CONFIG_INI_ATTR_TOTAL_NUMBER          "ATTR_TOTAL_NUMBER"
#define AUDIO_AMP_CONFIG_INI_ATTR_NAME                  "ATTR_%d_NAME"
#define AUDIO_AMP_CONFIG_INI_ATTR_DATA_TYPE             "ATTR_%d_DATA_TYPE"
#define AUDIO_AMP_CONFIG_INI_ATTR_DATA                  "ATTR_%d_DATA"

#define BOARD_INI_AUDIO_AMP_DEVICE_SECTION              "BOARD_AUDIO_AMP"
#define BOARD_INI_AUDIO_AMP_DEVICE_ID                   "BOARD_AUDIO_AMP_DEVICE_ID"
#define BOARD_INI_AUDIO_AMP_ATTR_TOTAL_NUMBER           "BOARD_AUDIO_AMP_ATTR_TOTAL_NUMBER"
#define BOARD_INI_AUDIO_AMP_ATTR_NAME                   "BOARD_AUDIO_AMP_ATTR_%d_NAME"
#define BOARD_INI_AUDIO_AMP_ATTR_DATA_TYPE              "BOARD_AUDIO_AMP_ATTR_%d_DATA_TYPE"
#define BOARD_INI_AUDIO_AMP_ATTR_DATA                   "BOARD_AUDIO_AMP_ATTR_%d_DATA"

#define JSON_ATTR_INFOS_ARRAY                           "stAttrInfos"

#define JSON_PUTS(pBuf, pBufMaxLen, pWrittenTotal, written) \
    do \
    { \
        int __ret; \
        __ret = jsmn_puts(pBuf, pBufMaxLen, pWrittenTotal, written); \
        if(0 > __ret) \
        { \
            UBOOT_DEBUG("%s: %d: jsmn_puts(...) is failed. __ret = %d.\n", __func__, __LINE__, __ret); \
            return __ret; \
        } \
    } while(0)

#define JSON_APPEND_INDENT(pBuf, pBufMaxLen, pWrittenTotal, indentLevel, indent) \
    do \
    { \
        int __ret; \
        __ret = jsmn_append_indent(pBuf, pBufMaxLen, pWrittenTotal, indentLevel, indentLevel); \
        if(0 > __ret) \
        { \
            UBOOT_DEBUG("%s: %d: jsmn_append_indent(...) is failed. __ret = %d.\n", __func__, __LINE__, __ret); \
            return __ret; \
        } \
    } while(0)

#define JSON_SNPRINTF(pBuf, pBufMaxLen, pWrittenTotal, args...) \
    do \
    { \
        int __ret; \
        __ret = jsmn_snprintf(pBuf, pBufMaxLen, pWrittenTotal, args); \
        if(0 > __ret) \
        { \
            UBOOT_DEBUG("%s: %d: jsmn_snprintf(...) is failed. __ret = %d.\n", __func__, __LINE__, __ret); \
            return __ret; \
        } \
    } while(0)

#define JSON_IS_NOT_TYPE_RETURN_ERROR_INVAL(toekn_type, token, token_pos) \
    do \
    { \
        if((toekn_type) != (token)[token_pos].type) \
        { \
            UBOOT_DEBUG("%s: %d: " #toekn_type " is expected, but it is %d.\n", __func__, __LINE__, (token)[token_pos].type); \
            (token_pos) = JSMN_ERROR_INVAL; \
            return (token_pos); \
        } \
    } while(0)

#define JSON_IS_NOT_EQ_RETURN_ERROR_INVAL(js, string_compared, token, token_pos) \
    do \
    { \
        if(0 != jsmn_eq(js, &((token)[token_pos]), string_compared)) \
        { \
            UBOOT_DEBUG("%s: %d: String %s expected\n", __func__, __LINE__, string_compared); \
            token_pos = JSMN_ERROR_INVAL; \
            return token_pos; \
        } \
    } while(0)

#define JSON_GET_VALUE(value, max_value_len, js, token, token_pos) \
    do \
    { \
        snprintf( \
                value, \
                max_value_len, \
                "%.*s", (token)[token_pos].end - (token)[token_pos].start, js + (token)[token_pos].start); \
    } while(0)

#define strtol                                          simple_strtol
#define strtoul                                         simple_strtoul

#define FREE(pointer) \
    if(IS_NOT_NULL(pointer)) \
    { \
        free(pointer); \
        (pointer) = NULL; \
    }

//-------------------------------------------------------------------------------------------------
//  Local Structures
//-------------------------------------------------------------------------------------------------

//-------------------------------------------------------------------------------------------------
//  Global Variables
//-------------------------------------------------------------------------------------------------

//-------------------------------------------------------------------------------------------------
//  Local Variables
//-------------------------------------------------------------------------------------------------

//-------------------------------------------------------------------------------------------------
//  Local Functions
//-------------------------------------------------------------------------------------------------

//-------------------------------------------------------------------------------------------------
//  Static Functions
//-------------------------------------------------------------------------------------------------
#ifdef PARSER_DEBUG
void _print_db_gpio_info(db_gpio_info_t stInfo, char *title)
{
    printf("%s\n", title);
    printf("m_u16GpioNum = %d\n", stInfo.m_u16GpioNum);
    printf("m_bInvert = %s\n", stInfo.m_bInvert? "TRUE": "FALSE");

    printf("\n");

    return;
}

void _print_db_iic_info(db_iic_info_t stInfo, char *title)
{
    printf("%s\n", title);
    printf("m_ePort = 0x%x\n", stInfo.m_ePort);
    printf("m_u16BusNumSlaveID =0x%x\n", stInfo.m_u16BusNumSlaveID);

    printf("\n");

    return;
}

void _print_db_amp_device_infos(db_amp_device_info_t stInfos[MAX_AMP_NUM])
{
    int i;
    char title[MAX_KEY_LEN];

    for(i = 0; i < MAX_AMP_NUM; i++)
    {
        if(AMPLIFIER_NONE == stInfos[i].m_u32DeviceId)
        {
            continue;
        }

        printf("\n");
        memset(title, 0, sizeof(title));
        snprintf(title, MAX_KEY_LEN, "stAmpDeviceInfos[%d]", i);
        printf("%s\n", title);

        printf("m_u32DeviceId = %d\n", stInfos[i].m_u32DeviceId);

        _print_db_gpio_info(stInfos[i].m_stGpioStandby, "m_stGpioStandby");
        _print_db_gpio_info(stInfos[i].m_stGpioReset, "m_stGpioReset");
        _print_db_iic_info(stInfos[i].m_stIicInfo, "m_stIicInfo");
    }

    return;
}
#endif //#ifdef PARSER_DEBUG

static int _parse_int(int *pValue, const char *psSection, char *psKey, int keyMaxLen, char *psKeyTemplate, int index)
{
    int ret = PARSE_SUCCESS;
    int value = 0;

    if(keyMaxLen <= snprintf(psKey, keyMaxLen, psKeyTemplate, index))
    {
        UBOOT_DEBUG("snprintf of %s %d is failed.\n", psKeyTemplate, index);
        ret = PARSE_PROCESSING_FAILED;

        return ret;
    }
    value = Profile_GetInteger(psSection, psKey, -1);
    UBOOT_DEBUG("%s = 0x%x\n", psKey, value);
    if(-1 == value)
    {
        UBOOT_DEBUG("Profile_GetInteger of %s: %s is failed.\n", psSection, psKey);
        ret = PARSE_NOTHING_FOUND;

        return ret;
    }

    *pValue = value;

    return ret;
}

static MS_BOOL _board_ini_get_attr_key(db_get_attr_key_params_t *pstGetAttrKeyParams)
{
    MS_BOOL bRet = TRUE;
    const char *psKeys[] =
    {
        BOARD_INI_AUDIO_AMP_ATTR_TOTAL_NUMBER,
        BOARD_INI_AUDIO_AMP_ATTR_NAME,
        BOARD_INI_AUDIO_AMP_ATTR_DATA_TYPE,
        BOARD_INI_AUDIO_AMP_ATTR_DATA,
    };

    switch(pstGetAttrKeyParams->m_eAttrKeyType)
    {
        case E_DB_ATTR_KEY_TYPE_ATTR_TOTAL_NUMBER:
            snprintf(
                    (char *)pstGetAttrKeyParams->m_stAttrKey.m_pu8Buffer,
                    pstGetAttrKeyParams->m_stAttrKey.m_u16BufferSize,
                    "%s", psKeys[pstGetAttrKeyParams->m_eAttrKeyType]);
            break;

        case E_DB_ATTR_KEY_TYPE_NAME:
        case E_DB_ATTR_KEY_TYPE_DATA_TYPE:
        case E_DB_ATTR_KEY_TYPE_DATA:
            snprintf(
                    (char *)pstGetAttrKeyParams->m_stAttrKey.m_pu8Buffer,
                    pstGetAttrKeyParams->m_stAttrKey.m_u16BufferSize,
                    psKeys[pstGetAttrKeyParams->m_eAttrKeyType],
                    pstGetAttrKeyParams->m_stAttrKeyParams.m_u16AttrNo);
            break;

        default:
            UBOOT_ERROR("%s: %d: pstAttrKeyInfo->eAttrKeyType(%d) is not supported.\n",
                    __func__,
                    __LINE__,
                    pstGetAttrKeyParams->m_eAttrKeyType);
            bRet = FALSE;
            break;

    }

    return bRet;
}

static MS_BOOL _audio_amp_config_ini_get_attr_key(db_get_attr_key_params_t *pstGetAttrKeyParams)
{
    MS_BOOL bRet = TRUE;
    const char *psKeys[] =
    {
        AUDIO_AMP_CONFIG_INI_ATTR_TOTAL_NUMBER,
        AUDIO_AMP_CONFIG_INI_ATTR_NAME,
        AUDIO_AMP_CONFIG_INI_ATTR_DATA_TYPE,
        AUDIO_AMP_CONFIG_INI_ATTR_DATA,
    };

    switch(pstGetAttrKeyParams->m_eAttrKeyType)
    {
        case E_DB_ATTR_KEY_TYPE_ATTR_TOTAL_NUMBER:
            snprintf(
                    (char *)pstGetAttrKeyParams->m_stAttrKey.m_pu8Buffer,
                    pstGetAttrKeyParams->m_stAttrKey.m_u16BufferSize,
                    "%s", psKeys[pstGetAttrKeyParams->m_eAttrKeyType]);
            break;

        case E_DB_ATTR_KEY_TYPE_NAME:
        case E_DB_ATTR_KEY_TYPE_DATA_TYPE:
        case E_DB_ATTR_KEY_TYPE_DATA:
            snprintf(
                    (char *)pstGetAttrKeyParams->m_stAttrKey.m_pu8Buffer,
                    pstGetAttrKeyParams->m_stAttrKey.m_u16BufferSize,
                    psKeys[pstGetAttrKeyParams->m_eAttrKeyType],
                    pstGetAttrKeyParams->m_stAttrKeyParams.m_u16AttrNo);
            break;

        default:
            UBOOT_ERROR("%s: %d: pstAttrKeyInfo->eAttrKeyType(%d) is not supported.\n",
                    __func__,
                    __LINE__,
                    pstGetAttrKeyParams->m_eAttrKeyType);
            bRet = FALSE;
            break;

    }

    return bRet;
}

static int _parse_board_ini_gpio_info(db_gpio_info_t stInfos[2][MAX_AMP_NUM], int ids[2][MAX_AMP_NUM])
{
    int ret = PARSE_SUCCESS;
    int i;
    int j;
    const char *psSection = BOARD_INI_GPIO_DEVICE_SECTION;
    int deviceTotalNumber;
    int value;
    int id;
    db_gpio_info_t stInfo = {0};
    char sKey[MAX_KEY_LEN];

    deviceTotalNumber = Profile_GetInteger(psSection, BOARD_INI_GPIO_DEVICE_TOTAL_NUMBER, 0);
    UBOOT_DEBUG("%s = %d\n", BOARD_INI_GPIO_DEVICE_TOTAL_NUMBER, deviceTotalNumber);

    for(i = 0; i < deviceTotalNumber; i++)
    {
        ret = _parse_int(&value, psSection, sKey, MAX_KEY_LEN, BOARD_INI_GPIO_DEVICE_ID, i);
        if(PARSE_SUCCESS != ret)
        {
            break;
        }
        id = value;

        ret = _parse_int(&value, psSection, sKey, MAX_KEY_LEN, BOARD_INI_GPIO_DEVICE_IS_INVERTED, i);
        if(PARSE_SUCCESS != ret)
        {
            break;
        }
        stInfo.m_bInvert = value;

        ret = _parse_int(&value, psSection, sKey, MAX_KEY_LEN, BOARD_INI_GPIO_DEVICE_PAD_ID, i);
        if(PARSE_SUCCESS != ret)
        {
            break;
        }
        stInfo.m_u16GpioNum = value;

        for(j = 0; j < MAX_AMP_NUM; j++)
        {
            if(id == ids[0][j])
            {
                memcpy(&stInfos[0][j], &stInfo, sizeof(db_gpio_info_t));

                break;
            }
            else if(id == ids[1][j])
            {
                memcpy(&stInfos[1][j], &stInfo, sizeof(db_gpio_info_t));

                break;
            }
        }
    }

    return ret;
}

static int _parse_board_ini_i2c_info(db_iic_info_t stInfos[MAX_AMP_NUM], int ids[MAX_AMP_NUM])
{
    int ret = PARSE_SUCCESS;
    int i;
    int j;
    const char *psSection = BOARD_INI_I2C_DEVICE_SECTION;
    int deviceTotalNumber;
    int value;
    int id;
    db_iic_info_t stInfo = {0};
    char sKey[MAX_KEY_LEN];

    deviceTotalNumber = Profile_GetInteger(psSection, BOARD_INI_I2C_DEVICE_TOTAL_NUMBER, 0);
    UBOOT_DEBUG("%s = %d\n", BOARD_INI_I2C_DEVICE_TOTAL_NUMBER, deviceTotalNumber);

    for(i = 0; i < deviceTotalNumber; i++)
    {
        ret = _parse_int(&value, psSection, sKey, MAX_KEY_LEN, BOARD_INI_I2C_DEVICE_ID, i);
        if(PARSE_SUCCESS != ret)
        {
            break;
        }
        id = value;

        ret = _parse_int(&value, psSection, sKey, MAX_KEY_LEN, BOARD_INI_I2C_DEVICE_PORT, i);
        if(PARSE_SUCCESS != ret)
        {
            break;
        }
        stInfo.m_ePort = value;

        ret = _parse_int(&value, psSection, sKey, MAX_KEY_LEN, BOARD_INI_I2C_DEVICE_SLAVE_ID, i);
        if(PARSE_SUCCESS != ret)
        {
            break;
        }
        stInfo.m_u16BusNumSlaveID = value;

        for(j = 0; j < MAX_AMP_NUM; j++)
        {
            if(id == ids[j])
            {
                memcpy(&stInfos[j], &stInfo, sizeof(db_iic_info_t));

                break;
            }
        }
    }

    return ret;
}

static int _parse_board_ini_sw_i2c_bus_info(db_iic_bus_info_t stInfos[MAX_IIC_BUS_NUM])
{
    int ret = PARSE_SUCCESS;
    int i;
    int j;
    const char *psSection = BOARD_INI_I2C_SWBUS_SECTION;
    int deviceTotalNumber;
    int value;
    db_iic_bus_info_t stInfo = {0};
    char sKey[MAX_KEY_LEN];

    deviceTotalNumber = Profile_GetInteger(psSection, BOARD_INI_I2C_SWBUS_TOTAL_NUMBER, 0);
    UBOOT_DEBUG("%s = %d\n", BOARD_INI_I2C_SWBUS_TOTAL_NUMBER, deviceTotalNumber);

    for(i = 0; i < deviceTotalNumber; i++)
    {
        stInfo.m_eBusType = E_DB_I2C_TYPE_SW;

        ret = _parse_int(&value, psSection, sKey, MAX_KEY_LEN, BOARD_INI_I2C_SWBUS_PORT, i);
        if(PARSE_SUCCESS != ret)
        {
            break;
        }
        stInfo.m_stSw.m_ePort = value;

        ret = _parse_int(&value, psSection, sKey, MAX_KEY_LEN, BOARD_INI_I2C_SWBUS_SCL_PAD, i);
        if(PARSE_SUCCESS != ret)
        {
            break;
        }
        stInfo.m_stSw.m_u16ClkPad = value;

        ret = _parse_int(&value, psSection, sKey, MAX_KEY_LEN, BOARD_INI_I2C_SWBUS_SDA_PAD, i);
        if(PARSE_SUCCESS != ret)
        {
            break;
        }
        stInfo.m_stSw.m_u16DataPad = value;

        ret = _parse_int(&value, psSection, sKey, MAX_KEY_LEN, BOARD_INI_I2C_SWBUS_DELAY, i);
        if(PARSE_SUCCESS != ret)
        {
            break;
        }
        stInfo.m_stSw.m_u16Delay = value;

        for(j = 0; j < MAX_IIC_BUS_NUM; j++)
        {
            if(E_DB_I2C_TYPE_NONE == stInfos[j].m_eBusType)
            {
                memcpy(&stInfos[j], &stInfo, sizeof(db_iic_bus_info_t));

                break;
            }
        }
        if(MAX_IIC_BUS_NUM <= j)
        {
            UBOOT_DEBUG("All stInfos are used.\n");
            ret = PARSE_PROCESSING_FAILED;

            break;
        }
    }

    return ret;
}

static int _parse_board_ini_hw_i2c_bus_info(db_iic_bus_info_t stInfos[MAX_IIC_BUS_NUM])
{
    int ret = PARSE_SUCCESS;
    int i;
    int j;
    const char *psSection = BOARD_INI_I2C_HWBUS_SECTION;
    int deviceTotalNumber;
    int value;
    db_iic_bus_info_t stInfo = {0};
    char sKey[MAX_KEY_LEN];

    deviceTotalNumber = Profile_GetInteger(psSection, BOARD_INI_I2C_HWBUS_TOTAL_NUMBER, 0);
    UBOOT_DEBUG("%s = %d\n", BOARD_INI_I2C_HWBUS_TOTAL_NUMBER, deviceTotalNumber);

    for(i = 0; i < deviceTotalNumber; i++)
    {
        stInfo.m_eBusType = E_DB_I2C_TYPE_HW;

        ret = _parse_int(&value, psSection, sKey, MAX_KEY_LEN, BOARD_INI_I2C_HWBUS_PIN_REG, i);
        if(PARSE_SUCCESS != ret)
        {
            break;
        }
        stInfo.m_stHw.m_stPinCfg.m_u32Reg = (unsigned)value;

        ret = _parse_int(&value, psSection, sKey, MAX_KEY_LEN, BOARD_INI_I2C_HWBUS_PIN_BITS, i);
        if(PARSE_SUCCESS != ret)
        {
            break;
        }
        stInfo.m_stHw.m_stPinCfg.m_u8BitPos = value;

        ret = _parse_int(&value, psSection, sKey, MAX_KEY_LEN, BOARD_INI_I2C_HWBUS_PIN_ENABLE, i);
        if(PARSE_SUCCESS != ret)
        {
            break;
        }
        stInfo.m_stHw.m_stPinCfg.m_bEnable = value;

        ret = _parse_int(&value, psSection, sKey, MAX_KEY_LEN, BOARD_INI_I2C_HWBUS_SPEED, i);
        if(PARSE_SUCCESS != ret)
        {
            break;
        }
        stInfo.m_stHw.m_u8Speed = value;

        ret = _parse_int(&value, psSection, sKey, MAX_KEY_LEN, BOARD_INI_I2C_HWBUS_PORT, i);
        if(PARSE_SUCCESS != ret)
        {
            break;
        }
        stInfo.m_stHw.m_ePort = value;

        for(j = 0; j < MAX_IIC_BUS_NUM; j++)
        {
            if(E_DB_I2C_TYPE_NONE == stInfos[j].m_eBusType)
            {
                memcpy(&stInfos[j], &stInfo, sizeof(db_iic_bus_info_t));

                break;
            }
        }
        if(MAX_IIC_BUS_NUM <= j)
        {
            UBOOT_DEBUG("All stInfos are used.\n");
            ret = PARSE_PROCESSING_FAILED;

            break;
        }
    }

    return ret;
}

/**
 * Object such as the following is expected.
 * {
 *  "stAttrInfos":
 *  [
 *     {
 *          "m_u8AttrTotalNumber":5,
 *          "m_pstAttr":
 *          [
 *                 {
 *                          "m_pszName":"Attr0",
 *                          "m_eDataType":0,
 *                          "m_stData":123
 *                 },
 *                 ...
 *          ]
 *     },
 *     ...
 *  ]
 * }
 */
static int _deserialize_db_attr_infos(db_attr_info_t stAttrInfos[MAX_AMP_NUM], const char *sSerializedAttrInfos, jsmntok_t *pToken)
{
    int currentTokenPos;
    int attrInfosSize = 0;
    int i;

    currentTokenPos = 0;

    JSON_IS_NOT_TYPE_RETURN_ERROR_INVAL(JSMN_OBJECT, pToken, currentTokenPos);
    currentTokenPos++;

    JSON_IS_NOT_EQ_RETURN_ERROR_INVAL(sSerializedAttrInfos, JSON_ATTR_INFOS_ARRAY, pToken, currentTokenPos);
    currentTokenPos++;

    JSON_IS_NOT_TYPE_RETURN_ERROR_INVAL(JSMN_ARRAY, pToken, currentTokenPos);
    attrInfosSize = pToken[currentTokenPos].size;
    currentTokenPos++;

    for(i = 0; i < attrInfosSize; i++)
    {
        currentTokenPos = deserialize_db_attr_info(&(stAttrInfos[i]), sSerializedAttrInfos, pToken, currentTokenPos);
        if(0 > currentTokenPos)
        {
            UBOOT_DEBUG("deserialize_db_attr_info(...) of stAttrInfos[%d] is failed.\n", i);
            stAttrInfos[i].m_u8AttrTotalNumber = 0;

            break;
        }
        currentTokenPos++;
    }

    return (currentTokenPos - 1);
}

/* Function _realloc_it() is a wrapper function for standard realloc()
 * with one difference - it frees old memory pointer in case of realloc
 * failure. Thus, DO NOT use old data pointer in anyway after call to
 * realloc_it(). If your code has some kind of fallback algorithm if
 * memory can't be re-allocated - use standard realloc() instead.
 */
static void *_realloc_it(void *pReallocateMem, size_t size)
{
    void *p = realloc(pReallocateMem, size);
    if(NULL == p)
    {
        FREE(pReallocateMem);
        UBOOT_DEBUG("realloc(..., %d) is failed\n", (int)size);
    }

    return p;
}

static int _get_sw_i2c_bus_info(db_iic_bus_info_t *pstInfo, db_iic_port_e ePort, db_iic_bus_info_t stIicBusInfos[MAX_IIC_BUS_NUM])
{
    int ret = CONVERT_INVALID;
    int i;

    if(NULL == pstInfo)
    {
        ret = -2;

        return ret;
    }

    for(i = 0; i < MAX_IIC_BUS_NUM; i++)
    {
        if((E_DB_I2C_TYPE_SW == stIicBusInfos[i].m_eBusType) &&
           (ePort == stIicBusInfos[i].m_stSw.m_ePort))
        {
            memcpy(pstInfo, &(stIicBusInfos[i]), sizeof(db_iic_bus_info_t));
            ret = CONVERT_SUCCESS;

            break;
        }
    }

    return ret;
}

//-------------------------------------------------------------------------------------------------
//  Global Functions
//-------------------------------------------------------------------------------------------------
int parse_board_ini_audio_amp_attr_info(db_attr_info_t* pstDbAttrInfo)
{
    int ret = PARSE_SUCCESS;
    const char *psSection = BOARD_INI_AUDIO_AMP_DEVICE_SECTION;
    db_attr_key_params_t stCfgAttrKeyParams;

    memset(&stCfgAttrKeyParams, 0, sizeof(db_attr_key_params_t));
    stCfgAttrKeyParams.m_psSection = psSection;
    stCfgAttrKeyParams.m_u16AttrNo = 0;
    ret = parse_db_attr(pstDbAttrInfo, _board_ini_get_attr_key, &stCfgAttrKeyParams);
    if(PARSE_SUCCESS != ret)
    {
        UBOOT_DEBUG("parse_db_attr(...) is failed.\n");
        pstDbAttrInfo->m_u8AttrTotalNumber = 0;

        return ret;
    }
#ifdef PARSER_DEBUG
    print_db_attr_info(pstDbAttrInfo);
#endif //#ifdef PARSER_DEBUG

    return ret;
}

int parse_board_ini_amp_info(db_amp_device_info_t stAmpDeviceInfos[MAX_AMP_NUM], db_iic_bus_info_t stIicBusInfos[MAX_IIC_BUS_NUM])
{
    int ret = PARSE_SUCCESS;
    int i;
    int value;
    db_iic_info_t stIicInfos[MAX_AMP_NUM];
    int i2cIds[MAX_AMP_NUM] =
    {
        E_MI_SYS_BOARD_INFO_ID_IIC_DEVICE_AUDIO_AMP,
        E_MI_SYS_BOARD_INFO_ID_IIC_DEVICE_AUDIO_AMP1,
        E_MI_SYS_BOARD_INFO_ID_IIC_DEVICE_AUDIO_AMP2,
        E_MI_SYS_BOARD_INFO_ID_IIC_DEVICE_AUDIO_AMP3,
    };
    db_gpio_info_t stGpioInfos[2][MAX_AMP_NUM];
    int gpioIds[2][MAX_AMP_NUM] =
    {
        {
            E_MI_SYS_BOARD_INFO_ID_GPIO_AUDIO_AMP_STANDBY,
            E_MI_SYS_BOARD_INFO_ID_GPIO_AUDIO_AMP_STANDBY1,
            E_MI_SYS_BOARD_INFO_ID_GPIO_AUDIO_AMP_STANDBY2,
            E_MI_SYS_BOARD_INFO_ID_GPIO_AUDIO_AMP_STANDBY3,
        },
        {
            E_MI_SYS_BOARD_INFO_ID_GPIO_AUDIO_AMP_RESET,
            E_MI_SYS_BOARD_INFO_ID_GPIO_AUDIO_AMP_RESET1,
            E_MI_SYS_BOARD_INFO_ID_GPIO_AUDIO_AMP_RESET2,
            E_MI_SYS_BOARD_INFO_ID_GPIO_AUDIO_AMP_RESET3,
        },
    };

    memset(stIicInfos, 0, sizeof(stIicInfos));
    memset(stGpioInfos, 0, sizeof(stGpioInfos));

    //BOARD_AUDIO_AMP
    value = Profile_GetInteger(BOARD_INI_AUDIO_AMP_DEVICE_SECTION, BOARD_INI_AUDIO_AMP_DEVICE_ID, 0);
    UBOOT_DEBUG("%s: %s = 0x%x \n", BOARD_INI_AUDIO_AMP_DEVICE_SECTION, BOARD_INI_AUDIO_AMP_DEVICE_ID, value);
    stAmpDeviceInfos[0].m_u32DeviceId = value;

    ret = _parse_board_ini_i2c_info(stIicInfos, i2cIds);
    if(0 != ret)
    {
        UBOOT_DEBUG("_parse_board_ini_i2c_info(...) is failed.\n");

        return ret;
    }
    for(i = 0; i < MAX_AMP_NUM; i++)
    {
        memcpy(&stAmpDeviceInfos[i].m_stIicInfo, &stIicInfos[i], sizeof(db_iic_info_t));
    }

    ret = _parse_board_ini_gpio_info(stGpioInfos, gpioIds);
    if(0 != ret)
    {
        UBOOT_DEBUG("_parse_board_ini_gpio_info(...) is failed.\n");

        return ret;
    }
    for(i = 0; i < MAX_AMP_NUM; i++)
    {
        memcpy(&stAmpDeviceInfos[i].m_stGpioStandby, &stGpioInfos[0][i], sizeof(db_gpio_info_t));
        memcpy(&stAmpDeviceInfos[i].m_stGpioReset, &stGpioInfos[1][i], sizeof(db_gpio_info_t));
    }
#ifdef PARSER_DEBUG
    _print_db_amp_device_infos(stAmpDeviceInfos);
#endif //#ifdef PARSER_DEBUG

    for(i = 0; i < MAX_IIC_BUS_NUM; i++)
    {
        memset(&stIicBusInfos[i], 0, sizeof(db_iic_bus_info_t));
        stIicBusInfos[i].m_eBusType = E_DB_I2C_TYPE_NONE;
    }
    ret = _parse_board_ini_sw_i2c_bus_info(stIicBusInfos);
    if(0 != ret)
    {
        UBOOT_DEBUG("_parse_board_ini_sw_i2c_bus_info(...) is failed.\n");

        return ret;
    }
    ret = _parse_board_ini_hw_i2c_bus_info(stIicBusInfos);
    if(0 != ret)
    {
        UBOOT_DEBUG("_parse_board_ini_hw_i2c_bus_info(...) is failed.\n");

        return ret;
    }

#ifdef PARSER_DEBUG
    print_db_iic_bus_infos(stIicBusInfos);
#endif //#ifdef PARSER_DEBUG

    return ret;
}

int parse_audio_amp_config_ini_info(db_amp_device_info_t stAmpDeviceInfos[MAX_AMP_NUM], db_attr_info_t stAttrInfos[MAX_AMP_NUM])
{
    int ret = PARSE_SUCCESS;
    int i;
    const char *psSection = "GENERAL";
    int deviceTotalNumber;
    int value;
    char section[MAX_SECTION_LEN];
    db_attr_key_params_t stCfgAttrKeyParams;

    deviceTotalNumber = Profile_GetInteger(psSection, AUDIO_AMP_CONFIG_INI_DEVICE_TOTAL_NUMBER, 0);
    UBOOT_DEBUG("%s = %d\n", AUDIO_AMP_CONFIG_INI_DEVICE_TOTAL_NUMBER, deviceTotalNumber);

    for(i = 0; i < deviceTotalNumber; i++)
    {
        if(MAX_SECTION_LEN <= snprintf(section, MAX_SECTION_LEN, AUDIO_AMP_CONFIG_INI_DEVICE_SECTION, i))
        {
            UBOOT_ERROR("snprintf of %s %d is failed.\n", AUDIO_AMP_CONFIG_INI_DEVICE_SECTION, i);
            ret = PARSE_PROCESSING_FAILED;

            break;
        }
        value = Profile_GetInteger(section, AUDIO_AMP_CONFIG_INI_DEVICE_ID, -1);
        UBOOT_DEBUG("%s = 0x%x \n", AUDIO_AMP_CONFIG_INI_DEVICE_ID, value);
        if(-1 == value)
        {
            UBOOT_ERROR("Profile_GetInteger of %s %d is not found.\n", AUDIO_AMP_CONFIG_INI_DEVICE_ID, i);
            ret = PARSE_NOTHING_FOUND;

            break;
        }
        stAmpDeviceInfos[i].m_u32DeviceId = value;

        memset(&stCfgAttrKeyParams, 0, sizeof(db_attr_key_params_t));
        stCfgAttrKeyParams.m_psSection = section;
        stCfgAttrKeyParams.m_u16AttrNo = 0;
        ret = parse_db_attr(&(stAttrInfos[i]), _audio_amp_config_ini_get_attr_key, &stCfgAttrKeyParams);
        if(PARSE_SUCCESS != ret)
        {
            UBOOT_DEBUG("parse_db_attr(...) of stAttrInfos[%d] is failed.\n", i);
            stAttrInfos[i].m_u8AttrTotalNumber = 0;

            break;
        }

        print_db_attr_info(&(stAttrInfos[i]));
    }

    return ret;
}

/**
 * String such as the following is generated.
 * {
 *  "stAttrInfos":
 *  [
 *     {
 *          "m_u8AttrTotalNumber":5,
 *          "m_pstAttr":
 *          [
 *                 {
 *                          "m_pszName":"Attr0",
 *                          "m_eDataType":0,
 *                          "m_stData":123
 *                 },
 *                 ...
 *          ]
 *     },
 *     ...
 *  ]
 * }
 */
int serialize_db_attr_infos(jsmn_serialize_params_t stSerializeParams, db_attr_info_t stAttrInfos[MAX_AMP_NUM])
{
    int written = 0;
    int writtenTotal = 0;
    int i;

    JSON_APPEND_INDENT(&stSerializeParams.psSerializedBuffer, &stSerializeParams.bufferMaxLen, &writtenTotal, stSerializeParams.indentlevel, stSerializeParams.indentPerLevel);
    JSON_SNPRINTF(&stSerializeParams.psSerializedBuffer, &stSerializeParams.bufferMaxLen, &writtenTotal, JSON_SERIALIZE_ITEM_OBJECT_START);
    JSON_SNPRINTF(&stSerializeParams.psSerializedBuffer, &stSerializeParams.bufferMaxLen, &writtenTotal, JSON_SERIALIZE_NEW_LINE);
    stSerializeParams.indentlevel += 1;

    JSON_APPEND_INDENT(&stSerializeParams.psSerializedBuffer, &stSerializeParams.bufferMaxLen, &writtenTotal, stSerializeParams.indentlevel, stSerializeParams.indentPerLevel);
    JSON_SNPRINTF(&stSerializeParams.psSerializedBuffer, &stSerializeParams.bufferMaxLen, &writtenTotal, JSON_SERIALIZE_NAME, JSON_ATTR_INFOS_ARRAY);
    JSON_SNPRINTF(&stSerializeParams.psSerializedBuffer, &stSerializeParams.bufferMaxLen, &writtenTotal, JSON_SERIALIZE_NEW_LINE);

    JSON_APPEND_INDENT(&stSerializeParams.psSerializedBuffer, &stSerializeParams.bufferMaxLen, &writtenTotal, stSerializeParams.indentlevel, stSerializeParams.indentPerLevel);
    JSON_SNPRINTF(&stSerializeParams.psSerializedBuffer, &stSerializeParams.bufferMaxLen, &writtenTotal, JSON_SERIALIZE_ITEM_ARRAY_START);
    JSON_SNPRINTF(&stSerializeParams.psSerializedBuffer, &stSerializeParams.bufferMaxLen, &writtenTotal, JSON_SERIALIZE_NEW_LINE);
    stSerializeParams.indentlevel += 1;

    for(i = 0; i < MAX_AMP_NUM; i++)
    {
        written = serialize_db_attr_info(stSerializeParams, &(stAttrInfos[i]));
        JSON_PUTS(&stSerializeParams.psSerializedBuffer, &stSerializeParams.bufferMaxLen, &writtenTotal, written);

        if((MAX_AMP_NUM - 1) != i)
        {
            JSON_SNPRINTF(&stSerializeParams.psSerializedBuffer, &stSerializeParams.bufferMaxLen, &writtenTotal, JSON_SERIALIZE_ITEM_SEPERATION);
        }
        JSON_SNPRINTF(&stSerializeParams.psSerializedBuffer, &stSerializeParams.bufferMaxLen, &writtenTotal, JSON_SERIALIZE_NEW_LINE);
    }

    stSerializeParams.indentlevel -= 1;
    JSON_APPEND_INDENT(&stSerializeParams.psSerializedBuffer, &stSerializeParams.bufferMaxLen, &writtenTotal, stSerializeParams.indentlevel, stSerializeParams.indentPerLevel);
    JSON_SNPRINTF(&stSerializeParams.psSerializedBuffer, &stSerializeParams.bufferMaxLen, &writtenTotal, JSON_SERIALIZE_ITEM_ARRAY_END);
    JSON_SNPRINTF(&stSerializeParams.psSerializedBuffer, &stSerializeParams.bufferMaxLen, &writtenTotal, JSON_SERIALIZE_NEW_LINE);

    stSerializeParams.indentlevel -= 1;
    JSON_APPEND_INDENT(&stSerializeParams.psSerializedBuffer, &stSerializeParams.bufferMaxLen, &writtenTotal, stSerializeParams.indentlevel, stSerializeParams.indentPerLevel);
    JSON_SNPRINTF(&stSerializeParams.psSerializedBuffer, &stSerializeParams.bufferMaxLen, &writtenTotal, JSON_SERIALIZE_ITEM_OBJECT_END);
    JSON_SNPRINTF(&stSerializeParams.psSerializedBuffer, &stSerializeParams.bufferMaxLen, &writtenTotal, JSON_SERIALIZE_NEW_LINE);

    if(stSerializeParams.bufferMaxLen < writtenTotal + 1)
    {
        UBOOT_DEBUG("writtenTotal(%d) is larger than (stSerializeParams.bufferMaxLen - 1)(%d).\n", writtenTotal, (stSerializeParams.bufferMaxLen - 1));

        return JSMN_ERROR_NOMEM;
    }
    stSerializeParams.psSerializedBuffer[0] = '\0';
    stSerializeParams.bufferMaxLen -= 1;
    writtenTotal += 1;

    UBOOT_DEBUG("writtenTotal = %d.\n", writtenTotal);

    return writtenTotal;
}

int deserialize_db_attr_infos(db_attr_info_t stAttrInfos[MAX_AMP_NUM], const char *psSerializedAttrInfos)
{
    int ret = 0;
    jsmn_parser p;
    jsmntok_t *pToken;
    size_t tokenCount = 512;

    /* Prepare parser */
    jsmn_init(&p);

    /* Allocate some tokens as a start */
    pToken = malloc(sizeof(jsmntok_t) * tokenCount);

    do
    {
        ret = jsmn_parse(&p, psSerializedAttrInfos, strlen(psSerializedAttrInfos), pToken, tokenCount);
        if(JSMN_ERROR_NOMEM == ret)
        {
            tokenCount = tokenCount * 2;
            pToken = _realloc_it(pToken, sizeof(jsmntok_t) * tokenCount);
            if(pToken == NULL)
            {
                // return error if it is failed to reallocate the memory.
                break;
            }
        }
    } while(JSMN_ERROR_NOMEM == ret);

    if(0 <= ret)
    {
        ret = _deserialize_db_attr_infos(stAttrInfos, psSerializedAttrInfos, pToken);
        if(0 > ret)
        {
            UBOOT_DEBUG("_deserialize_db_attr_infos(...) is failed.\n");
        }
    }

    FREE(pToken);

    return ret;
}

int convert_2_cus_iic_port_from_db(MS_IIC_PORT *pePortTo, db_iic_port_e ePortFrom, db_iic_bus_info_t stIicBusInfos[MAX_IIC_BUS_NUM])
{
    int ret = CONVERT_SUCCESS;
    db_iic_bus_info_t stIicInfo;
    SWI2C_BusCfg stI2CBusCfg;
    MS_U8 u8BusNum = 0;

    if(NULL == pePortTo)
    {
        UBOOT_DEBUG("pePortTo is NULL.\n");

        return CONVERT_INVALID;
    }

    *pePortTo = ePortFrom;
    if(E_DB_IIC_SW_PORT_0 <= ePortFrom)
    {
        memset(&stIicInfo, 0, sizeof(db_iic_bus_info_t));
        ret = _get_sw_i2c_bus_info(&stIicInfo, ePortFrom, stIicBusInfos);
        if(CONVERT_SUCCESS != ret)
        {
            UBOOT_DEBUG("_get_sw_i2c_bus_info(...) is failed.\n");

            return ret;
        }
        memset(&stI2CBusCfg, 0, sizeof(SWI2C_BusCfg));
        stI2CBusCfg.padSCL = stIicInfo.m_stSw.m_u16ClkPad;
        stI2CBusCfg.padSDA = stIicInfo.m_stSw.m_u16DataPad;
        ret = msI2C_convert_2_port(pePortTo, stI2CBusCfg);
        if(0 != ret)
        {
            UBOOT_DEBUG("msI2C_convert_2_port(...) is failed.\n");
            UBOOT_DEBUG("Set as E_MS_IIC_PORT_NOSUP instead.\n");
            *pePortTo = E_MS_IIC_PORT_NOSUP;
        }
        UBOOT_DEBUG("Convert from ePortFrom(0x%x) to 0x%x.\n", ePortFrom, *pePortTo);
    }
    else
    {
        // Do nothing
        UBOOT_DEBUG("No need to convert. ePortFrom = 0x%x.\n", ePortFrom);
    }

    return ret;
}

void print_db_iic_bus_infos(db_iic_bus_info_t stInfos[MAX_IIC_BUS_NUM])
{
    int i;
    char title[MAX_KEY_LEN];

    for(i = 0; i < MAX_IIC_BUS_NUM; i++)
    {
        printf("\n");
        memset(title, 0, sizeof(title));
        snprintf(title, MAX_KEY_LEN, "stIicBusInfos[%d]", i);
        printf("%s\n", title);

        switch(stInfos[i].m_eBusType)
        {
            case E_DB_I2C_TYPE_SW:
            case E_DB_I2C_TYPE_HW:
                printf("m_eBusType = %s\n", (E_DB_I2C_TYPE_SW == stInfos[i].m_eBusType)? "E_DB_I2C_TYPE_SW": "E_DB_I2C_TYPE_HW");
                break;

            default:
                break;
        }

        switch(stInfos[i].m_eBusType)
        {
            case E_DB_I2C_TYPE_SW:
                printf("m_ePort = 0x%x\n", stInfos[i].m_stSw.m_ePort);
                printf("m_u16ClkPad = %d\n", stInfos[i].m_stSw.m_u16ClkPad);
                printf("m_u16DataPad = %d\n", stInfos[i].m_stSw.m_u16DataPad);
                printf("m_u16Delay = %d\n", stInfos[i].m_stSw.m_u16Delay);
                break;

            case E_DB_I2C_TYPE_HW:
                printf("m_ePort = 0x%x\n", stInfos[i].m_stHw.m_ePort);
                printf("m_bEnable = %s\n", stInfos[i].m_stHw.m_stPinCfg.m_bEnable? "TRUE": "FALSE");
                printf("m_u32Reg = 0x%x\n", stInfos[i].m_stHw.m_stPinCfg.m_u32Reg);
                printf("m_u8BitPos = %d\n", stInfos[i].m_stHw.m_stPinCfg.m_u8BitPos);
                printf("m_u8Speed = %d\n", stInfos[i].m_stHw.m_u8Speed);
                break;

            default:
                break;
        }
    }

    return;
}
