///////////////////////////////////////////////////////////////////////////////////////////////////
/// @file   DbAttr.c
/// @brief  Parse Attr information from ini files.
/// @author MediaTek Inc.
///////////////////////////////////////////////////////////////////////////////////////////////////

//-------------------------------------------------------------------------------------------------
//  Include Files
//-------------------------------------------------------------------------------------------------
#include <stdio.h>
#include <stdlib.h>
#include <linux/string.h>

#include <MsTypes.h>

#include <common.h>
#include <command.h>
#include <malloc.h>
#include <MsDebug.h>

#include <bootlogo/iniparser.h>
#include <bootlogo/vector.h>
#include <bootlogo/DbAttr.h>

//-------------------------------------------------------------------------------------------------
//  Local Compiler Options
//-------------------------------------------------------------------------------------------------
#define PARSER_DEBUG

//-------------------------------------------------------------------------------------------------
//  Debug Defines
//-------------------------------------------------------------------------------------------------
#ifdef IS_NULL
#undef IS_NULL
#endif //#ifdef IS_NULL
#ifdef IS_NOT_NULL
#undef IS_NOT_NULL
#endif //#ifdef IS_NOT_NULL
#define IS_NULL(pointer)                                (NULL == (pointer))
#define IS_NOT_NULL(pointer)                            (!(IS_NULL(pointer)))

#ifndef ASSERT_FAIL_RETURN_FALSE
#define ASSERT_FAIL_RETURN_FALSE(_x_)                                                            \
    do  {                                                                                   \
        if ( ! ( _x_ ) )                                                                    \
        {                                                                                   \
            UBOOT_DEBUG("ASSERT FAIL RETURN FALSE: %s %s %d\n", __FILE__, __PRETTY_FUNCTION__, __LINE__);     \
            return FALSE;                                                                   \
        }                                                                                   \
    } while (0)
#endif //#ifndef ASSERT_FAIL_RETURN_FALSE

#ifndef CHECK_PARAM_NULL_RETURN
#define CHECK_PARAM_NULL_RETURN(parameter, value) \
    do \
    { \
        if(IS_NULL(parameter)) \
        { \
            UBOOT_DEBUG("%s: %d: " #parameter " is NULL.\n", __func__, __LINE__); \
            return (value); \
        } \
    } while(0)
#endif //#ifndef CHECK_PARAM_NULL_RETURN

//-------------------------------------------------------------------------------------------------
//  Local Defines
//-------------------------------------------------------------------------------------------------
#define INT_MIN                                         (-INT_MAX - 1)
#define INT_MAX                                         2147483647
#define UINT_MAX                                        4294967295U

#define CFG_BLOCK_COMMENT_START                         "/*"
#define CFG_BLOCK_COMMENT_END                           "*/"
#define ARRAY_DELIM                                     " ,"

#define JSON_ATTR_TOTAL_NUMBER                          "m_u8AttrTotalNumber"
#define JSON_ATTR_ARRAY                                 "m_pstAttr"
#define JSON_ATTR_NAME                                  "m_pszName"
#define JSON_ATTR_DATA_TYPE                             "m_eDataType"
#define JSON_ATTR_DATA                                  "m_stData"

#define JSON_PUTS(pBuf, pBufMaxLen, pWrittenTotal, written) \
    do \
    { \
        int __ret; \
        __ret = jsmn_puts(pBuf, pBufMaxLen, pWrittenTotal, written); \
        if(0 > __ret) \
        { \
            UBOOT_DEBUG("%s: %d: jsmn_puts(...) is failed. __ret = %d.\n", __func__, __LINE__, __ret); \
            return __ret; \
        } \
    } while(0)

#define JSON_APPEND_INDENT(pBuf, pBufMaxLen, pWrittenTotal, indentLevel, indent) \
    do \
    { \
        int __ret; \
        __ret = jsmn_append_indent(pBuf, pBufMaxLen, pWrittenTotal, indentLevel, indentLevel); \
        if(0 > __ret) \
        { \
            UBOOT_DEBUG("%s: %d: jsmn_append_indent(...) is failed. __ret = %d.\n", __func__, __LINE__, __ret); \
            return __ret; \
        } \
    } while(0)

#define JSON_SNPRINTF(pBuf, pBufMaxLen, pWrittenTotal, args...) \
    do \
    { \
        int __ret; \
        __ret = jsmn_snprintf(pBuf, pBufMaxLen, pWrittenTotal, args); \
        if(0 > __ret) \
        { \
            UBOOT_DEBUG("%s: %d: jsmn_snprintf(...) is failed. __ret = %d.\n", __func__, __LINE__, __ret); \
            return __ret; \
        } \
    } while(0)

#define JSON_IS_NOT_TYPE_RETURN_ERROR_INVAL(toekn_type, token, token_pos) \
    do \
    { \
        if((toekn_type) != (token)[token_pos].type) \
        { \
            UBOOT_DEBUG("%s: %d: " #toekn_type " is expected, but it is %d.\n", __func__, __LINE__, (token)[token_pos].type); \
            (token_pos) = JSMN_ERROR_INVAL; \
            return (token_pos); \
        } \
    } while(0)

#define JSON_IS_NOT_EQ_RETURN_ERROR_INVAL(js, string_compared, token, token_pos) \
    do \
    { \
        if(0 != jsmn_eq(js, &((token)[token_pos]), string_compared)) \
        { \
            UBOOT_DEBUG("%s: %d: String %s expected\n", __func__, __LINE__, string_compared); \
            token_pos = JSMN_ERROR_INVAL; \
            return token_pos; \
        } \
    } while(0)

#define JSON_GET_VALUE(value, max_value_len, js, token, token_pos) \
    do \
    { \
        snprintf( \
                value, \
                max_value_len, \
                "%.*s", (token)[token_pos].end - (token)[token_pos].start, js + (token)[token_pos].start); \
    } while(0)

#define FREE(pointer) \
    if(IS_NOT_NULL(pointer)) \
    { \
        free(pointer); \
        (pointer) = NULL; \
    }

#define OSAL_OS_STRTOK_R(str, delimit, saved_str) \
    ({ \
        char *token; \
        if(NULL != (str)) \
        { \
            (saved_str) = (char *)(str); \
        } \
        token = (char *)strtok_r((char *)str, (const char *)delimit, (char **)&(saved_str)); \
        UBOOT_INFO("token = %s.\n", (char *)token); \
        token; \
    })

#define strtol                                          simple_strtol
#define strtoul                                         simple_strtoul

//-------------------------------------------------------------------------------------------------
//  Local Structures
//-------------------------------------------------------------------------------------------------

//-------------------------------------------------------------------------------------------------
//  Global Variables
//-------------------------------------------------------------------------------------------------

//-------------------------------------------------------------------------------------------------
//  Local Variables
//-------------------------------------------------------------------------------------------------

//-------------------------------------------------------------------------------------------------
//  Local Functions
//-------------------------------------------------------------------------------------------------

//-------------------------------------------------------------------------------------------------
//  Static Functions
//-------------------------------------------------------------------------------------------------
#ifdef PARSER_DEBUG
static void _print_db_attr_array(db_attr_array_t *pstValue, db_attr_data_type_e eDataType)
{
    MS_U16 i = 0;

    if(NULL == pstValue)
    {
        printf("pstValue is NULL.\n");
        return;
    }

    switch(eDataType)
    {
        case E_DB_ATTR_DATA_TYPE_INT_ARRAY:
            printf("m_ps32Array = \n");

            break;

        case E_DB_ATTR_DATA_TYPE_UINT_ARRAY:
            printf("m_pu32Array = \n");

            break;

        default:

            break;
    }

    for(i = 0 ; i < pstValue->m_u16Size; i++)
    {
        switch(eDataType)
        {
            case E_DB_ATTR_DATA_TYPE_INT_ARRAY:
                if(pstValue->m_stArray.m_ps32Array[i] >= 0)
                {
                    printf("[%02d] = 0x%08x\n", i, (unsigned)pstValue->m_stArray.m_ps32Array[i]);
                }
                else
                {
                    printf("[%02d] = -0x%08x\n", i, (unsigned)-pstValue->m_stArray.m_ps32Array[i]);
                }

                break;

            case E_DB_ATTR_DATA_TYPE_UINT_ARRAY:
                printf("[%02d] = 0x%08x\n", i, (unsigned)pstValue->m_stArray.m_pu32Array[i]);

                break;

            default:
                // Do nothing.

                break;
        }
    }

    return;
}

static void _print_attr_int_array(MS_U16 u16Size, MS_S32 *ps32Array)
{
    MS_U16 i;

    if(NULL == ps32Array)
    {
        printf("ps32Array is NULL.\n");
        return;
    }
    if(0 == u16Size)
    {
        printf("u16Size is 0.\n");
        return;
    }

    for(i = 0; i < u16Size; i++)
    {
        printf("%08d, ", (int)ps32Array[i]);
    }

    printf("\n");
}

static void _print_attr_uint_array(MS_U16 u16Size, MS_U32 *pu32Array)
{
    MS_U16 i;

    if(NULL == pu32Array)
    {
        printf("pu32Array is NULL.\n");
        return;
    }
    if(0 == u16Size)
    {
        printf("u16Size is 0.\n");
        return;
    }

    for(i = 0; i < u16Size; i++)
    {
        printf("%08u, ", (unsigned)pu32Array[i]);
    }

    printf("\n");
}

static void _print_db_attr(db_attr_t *pstAttr)
{
    if(NULL == pstAttr)
    {
        printf("pstAttr is NULL.\n");
        return;
    }

    switch(pstAttr->m_eDataType)
    {
        case E_DB_ATTR_DATA_TYPE_INT:
            printf("m_s32Value = %08d.\n", pstAttr->m_stData.m_s32Value);
            break;

        case E_DB_ATTR_DATA_TYPE_UINT:
            printf("m_u32Value = %08u.\n", pstAttr->m_stData.m_u32Value);
            break;

        case E_DB_ATTR_DATA_TYPE_STRING:
            printf("m_pszValue = \"%s\".\n", (pstAttr->m_stData.m_pszValue)? (pstAttr->m_stData.m_pszValue): "");
            break;

        case E_DB_ATTR_DATA_TYPE_INT_ARRAY:
            _print_db_attr_array(&(pstAttr->m_stData.m_stValue), pstAttr->m_eDataType);
            break;

        case E_DB_ATTR_DATA_TYPE_UINT_ARRAY:
            _print_db_attr_array(&(pstAttr->m_stData.m_stValue), pstAttr->m_eDataType);
            break;
    }

    return;
}
#endif //#ifdef PARSER_DEBUG

static MS_BOOL _set_db_attr_array(db_attr_array_t *pstValue, db_attr_data_type_e eDataType, MS_U16 u16Size, void *pArray)
{
    MS_BOOL bRet = TRUE;

    UBOOT_INFO("%s: %d: eDataType = %d.\n",
            __func__,
            __LINE__,
            eDataType);
    switch(eDataType)
    {
        case E_DB_ATTR_DATA_TYPE_INT_ARRAY:
            pstValue->m_u16Size = u16Size;
            pstValue->m_stArray.m_ps32Array = pArray;
            break;

        case E_DB_ATTR_DATA_TYPE_UINT_ARRAY:
            pstValue->m_u16Size = u16Size;
            pstValue->m_stArray.m_pu32Array = pArray;
            break;

        default:
            UBOOT_DEBUG("%s: %d: eDataType(%d) is not supported.\n",
                    __func__,
                    __LINE__,
                    eDataType);

            pstValue->m_u16Size = 0;
            pstValue->m_stArray.m_ps32Array = NULL;
            bRet = FALSE;

            break;
    }

    return bRet;
}

static MS_BOOL _check_db_attr_array_value(db_attr_data_type_e eDataType, MS_S64 s64Value)
{
    MS_BOOL bRet = TRUE;

    UBOOT_INFO("%s: %d: eDataType = %d.\n",
            __func__,
            __LINE__,
            eDataType);
    switch(eDataType)
    {
        case E_DB_ATTR_DATA_TYPE_INT_ARRAY:
            if((s64Value < INT_MIN) || (s64Value > INT_MAX))
            {
                UBOOT_INFO("%s: %d: (s64Value(0x%llx) < INT_MIN(0x%llx)) || (s64Value > INT_MAX(0x%llx)).\n",
                        __func__,
                        __LINE__,
                        (long long)s64Value,
                        (long long)INT_MIN,
                        (long long)INT_MAX);
                bRet = FALSE;
            }

            break;

        case E_DB_ATTR_DATA_TYPE_UINT_ARRAY:
            if(s64Value > UINT_MAX)
            {
                UBOOT_INFO("%s: %d: s64Value(0x%llu) > UINT_MAX(0x%llu).\n",
                        __func__,
                        __LINE__,
                        (unsigned long long)s64Value,
                        (unsigned long long)UINT_MAX);
                bRet = FALSE;
            }

            break;

        default:
            UBOOT_DEBUG("%s: %d: eDataType(%d) is not supported.\n",
                    __func__,
                    __LINE__,
                    eDataType);
            bRet = FALSE;

            break;
    }

    return bRet;
}

static MS_BOOL _parse_string_to_vector_array(const char *psParsedStr, vector_Vector_t *pstOutputStrArray)
{
    MS_S16 s16LeftIdx = -1;
    MS_S16 s16RightIdx = -2;
    MS_S16 s16Len;
    char *psStr;
    char *psRestOfStr;
    char *psToken;
    MS_S16 i;

    ASSERT_FAIL_RETURN_FALSE(IS_NOT_NULL(psParsedStr));
    ASSERT_FAIL_RETURN_FALSE(IS_NOT_NULL(pstOutputStrArray));
    if(0 == strlen(psParsedStr))
    {
        UBOOT_DEBUG("psParsedStr's length is 0.\n");
        return FALSE;
    }

    for(i = 0; i < (MS_S16)strlen(psParsedStr); i++)
    {
        if (*(psParsedStr+i) == '{')
        {
            s16LeftIdx = i + 1;
            break;
        }
    }

    for(i = strlen(psParsedStr) - 1; i >= 0; i--)
    {
        if(*(psParsedStr+i) == '}')
        {
            s16RightIdx = i - 1;
            break;
        }
    }

    if(s16LeftIdx > s16RightIdx)
    {
        UBOOT_DEBUG("s16LeftIdx(%d) > s16RightIdx(%d) \n", s16LeftIdx, s16RightIdx);
        return FALSE;
    }

    s16Len = s16RightIdx - s16LeftIdx + 1;
    psStr = calloc(s16Len + 1, sizeof(char));
    ASSERT_FAIL_RETURN_FALSE(IS_NOT_NULL(psStr));
    strncpy((char *)psStr, (char *)psParsedStr + s16LeftIdx, s16Len);

    psToken = OSAL_OS_STRTOK_R(psStr, ARRAY_DELIM, psRestOfStr);
    while(psToken != NULL)
    {
        if(!vector_PushBack(pstOutputStrArray, psToken, (strlen(psToken) + 1) * sizeof(char)))
        {
            free(psStr);
            return FALSE;
        }
        psToken = OSAL_OS_STRTOK_R(psRestOfStr, ARRAY_DELIM, psRestOfStr);
    }

    free(psStr);

    return TRUE;
}

static MS_BOOL _parse_db_attr_array(db_attr_array_t *pstValue, db_attr_data_type_e eDataType, const char *psParsedStr)
{
    vector_Vector_t *pstVector = NULL;
    MS_U16 u16Idx = 0;
    const MS_U8 sErrFormtMsg[] = "Wrong Attr format from ini file.";
    const MS_U8 sErrParsingMsg[] = "Fail to parse string to array.";
    MS_S64 s64Value = 0;
    MS_S32 s32FilledResult = 0;
    MS_S32 *ps32Array = NULL;

    ASSERT_FAIL_RETURN_FALSE(IS_NOT_NULL(pstValue));
    ASSERT_FAIL_RETURN_FALSE(IS_NOT_NULL(psParsedStr));

    pstVector = vector_Init();
    ASSERT_FAIL_RETURN_FALSE(IS_NOT_NULL(pstVector));
    if(!_parse_string_to_vector_array(psParsedStr, pstVector))
    {
        vector_DeInit(pstVector);
        UBOOT_DEBUG("%s: %d: %s\n",
                __func__,
                __LINE__,
                sErrParsingMsg);
        UBOOT_DEBUG("%s: %d: pszParsedStr = \n%s\n",
                __func__,
                __LINE__,
                psParsedStr);

        return FALSE;
    }

    if(vector_Size(pstVector) == 0)
    {
        vector_DeInit(pstVector);
        UBOOT_DEBUG("%s: %d: %s\n",
                __func__,
                __LINE__,
                sErrFormtMsg);

        return FALSE;
    }

    ps32Array = calloc(vector_Size(pstVector), sizeof(MS_S32));
    if(IS_NULL(ps32Array))
    {
        UBOOT_DEBUG("%s: %d: It is failed to calloc(%d, %d).\n",
                __func__,
                __LINE__,
                vector_Size(pstVector),
                (int)sizeof(MS_S16));
        vector_DeInit(pstVector);

        return FALSE;
    }

    for(u16Idx = 0; u16Idx < vector_Size(pstVector); u16Idx++)
    {
        if(IS_NULL(vector_At(pstVector, u16Idx)))
        {
            UBOOT_DEBUG("%s: %d: %s\n",
                    __func__,
                    __LINE__,
                    sErrFormtMsg);
            vector_DeInit(pstVector);
            free(ps32Array);

            return FALSE;
        }

        s64Value = 0;
        s32FilledResult = sscanf(
                                VECTOR_GET_STRING(pstVector, u16Idx),
                                strstr(VECTOR_GET_STRING(pstVector, u16Idx), "0x")? "%x": "%d",
                                (unsigned int *)&s64Value);

        if(s32FilledResult <= 0)
        {
            UBOOT_DEBUG("%s: %d: %s: [%s]\n",
                    __func__,
                    __LINE__,
                    sErrFormtMsg,
                    VECTOR_GET_STRING(pstVector, u16Idx));
            vector_DeInit(pstVector);
            free(ps32Array);

            return FALSE;
        }
        if(!_check_db_attr_array_value(eDataType, s64Value))
        {
            UBOOT_DEBUG("%s: %d: %s\n",
                    __func__,
                    __LINE__,
                    sErrFormtMsg);
            vector_DeInit(pstVector);
            free(ps32Array);

            return FALSE;
        }

        ps32Array[u16Idx] = (MS_S32)(s64Value);
    }
    if(!_set_db_attr_array(pstValue, eDataType, vector_Size(pstVector), ps32Array))
    {
        // The return value is not handled because it will be reset when it is failed in _set_db_attr_array.
        UBOOT_DEBUG("%s: %d: It is failed to _set_db_attr_array(...).\n",
                __func__,
                __LINE__);
    }

    vector_DeInit(pstVector);

    return TRUE;
}

static MS_BOOL _strip_block_comment(char *psStart)
{
    char *psEnd = NULL;

    CHECK_PARAM_NULL_RETURN(psStart, FALSE);
    if(0 == strlen(psStart))
    {
        UBOOT_INFO("%s: %d: strlen(pszStripped) is 0.\n",
                __func__,
                __LINE__);

        return TRUE;
    }

    if(IS_NULL(psStart = strstr(psStart, CFG_BLOCK_COMMENT_START)))
    {
        return TRUE;
    }
    if(IS_NULL(psEnd = strstr(psStart + strlen(CFG_BLOCK_COMMENT_START), CFG_BLOCK_COMMENT_END)))
    {
        return TRUE;
    }
    psEnd += strlen(CFG_BLOCK_COMMENT_END);
    memmove(
        psStart,
        psEnd,
        strlen(psStart) + 1 - (psEnd - psStart));

    return _strip_block_comment(psStart);
}

static int _parse_db_attr_name(db_attr_t *pstAttr, db_get_attr_key pfGetAttrKey, db_attr_key_params_t *pstAttrKeyParams)
{
    int ret = PARSE_SUCCESS;
    char sKey[MAX_KEY_LEN];
    db_get_attr_key_params_t stGetAttrKeyParams;
    int strLen;

    memset(&stGetAttrKeyParams, 0 , sizeof(db_get_attr_key_params_t));
    stGetAttrKeyParams.m_stAttrKey.m_pu8Buffer = (MS_U8 *)sKey;
    stGetAttrKeyParams.m_stAttrKey.m_u16BufferSize = MAX_KEY_LEN;
    stGetAttrKeyParams.m_eAttrKeyType = E_DB_ATTR_KEY_TYPE_NAME;
    stGetAttrKeyParams.m_stAttrKeyParams.m_u16AttrNo = pstAttrKeyParams->m_u16AttrNo;
    if(!pfGetAttrKey(&stGetAttrKeyParams))
    {
        UBOOT_DEBUG("%s: %d: Error: It is failed to pfGetAttrKey(...).\n",
                __func__,
                __LINE__);
        ret = PARSE_PROCESSING_FAILED;

        return ret;
    }

    pstAttr->m_pszName = calloc(MAX_NAME_LEN, sizeof(char));
    if(NULL == pstAttr->m_pszName)
    {
        UBOOT_DEBUG("calloc(%d, %d) of pstAttr->m_pszName is failed.\n", MAX_NAME_LEN, (int)sizeof(char));
        ret = PARSE_PROCESSING_FAILED;

        return ret;
    }

    do
    {
        strLen = Profile_GetString(pstAttrKeyParams->m_psSection, sKey, "", pstAttr->m_pszName, MAX_NAME_LEN);
        if(0 >= strLen)
        {
            UBOOT_DEBUG("[%s] %s is NULL.\n", pstAttrKeyParams->m_psSection, sKey);
            if(pstAttr->m_pszName)
            {
                free(pstAttr->m_pszName);
                pstAttr->m_pszName = NULL;
            }
            ret = PARSE_NOTHING_FOUND;

            break;
        }
        if(MAX_NAME_LEN < strLen + 1)
        {
            UBOOT_DEBUG("[%s] %s is larger than %d.\n", pstAttrKeyParams->m_psSection, sKey, (MAX_NAME_LEN - 1));
            if(pstAttr->m_pszName)
            {
                free(pstAttr->m_pszName);
                pstAttr->m_pszName = NULL;
            }
            ret = PARSE_PROCESSING_FAILED;

            break;
        }
        UBOOT_DEBUG("%s = \"%s\"\n", sKey, (pstAttr->m_pszName)? (pstAttr->m_pszName): "");
    } while(0);

    return ret;
}

static int _parse_db_attr_data_type(db_attr_t *pstAttr, db_get_attr_key pfGetAttrKey, db_attr_key_params_t *pstAttrKeyParams)
{
    int ret = PARSE_SUCCESS;
    char sKey[MAX_KEY_LEN];
    db_get_attr_key_params_t stGetAttrKeyParams;
    int value;

    memset(&stGetAttrKeyParams, 0 , sizeof(db_get_attr_key_params_t));
    stGetAttrKeyParams.m_stAttrKey.m_pu8Buffer = (MS_U8 *)sKey;
    stGetAttrKeyParams.m_stAttrKey.m_u16BufferSize = MAX_KEY_LEN;
    stGetAttrKeyParams.m_eAttrKeyType = E_DB_ATTR_KEY_TYPE_DATA_TYPE;
    stGetAttrKeyParams.m_stAttrKeyParams.m_u16AttrNo = pstAttrKeyParams->m_u16AttrNo;
    if(!pfGetAttrKey(&stGetAttrKeyParams))
    {
        UBOOT_DEBUG("%s: %d: Error: It is failed to pfGetAttrKey(...).\n",
                __func__,
                __LINE__);
        ret = PARSE_PROCESSING_FAILED;

        return ret;
    }
    value = Profile_GetInteger(pstAttrKeyParams->m_psSection, sKey, -1);
    UBOOT_DEBUG("%s = 0x%x \n", sKey, value);
    if(-1 == value)
    {
        UBOOT_DEBUG("[%s] %s is failed.\n", pstAttrKeyParams->m_psSection, sKey);
        ret = PARSE_NOTHING_FOUND;

        return ret;
    }
    pstAttr->m_eDataType = value;

    return ret;
}

static int _parse_attr_data(db_attr_t *pstAttr, db_get_attr_key pfGetAttrKey, db_attr_key_params_t *pstAttrKeyParams)
{
    int ret = PARSE_SUCCESS;
    char sKey[MAX_KEY_LEN];
    db_get_attr_key_params_t stGetAttrKeyParams;
    int value;
    int strLen;
    char *psStrippedStr = NULL;

    memset(&stGetAttrKeyParams, 0 , sizeof(db_get_attr_key_params_t));
    stGetAttrKeyParams.m_stAttrKey.m_pu8Buffer = (MS_U8 *)sKey;
    stGetAttrKeyParams.m_stAttrKey.m_u16BufferSize = MAX_KEY_LEN;
    stGetAttrKeyParams.m_eAttrKeyType = E_DB_ATTR_KEY_TYPE_DATA;
    stGetAttrKeyParams.m_stAttrKeyParams.m_u16AttrNo = pstAttrKeyParams->m_u16AttrNo;
    if(!pfGetAttrKey(&stGetAttrKeyParams))
    {
        UBOOT_DEBUG("%s: %d: Error: It is failed to pfGetAttrKey(...).\n",
                __func__,
                __LINE__);
        ret = PARSE_PROCESSING_FAILED;

        return ret;
    }
    switch(pstAttr->m_eDataType)
    {
        case E_DB_ATTR_DATA_TYPE_INT:
            value = Profile_GetInteger(pstAttrKeyParams->m_psSection, sKey, 0);
            pstAttr->m_stData.m_s32Value = value;
            UBOOT_DEBUG("%s = 0x%x\n", sKey, value);
            break;

        case E_DB_ATTR_DATA_TYPE_UINT:
            value = Profile_GetInteger(pstAttrKeyParams->m_psSection, sKey, 0);
            pstAttr->m_stData.m_u32Value = (unsigned)value;
            UBOOT_DEBUG("%s = 0x%x\n", sKey, value);
            break;

        case E_DB_ATTR_DATA_TYPE_STRING:
            pstAttr->m_stData.m_pszValue = calloc(MAX_VALUE_LEN, sizeof(char));
            if(NULL == pstAttr->m_stData.m_pszValue)
            {
                UBOOT_DEBUG("calloc(%d, %d) of pstAttr->m_stData.m_pszValue is failed.\n", MAX_VALUE_LEN, (int)sizeof(char));
                ret = PARSE_PROCESSING_FAILED;

                break;
            }

            strLen = Profile_GetString(pstAttrKeyParams->m_psSection, sKey, "", pstAttr->m_stData.m_pszValue, MAX_VALUE_LEN * sizeof(char));
            if(MAX_VALUE_LEN < strLen + 1)
            {
                UBOOT_DEBUG("[%s] %s is larger than %d.\n", pstAttrKeyParams->m_psSection, sKey, (MAX_VALUE_LEN - 1));
                if(pstAttr->m_stData.m_pszValue)
                {
                    free(pstAttr->m_stData.m_pszValue);
                    pstAttr->m_stData.m_pszValue = NULL;
                }
                ret = PARSE_PROCESSING_FAILED;

                break;
            }
            UBOOT_DEBUG("%s = \"%s\"\n", sKey, pstAttr->m_stData.m_pszValue);
            break;

        case E_DB_ATTR_DATA_TYPE_INT_ARRAY:
        case E_DB_ATTR_DATA_TYPE_UINT_ARRAY:
            do
            {
                psStrippedStr = calloc(MAX_VALUE_LEN, sizeof(char));
                if(IS_NULL(psStrippedStr))
                {
                    UBOOT_DEBUG("calloc(%d, %d) is failed.\n", MAX_VALUE_LEN, (int)sizeof(char));
                    ret = PARSE_PROCESSING_FAILED;

                    break;
                }
                strLen = Profile_GetString(pstAttrKeyParams->m_psSection, sKey, "", psStrippedStr, MAX_VALUE_LEN * sizeof(char));
                UBOOT_DEBUG("%s = %s\n", sKey, (psStrippedStr)? (psStrippedStr): "");
                if(MAX_VALUE_LEN < strLen + 1)
                {
                    UBOOT_DEBUG("[%s] %s is larger than %d.\n", pstAttrKeyParams->m_psSection, sKey, (MAX_VALUE_LEN - 1));
                    ret = PARSE_PROCESSING_FAILED;

                    break;
                }

                if(!_strip_block_comment(psStrippedStr))
                {
                    UBOOT_DEBUG("_strip_block_comment(...) is failed.\n");
                    ret = PARSE_PROCESSING_FAILED;

                    break;
                }
                UBOOT_DEBUG("Stripped\n");
                UBOOT_DEBUG("%s = %s\n", sKey, (psStrippedStr)? (psStrippedStr): "");
                if(!_parse_db_attr_array(&(pstAttr->m_stData.m_stValue), pstAttr->m_eDataType, psStrippedStr))
                {
                    UBOOT_DEBUG("_parse_db_attr_array(...) is failed.\n");
                    ret = PARSE_PROCESSING_FAILED;

                    break;
                }
            } while(0);

            if(psStrippedStr)
            {
                free(psStrippedStr);
                psStrippedStr = NULL;
            }
            if(PARSE_SUCCESS != ret)
            {
                pstAttr->m_stData.m_stValue.m_u16Size = 0;
                pstAttr->m_stData.m_stValue.m_stArray.m_pu32Array = NULL;
            }
            break;

        default:
            UBOOT_DEBUG("pstAttr->m_eDataType(%d) is not supported.\n", pstAttr->m_eDataType);
            break;
    }

    return ret;
}

static int _serialize_DbAttr(jsmn_serialize_params_t stSerializeParams, db_attr_t *pstAttr)
{
    int writtenTotal = 0;
    int i;

    JSON_APPEND_INDENT(&stSerializeParams.psSerializedBuffer, &stSerializeParams.bufferMaxLen, &writtenTotal, stSerializeParams.indentlevel, stSerializeParams.indentPerLevel);
    JSON_SNPRINTF(&stSerializeParams.psSerializedBuffer, &stSerializeParams.bufferMaxLen, &writtenTotal, JSON_SERIALIZE_ITEM_OBJECT_START);
    JSON_SNPRINTF(&stSerializeParams.psSerializedBuffer, &stSerializeParams.bufferMaxLen, &writtenTotal, JSON_SERIALIZE_NEW_LINE);
    stSerializeParams.indentlevel += 1;

    JSON_APPEND_INDENT(&stSerializeParams.psSerializedBuffer, &stSerializeParams.bufferMaxLen, &writtenTotal, stSerializeParams.indentlevel, stSerializeParams.indentPerLevel);
    JSON_SNPRINTF(&stSerializeParams.psSerializedBuffer, &stSerializeParams.bufferMaxLen, &writtenTotal, JSON_SERIALIZE_NAME, JSON_ATTR_NAME);
    JSON_SNPRINTF(&stSerializeParams.psSerializedBuffer, &stSerializeParams.bufferMaxLen, &writtenTotal, JSON_SERIALIZE_VALUE_STRING, pstAttr->m_pszName);
    JSON_SNPRINTF(&stSerializeParams.psSerializedBuffer, &stSerializeParams.bufferMaxLen, &writtenTotal, JSON_SERIALIZE_ITEM_SEPERATION);
    JSON_SNPRINTF(&stSerializeParams.psSerializedBuffer, &stSerializeParams.bufferMaxLen, &writtenTotal, JSON_SERIALIZE_NEW_LINE);

    JSON_APPEND_INDENT(&stSerializeParams.psSerializedBuffer, &stSerializeParams.bufferMaxLen, &writtenTotal, stSerializeParams.indentlevel, stSerializeParams.indentPerLevel);
    JSON_SNPRINTF(&stSerializeParams.psSerializedBuffer, &stSerializeParams.bufferMaxLen, &writtenTotal, JSON_SERIALIZE_NAME, JSON_ATTR_DATA_TYPE);
    JSON_SNPRINTF(&stSerializeParams.psSerializedBuffer, &stSerializeParams.bufferMaxLen, &writtenTotal, JSON_SERIALIZE_VALUE_INT, pstAttr->m_eDataType);
    JSON_SNPRINTF(&stSerializeParams.psSerializedBuffer, &stSerializeParams.bufferMaxLen, &writtenTotal, JSON_SERIALIZE_ITEM_SEPERATION);
    JSON_SNPRINTF(&stSerializeParams.psSerializedBuffer, &stSerializeParams.bufferMaxLen, &writtenTotal, JSON_SERIALIZE_NEW_LINE);

    JSON_APPEND_INDENT(&stSerializeParams.psSerializedBuffer, &stSerializeParams.bufferMaxLen, &writtenTotal, stSerializeParams.indentlevel, stSerializeParams.indentPerLevel);
    JSON_SNPRINTF(&stSerializeParams.psSerializedBuffer, &stSerializeParams.bufferMaxLen, &writtenTotal, JSON_SERIALIZE_NAME, JSON_ATTR_DATA);
    switch(pstAttr->m_eDataType)
    {
        case E_DB_ATTR_DATA_TYPE_INT:
            JSON_SNPRINTF(&stSerializeParams.psSerializedBuffer, &stSerializeParams.bufferMaxLen, &writtenTotal, JSON_SERIALIZE_VALUE_INT, pstAttr->m_stData.m_s32Value);
            break;

        case E_DB_ATTR_DATA_TYPE_UINT:
            JSON_SNPRINTF(&stSerializeParams.psSerializedBuffer, &stSerializeParams.bufferMaxLen, &writtenTotal, JSON_SERIALIZE_VALUE_INT, pstAttr->m_stData.m_u32Value);
            break;

        case E_DB_ATTR_DATA_TYPE_STRING:
            JSON_SNPRINTF(&stSerializeParams.psSerializedBuffer, &stSerializeParams.bufferMaxLen, &writtenTotal, JSON_SERIALIZE_VALUE_STRING, pstAttr->m_stData.m_pszValue);
            break;

        case E_DB_ATTR_DATA_TYPE_INT_ARRAY:
            JSON_SNPRINTF(&stSerializeParams.psSerializedBuffer, &stSerializeParams.bufferMaxLen, &writtenTotal, JSON_SERIALIZE_ITEM_ARRAY_START);
            for(i = 0; i < pstAttr->m_stData.m_stValue.m_u16Size; i++)
            {
                JSON_SNPRINTF(&stSerializeParams.psSerializedBuffer, &stSerializeParams.bufferMaxLen, &writtenTotal, JSON_SERIALIZE_VALUE_INT, pstAttr->m_stData.m_stValue.m_stArray.m_ps32Array[i]);
                if(i != (pstAttr->m_stData.m_stValue.m_u16Size - 1))
                {
                    JSON_SNPRINTF(&stSerializeParams.psSerializedBuffer, &stSerializeParams.bufferMaxLen, &writtenTotal, JSON_SERIALIZE_ITEM_SEPERATION);
                }
            }
            JSON_SNPRINTF(&stSerializeParams.psSerializedBuffer, &stSerializeParams.bufferMaxLen, &writtenTotal, JSON_SERIALIZE_ITEM_ARRAY_END);
            break;

        case E_DB_ATTR_DATA_TYPE_UINT_ARRAY:
            JSON_SNPRINTF(&stSerializeParams.psSerializedBuffer, &stSerializeParams.bufferMaxLen, &writtenTotal, JSON_SERIALIZE_ITEM_ARRAY_START);
            for(i = 0; i < pstAttr->m_stData.m_stValue.m_u16Size; i++)
            {
                JSON_SNPRINTF(&stSerializeParams.psSerializedBuffer, &stSerializeParams.bufferMaxLen, &writtenTotal, JSON_SERIALIZE_VALUE_INT, pstAttr->m_stData.m_stValue.m_stArray.m_pu32Array[i]);
                if(i != (pstAttr->m_stData.m_stValue.m_u16Size - 1))
                {
                    JSON_SNPRINTF(&stSerializeParams.psSerializedBuffer, &stSerializeParams.bufferMaxLen, &writtenTotal, JSON_SERIALIZE_ITEM_SEPERATION);
                }
            }
            JSON_SNPRINTF(&stSerializeParams.psSerializedBuffer, &stSerializeParams.bufferMaxLen, &writtenTotal, JSON_SERIALIZE_ITEM_ARRAY_END);
            break;

        default:
            UBOOT_DEBUG("%s: %d: pstAttr->m_eDataType(%d) is not supported.\n", __func__, __LINE__, pstAttr->m_eDataType);
            writtenTotal = -1;
            break;
    }
    JSON_SNPRINTF(&stSerializeParams.psSerializedBuffer, &stSerializeParams.bufferMaxLen, &writtenTotal, JSON_SERIALIZE_NEW_LINE);

    stSerializeParams.indentlevel -= 1;
    JSON_APPEND_INDENT(&stSerializeParams.psSerializedBuffer, &stSerializeParams.bufferMaxLen, &writtenTotal, stSerializeParams.indentlevel, stSerializeParams.indentPerLevel);
    JSON_SNPRINTF(&stSerializeParams.psSerializedBuffer, &stSerializeParams.bufferMaxLen, &writtenTotal, JSON_SERIALIZE_ITEM_OBJECT_END);

    return writtenTotal;
}

static int _deserialize_db_int_array(db_attr_t *pstAttr, const char *sSerializedAttrInfos, jsmntok_t *pToken, int currentTokenPos)
{
    char value[MAX_VALUE_LEN];
    int i;

    JSON_IS_NOT_TYPE_RETURN_ERROR_INVAL(JSMN_ARRAY, pToken, currentTokenPos);
    pstAttr->m_stData.m_stValue.m_u16Size = pToken[currentTokenPos].size;
    currentTokenPos++;

    pstAttr->m_stData.m_stValue.m_stArray.m_ps32Array = calloc(pstAttr->m_stData.m_stValue.m_u16Size, sizeof(MS_S32));
    if(NULL == pstAttr->m_stData.m_stValue.m_stArray.m_ps32Array)
    {
        UBOOT_DEBUG("calloc(%d, %d) of pstAttr->m_stData.m_stValue.m_stArray.m_ps32Array is failed.\n", pstAttr->m_stData.m_stValue.m_u16Size, (int)sizeof(MS_S32));
        currentTokenPos = JSMN_ERROR_NOMEM;

        return currentTokenPos;
//        return (0 < currentTokenPos)? (currentTokenPos - 1): (currentTokenPos);
    }

    for(i = 0; i < pstAttr->m_stData.m_stValue.m_u16Size; i++)
    {
        JSON_IS_NOT_TYPE_RETURN_ERROR_INVAL(JSMN_PRIMITIVE, pToken, currentTokenPos);
        JSON_GET_VALUE(value, MAX_VALUE_LEN, sSerializedAttrInfos, pToken, currentTokenPos);
        pstAttr->m_stData.m_stValue.m_stArray.m_ps32Array[i] = strtol(value, NULL, 0);
        currentTokenPos++;
    }

    return (0 < currentTokenPos)? (currentTokenPos - 1): (currentTokenPos);
}

static int _deserialize_db_uint_array(db_attr_t *pstAttr, const char *sSerializedAttrInfos, jsmntok_t *pToken, int currentTokenPos)
{
    char value[MAX_VALUE_LEN];
    int i;

    JSON_IS_NOT_TYPE_RETURN_ERROR_INVAL(JSMN_ARRAY, pToken, currentTokenPos);
    pstAttr->m_stData.m_stValue.m_u16Size = pToken[currentTokenPos].size;
    currentTokenPos++;

    pstAttr->m_stData.m_stValue.m_stArray.m_pu32Array = calloc(pstAttr->m_stData.m_stValue.m_u16Size, (int)sizeof(MS_S32));
    if(NULL == pstAttr->m_stData.m_stValue.m_stArray.m_pu32Array)
    {
        UBOOT_DEBUG("calloc(%d, %d) of pstAttr->m_stData.m_stValue.m_stArray.m_pu32Array is failed.\n", pstAttr->m_stData.m_stValue.m_u16Size, (int)sizeof(MS_S32));
        currentTokenPos = JSMN_ERROR_NOMEM;

        return currentTokenPos;
//        return (0 < currentTokenPos)? (currentTokenPos - 1): (currentTokenPos);
    }

    for(i = 0; i < pstAttr->m_stData.m_stValue.m_u16Size; i++)
    {
        JSON_IS_NOT_TYPE_RETURN_ERROR_INVAL(JSMN_PRIMITIVE, pToken, currentTokenPos);
        JSON_GET_VALUE(value, MAX_VALUE_LEN, sSerializedAttrInfos, pToken, currentTokenPos);
        pstAttr->m_stData.m_stValue.m_stArray.m_pu32Array[i] = strtoul(value, NULL, 0);
        currentTokenPos++;
    }

    return (0 < currentTokenPos)? (currentTokenPos - 1): (currentTokenPos);
}

static int _deserialize_db_attr(db_attr_t *pstAttr, const char *sSerializedAttrInfos, jsmntok_t *pToken, int currentTokenPos)
{
    char value[MAX_VALUE_LEN];

    JSON_IS_NOT_TYPE_RETURN_ERROR_INVAL(JSMN_OBJECT, pToken, currentTokenPos);
    currentTokenPos++;

    JSON_IS_NOT_EQ_RETURN_ERROR_INVAL(sSerializedAttrInfos, JSON_ATTR_NAME, pToken, currentTokenPos);
    currentTokenPos++;

    JSON_IS_NOT_TYPE_RETURN_ERROR_INVAL(JSMN_STRING, pToken, currentTokenPos);
    JSON_GET_VALUE(value, MAX_VALUE_LEN, sSerializedAttrInfos, pToken, currentTokenPos);
    pstAttr->m_pszName = calloc(strlen(value) + 1, sizeof(char));
    if(NULL == pstAttr->m_pszName)
    {
        UBOOT_DEBUG("calloc(%d, %d) of pstAttr->m_pszName is failed.\n", (int)(strlen(value) + 1), (int)sizeof(char));
        currentTokenPos = JSMN_ERROR_NOMEM;

        return currentTokenPos;
//        return (0 < currentTokenPos)? (currentTokenPos - 1): (currentTokenPos);
    }
    strncpy(pstAttr->m_pszName, value, strlen(value) + 1);
    UBOOT_DEBUG("pstAttr->m_pszName = %s.\n", (pstAttr->m_pszName)? (pstAttr->m_pszName): "");
    currentTokenPos++;

    JSON_IS_NOT_EQ_RETURN_ERROR_INVAL(sSerializedAttrInfos, JSON_ATTR_DATA_TYPE, pToken, currentTokenPos);
    currentTokenPos++;

    JSON_IS_NOT_TYPE_RETURN_ERROR_INVAL(JSMN_PRIMITIVE, pToken, currentTokenPos);
    JSON_GET_VALUE(value, MAX_VALUE_LEN, sSerializedAttrInfos, pToken, currentTokenPos);
    pstAttr->m_eDataType = strtol(value, NULL, 0);
    UBOOT_DEBUG("pstAttr->m_eDataType = %d.\n", pstAttr->m_eDataType);
    currentTokenPos++;

    JSON_IS_NOT_EQ_RETURN_ERROR_INVAL(sSerializedAttrInfos, JSON_ATTR_DATA, pToken, currentTokenPos);
    currentTokenPos++;

    switch(pstAttr->m_eDataType)
    {
        case E_DB_ATTR_DATA_TYPE_INT:
            JSON_IS_NOT_TYPE_RETURN_ERROR_INVAL(JSMN_PRIMITIVE, pToken, currentTokenPos);
            JSON_GET_VALUE(value, MAX_VALUE_LEN, sSerializedAttrInfos, pToken, currentTokenPos);
            pstAttr->m_stData.m_s32Value = strtol(value, NULL, 0);
            currentTokenPos++;
            break;

        case E_DB_ATTR_DATA_TYPE_UINT:
            JSON_IS_NOT_TYPE_RETURN_ERROR_INVAL(JSMN_PRIMITIVE, pToken, currentTokenPos);
            JSON_GET_VALUE(value, MAX_VALUE_LEN, sSerializedAttrInfos, pToken, currentTokenPos);
            pstAttr->m_stData.m_u32Value = strtoul(value, NULL, 0);
            currentTokenPos++;
            break;

        case E_DB_ATTR_DATA_TYPE_STRING:
            JSON_IS_NOT_TYPE_RETURN_ERROR_INVAL(JSMN_STRING, pToken, currentTokenPos);
            JSON_GET_VALUE(value, MAX_VALUE_LEN, sSerializedAttrInfos, pToken, currentTokenPos);
            pstAttr->m_stData.m_pszValue = calloc(strlen(value) + 1, sizeof(char));
            if(NULL == pstAttr->m_pszName)
            {
                UBOOT_DEBUG("calloc(%d, %d) of pstAttr->m_stData.m_pszValue is failed.\n", (int)(strlen(value) + 1), (int)sizeof(char));
                currentTokenPos = JSMN_ERROR_NOMEM;

                break;
            }
            strncpy(pstAttr->m_stData.m_pszValue, value, strlen(value) + 1);
            currentTokenPos++;
            break;

        case E_DB_ATTR_DATA_TYPE_INT_ARRAY:
            currentTokenPos = _deserialize_db_int_array(pstAttr, sSerializedAttrInfos, pToken, currentTokenPos);
            if(0 > currentTokenPos)
            {
                UBOOT_DEBUG("deserialize_db_int_array(...) is failed.\n");
                pstAttr->m_stData.m_stValue.m_u16Size = 0;
                FREE(pstAttr->m_stData.m_stValue.m_stArray.m_ps32Array);

                break;
            }
            currentTokenPos++;
            break;

        case E_DB_ATTR_DATA_TYPE_UINT_ARRAY:
            currentTokenPos = _deserialize_db_uint_array(pstAttr, sSerializedAttrInfos, pToken, currentTokenPos);
            if(0 > currentTokenPos)
            {
                UBOOT_DEBUG("deserialize_db_uint_array(...) is failed.\n");
                pstAttr->m_stData.m_stValue.m_u16Size = 0;
                FREE(pstAttr->m_stData.m_stValue.m_stArray.m_pu32Array);

                break;
            }
            currentTokenPos++;
            break;
    }

#ifdef PARSER_DEBUG
    _print_db_attr(pstAttr);
#endif //#ifdef PARSER_DEBUG

    return (0 < currentTokenPos)? (currentTokenPos - 1): (currentTokenPos);
}

//-------------------------------------------------------------------------------------------------
//  Global Functions
//-------------------------------------------------------------------------------------------------
void free_db_attr_infos(db_attr_info_t *pstAttrInfos)
{
    int i;

    for(i = 0; i < pstAttrInfos->m_u8AttrTotalNumber; i++)
    {
        FREE(pstAttrInfos->m_pstAttr[i].m_pszName);

        switch(pstAttrInfos->m_pstAttr[i].m_eDataType)
        {
            case E_DB_ATTR_DATA_TYPE_STRING:
                FREE(pstAttrInfos->m_pstAttr[i].m_stData.m_pszValue);
                break;

            case E_DB_ATTR_DATA_TYPE_INT_ARRAY:
                pstAttrInfos->m_pstAttr[i].m_stData.m_stValue.m_u16Size = 0;
                FREE(pstAttrInfos->m_pstAttr[i].m_stData.m_stValue.m_stArray.m_ps32Array);
                break;

            case E_DB_ATTR_DATA_TYPE_UINT_ARRAY:
                pstAttrInfos->m_pstAttr[i].m_stData.m_stValue.m_u16Size = 0;
                FREE(pstAttrInfos->m_pstAttr[i].m_stData.m_stValue.m_stArray.m_pu32Array);
                break;

            default:
                break;
        }
    }

    pstAttrInfos->m_u8AttrTotalNumber = 0;
    FREE(pstAttrInfos->m_pstAttr);

    return;
 }


void print_db_attr_info(db_attr_info_t *pstAttrInfo)
{
#ifdef PARSER_DEBUG
    char title[MAX_KEY_LEN];
    int i;

    printf("\n");
    printf("m_u8AttrTotalNumber = %d\n", pstAttrInfo->m_u8AttrTotalNumber);
    for(i = 0; i < pstAttrInfo->m_u8AttrTotalNumber; i++)
    {
        memset(title, 0, sizeof(title));
        snprintf(title, MAX_KEY_LEN, "m_pstAttr[%d]", i);

        printf("\n");
        printf("%s\n", title);
        printf("m_pszName = %s\n", (pstAttrInfo->m_pstAttr[i].m_pszName)? (pstAttrInfo->m_pstAttr[i].m_pszName): "");
        printf("m_eDataType = %d\n", pstAttrInfo->m_pstAttr[i].m_eDataType);
        _print_db_attr(&(pstAttrInfo->m_pstAttr[i]));
    }
#endif //#ifdef PARSER_DEBUG

    return;
}

int parse_db_attr(db_attr_info_t *pstAttrInfo, db_get_attr_key pfGetAttrKey, db_attr_key_params_t *pstAttrKeyParams)
{
    int ret = PARSE_SUCCESS;
    char sKey[MAX_KEY_LEN];
    int i;
    int value;
    db_get_attr_key_params_t stGetAttrKeyParams;
    db_attr_key_params_t stAttrKeyParams;

    memset(&stGetAttrKeyParams, 0 , sizeof(db_get_attr_key_params_t));
    stGetAttrKeyParams.m_stAttrKey.m_pu8Buffer = (MS_U8 *)sKey;
    stGetAttrKeyParams.m_stAttrKey.m_u16BufferSize = MAX_KEY_LEN;
    stGetAttrKeyParams.m_eAttrKeyType = E_DB_ATTR_KEY_TYPE_ATTR_TOTAL_NUMBER;
    stGetAttrKeyParams.m_stAttrKeyParams.m_u16AttrNo = pstAttrKeyParams->m_u16AttrNo; // It is expected to be 0.
    if(!pfGetAttrKey(&stGetAttrKeyParams))
    {
        UBOOT_DEBUG("%s: %d: Error: It is failed to pfGetAttrKey(...).\n",
                __func__,
                __LINE__);
        ret = PARSE_PROCESSING_FAILED;

        return ret;
    }
    value = Profile_GetInteger(pstAttrKeyParams->m_psSection, sKey, -1);
    UBOOT_DEBUG("%s = 0x%x \n", sKey, value);
    if(-1 == value)
    {
        UBOOT_DEBUG("%s: %s is not found.\n", pstAttrKeyParams->m_psSection, sKey);
        pstAttrInfo->m_u8AttrTotalNumber = 0;
        // It is possible attributes are not defined.
        ret = PARSE_SUCCESS;

        return ret;
    }
    pstAttrInfo->m_u8AttrTotalNumber = value;

    pstAttrInfo->m_pstAttr = calloc(pstAttrInfo->m_u8AttrTotalNumber, sizeof(db_attr_t));
    if(NULL == pstAttrInfo->m_pstAttr)
    {
        UBOOT_DEBUG("calloc(%d, %d) of pstAttrInfo->m_pstAttr is failed.\n", pstAttrInfo->m_u8AttrTotalNumber, (int)sizeof(db_attr_t));
        pstAttrInfo->m_u8AttrTotalNumber = 0;
        ret = PARSE_PROCESSING_FAILED;

        return ret;
    }

    for(i = 0; i < pstAttrInfo->m_u8AttrTotalNumber; i++)
    {
        memset(&stAttrKeyParams, 0, sizeof(db_attr_key_params_t));
        stAttrKeyParams.m_psSection = pstAttrKeyParams->m_psSection;
        stAttrKeyParams.m_u16AttrNo = i;

        ret = _parse_db_attr_name(&(pstAttrInfo->m_pstAttr[i]), pfGetAttrKey, &stAttrKeyParams);
        if(PARSE_SUCCESS != ret)
        {
            UBOOT_DEBUG("%s: %d: _parse_db_attr_name(...) of Attr %d is failed.\n", __func__, __LINE__, i);
            break;
        }

        ret = _parse_db_attr_data_type(&(pstAttrInfo->m_pstAttr[i]), pfGetAttrKey, &stAttrKeyParams);
        if(PARSE_SUCCESS != ret)
        {
            UBOOT_DEBUG("%s: %d: _parse_db_attr_data_type(...) of Attr %d is failed.\n", __func__, __LINE__, i);
            break;
        }

        ret = _parse_attr_data(&(pstAttrInfo->m_pstAttr[i]), pfGetAttrKey, &stAttrKeyParams);
        if(PARSE_SUCCESS != ret)
        {
            UBOOT_DEBUG("%s: %d: _parse_attr_data(...) of Attr %d is failed.\n", __func__, __LINE__, i);
            break;
        }
    }

    return ret;
}

int serialize_db_attr_info(jsmn_serialize_params_t stSerializeParams, db_attr_info_t *pstAttrInfo)
{
    int written = 0;
    int writtenTotal = 0;
    int i;

    JSON_APPEND_INDENT(&stSerializeParams.psSerializedBuffer, &stSerializeParams.bufferMaxLen, &writtenTotal, stSerializeParams.indentlevel, stSerializeParams.indentPerLevel);
    JSON_SNPRINTF(&stSerializeParams.psSerializedBuffer, &stSerializeParams.bufferMaxLen, &writtenTotal, JSON_SERIALIZE_ITEM_OBJECT_START);
    JSON_SNPRINTF(&stSerializeParams.psSerializedBuffer, &stSerializeParams.bufferMaxLen, &writtenTotal, JSON_SERIALIZE_NEW_LINE);
    stSerializeParams.indentlevel += 1;

    JSON_APPEND_INDENT(&stSerializeParams.psSerializedBuffer, &stSerializeParams.bufferMaxLen, &writtenTotal, stSerializeParams.indentlevel, stSerializeParams.indentPerLevel);
    JSON_SNPRINTF(&stSerializeParams.psSerializedBuffer, &stSerializeParams.bufferMaxLen, &writtenTotal, JSON_SERIALIZE_NAME, JSON_ATTR_TOTAL_NUMBER);
    JSON_SNPRINTF(&stSerializeParams.psSerializedBuffer, &stSerializeParams.bufferMaxLen, &writtenTotal, JSON_SERIALIZE_VALUE_INT, pstAttrInfo->m_u8AttrTotalNumber);
    if(0 < pstAttrInfo->m_u8AttrTotalNumber)
    {
        JSON_SNPRINTF(&stSerializeParams.psSerializedBuffer, &stSerializeParams.bufferMaxLen, &writtenTotal, JSON_SERIALIZE_ITEM_SEPERATION);
    }
    JSON_SNPRINTF(&stSerializeParams.psSerializedBuffer, &stSerializeParams.bufferMaxLen, &writtenTotal, JSON_SERIALIZE_NEW_LINE);

    if(0 < pstAttrInfo->m_u8AttrTotalNumber)
    {
        JSON_APPEND_INDENT(&stSerializeParams.psSerializedBuffer, &stSerializeParams.bufferMaxLen, &writtenTotal, stSerializeParams.indentlevel, stSerializeParams.indentPerLevel);
        JSON_SNPRINTF(&stSerializeParams.psSerializedBuffer, &stSerializeParams.bufferMaxLen, &writtenTotal, JSON_SERIALIZE_NAME, JSON_ATTR_ARRAY);
        JSON_SNPRINTF(&stSerializeParams.psSerializedBuffer, &stSerializeParams.bufferMaxLen, &writtenTotal, JSON_SERIALIZE_NEW_LINE);

        JSON_APPEND_INDENT(&stSerializeParams.psSerializedBuffer, &stSerializeParams.bufferMaxLen, &writtenTotal, stSerializeParams.indentlevel, stSerializeParams.indentPerLevel);
        JSON_SNPRINTF(&stSerializeParams.psSerializedBuffer, &stSerializeParams.bufferMaxLen, &writtenTotal, JSON_SERIALIZE_ITEM_ARRAY_START);
        JSON_SNPRINTF(&stSerializeParams.psSerializedBuffer, &stSerializeParams.bufferMaxLen, &writtenTotal, JSON_SERIALIZE_NEW_LINE);
        stSerializeParams.indentlevel += 1;

        for(i = 0; i < pstAttrInfo->m_u8AttrTotalNumber; i++)
        {
            written = _serialize_DbAttr(stSerializeParams, &(pstAttrInfo->m_pstAttr[i]));
            JSON_PUTS(&stSerializeParams.psSerializedBuffer, &stSerializeParams.bufferMaxLen, &writtenTotal, written);

            if((pstAttrInfo->m_u8AttrTotalNumber - 1) != i)
            {
                JSON_SNPRINTF(&stSerializeParams.psSerializedBuffer, &stSerializeParams.bufferMaxLen, &writtenTotal, JSON_SERIALIZE_ITEM_SEPERATION);
            }
            JSON_SNPRINTF(&stSerializeParams.psSerializedBuffer, &stSerializeParams.bufferMaxLen, &writtenTotal, JSON_SERIALIZE_NEW_LINE);
        }

        stSerializeParams.indentlevel -= 1;
        JSON_APPEND_INDENT(&stSerializeParams.psSerializedBuffer, &stSerializeParams.bufferMaxLen, &writtenTotal, stSerializeParams.indentlevel, stSerializeParams.indentPerLevel);
        JSON_SNPRINTF(&stSerializeParams.psSerializedBuffer, &stSerializeParams.bufferMaxLen, &writtenTotal, JSON_SERIALIZE_ITEM_ARRAY_END);
        JSON_SNPRINTF(&stSerializeParams.psSerializedBuffer, &stSerializeParams.bufferMaxLen, &writtenTotal, JSON_SERIALIZE_NEW_LINE);
    }

    stSerializeParams.indentlevel -= 1;
    JSON_APPEND_INDENT(&stSerializeParams.psSerializedBuffer, &stSerializeParams.bufferMaxLen, &writtenTotal, stSerializeParams.indentlevel, stSerializeParams.indentPerLevel);
    JSON_SNPRINTF(&stSerializeParams.psSerializedBuffer, &stSerializeParams.bufferMaxLen, &writtenTotal, JSON_SERIALIZE_ITEM_OBJECT_END);

    return writtenTotal;
}

int deserialize_db_attr_info(db_attr_info_t *pstAttrInfos, const char *sSerializedAttrInfos, jsmntok_t *pToken, int currentTokenPos)
{
    char value[MAX_VALUE_LEN];
    int i;

    JSON_IS_NOT_TYPE_RETURN_ERROR_INVAL(JSMN_OBJECT, pToken, currentTokenPos);
    currentTokenPos++;

    JSON_IS_NOT_EQ_RETURN_ERROR_INVAL(sSerializedAttrInfos, JSON_ATTR_TOTAL_NUMBER, pToken, currentTokenPos);
    currentTokenPos++;

    JSON_IS_NOT_TYPE_RETURN_ERROR_INVAL(JSMN_PRIMITIVE, pToken, currentTokenPos);
    JSON_GET_VALUE(value, MAX_VALUE_LEN, sSerializedAttrInfos, pToken, currentTokenPos);
    pstAttrInfos->m_u8AttrTotalNumber = strtol(value, NULL, 0);
    UBOOT_DEBUG("pstAttrInfos->m_u8AttrTotalNumber = %d.\n", pstAttrInfos->m_u8AttrTotalNumber);
    currentTokenPos++;

    if(0 == pstAttrInfos->m_u8AttrTotalNumber)
    {
        return (0 < currentTokenPos)? (currentTokenPos - 1): (currentTokenPos);
    }
    pstAttrInfos->m_pstAttr = calloc(pstAttrInfos->m_u8AttrTotalNumber, sizeof(db_attr_t));
    if(NULL == pstAttrInfos->m_pstAttr)
    {
        UBOOT_DEBUG("calloc(%d, %d) of pstAttrInfos->m_pstAttr is failed.\n", pstAttrInfos->m_u8AttrTotalNumber, (int)sizeof(db_attr_t));
        currentTokenPos = JSMN_ERROR_NOMEM;

        return currentTokenPos;
//        return (0 < currentTokenPos)? (currentTokenPos - 1): (currentTokenPos);
    }

    JSON_IS_NOT_EQ_RETURN_ERROR_INVAL(sSerializedAttrInfos, JSON_ATTR_ARRAY, pToken, currentTokenPos);
    currentTokenPos++;

    JSON_IS_NOT_TYPE_RETURN_ERROR_INVAL(JSMN_ARRAY, pToken, currentTokenPos);
    currentTokenPos++;

    for(i = 0; i < pstAttrInfos->m_u8AttrTotalNumber; i++)
    {
        currentTokenPos = _deserialize_db_attr(&(pstAttrInfos->m_pstAttr[i]), sSerializedAttrInfos, pToken, currentTokenPos);
        if(0 > currentTokenPos)
        {
            UBOOT_DEBUG("_deserialize_db_attr(...) of pstAttrInfos->m_pstAttr[%d] is failed.\n", i);
            free_db_attr_infos(pstAttrInfos);

            break;
        }
        currentTokenPos++;
    }

    return (0 < currentTokenPos)? (currentTokenPos - 1): (currentTokenPos);
}
