//<MStar Software>
//******************************************************************************
// MStar Software
// Copyright (c) 2010 - 2012 MStar Semiconductor, Inc. All rights reserved.
// All software, firmware and related documentation herein ("MStar Software") are
// intellectual property of MStar Semiconductor, Inc. ("MStar") and protected by
// law, including, but not limited to, copyright law and international treaties.
// Any use, modification, reproduction, retransmission, or republication of all
// or part of MStar Software is expressly prohibited, unless prior written
// permission has been granted by MStar.
//
// By accessing, browsing and/or using MStar Software, you acknowledge that you
// have read, understood, and agree, to be bound by below terms ("Terms") and to
// comply with all applicable laws and regulations:
//
// 1. MStar shall retain any and all right, ownership and interest to MStar
//    Software and any modification/derivatives thereof.
//    No right, ownership, or interest to MStar Software and any
//    modification/derivatives thereof is transferred to you under Terms.
//
// 2. You understand that MStar Software might include, incorporate or be
//    supplied together with third party`s software and the use of MStar
//    Software may require additional licenses from third parties.
//    Therefore, you hereby agree it is your sole responsibility to separately
//    obtain any and all third party right and license necessary for your use of
//    such third party`s software.
//
// 3. MStar Software and any modification/derivatives thereof shall be deemed as
//    MStar`s confidential information and you agree to keep MStar`s
//    confidential information in strictest confidence and not disclose to any
//    third party.
//
// 4. MStar Software is provided on an "AS IS" basis without warranties of any
//    kind. Any warranties are hereby expressly disclaimed by MStar, including
//    without limitation, any warranties of merchantability, non-infringement of
//    intellectual property rights, fitness for a particular purpose, error free
//    and in conformity with any international standard.  You agree to waive any
//    claim against MStar for any loss, damage, cost or expense that you may
//    incur related to your use of MStar Software.
//    In no event shall MStar be liable for any direct, indirect, incidental or
//    consequential damages, including without limitation, lost of profit or
//    revenues, lost or damage of data, and unauthorized system use.
//    You agree that this Section 4 shall still apply without being affected
//    even if MStar Software has been modified by MStar in accordance with your
//    request or instruction for your use, except otherwise agreed by both
//    parties in writing.
//
// 5. If requested, MStar may from time to time provide technical supports or
//    services in relation with MStar Software to you for your use of
//    MStar Software in conjunction with your or your customer`s product
//    ("Services").
//    You understand and agree that, except otherwise agreed by both parties in
//    writing, Services are provided on an "AS IS" basis and the warranty
//    disclaimer set forth in Section 4 above shall apply.
//
// 6. Nothing contained herein shall be construed as by implication, estoppels
//    or otherwise:
//    (a) conferring any license or right to use MStar name, trademark, service
//        mark, symbol or any other identification;
//    (b) obligating MStar or any of its affiliates to furnish any person,
//        including without limitation, you and your customers, any assistance
//        of any kind whatsoever, or any information; or
//    (c) conferring any license or right under any intellectual property right.
//
// 7. These terms shall be governed by and construed in accordance with the laws
//    of Taiwan, R.O.C., excluding its conflict of law rules.
//    Any and all dispute arising out hereof or related hereto shall be finally
//    settled by arbitration referred to the Chinese Arbitration Association,
//    Taipei in accordance with the ROC Arbitration Law and the Arbitration
//    Rules of the Association by three (3) arbitrators appointed in accordance
//    with the said Rules.
//    The place of arbitration shall be in Taipei, Taiwan and the language shall
//    be English.
//    The arbitration award shall be final and binding to both parties.
//
//******************************************************************************
//<MStar Software>

#include <common.h>
#include <malloc.h>
#include <ShareType.h>
#include <MsApiJpd.h>
#include <MsApiGEGOP.h>
#include <MsVfs.h>
#include <MsSystem.h>
#include <MsUboot.h>
#include <MsDebug.h>
#include <MsRawIO.h>
#include <CusConfig.h>
#include <MsMmap.h>
#include <drvGPIO.h>
#include <MsBoot.h>
#include <miu/MsDrvMiu.h>
#include <MsEnvironment.h>
#include "MsTypes.h"
#include "MsOS.h"
#include "ScriptTypes.h"
#include "ScriptBootLogo.h"
#include "MsScriptOSD.h"
#include "stdio.h"


MS_U32 gu32JPD_InputADDR=0;
MS_U32 gu32JPD_OUTADDR=0;
MS_PHYADDR gphyInputAddr = 0;

#define BOOT_LOGO_FLASH_BASE_ADDR 0x94000000//+0x60000
#define BOOT_LOGO_FLASH_DEFAULT_OFFSET 0x60000
extern int g_JpgPitch;
extern int g_JpgWidth;
extern int g_JpgHeight;

static int LogoInMboot2Dram(MS_U32 *Dram_Addr)
{
    int ret = -1;
    MS_U32 u32LogoSize = 0;
    MS_U32 u32LogoOffset = 0;
    UBOOT_TRACE("IN\n");

    ret = mboot_raw_io_Config();
    if(ret != 0)
    {
        UBOOT_ERROR("raw_io_config setting fail!\n");
        return -1;
    }

    ret=MsApiChunkHeader_GetValue(CH_LOGO_ROM_OFFSET,&u32LogoOffset);
    if(ret!=0)
    {
        UBOOT_ERROR("get u32LogoOffset fail\n");
        return -1;
    }
    ret=MsApiChunkHeader_GetValue(CH_LOGO_ROM_SIZE,&u32LogoSize);
    if(ret!=0)
    {
        UBOOT_ERROR("get u32LogoSize fail\n");
        return -1;
    }
    
    *Dram_Addr=(MS_U32)malloc(u32LogoSize);
    if(Dram_Addr!=NULL)
    {
        if(raw_read(*Dram_Addr,u32LogoOffset,u32LogoSize)!=0)
        {
            UBOOT_ERROR("LogoInMboot2Dram Fail!!\n");
            return -1;
        }
    }

    UBOOT_TRACE("OK\n");
    return u32LogoSize;
}

#define BOOT_LOGO_DEFAULT_SIZE 0xC7C3
#define BOOT_LOGO_MAX_WIDTH 1920
#define BOOT_LOGO_MAX_HEIGHT 1080
#define BOOT_LOGO_MAX_SIZE 0x100000 // 1M
MS_BOOL do_bootlogo_lite(void)
{
    UBOOT_TRACE("\nStart bootlogo\n");
    
    char *pEnv=NULL;
    MS_U32 u32Addr = 0;
    ST_RectInfo stRectInfoSrc = {0, 0};
    MS_U8 resolution_lite = 0;
    MS_U32 u32LogoSize = 0;
    
    #if 0
    pEnv = getenv("upgrade_mode");
    if(strcmp(pEnv,"usb") == 0 || strcmp(pEnv,"oad") == 0)//when oad or usb upgrade we don't show logo first
    {   
        return FALSE;
    }
    #endif
    
    pEnv = getenv("resolution");
    if(pEnv == NULL)
    {
        resolution_lite = E_OUTPUT_TIMING_1920X1080_50I;// 8
        setenv("resolution", "8");//E_OUTPUT_TIMING_1920X1080_50I
        saveenv();
    }
    else
    {
        resolution_lite = (MS_U8)simple_strtoull(pEnv, NULL, 16);
    }
    UBOOT_TRACE("\nresolution 0x%x\n",resolution_lite);
    


    
    EN_OUTPUT_TIMING_TYPE enOutputTiming = resolution_lite;//E_OUTPUT_TIMING_1280X720_50P;//E_OUTPUT_TIMING_1920X1080_60P;//E_OUTPUT_TIMING_1920X1080_50P;//E_OUTPUT_TIMING_1920X1080_50I;//_BootLogo_GetOutputTiming();
    EN_HDMI_COLOR_FORMAT enHdmiColorFormat = E_HDMI_COLOR_FORMAT_YUV;//_BootLogo_GetColorFormat();

    Script_XC_Init(enOutputTiming);
    Script_VE_Init(enOutputTiming);

    if(enHdmiColorFormat == E_HDMI_COLOR_FORMAT_RGB)
    {
        Script_XC_HDMI_YUV2RGB();
    }

   
    if (get_map_addr_from_env(E_DFB_FRAMEBUFFER, MMAP_E_DFB_FRAMEBUFFER_ADDR, &gphyInputAddr)!=0)
    {
        UBOOT_ERROR("get %s mmap fail\n",E_DFB_FRAMEBUFFER_ADR);
        return FALSE;
     }

    u32LogoSize = LogoInMboot2Dram(&gu32JPD_InputADDR);
    gu32JPD_OUTADDR=MsApi_JPD_Decode((MS_U32)gu32JPD_InputADDR, u32LogoSize);

    stRectInfoSrc.u16Height = g_JpgHeight;
    stRectInfoSrc.u16Width = g_JpgWidth;
    stRectInfoSrc.u16Pitch = g_JpgPitch;
    
    if(g_JpgWidth > BOOT_LOGO_MAX_WIDTH ||g_JpgHeight > BOOT_LOGO_MAX_HEIGHT ||u32LogoSize > BOOT_LOGO_MAX_SIZE)
    {
        return FALSE;
    }
    
    Script_GEGOP_Init(stRectInfoSrc, enOutputTiming);
    
    UBOOT_TRACE("\nEnd bootlogo\n");
    return TRUE;
}

//stub funcs
typedef MS_U32 (*FUtopiaOpen)(void** ppInstance, const void* const pAttribute);
void PNLRegisterToUtopia(FUtopiaOpen ModuleType)
{
}
void GOPRegisterToUtopia(FUtopiaOpen ModuleType)
{
}
void GFXRegisterToUtopia(FUtopiaOpen ModuleType)
{
}
void XCRegisterToUtopia(FUtopiaOpen ModuleType)
{
}
void TVENCODERRegisterToUtopia(FUtopiaOpen ModuleType)
{
}
void HDMITXRegisterToUtopia(FUtopiaOpen ModuleType)
{
}
void DACRegisterToUtopia(FUtopiaOpen ModuleType)
{
}
/*
void CECRegisterToUtopia(FUtopiaOpen ModuleType)
{
}*/
