//<MStar Software>
//***********************************************************************************
//MStar Software
//Copyright (c) 2010 - 2015 MStar Semiconductor, Inc. All rights reserved.
//All software, firmware and related documentation herein (MStar Software) are intellectual property of MStar Semiconductor, Inc. (MStar) and protected by law, including, but not limited to, copyright law and international treaties.  Any use, modification, reproduction, retransmission, or republication of all or part of MStar Software is expressly prohibited, unless prior written permission has been granted by MStar.
//By accessing, browsing and/or using MStar Software, you acknowledge that you have read, understood, and agree, to be bound by below terms (Terms) and to comply with all applicable laws and regulations:
//
//1. MStar shall retain any and all right, ownership and interest to MStar Software and any modification/derivatives thereof.  No right, ownership, or interest to MStar Software and any modification/derivatives thereof is transferred to you under Terms.
//2. You understand that MStar Software might include, incorporate or be supplied together with third partys software and the use of MStar Software may require additional licenses from third parties.  Therefore, you hereby agree it is your sole responsibility to separately obtain any and all third party right and license necessary for your use of such third partys software.
//3. MStar Software and any modification/derivatives thereof shall be deemed as MStars confidential information and you agree to keep MStars confidential information in strictest confidence and not disclose to any third party.
//4. MStar Software is provided on an AS IS basis without warranties of any kind. Any warranties are hereby expressly disclaimed by MStar, including without limitation, any warranties of merchantability, non-infringement of intellectual property rights, fitness for a particular purpose, error free and in conformity with any international standard.  You agree to waive any claim against MStar for any loss, damage, cost or expense that you may incur related to your use of MStar Software.
//   In no event shall MStar be liable for any direct, indirect, incidental or consequential damages, including without limitation, lost of profit or revenues, lost or damage of data, and unauthorized system use.  You agree that this Section 4 shall still apply without being affected even if MStar Software has been modified by MStar in accordance with your request or instruction for your use, except otherwise agreed by both parties in writing.
//5. If requested, MStar may from time to time provide technical supports or services in relation with MStar Software to you for your use of MStar Software in conjunction with your or your customers product (Services).  You understand and agree that, except otherwise agreed by both parties in writing, Services are provided on an AS IS basis and the warranty disclaimer set forth in Section 4 above shall apply.
//6. Nothing contained herein shall be construed as by implication, estoppels or otherwise: (a) conferring any license or right to use MStar name, trademark, service mark, symbol or any other identification; (b) obligating MStar or any of its affiliates to furnish any person, including without limitation, you and your customers, any assistance of any kind whatsoever, or any information; or (c) conferring any license or right under any intellectual property right.
//7. These terms shall be governed by and construed in accordance with the laws of Taiwan, R.O.C., excluding its conflict of law rules.  Any and all dispute arising out hereof or related hereto shall be finally settled by arbitration referred to the Chinese Arbitration Association, Taipei in accordance with the ROC Arbitration Law and the Arbitration Rules of the Association by three (3) arbitrators appointed in accordance with the said Rules.  The place of arbitration shall be in Taipei, Taiwan and the language shall be English.
//   The arbitration award shall be final and binding to both parties.
//***********************************************************************************
//<MStar Software>
#define CRYPTO_HMAC_C

#include <secure/crypto_hmac.h>
#include <secure/crypto_sha.h>
#include <secure/crypto_aes.h>
#include <secure/MsSecureCommon.h>
#include <common.h>
#include <MsSystem.h>
#include <MsString.h>
#include <MsDebug.h>

#define SHA256_DIGEST_LENGTH 32
#define HMAC_KEY_LENGTH 32
#define HMAC_DRM_ID "MStarHmacKey"
#define DRM_ID_LEN 16


void
hmac_sha256(
    const unsigned char *text,      /* pointer to data stream        */
    int                 text_len,   /* length of data stream         */
    const unsigned char *key,       /* pointer to authentication key */
    int                 key_len,    /* length of authentication key  */
    void                *digest)    /* caller digest to be filled in */
{
    unsigned char k_ipad[65];   /* inner padding -
                                 * key XORd with ipad
                                 */
    unsigned char k_opad[65];   /* outer padding -
                                 * key XORd with opad
                                 */
    unsigned char tk[SHA256_DIGEST_LENGTH];
    unsigned char tk2[SHA256_DIGEST_LENGTH];
    unsigned char bufferIn[1024];
    unsigned char bufferOut[1024];
    int           i;

    /* if key is longer than 64 bytes reset it to key=sha256(key) */
    if ( key_len > 64 ) {
        sha256( key, key_len, tk );
        key     = tk;
        key_len = SHA256_DIGEST_LENGTH;
    }

    /*
     * the HMAC_SHA256 transform looks like:
     *
     * SHA256(K XOR opad, SHA256(K XOR ipad, text))
     *
     * where K is an n byte key
     * ipad is the byte 0x36 repeated 64 times
     * opad is the byte 0x5c repeated 64 times
     * and text is the data being protected
     */

    /* start out by storing key in pads */
    memset( k_ipad, 0, sizeof k_ipad );
    memset( k_opad, 0, sizeof k_opad );
    memcpy( k_ipad, key, key_len );
    memcpy( k_opad, key, key_len );

    /* XOR key with ipad and opad values */
    for ( i = 0; i < 64; i++ ) {
        k_ipad[i] ^= 0x36;
        k_opad[i] ^= 0x5c;
    }

    /*
     * perform inner SHA256
     */
    memset( bufferIn, 0x00, 1024 );
    memcpy( bufferIn, k_ipad, 64 );
    memcpy( bufferIn + 64, text, text_len );

    sha256( bufferIn, 64 + text_len, tk2 );

    /*
     * perform outer SHA256
     */
    memset( bufferOut, 0x00, 1024 );
    memcpy( bufferOut, k_opad, 64 );
    memcpy( bufferOut + 64, tk2, SHA256_DIGEST_LENGTH );

    sha256( bufferOut, 64 + SHA256_DIGEST_LENGTH, digest );
}

int GetHmacKey(unsigned char *u8pHmacKey)
{
    unsigned char VirtAddr[DRM_ID_LEN]={0};
    unsigned char u8Key[AES_KEY_LEN];

    UBOOT_TRACE("IN\n");
    if(strlen(HMAC_DRM_ID)<sizeof(VirtAddr))
    {
        memcpy(VirtAddr, HMAC_DRM_ID, strlen(HMAC_DRM_ID));
    }
    else
    {
        memcpy(VirtAddr, HMAC_DRM_ID, sizeof(VirtAddr));
    }

    UBOOT_DEBUG("HMAC_DRM_ID\n\n");
    UBOOT_DUMP((U32)VirtAddr, DRM_ID_LEN);

    Secure_Get_DeviceKey(u8Key);
    Secure_AES_ECB_Encrypt((U32)VirtAddr,DRM_ID_LEN,u8Key);

    memcpy(u8pHmacKey,(void*)VirtAddr, DRM_ID_LEN);
    array_reverse((char*)VirtAddr,DRM_ID_LEN);
    memcpy(&u8pHmacKey[DRM_ID_LEN],(void*)VirtAddr, DRM_ID_LEN);

    UBOOT_DEBUG("HmacKey\n\n");
    UBOOT_DUMP((U32)u8pHmacKey, HMAC_KEY_LEN);

    UBOOT_TRACE("OK\n");
    return TRUE;
}
#undef CRYPTO_HMAC_C
