//<MStar Software>
//******************************************************************************
// MStar Software
// Copyright (c) 2010 - 2012 MStar Semiconductor, Inc. All rights reserved.
// All software, firmware and related documentation herein ("MStar Software") are
// intellectual property of MStar Semiconductor, Inc. ("MStar") and protected by
// law, including, but not limited to, copyright law and international treaties.
// Any use, modification, reproduction, retransmission, or republication of all
// or part of MStar Software is expressly prohibited, unless prior written
// permission has been granted by MStar.
//
// By accessing, browsing and/or using MStar Software, you acknowledge that you
// have read, understood, and agree, to be bound by below terms ("Terms") and to
// comply with all applicable laws and regulations:
//
// 1. MStar shall retain any and all right, ownership and interest to MStar
//    Software and any modification/derivatives thereof.
//    No right, ownership, or interest to MStar Software and any
//    modification/derivatives thereof is transferred to you under Terms.
//
// 2. You understand that MStar Software might include, incorporate or be
//    supplied together with third party`s software and the use of MStar
//    Software may require additional licenses from third parties.
//    Therefore, you hereby agree it is your sole responsibility to separately
//    obtain any and all third party right and license necessary for your use of
//    such third party`s software.
//
// 3. MStar Software and any modification/derivatives thereof shall be deemed as
//    MStar`s confidential information and you agree to keep MStar`s
//    confidential information in strictest confidence and not disclose to any
//    third party.
//
// 4. MStar Software is provided on an "AS IS" basis without warranties of any
//    kind. Any warranties are hereby expressly disclaimed by MStar, including
//    without limitation, any warranties of merchantability, non-infringement of
//    intellectual property rights, fitness for a particular purpose, error free
//    and in conformity with any international standard.  You agree to waive any
//    claim against MStar for any loss, damage, cost or expense that you may
//    incur related to your use of MStar Software.
//    In no event shall MStar be liable for any direct, indirect, incidental or
//    consequential damages, including without limitation, lost of profit or
//    revenues, lost or damage of data, and unauthorized system use.
//    You agree that this Section 4 shall still apply without being affected
//    even if MStar Software has been modified by MStar in accordance with your
//    request or instruction for your use, except otherwise agreed by both
//    parties in writing.
//
// 5. If requested, MStar may from time to time provide technical supports or
//    services in relation with MStar Software to you for your use of
//    MStar Software in conjunction with your or your customer`s product
//    ("Services").
//    You understand and agree that, except otherwise agreed by both parties in
//    writing, Services are provided on an "AS IS" basis and the warranty
//    disclaimer set forth in Section 4 above shall apply.
//
// 6. Nothing contained herein shall be construed as by implication, estoppels
//    or otherwise:
//    (a) conferring any license or right to use MStar name, trademark, service
//        mark, symbol or any other identification;
//    (b) obligating MStar or any of its affiliates to furnish any person,
//        including without limitation, you and your customers, any assistance
//        of any kind whatsoever, or any information; or
//    (c) conferring any license or right under any intellectual property right.
//
// 7. These terms shall be governed by and construed in accordance with the laws
//    of Taiwan, R.O.C., excluding its conflict of law rules.
//    Any and all dispute arising out hereof or related hereto shall be finally
//    settled by arbitration referred to the Chinese Arbitration Association,
//    Taipei in accordance with the ROC Arbitration Law and the Arbitration
//    Rules of the Association by three (3) arbitrators appointed in accordance
//    with the said Rules.
//    The place of arbitration shall be in Taipei, Taiwan and the language shall
//    be English.
//    The arbitration award shall be final and binding to both parties.
//
//******************************************************************************
//<MStar Software>
////////////////////////////////////////////////////////////////////////////////
//
// Copyright (c) 2006-2009 MStar Semiconductor, Inc.
// All rights reserved.
//
// Unless otherwise stipulated in writing, any and all information contained
// herein regardless in any format shall remain the sole proprietary of
// MStar Semiconductor Inc. and be kept in strict confidence
// ("MStar Confidential Information") by the recipient.
// Any unauthorized act including without limitation unauthorized disclosure,
// copying, use, reproduction, sale, distribution, modification, disassembling,
// reverse engineering and compiling of the contents of MStar Confidential
// Information is unlawful and strictly prohibited. MStar hereby reserves the
// rights to any and all damages, losses, costs and expenses resulting therefrom.
//
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////////////
///
/// @file   apiSWI2C_priv.h
/// @brief  SWI2C Driver Interface
/// @author MStar Semiconductor,Inc.
///
////////////////////////////////////////////////////////////////////////////////////////////////////

#ifndef _APISWI2C_PRIV_H_
#define _APISWI2C_PRIV_H_

#ifdef __cplusplus
extern "C"
{
#endif

#include "MsTypes.h"
#include "apiSWI2C.h"

#if defined (CONFIG_MSTAR_SWI2C_KERNEL_MODIIC_DEVICE_ENABLE)
typedef struct DLL_PACKED _SWI2C_IIC_Param
{
    MS_U8 u8IICType;         /// IICType: 0:SWI2C 1:HWI2C
    MS_U8 u8IdIIC;           /// IIC ID: Channel 1~7
    MS_U8 u8ClockIIC;        /// IIC clock speed
    MS_U8 u8SlaveIdIIC;      /// Device slave ID
    MS_U8 u8AddrSizeIIC;     /// Address length in bytes
    MS_U8 *u8AddrIIC;        /// Starting address inside the device
    MS_U8 *u8pbufIIC;        /// buffer
    MS_U32 u32DataSizeIIC;   /// size of buffer
} SWI2C_IIC_Param, *PSWI2C_IIC_Param;
#endif

typedef struct DLL_PACKED _SWI2C_PRIVATE_Init_PARAM
{
    SWI2C_BusCfg* SWI2CBusCfg;
#if !defined (__aarch64__)
    MS_U32 SWI2CBusCfgDummy;
#endif
    MS_U8 u8CfgBusNum;
} SWI2C_PRIVATE_Init_PARAM, *PSWI2C_PRIVATE_Init_PARAM;

typedef struct DLL_PACKED _SWI2C_PRIVATE_PARAM_WriteBytes
{
    MS_U16 u16BusNumSlaveID;
    MS_U8 u8addrcount;
    MS_U8* pu8addr;
    MS_U16 u16size;
    MS_U8* pu8data;
} SWI2C_PRIVATE_WriteBytes_PARAM, *PSWI2C_PRIVATE_WriteBytes_PARAM;

typedef struct DLL_PACKED _SWI2C_PRIVATE_WriteBytesStop_PARAM
{
    MS_U16 u16BusNumSlaveID;
    MS_U8 AddrCnt;
    MS_U8* pu8addr;
    MS_U16 u16size;
    MS_U8* pBuf;
    MS_BOOL bGenStop;
} SWI2C_PRIVATE_WriteBytesStop_PARAM, *PSWI2C_PRIVATE_WriteBytesStop_PARAM;

typedef struct DLL_PACKED _SWI2C_PRIVATE_ReadBytes_PARAM
{
    MS_U16 u16BusNumSlaveID;
    MS_U8 u8AddrNum;
    MS_U8* paddr;
    MS_U16 u16size;
    MS_U8* pu8data;
} SWI2C_PRIVATE_ReadBytes_PARAM, *PSWI2C_PRIVATE_ReadBytes_PARAM;

typedef struct DLL_PACKED _SWI2C_PRIVATE_ReadBytes_ThruMode_PARAM
{
    MS_U16 u16BusNumSlaveID;
    MS_U8 ucSubAdr;
    MS_U8* paddr;
    MS_U16 ucBufLen;
    MS_U8* pBuf;
} SWI2C_PRIVATE_ReadBytes_ThruMode_PARAM, *PSWI2C_PRIVATE_ReadBytes_ThruMode_PARAM;

typedef struct DLL_PACKED _SWI2C_PRIVATE_ReadByte_PARAM
{
    MS_U16 u16BusNumSlaveID;
    MS_U8 u8RegAddr;
    MS_U8 *pu8Data;
} SWI2C_PRIVATE_ReadByte_PARAM, *PSWI2C_PRIVATE_ReadByte_PARAM;

typedef struct DLL_PACKED _SWI2C_PRIVATE_WriteByte_PARAM
{
    MS_U16 u16BusNumSlaveID;
    MS_U8 u8RegAddr;
    MS_U8 u8Data;
} SWI2C_PRIVATE_WriteByte_PARAM, *PSWI2C_PRIVATE_WriteByte_PARAM;

typedef struct DLL_PACKED _SWI2C_PRIVATE_Write2Bytes_PARAM
{
    MS_U16 u16BusNumSlaveID;
    MS_U8 u8addr;
    MS_U16 u16data;
} SWI2C_PRIVATE_Write2Bytes_PARAM, *PSWI2C_PRIVATE_Write2Bytes_PARAM;

typedef struct DLL_PACKED _SWI2C_PRIVATE_Read2Bytes_PARAM
{
    MS_U16 u16BusNumSlaveID;
    MS_U8 u8addr;
} SWI2C_PRIVATE_Read2Bytes_PARAM, *PSWI2C_PRIVATE_Read2Bytes_PARAM;

typedef struct DLL_PACKED _SWI2C_PRIVATE_Write4Bytes_PARAM
{
    MS_U16 u16BusNumSlaveID;
    MS_U32 u32Data;
    MS_U8 u8EndData;
} SWI2C_PRIVATE_Write4Bytes_PARAM, *PSWI2C_PRIVATE_Write4Bytes_PARAM;

typedef struct DLL_PACKED _SWI2C_PRIVATE_WriteGroupBytes_PARAM
{
    MS_U16 u16BusNumSlaveID;
    MS_U8 u8SubGroup;
    MS_U16 u16Addr;
    MS_U16 u16Data;
} SWI2C_PRIVATE_WriteGroupBytes_PARAM, *PSWI2C_PRIVATE_WriteGroupBytes_PARAM;

typedef struct DLL_PACKED _SWI2C_PRIVATE_ReadGroupBytes_PARAM
{
    MS_U16 u16BusNumSlaveID;
    MS_U8 u8SubGroup;
    MS_U16 u16Addr;
} SWI2C_PRIVATE_ReadGroupBytes_PARAM, *PSWI2C_PRIVATE_ReadGroupBytes_PARAM;

typedef struct DLL_PACKED _SWI2C_PRIVATE_PARAM_Speed_Setting_PARAM
{
    MS_U8 u8BusNum;
    MS_U32 u32Speed_K;
} SWI2C_PRIVATE_Speed_Setting_PARAM, *PSWI2C_PRIVATE_Speed_Setting_PARAM;


typedef struct DLL_PACKED _SWI2C_PRIVATE_SetReadMode_PARAM
{
    SWI2C_ReadMode eReadMode;
} SWI2C_PRIVATE_SetReadMode_PARAM, *PSWI2C_PRIVATE_SetReadMode_PARAM;

typedef struct DLL_PACKED _SWI2C_PRIVATE_SetBusReadMode_PARAM
{
    MS_U8 u8BusNum;
    SWI2C_ReadMode eReadMode;
} SWI2C_PRIVATE_SetBusReadMode_PARAM, *PSWI2C_PRIVATE_SetBusReadMode_PARAM;

typedef struct DLL_PACKED _SWI2C_PRIVATE_GetLibVer_PARAM
{
    const MSIF_Version **ppVersion;
#if !defined (__aarch64__)
    MS_U32 ppVersionDummy;
#endif
} SWI2C_PRIVATE_GetLibVer_PARAM, *PSWI2C_PRIVATE_GetLibVer_PARAM;

typedef struct DLL_PACKED _SWI2C_PRIVATE_SetDbgLevel_PARAM
{
    SWI2C_DbgLvl eLevel;
} SWI2C_PRIVATE_SetDbgLevel_PARAM, *PSWI2C_PRIVATE_SetDbgLevel_PARAM;

#if 0
typedef struct _SWI2C_PRIVATE_PARAM_SetPowerState
{
// one pointer memberEN_POWER_MODE u16PowerState
} SWI2C_PRIVATE_PARAM_SetPowerState;
#endif

//ioctl function pointer type
typedef MS_BOOL (*IOCTL_SWI2C_INIT)(SWI2C_BusCfg[], MS_U8 u8CfgBusNum);
typedef MS_BOOL (*IOCTL_SWI2C_WRITEBYTES)(MS_U16, MS_U8, MS_U8*, MS_U16, MS_U8*);
typedef MS_BOOL (*IOCTL_SWI2C_WRITEBYTESSTOP)(MS_U16, MS_U8, MS_U8*, MS_U16, MS_U8*,MS_BOOL);
typedef MS_BOOL (*IOCTL_SWI2C_READBYTES)(MS_U16, MS_U8, MS_U8*, MS_U16, MS_U8*);
typedef MS_BOOL (*IOCTL_SWI2C_READBYTESTHRUMODE)(MS_U16, MS_U8, MS_U8*, MS_U16, MS_U8*);
typedef MS_BOOL (*IOCTL_SWI2C_READBYTE)(MS_U16, MS_U8, MS_U8*);
typedef MS_BOOL (*IOCTL_SWI2C_WRITEBYTE)(MS_U16, MS_U8, MS_U8);
typedef MS_BOOL (*IOCTL_SWI2C_WRITE2BYTES)(MS_U16, MS_U8, MS_U16);
typedef MS_BOOL (*IOCTL_SWI2C_READ2BYTES)(MS_U16, MS_U8);
typedef MS_BOOL (*IOCTL_SWI2C_WRITE4BYTES)(MS_U16, MS_U32, MS_U8);
typedef MS_BOOL (*IOCTL_SWI2C_WRITEGROUPBYTES)(MS_U16, MS_U8, MS_U16, MS_U16);
typedef MS_BOOL (*IOCTL_SWI2C_READGROUPBYTES)(MS_U16, MS_U8, MS_U16);
typedef MS_BOOL (*IOCTL_SWI2C_GETMAXBUSES)(void);
typedef MS_BOOL (*IOCTL_SWI2C_SPEEDSETTING)(MS_U8, MS_U32);
typedef MS_BOOL (*IOCTL_SWI2C_SETREADMODE)(SWI2C_ReadMode);
typedef MS_BOOL (*IOCTL_SWI2C_SETBUSREADMODE)(MS_U8, SWI2C_ReadMode);
typedef MS_BOOL (*IOCTL_SWI2C_GETLIBVER)(const MSIF_Version**);
typedef MS_BOOL (*IOCTL_SWI2C_SETDBGLEVEL)(SWI2C_DbgLvl);

typedef struct _SWI2C_RESOURCE_PRIVATE
{
    MS_U32 Dummy;
}SWI2C_RESOURCE_PRIVATE;

typedef struct _SWI2C_INSTANT_PRIVATE
{
    IOCTL_SWI2C_INIT fpSWI2CInit;
    IOCTL_SWI2C_WRITEBYTES fpSWI2CWriteBytes;
    IOCTL_SWI2C_WRITEBYTESSTOP fpSWI2CWriteBytesStop;
    IOCTL_SWI2C_READBYTES fpSWI2CReadBytes;
    IOCTL_SWI2C_READBYTESTHRUMODE fpSWI2CReadBytesThruMode;
    IOCTL_SWI2C_READBYTE fpSWI2CReadByte;
    IOCTL_SWI2C_WRITEBYTE fpSWI2CWriteByte;
    IOCTL_SWI2C_WRITE2BYTES fpSWI2CWrite2Bytes;
    IOCTL_SWI2C_READ2BYTES fpSWI2CRead2Bytes;
    IOCTL_SWI2C_WRITE4BYTES fpSWI2CWrite4Bytes;
    IOCTL_SWI2C_WRITEGROUPBYTES fpSWI2CWriteGroupBytes;
    IOCTL_SWI2C_READGROUPBYTES fpSWI2CReadGroupBytes;
    IOCTL_SWI2C_GETMAXBUSES fpSWI2CGetMaxBuses;
    IOCTL_SWI2C_SPEEDSETTING fpSWI2CSpeedSetting;
    IOCTL_SWI2C_SETREADMODE fpSWI2CSetReadMode;
    IOCTL_SWI2C_SETBUSREADMODE fpSWI2CSetBusReadMode;
    IOCTL_SWI2C_GETLIBVER fpSWI2CGetLibVer;
    IOCTL_SWI2C_SETDBGLEVEL fpSWI2CSetDbgLevel;
}SWI2C_INSTANT_PRIVATE;

typedef enum {
    MApi_CMD_SWI2C_Init,
    MApi_CMD_SWI2C_WriteBytes,
    MApi_CMD_SWI2C_WriteBytesStop,
    MApi_CMD_SWI2C_ReadBytes,
    MApi_CMD_SWI2C_ReadBytes_ThruMode,
    MApi_CMD_SWI2C_ReadByte,
    MApi_CMD_SWI2C_WriteByte,
    MApi_CMD_SWI2C_Write2Bytes,
    MApi_CMD_SWI2C_Read2Bytes,
    MApi_CMD_SWI2C_Write4Bytes,
    MApi_CMD_SWI2C_WriteGroupBytes,
    MApi_CMD_SWI2C_ReadGroupBytes,
    MApi_CMD_SWI2C_GetMaxBuses,
    MApi_CMD_SWI2C_Speed_Setting,
    MApi_CMD_SWI2C_SetReadMode,
    MApi_CMD_SWI2C_SetBusReadMode,
    MApi_CMD_SWI2C_GetLibVer,
    MApi_CMD_SWI2C_SetDbgLevel,
    //MApi_CMD_SWI2C_SetPowerState,
} eSWI2CIoctlOpt;
//////////////////////////////////////
// function for utopia2.0 ////////////
//////////////////////////////////////
void SWI2CRegisterToUtopia(void);
MS_U32 SWI2COpen(void** pInstance, MS_U32 u32ModuleVersion, void* pAttribute);
MS_U32 SWI2CClose(void* pInstance);
MS_U32 SWI2CIoctl(void* pInstance, MS_U32 u32Cmd, void *pArgs);

void _MApi_SWI2C_Init(SWI2C_BusCfg SWI2CBusCfg[],MS_U8 u8CfgBusNum);
MS_BOOL _MApi_SWI2C_WriteBytes(MS_U16 u16BusNumSlaveID, MS_U8 u8addrcount, MS_U8* pu8addr, MS_U16 u16size, MS_U8* pu8data);
MS_BOOL _MApi_SWI2C_WriteBytesStop(MS_U16 u16BusNumSlaveID, MS_U8 AddrCnt, MS_U8* pu8addr, MS_U16 u16size, MS_U8* pBuf,MS_BOOL bGenStop);
MS_BOOL _MApi_SWI2C_ReadBytes(MS_U16 u16BusNumSlaveID, MS_U8 u8AddrNum, MS_U8* paddr, MS_U16 u16size, MS_U8* pu8data);
MS_BOOL _MApi_SWI2C_ReadBytes_ThruMode(MS_U16 u16BusNumSlaveID, MS_U8 ucSubAdr, MS_U8* paddr, MS_U16 ucBufLen, MS_U8* pBuf);
MS_BOOL _MApi_SWI2C_ReadByte(MS_U16 u16BusNumSlaveID, MS_U8 u8RegAddr, MS_U8 *pu8Data);
MS_BOOL _MApi_SWI2C_WriteByte(MS_U16 u16BusNumSlaveID, MS_U8 u8RegAddr, MS_U8 u8Data);
MS_BOOL _MApi_SWI2C_Write2Bytes(MS_U16 u16BusNumSlaveID, MS_U8 u8addr, MS_U16 u16data);
MS_U16 _MApi_SWI2C_Read2Bytes(MS_U16 u16BusNumSlaveID, MS_U8 u8addr);
MS_BOOL _MApi_SWI2C_Write4Bytes(MS_U16 u16BusNumSlaveID, MS_U32 u32Data, MS_U8 u8EndData);
//MS_BOOL _MApi_SWI2C_ReadByteDirectly(MS_U16 u16BusNumSlaveID, MS_U8 * pu8Data);
MS_BOOL _MApi_SWI2C_WriteGroupBytes(MS_U16 u16BusNumSlaveID, MS_U8 u8SubGroup, MS_U16 u16Addr, MS_U16 u16Data);
MS_U16 _MApi_SWI2C_ReadGroupBytes(MS_U16 u16BusNumSlaveID, MS_U8 u8SubGroup, MS_U16 u16Addr);
MS_U8 _MApi_SWI2C_GetMaxBuses(void);
MS_U32 _MApi_SWI2C_Speed_Setting(MS_U8 u8BusNum, MS_U32 u32Speed_K);
MS_BOOL _MApi_SWI2C_SetReadMode(SWI2C_ReadMode eReadMode);
MS_BOOL _MApi_SWI2C_SetBusReadMode(MS_U8 u8BusNum, SWI2C_ReadMode eReadMode);
MS_BOOL _MApi_SWI2C_GetLibVer(const MSIF_Version **ppVersion);
SWI2C_Result _MApi_SWI2C_SetDbgLevel(SWI2C_DbgLvl eLevel);

#if defined(MSOS_TYPE_LINUX_KERNEL)
extern int MDrv_SW_IIC_WriteBytes(u8 u8BusNum, u8 u8SlaveID, u8 u8addrcount, u8* pu8addr, u16 u16size, u8* pu8data);
extern int MDrv_SW_IIC_ReadBytes(u8 u8BusNum, u8 u8SlaveID, u8 u8AddrNum, u8* paddr, u16 u16size, u8* pu8data);
extern int MDrv_SW_IIC_WriteBytesStop(u8 u8BusNum, u8 u8SlaveID, u8 AddrCnt, u8* pu8addr, u16 u16size, u8* pBuf, int bGenStop);
#endif

#endif
