//<MStar Software>
//******************************************************************************
// MStar Software
// Copyright (c) 2010 - 2012 MStar Semiconductor, Inc. All rights reserved.
// All software, firmware and related documentation herein ("MStar Software") are
// intellectual property of MStar Semiconductor, Inc. ("MStar") and protected by
// law, including, but not limited to, copyright law and international treaties.
// Any use, modification, reproduction, retransmission, or republication of all
// or part of MStar Software is expressly prohibited, unless prior written
// permission has been granted by MStar.
//
// By accessing, browsing and/or using MStar Software, you acknowledge that you
// have read, understood, and agree, to be bound by below terms ("Terms") and to
// comply with all applicable laws and regulations:
//
// 1. MStar shall retain any and all right, ownership and interest to MStar
//    Software and any modification/derivatives thereof.
//    No right, ownership, or interest to MStar Software and any
//    modification/derivatives thereof is transferred to you under Terms.
//
// 2. You understand that MStar Software might include, incorporate or be
//    supplied together with third party`s software and the use of MStar
//    Software may require additional licenses from third parties.
//    Therefore, you hereby agree it is your sole responsibility to separately
//    obtain any and all third party right and license necessary for your use of
//    such third party`s software.
//
// 3. MStar Software and any modification/derivatives thereof shall be deemed as
//    MStar`s confidential information and you agree to keep MStar`s
//    confidential information in strictest confidence and not disclose to any
//    third party.
//
// 4. MStar Software is provided on an "AS IS" basis without warranties of any
//    kind. Any warranties are hereby expressly disclaimed by MStar, including
//    without limitation, any warranties of merchantability, non-infringement of
//    intellectual property rights, fitness for a particular purpose, error free
//    and in conformity with any international standard.  You agree to waive any
//    claim against MStar for any loss, damage, cost or expense that you may
//    incur related to your use of MStar Software.
//    In no event shall MStar be liable for any direct, indirect, incidental or
//    consequential damages, including without limitation, lost of profit or
//    revenues, lost or damage of data, and unauthorized system use.
//    You agree that this Section 4 shall still apply without being affected
//    even if MStar Software has been modified by MStar in accordance with your
//    request or instruction for your use, except otherwise agreed by both
//    parties in writing.
//
// 5. If requested, MStar may from time to time provide technical supports or
//    services in relation with MStar Software to you for your use of
//    MStar Software in conjunction with your or your customer`s product
//    ("Services").
//    You understand and agree that, except otherwise agreed by both parties in
//    writing, Services are provided on an "AS IS" basis and the warranty
//    disclaimer set forth in Section 4 above shall apply.
//
// 6. Nothing contained herein shall be construed as by implication, estoppels
//    or otherwise:
//    (a) conferring any license or right to use MStar name, trademark, service
//        mark, symbol or any other identification;
//    (b) obligating MStar or any of its affiliates to furnish any person,
//        including without limitation, you and your customers, any assistance
//        of any kind whatsoever, or any information; or
//    (c) conferring any license or right under any intellectual property right.
//
// 7. These terms shall be governed by and construed in accordance with the laws
//    of Taiwan, R.O.C., excluding its conflict of law rules.
//    Any and all dispute arising out hereof or related hereto shall be finally
//    settled by arbitration referred to the Chinese Arbitration Association,
//    Taipei in accordance with the ROC Arbitration Law and the Arbitration
//    Rules of the Association by three (3) arbitrators appointed in accordance
//    with the said Rules.
//    The place of arbitration shall be in Taipei, Taiwan and the language shall
//    be English.
//    The arbitration award shall be final and binding to both parties.
//
//******************************************************************************
//<MStar Software>
////////////////////////////////////////////////////////////////////////////////
//
// Copyright (c) 2008-2009 MStar Semiconductor, Inc.
// All rights reserved.
//
// Unless otherwise stipulated in writing, any and all information contained
// herein regardless in any format shall remain the sole proprietary of
// MStar Semiconductor Inc. and be kept in strict confidence
// ("MStar Confidential Information") by the recipient.
// Any unauthorized act including without limitation unauthorized disclosure,
// copying, use, reproduction, sale, distribution, modification, disassembling,
// reverse engineering and compiling of the contents of MStar Confidential
// Information is unlawful and strictly prohibited. MStar hereby reserves the
// rights to any and all damages, losses, costs and expenses resulting therefrom.
//
////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////
///
/// @file   drvCH34.h
/// @brief  CH34 Driver Interface
/// @author MStar Semiconductor Inc.
///////////////////////////////////////////////////////////////////////////////////////////////////

#ifndef __DRV_CH34_H__
#define __DRV_CH34_H__

#ifdef __cplusplus
extern "C"
{
#endif


//-------------------------------------------------------------------------------------------------
//  Driver Capability
//-------------------------------------------------------------------------------------------------

//-------------------------------------------------------------------------------------------------
//  Macro and Define
//-------------------------------------------------------------------------------------------------

/// ATTRIBUTE_INFO_EX version of current CH34 lib
#define ATTRIBUTE_INFO_VERSION                          1
#define SET_DAC_OUTPUT_SWING_VERSION             1
#define SET_SIGMA_DELTA_GAIN_VERSION               1

/// Version string.
#define CH34_DRV_VERSION                /* Character String for DRV/API version             */  \
    MSIF_TAG,                           /* 'MSIF'                                           */  \
    MSIF_CLASS,                         /* '00'                                             */  \
    MSIF_CUS,                           /* 0x0000                                           */  \
    MSIF_MOD,                           /* 0x0000                                           */  \
    MSIF_CHIP,                                                                                  \
    MSIF_CPU,                                                                                   \
    {'C','H','3','4'},                  /* IP__                                             */  \
    {'0','1'},                          /* 0.0 ~ Z.Z                                        */  \
    {'0','1'},                          /* 00 ~ 99                                          */  \
    {'0','0','0','0','0','0','0','0'},  /* CL#                                              */  \
    MSIF_OS


//-------------------------------------------------------------------------------------------------
//  Type and Structure
//-------------------------------------------------------------------------------------------------

typedef enum
{
    ///failed
    En_Drv_CH34_FAIL = 0,
    ///success
    En_Drv_CH34_OK,
    ///invalid parameter
    En_Drv_CH34_INVALID_PARAM,
    ///access not allow
    En_Drv_CH34_NotSupport,
}CH34_Drv_Result;

typedef enum
{
    E_CH34_VIDEO_SYS_NTSC = 0,     ///< NTSC
    E_CH34_VIDEO_SYS_PAL,          ///< PAL
    E_CH34_VIDEO_SYS_NUM,
} E_CH34_VIDEO_SYS;

typedef enum
{
    E_CH34_AUDIO_SYS_MONO = 0,     ///< MONO
    E_CH34_AUDIO_SYS_BTSC,         ///< BTSC_STEREO
    E_CH34_AUDIO_SYS_BTSC_SAP,     ///< BTSC_SAP
    E_CH34_AUDIO_SYS_A2,           ///< A2
    E_CH34_AUDIO_SYS_NUM,
} E_CH34_AUDIO_SYS;

typedef enum
{
    E_CH34_CH3 = 0,     ///< ch3
    E_CH34_CH4,          ///< ch4
    E_CH34_NUM,
} E_CH34_CHANNEL;

typedef enum{
    E_CH34_SPEC_DEFAULT = 0,               ///< Default
    E_CH34_SPEC_ABNT_NBR_15604,       ///< ABNT_NBR_15604 spec
} E_CH34_SPEC_SELECT;

typedef enum{
    E_CH34_DAC_SWING_LEVEL0 = 0,     ///< Dac output swing level0
    E_CH34_DAC_SWING_LEVEL1,           ///< Dac output swing level1
    E_CH34_DAC_SWING_LEVEL2,           ///< Dac output swing level2
    E_CH34_DAC_SWING_LEVEL3,           ///< Dac output swing level3
} E_CH34_DAC_SWING_LEVEL;

typedef enum
{
    E_CH34_SIGMA_DELTA_GAIN_LEVEL0 = 0,     ///< Sigma Delta Gain level0, 0.9375
    E_CH34_SIGMA_DELTA_GAIN_LEVEL1,           ///< Sigma Delta Gain level1, 0.875
    E_CH34_SIGMA_DELTA_GAIN_LEVEL2,           ///< Sigma Delta Gain level2, 0.75
    E_CH34_SIGMA_DELTA_GAIN_LEVEL3,           ///< Sigma Delta Gain level3, 0.5
} E_CH34_SIGMA_DELTA_GAIN_LEVEL;

typedef enum
{
    CH34_CMD_NONE = 0,
    CH34_CMD_SET_DAC_OUTPUT_SWING,
    CH34_CMD_SET_SIGMA_DELTA_GAIN,
    CH34_CMD_NUMBER,
} CH34_CTRL_ID;

typedef struct{
    MS_U32 u32AttributeInfo_Version;   ///<Version of current structure. Please always set to "ATTRIBUTE_INFO_VERSION" as input
    MS_U16 u16AttributeInfo_Length;    ///<Length of this structure, u16AttributeInfo_Length=sizeof(CH34_ATTRIBUTE_INFO)
    E_CH34_CHANNEL eChannel;
    E_CH34_VIDEO_SYS eVideoSys;
    E_CH34_AUDIO_SYS eAudioSys;
    E_CH34_SPEC_SELECT eSpecSelect;
} CH34_ATTRIBUTE_INFO;

typedef struct{
    MS_U32 u32SetDACOutputSwing_Version;   ///<Version of current structure. Please always set to "SET_DAC_OUTPUT_SWING_VERSION" as input
    MS_U16 u16SetDACOutputSwing_Length;    ///<Length of this structure, u16SetDACOutputSwing_Length=sizeof(CH34_SET_DAC_OUTPUT_SWING)
    E_CH34_DAC_SWING_LEVEL eDACSwingLevel;
} CH34_SET_DAC_OUTPUT_SWING;

typedef struct{
    MS_U32 u32SetSigmaDeltaGain_Version;   ///<Version of current structure. Please always set to "SET_SIGMA_DELTA_GAIN_VERSION" as input
    MS_U16 u16SetSigmaDeltaGain_Length;    ///<Length of this structure, u16SetSigmaDeltaGain_Length=sizeof(CH34_SET_SIGMA_DELTA_GAIN)
    E_CH34_SIGMA_DELTA_GAIN_LEVEL eSigmaDeltaGainLevel;
} CH34_SET_SIGMA_DELTA_GAIN;

#define CH34_TABLE_OFFSET                  4
#define CH34_TABLE_MAX_DEPTH               2000

//-------------------------------------------------------------------------------------------------
//  Function and Variable
//-------------------------------------------------------------------------------------------------

CH34_Drv_Result MDrv_Ch34_Init(void);
CH34_Drv_Result MDrv_Ch34_TurnOnOff(MS_BOOL bOn);
CH34_Drv_Result MDrv_Ch34_SetMode(E_CH34_CHANNEL channel, E_CH34_VIDEO_SYS videosys, E_CH34_AUDIO_SYS audiosys);
CH34_Drv_Result MDrv_Ch34_WriteCH34Table(MS_U8* pTable);
CH34_Drv_Result MDrv_Ch34_SetAttribute(CH34_ATTRIBUTE_INFO* pInfo);
MS_BOOL MDrv_Ch34_GeneralCtrl(MS_U32 u32Cmd, void* pBuf, MS_U32 u32BufSize);

#ifdef __cplusplus
}
#endif


#endif // __DRV_CH34_H__

