/* SPDX-License-Identifier: GPL-2.0-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2 of the GNU General Public License ("GPLv2 License")
 * or BSD License.
 *
 * GPLv2 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See http://www.gnu.org/licenses/gpl-2.0.html for more details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/
#include <MsTypes.h>
#include <common.h>
#include <MsCommon.h>
#include <MsApiAesdma.h>
#include <drvAESDMA.h>
#include <MsDebug.h>

#define TEST_ROUNDS         (10000)
#define TEST_ITEM           (13)
#define KEY_MAX             (16)
#define AES_IN_ADDR         (0x0)
#define AES_OUT_ADDR        (0x10000)
#define ERR                 (-1)
#define OK                  (0)
static MS_U8 _au8keyData[KEY_MAX] = {0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 
                                     0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f};

static MS_U8 _au8IvData[KEY_MAX] =  {0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 
                                     0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f};

                                
static MS_U32 _au32TestSize[TEST_ITEM] = {128 , 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, 65536, 131072, 20480, 102400};
static MS_U8* _szTestSize[TEST_ITEM] = 
{
    (MS_U8*)"128    Byte",
    (MS_U8*)"256    Byte",
    (MS_U8*)"512    Byte",
    (MS_U8*)"1      KB",
    (MS_U8*)"2      KB",
    (MS_U8*)"4      KB",
    (MS_U8*)"8      KB",
    (MS_U8*)"16     KB",
    (MS_U8*)"32     KB",
    (MS_U8*)"64     KB",
    (MS_U8*)"128    KB",

    (MS_U8*)"20     KB",
    (MS_U8*)"100    KB"
};
    
static int _convert_cipher_mode(cmd_tbl_t *cmdtp, int argc, char * const argv[], DrvAESDMA_CipherMode *peCipherMode)
{
    DrvAESDMA_CipherMode eCipherMode = E_DRVAESDMA_CIPHER_ECB;

    if (argc < 4)
    {
        UBOOT_ERROR("Error: Invalid parameter nums(argc = %d).\n", argc);
        cmd_usage(cmdtp);
        return ERR;
    }

    if (peCipherMode == NULL)
    {
        UBOOT_ERROR("Error: peCipherMode is null.\n");
        return ERR;
    }

    // argv[2]: "aes", "des", "tdes"
    // argv[3]: "ecb", "ctr", "cbc", "cts_cbc", "cts_ecb", "cbcs", "cens"

    if (memcmp(argv[2], "aes", strlen(argv[2])) == 0)
    {
        if (memcmp(argv[3], "ecb", strlen(argv[3])) == 0)
        {
            eCipherMode = E_DRVAESDMA_CIPHER_ECB;
        }
        else if (memcmp(argv[3], "ctr", strlen(argv[3])) == 0)
        {
            eCipherMode = E_DRVAESDMA_CIPHER_CTR;
        }
        else if (memcmp(argv[3], "cbc", strlen(argv[3])) == 0)
        {
            eCipherMode = E_DRVAESDMA_CIPHER_CBC;
        }
        else if (memcmp(argv[3], "cts_ecb", strlen(argv[3])) == 0)
        {
            eCipherMode = E_DRVAESDMA_CIPHER_CTS_ECB;
        }
        else if (memcmp(argv[3], "cts_cbc", strlen(argv[3])) == 0)
        {
            eCipherMode = E_DRVAESDMA_CIPHER_CTS_CBC;
        }
#if 0
        else if (memcmp(argv[3], "cbcs", strlen(argv[3])) == 0)
        {
            eCipherMode = E_DRVAESDMA_CIPHER_CBCS;
        }
        else if (strcmp(argv[3], "cens", strlen(argv[3])) == 0)
        {
            eCipherMode = E_DRVAESDMA_CIPHER_CENS;
        }
#endif
        else 
        {
            UBOOT_ERROR("Error: Invalid parameter(argv[3] = %s).\n", argv[3]);
            return ERR;
        }
    }
    else if (memcmp(argv[2], "des", strlen(argv[2])) == 0)
    {
        if (memcmp(argv[3], "ecb", strlen(argv[3])) == 0)
        {
            eCipherMode = E_DRVAESDMA_CIPHER_DES_ECB;
        }
        else if (memcmp(argv[3], "ctr", strlen(argv[3])) == 0)
        {
            eCipherMode = E_DRVAESDMA_CIPHER_DES_CTR;
        }
        else if (memcmp(argv[3], "cbc", strlen(argv[3])) == 0)
        {
            eCipherMode = E_DRVAESDMA_CIPHER_DES_CBC;
        }
        else if (memcmp(argv[3], "cts_ecb", strlen(argv[3])) == 0)
        {
            eCipherMode = E_DRVAESDMA_CIPHER_DES_CTS_ECB;
        }
        else if (memcmp(argv[3], "cts_cbc", strlen(argv[3])) == 0)
        {
            eCipherMode = E_DRVAESDMA_CIPHER_DES_CTS_CBC;
        }
#if 0
        else if (memcmp(argv[3], "cbcs", strlen(argv[3])) == 0)
        {
            eCipherMode = E_DRVAESDMA_CIPHER_DES_CBCS;
        }
        else if (strcmp(argv[3], "cens", strlen(argv[3])) == 0)
        {
            eCipherMode = E_DRVAESDMA_CIPHER_DES_CENS;
        }
 #endif
       else 
        {
            UBOOT_ERROR("Error: Invalid parameter(argv[3] = %s).\n", argv[3]);
            return ERR;
        }   
    }
    else if (memcmp(argv[2], "tdes", strlen(argv[2])) == 0)
    {
        if (memcmp(argv[3], "ecb", strlen(argv[3])) == 0)
        {
            eCipherMode = E_DRVAESDMA_CIPHER_TDES_ECB;
        }
        else if (memcmp(argv[3], "ctr", strlen(argv[3])) == 0)
        {
            eCipherMode = E_DRVAESDMA_CIPHER_TDES_CTR;
        }
        else if (memcmp(argv[3], "cbc", strlen(argv[3])) == 0)
        {
            eCipherMode = E_DRVAESDMA_CIPHER_TDES_CBC;
        }
        else if (memcmp(argv[3], "cts_ecb", strlen(argv[3])) == 0)
        {
            eCipherMode = E_DRVAESDMA_CIPHER_TDES_CTS_ECB;
        }
        else if (memcmp(argv[3], "cts_cbc", strlen(argv[3])) == 0)
        {
            eCipherMode = E_DRVAESDMA_CIPHER_TDES_CTS_CBC;
        }
#if 0
        else if (memcmp(argv[3], "cbcs", strlen(argv[3])) == 0)
        {
            eCipherMode = E_DRVAESDMA_CIPHER_TDES_CBCS;
        }
        else if (memcmp(argv[3], "cens", strlen(argv[3])) == 0)
        {
            eCipherMode = E_DRVAESDMA_CIPHER_TDES_CENS;
        } 
#endif   
       else 
        {
            UBOOT_ERROR("Error: Invalid parameter(argv[3] = %s) for cipher mode.\n", argv[3]);
            return ERR;
        } 

    }
    else
    {
        UBOOT_ERROR("Error: Invalid parameter(argv[2] = %s) for cipher algorithm.\n", argv[2]);
        return ERR;
    }
        

    *peCipherMode = eCipherMode;
  
    return OK;
}

static int do_run_aesdma_performance(cmd_tbl_t *cmdtp, int argc, char * const argv[])
{
    UBOOT_TRACE("Enter the do_run_aesdma_performance().\n"); 

    int i, j;
    MS_PHY phyAddr = 0; // don't care
    DrvAESDMA_CipherMode eCipherMode = E_DRVAESDMA_CIPHER_ECB;
    
    MS_U32 u32StartTime = 0;
    MS_U32 u32EndTime = 0;

    MS_U32 u32SrcAddr = AES_IN_ADDR;
    MS_U32 u32DstAddr = AES_OUT_ADDR;
    MS_U32 u32DataSize = 0;
    MS_BOOL bExeSpecifySize = FALSE;
#if 0
    MS_BOOL bEnableCencMode = FALSE;
    MS_U32 u32CryptedBlock = 1;
    MS_U32 u32SkippedBlock = 9;
    MS_U32 u32Offset = 0;
    DrvAESDMA_CencMode eCencMode = E_DRVAESDMA_AES_MAX;
#endif

    if (_convert_cipher_mode(cmdtp, argc, argv, &eCipherMode) != OK)
    {
        UBOOT_ERROR("Error: Fail to call the _convert_cipher_mode().\n");
        return ERR;
    }

    if (argc >= 5)
    {   
        u32DataSize = (MS_U32)simple_strtoul(argv[4], NULL, 10);
        UBOOT_INFO("u32DataSize = 0x%x.\n", u32DataSize);

        if (u32DataSize != 0)
        {
            bExeSpecifySize = TRUE;
        }
    }
#if 0
    if (eCipherMode == E_DRVAESDMA_CIPHER_CBCS || eCipherMode == E_DRVAESDMA_CIPHER_CENS)
    {
        if (argc >= 7)
        {
            u32CryptedBlock = (MS_U32)simple_strtoul(argv[6], NULL, 10);
            u32SkippedBlock = (MS_U32)simple_strtoul(argv[7], NULL, 10);
        }
        UBOOT_INFO("u32CryptedBlock = 0x%x.\n", u32CryptedBlock);
        UBOOT_INFO("u32SkippedBlock = 0x%x.\n", u32SkippedBlock);

        switch (eCipherMode)
        {
            case E_DRVAESDMA_CIPHER_CBCS:
                eCencMode = E_DRVAESDMA_AES_CBCS;
                break;
            case E_DRVAESDMA_CIPHER_CENS:
                eCencMode = E_DRVAESDMA_AES_CENS;
                break;
            default:
                UBOOT_ERROR("Error: Invalid eCipherMode(%d).\n", eCipherMode);
                return ERR;
        }
        bEnableCencMode = TRUE;
    }
#endif

    MDrv_AESDMA_Init(phyAddr, phyAddr, 2);
    
    for (i = 0; i < TEST_ITEM; i++)
    {        
        if (bExeSpecifySize == FALSE)
        {
            u32DataSize = _au32TestSize[i];
            //UBOOT_INFO("u32DataSize = 0x%x.\n", u32DataSize);
        }

        u32StartTime = MsOS_GetSystemTime();
        for (j = 0; j < TEST_ROUNDS; j++)
        {   
            MDrv_AESDMA_Reset();
            MDrv_AESDMA_SetFileInOut(u32SrcAddr, u32DataSize, u32DstAddr, u32DstAddr+u32DataSize-1);
            MDrv_AESDMA_SetKey_Ex((MS_U8*)_au8keyData, KEY_MAX);
            MDrv_AESDMA_SetIV_Ex((MS_U8*)_au8IvData, KEY_MAX);
            MDrv_AESDMA_SelEng(eCipherMode, FALSE);
#if 0
            if (bEnableCencMode == TRUE)
            {
                MDrv_AESDMA_SetCencParams(eCencMode, u32CryptedBlock, u32SkippedBlock, u32Offset);
            }
#endif
            MDrv_AESDMA_Start(TRUE);
            while (MDrv_AESDMA_IsFinished() != DRVAESDMA_OK);          
        }

        u32EndTime = MsOS_GetSystemTime();

        if (bExeSpecifySize == TRUE)
        {
            printf("\nBytes(%lu) * rounds(%d) = %lu ms.\n", u32DataSize, TEST_ROUNDS, u32EndTime - u32StartTime);
            break;
        }
        else if (bExeSpecifySize == FALSE)
        {
            printf("\n%s * rounds(%d) = %lu ms.\n", _szTestSize[i], TEST_ROUNDS, u32EndTime - u32StartTime);
        }
        
    }

    UBOOT_TRACE("Exit the do_run_aesdma_performance().\n");
    return OK;
}

int do_run_aesdma_cmd(cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
    UBOOT_TRACE("Enter the do_run_aesdma_cmd()\n");
    
    // argv[1]: "performance"
    if (memcmp(argv[1], "performance", strlen(argv[1])) == 0)
    {
        UBOOT_TRACE("Call the do_run_aesdma_performance().\n");
        if (do_run_aesdma_performance(cmdtp, argc, argv) != OK)
        {
            UBOOT_ERROR("Error: Fail to call the do_run_aesdma_performance().\n");
            return ERR;
        }
    }
    else 
    {
        UBOOT_ERROR("Error: Invalid parameter(argv[1] = %s) for command type.\n", argv[1]);
        cmd_usage(cmdtp);
        return ERR;
    }
  
    UBOOT_TRACE("Exit the do_run_aesdma_cmd()\n");
    return OK;
}

