////////////////////////////////////////////////////////////////////////////////////////////////////
///
/// file    CfgAttr.c
/// @brief  Helpers of Attributes in ini files
/// @author MediaTek Inc.
////////////////////////////////////////////////////////////////////////////////////////////////////

//-------------------------------------------------------------------------------------------------
//  Include Files
//-------------------------------------------------------------------------------------------------
#include <stdlib.h>

#include <MsTypes.h>

#include <common.h>
#include <command.h>
#include <bootlogo/MsPoolDB.h>
#include <malloc.h>

#include <MsDebug.h>

#include "AmpDeviceUtil.h"

//-------------------------------------------------------------------------------------------------
//  Debug Defines
//-------------------------------------------------------------------------------------------------
#define PARSER_DEBUG

#ifdef IS_NULL
#undef IS_NULL
#endif //#ifdef IS_NULL
#ifdef IS_NOT_NULL
#undef IS_NOT_NULL
#endif //#ifdef IS_NOT_NULL
#define IS_NULL(pointer)                                (NULL == (pointer))
#define IS_NOT_NULL(pointer)                            (!(IS_NULL(pointer)))

//-------------------------------------------------------------------------------------------------
//  Local Defines
//-------------------------------------------------------------------------------------------------
#define FREE(pointer) \
    if(IS_NOT_NULL(pointer)) \
    { \
        free(pointer); \
        (pointer) = NULL; \
    }

//-------------------------------------------------------------------------------------------------
//  Local Variables
//-------------------------------------------------------------------------------------------------

//--------------------------------------------------------------------------------------------------
//  Global Variables
//--------------------------------------------------------------------------------------------------

//-------------------------------------------------------------------------------------------------
//  Local Functions
//-------------------------------------------------------------------------------------------------

//-------------------------------------------------------------------------------------------------
//  Static Functions
//-------------------------------------------------------------------------------------------------
#ifdef PARSER_DEBUG
void _print_amp_device_gpio_info(amp_device_gpio_info_t stInfo, char *title)
{
    printf("%s\n", title);
    printf("u16GpioNum = %d\n", stInfo.u16GpioNum);
    printf("bInvert = %s\n", stInfo.bInvert? "TRUE": "FALSE");

    printf("\n");

    return;
}

void _print_amp_device_iic_info(amp_device_iic_info_t stInfo, char *title)
{
    printf("%s\n", title);
    printf("ePort = 0x%x\n", stInfo.ePort);
    printf("u16BusNumSlaveID =0x%x\n", stInfo.u16BusNumSlaveID);

    printf("\n");

    return;
}

static void _print_amp_device_attr_array(amp_device_attr_array_t *pstValue, amp_device_attr_data_type_e eDataType)
{
    MS_U16 i = 0;

    if(NULL == pstValue)
    {
        printf("pstValue is NULL.\n");
        return;
    }

    switch(eDataType)
    {
        case E_AMP_DEVICE_ATTR_DATA_TYPE_INT_ARRAY:
            printf("ps32Array = \n");

            break;

        case E_AMP_DEVICE_ATTR_DATA_TYPE_UINT_ARRAY:
            printf("pu32Array = \n");

            break;

        default:

            break;
    }

    for(i = 0 ; i < pstValue->u16Size; i++)
    {
        switch(eDataType)
        {
            case E_AMP_DEVICE_ATTR_DATA_TYPE_INT_ARRAY:
                if(pstValue->stArray.ps32Array[i] >= 0)
                {
                    printf("[%02d] = 0x%08x\n", i, (unsigned)pstValue->stArray.ps32Array[i]);
                }
                else
                {
                    printf("[%02d] = -0x%08x\n", i, (unsigned)-pstValue->stArray.ps32Array[i]);
                }

                break;

            case E_AMP_DEVICE_ATTR_DATA_TYPE_UINT_ARRAY:
                printf("[%02d] = 0x%08x\n", i, (unsigned)pstValue->stArray.pu32Array[i]);

                break;

            default:
                // Do nothing.

                break;
        }
    }

    return;
}

static void _print_amp_device_attr(amp_device_attr_t *pstAttr)
{
    if(NULL == pstAttr)
    {
        printf("pstAttr is NULL.\n");
        return;
    }

    switch(pstAttr->eDataType)
    {
        case E_AMP_DEVICE_ATTR_DATA_TYPE_INT:
            printf("s32Value = %08d.\n", pstAttr->stData.s32Value);
            break;

        case E_AMP_DEVICE_ATTR_DATA_TYPE_UINT:
            printf("u32Value = %08u.\n", pstAttr->stData.u32Value);
            break;

        case E_AMP_DEVICE_ATTR_DATA_TYPE_STRING:
            printf("pszValue = \"%s\".\n", (pstAttr->stData.pszValue)? (pstAttr->stData.pszValue): "");
            break;

        case E_AMP_DEVICE_ATTR_DATA_TYPE_INT_ARRAY:
            _print_amp_device_attr_array(&(pstAttr->stData.stValue), pstAttr->eDataType);
            break;

        case E_AMP_DEVICE_ATTR_DATA_TYPE_UINT_ARRAY:
            _print_amp_device_attr_array(&(pstAttr->stData.stValue), pstAttr->eDataType);
            break;
    }

    return;
}

static void _print_amp_device_attr_info(amp_device_attr_info_t *pstAttrInfo)
{
    char title[MAX_KEY_LEN];
    int i;

    printf("\n");
    printf("u8AttrTotalNumber = %d\n", pstAttrInfo->u8AttrTotalNumber);
    for(i = 0; i < pstAttrInfo->u8AttrTotalNumber; i++)
    {
        memset(title, 0, sizeof(title));
        snprintf(title, MAX_KEY_LEN, "pstAttr[%d]", i);

        printf("\n");
        printf("%s\n", title);
        printf("pszName = %s\n", (pstAttrInfo->pstAttr[i].pszName)? (pstAttrInfo->pstAttr[i].pszName): "");
        printf("eDataType = %d\n", pstAttrInfo->pstAttr[i].eDataType);
        _print_amp_device_attr(&(pstAttrInfo->pstAttr[i]));
    }

    return;
}
#endif //#ifdef PARSER_DEBUG

static int _free_amp_device_attr_info(amp_device_attr_info_t *pstAmpDeviceAttrInfo)
{
    int ret = CONVERT_SUCCESS;
    int i;

    if(NULL == pstAmpDeviceAttrInfo)
    {
        ret = CONVERT_INVALID;

        return ret;
    }

    for(i = 0; i < pstAmpDeviceAttrInfo->u8AttrTotalNumber; i++)
    {
        FREE(pstAmpDeviceAttrInfo->pstAttr[i].pszName);

        switch(pstAmpDeviceAttrInfo->pstAttr[i].eDataType)
        {
            case E_AMP_DEVICE_ATTR_DATA_TYPE_STRING:
                FREE(pstAmpDeviceAttrInfo->pstAttr[i].stData.pszValue);
                break;

            case E_AMP_DEVICE_ATTR_DATA_TYPE_INT_ARRAY:
                pstAmpDeviceAttrInfo->pstAttr[i].stData.stValue.u16Size = 0;
                FREE(pstAmpDeviceAttrInfo->pstAttr[i].stData.stValue.stArray.ps32Array);
                break;

            case E_AMP_DEVICE_ATTR_DATA_TYPE_UINT_ARRAY:
                pstAmpDeviceAttrInfo->pstAttr[i].stData.stValue.u16Size = 0;
                FREE(pstAmpDeviceAttrInfo->pstAttr[i].stData.stValue.stArray.pu32Array);
                break;

            default:
                break;
        }
    }

    pstAmpDeviceAttrInfo->u8AttrTotalNumber = 0;
    FREE(pstAmpDeviceAttrInfo->pstAttr);

    return ret;
}

static int _convert_amp_device_attr_info_from_db(amp_device_attr_info_t *pstAmpDeviceAttrInfo, db_attr_info_t *pstDbAttrInfo)
{
    int ret = CONVERT_SUCCESS;

    if(NULL == pstAmpDeviceAttrInfo)
    {
        return CONVERT_INVALID;
    }
    if(NULL == pstDbAttrInfo)
    {
        return CONVERT_INVALID;
    }

    pstAmpDeviceAttrInfo->u8AttrTotalNumber = pstDbAttrInfo->m_u8AttrTotalNumber;
    pstAmpDeviceAttrInfo->pstAttr = pstDbAttrInfo->m_pstAttr;

    pstDbAttrInfo->m_u8AttrTotalNumber = 0;
    pstDbAttrInfo->m_pstAttr = NULL;

    return ret;
}

static int _convert_amp_device_info_from_db(amp_device_info_t *pstAmpDeviceInfo, db_amp_device_info_t *pstDbAmpDeviceInfo, db_attr_info_t *pstDbAttrInfo, db_iic_bus_info_t stIicBusInfos[MAX_IIC_BUS_NUM])
{
    int ret = 0;
    MS_IIC_PORT eCusIicPort;

    pstAmpDeviceInfo->u32DeviceId = pstDbAmpDeviceInfo->m_u32DeviceId;
    memcpy(&(pstAmpDeviceInfo->stGpioReset), &(pstDbAmpDeviceInfo->m_stGpioReset), sizeof(amp_device_gpio_info_t));
    memcpy(&(pstAmpDeviceInfo->stGpioStandby), &(pstDbAmpDeviceInfo->m_stGpioStandby), sizeof(amp_device_gpio_info_t));
    memcpy(&(pstAmpDeviceInfo->stIicInfo), &(pstDbAmpDeviceInfo->m_stIicInfo), sizeof(amp_device_iic_info_t));
    ret = convert_2_cus_iic_port_from_db(&eCusIicPort, pstAmpDeviceInfo->stIicInfo.ePort, stIicBusInfos);
    if(0 != ret)
    {
        UBOOT_DEBUG("convert_2_cus_iic_port_from_db(...) is failed.\n");
        // It is possible that BOARD_I2C_SWBUS in board.ini can't be mapped to settings in board.h.
    }
    pstAmpDeviceInfo->stIicInfo.ePort = eCusIicPort;

    ret = _convert_amp_device_attr_info_from_db(&(pstAmpDeviceInfo->stAttrInfo), pstDbAttrInfo);
    if(0 != ret)
    {
        UBOOT_DEBUG("_convert_amp_device_attr_info_from_db(...) is failed.\n");

        return ret;
    }

    return ret;
}

//-------------------------------------------------------------------------------------------------
//  Internal Global Functions
//-------------------------------------------------------------------------------------------------

//-------------------------------------------------------------------------------------------------
//  Global Functions
//-------------------------------------------------------------------------------------------------
void print_amp_device_info(amp_device_info_t *pstAttrInfo)
{
#ifdef PARSER_DEBUG
    printf("u32DeviceId = %d\n", pstAttrInfo->u32DeviceId);

    _print_amp_device_gpio_info(pstAttrInfo->stGpioStandby, "stGpioStandby");
    _print_amp_device_gpio_info(pstAttrInfo->stGpioReset, "stGpioReset");
    _print_amp_device_iic_info(pstAttrInfo->stIicInfo, "stIicInfo");

    _print_amp_device_attr_info(&(pstAttrInfo->stAttrInfo));
#endif //#ifdef PARSER_DEBUG

    return;
}

int read_iic_para_from_flash(db_iic_bus_info_t stIicBusInfos[MAX_IIC_BUS_NUM])
{
    int ret = 0;
    st_board_para stBoardPara;

    ret = Read_BoardParaFromflash(&stBoardPara);
    if(0 != ret)
    {
        UBOOT_DEBUG("Read_BoardParaFromflash(...) is failed.\n");

        return ret;
    }

    // Notice: Use sizeof(stBoardPara.m_stIIC_BUS_INFO) instead of sizeof(stIicBusInfos).
    // sizeof(stIicBusInfos) = 4: the size of pointer
    // sizeof(stBoardPara.m_stIIC_BUS_INFO) = 320
    memcpy(stIicBusInfos, stBoardPara.m_stIIC_BUS_INFO, sizeof(stBoardPara.m_stIIC_BUS_INFO));

#ifdef PARSER_DEBUG
    print_db_iic_bus_infos(stIicBusInfos);
#endif //#ifdef PARSER_DEBUG

    return ret;
}

int read_amp_para_from_flash(amp_device_info_t stAmpDeviceInfo[MAX_AMP_NUM])
{
    int ret = 0;
    st_board_para stBoardPara;
    st_amp_attr_para stAmpAttrPara = {NULL, 0};
    db_attr_info_t stAttrInfos[MAX_AMP_NUM];
    int i;

    memset(&stBoardPara, 0, sizeof(st_board_para));
    memset(&stAttrInfos, 0, sizeof(stAttrInfos));
    stAmpAttrPara.m_u16Size = MAX_ATTR_INFOS_LEN * sizeof(char);
    stAmpAttrPara.m_psSerializedAmpAttrInfos = calloc(stAmpAttrPara.m_u16Size, sizeof(char));
    if(NULL == stAmpAttrPara.m_psSerializedAmpAttrInfos)
    {
        UBOOT_DEBUG("calloc(%d, %d) is failed.\n", stAmpAttrPara.m_u16Size, (int)sizeof(char));
        ret = -1;

        return ret;
    }

    do
    {
        ret = Read_BoardParaFromflash(&stBoardPara);
        if(0 > ret)
        {
            UBOOT_DEBUG("Read_BoardParaFromflash(...) is failed.\n");

            break;
        }

        ret = Read_AmpAttrParaFromFlash(stAmpAttrPara.m_psSerializedAmpAttrInfos);
        if(0 > ret)
        {
            UBOOT_DEBUG("Read_AmpAttrParaFromFlash(...) is failed.\n");

            break;
        }

        ret = deserialize_db_attr_infos(stAttrInfos, stAmpAttrPara.m_psSerializedAmpAttrInfos);
        if(0 > ret)
        {
            UBOOT_DEBUG("deserialize_db_attr_infos(...) is failed.\n");

            break;
        }

        for(i = 0; i < MAX_AMP_NUM; i++)
        {
            ret = _convert_amp_device_info_from_db(&(stAmpDeviceInfo[i]), &(stBoardPara.m_stAMP_INFO[i]), &(stAttrInfos[i]), stBoardPara.m_stIIC_BUS_INFO);
            if(0 > ret)
            {
                UBOOT_DEBUG("_convert_amp_device_info_from_db(...) of stAttrInfos[%d] is failed.\n", i);
                _free_amp_device_attr_info(&(stAmpDeviceInfo[i].stAttrInfo));

                break;
            }
#ifdef PARSER_DEBUG
            print_amp_device_info(&(stAmpDeviceInfo[i]));
#endif //#ifdef PARSER_DEBUG
        }
    } while(0);

    FREE(stAmpAttrPara.m_psSerializedAmpAttrInfos);

    return ret;
}
