////////////////////////////////////////////////////////////////////////////////////////////////////
///
/// file    Audio_amplifier2.c
/// @brief  Glue to audio amplifier driver
/// @author MediaTek Inc.
////////////////////////////////////////////////////////////////////////////////////////////////////


//-------------------------------------------------------------------------------------------------
//  Include Files
//-------------------------------------------------------------------------------------------------
#include <config.h>
#include <MsCommon.h>
#include <drvIIC.h>

#include "Audio_amplifier2.h"
#include "AmpDeviceUtil.h"

//-------------------------------------------------------------------------------------------------
//  Debug Defines
//-------------------------------------------------------------------------------------------------
#define DBG_LEVEL                                       _u32ApiAudioAmpDbgLevel

//-------------------------------------------------------------------------------------------------
//  Local Defines
//-------------------------------------------------------------------------------------------------
#if (ENABLE_POWER_MUSIC == 1) && (ENABLE_AMP_VERSION_SEL_2 == 1)
#define ENABLE_API_AUDIO_AMP_SWI2C_HWI2C_SWITCH         (ENABLE_AUDIO_AMP_SWI2C_HWI2C_SWITCH)
#else
#define ENABLE_API_AUDIO_AMP_SWI2C_HWI2C_SWITCH         (0)
#endif //#if ENABLE_POWER_MUSIC && ENABLE_AMP_VERSION_SEL_2

#define API_AUDIO_AMP_SUPPORT_DYNAMIC_LOAD      1
#define API_AUDIO_AMP_NAME_LENGTH               64

#if (API_AUDIO_AMP_SUPPORT_DYNAMIC_LOAD == 1)
#define API_AUDIO_AMP_TABLE_ENTRY(amp)          {amp, &(GET_AMPLIFIER_ENTRY_NODE(amp))}
#define AMP_ENTRY_COUNT                         (sizeof(_astAmpTable) / sizeof(_astAmpTable[0]))
#endif //#if (API_AUDIO_AMP_SUPPORT_DYNAMIC_LOAD == 1)

// It should be the same w/ MI_SYS_AUDIO_AMP_DEVICE_INFO_MAX_NUM
#define API_AUDIO_AMP_DEVICE_INFO_MAX_NUM       (4)

#define API_AUDIO_AMP_CHECK_CONFIG_RETURN(value) \
    do \
    { \
        if(!_bIsAudioAmpConfigured) \
        { \
            printf("%s: MApi_AudioAmp_FillDeviceInfo(...) should be called in advance.\n", __FUNCTION__); \
            return (value); \
        } \
    } while(0)

#define API_AUDIO_AMP_CHECK_INIT_RETURN(value) \
    do \
    { \
        if(!_bIsAudioAmpInit) \
        { \
            printf("%s: MApi_AudioAmp_Init(...) should be called in advance.\n", __FUNCTION__); \
            return (value); \
        } \
    } while(0)

#define API_AUDIO_AMP_CHECK_AMP_TABLE_EXISTED_RETURN(value) \
    do \
    { \
        if(_MApi_AudioAmp_IsNoAudioAmpTableExisted()) \
        { \
            printf("%s: This chip has no matching amplifier device\n", __FUNCTION__); \
            return (value); \
        } \
    } while(0)

typedef struct amp_device_supplemental_info_s
{
     MS_U32 u32DeviceId;
     MS_IIC_PORT ePort;
} amp_device_supplemental_info_t;

//-------------------------------------------------------------------------------------------------
//  Local Variables
//-------------------------------------------------------------------------------------------------
static MS_U32 _u32ApiAudioAmpDbgLevel = API_AUDIO_AMP_DBG_ERR;
static MS_BOOL _bIsAudioAmpConfigured = FALSE;
static MS_BOOL _bIsAudioAmpInit = FALSE;
static amp_device_supplemental_info_t _astAmpDeviceSupplementalInfos[API_AUDIO_AMP_DEVICE_INFO_MAX_NUM] = {AMPLIFIER_NONE, E_MS_IIC_PORT_NOSUP};
static MS_BOOL _abIsAudioAmpInit[API_AUDIO_AMP_DEVICE_INFO_MAX_NUM] = {FALSE};

//--------------------------------------------------------------------------------------------------
//  Global Variables
//--------------------------------------------------------------------------------------------------
static DRV_AUDIO_AMP_TABLE_TYPE * _pDRV_AUDIO_AMP_TABLE_TYPE[API_AUDIO_AMP_DEVICE_INFO_MAX_NUM] = {NULL};
#if (ENABLE_MULTI_DIGITAL_RT9113B == 1)
extern DRV_AUDIO_AMP_TABLE_TYPE GET_AMPLIFIER_ENTRY_NODE(AMPLIFIER_I2S_RT9113B);
#endif // #if (ENABLE_MULTI_DIGITAL_RT9113B == 1)

#if (ENABLE_MULTI_DIGITAL_TI5707 == 1)
extern DRV_AUDIO_AMP_TABLE_TYPE GET_AMPLIFIER_ENTRY_NODE(AMPLIFIER_I2S_TI5707);
#endif // #if (ENABLE_MULTI_DIGITAL_TI5707 == 1)

#if (ENABLE_MULTI_DIGITAL_TI5719 == 1)
extern DRV_AUDIO_AMP_TABLE_TYPE GET_AMPLIFIER_ENTRY_NODE(AMPLIFIER_I2S_TI5719);
#endif // #if (ENABLE_MULTI_DIGITAL_TI5719 == 1)

#if (ENABLE_MULTI_DIGITAL_TI5733 == 1)
extern DRV_AUDIO_AMP_TABLE_TYPE GET_AMPLIFIER_ENTRY_NODE(AMPLIFIER_I2S_TI5733);
#endif // #if (ENABLE_MULTI_DIGITAL_TI5733 == 1)

#if (ENABLE_MULTI_DIGITAL_AD82010 == 1)
extern DRV_AUDIO_AMP_TABLE_TYPE GET_AMPLIFIER_ENTRY_NODE(AMPLIFIER_I2S_AD82010);
#endif // #if (ENABLE_MULTI_DIGITAL_AD82010 == 1)

#if (ENABLE_MULTI_DIGITAL_DIGITAL_AD82010 == 1)
extern DRV_AUDIO_AMP_TABLE_TYPE GET_AMPLIFIER_ENTRY_NODE(AMPLIFIER_I2S_AD82010);
#endif // #if (ENABLE_MULTI_DIGITAL_DIGITAL_AD82010 == 1)

#if (ENABLE_MULTI_DIGITAL_AD82586 == 1)
extern DRV_AUDIO_AMP_TABLE_TYPE GET_AMPLIFIER_ENTRY_NODE(AMPLIFIER_I2S_AD82586);
#endif // #if (ENABLE_MULTI_DIGITAL_AD82586 == 1)

#if (ENABLE_MULTI_DIGITAL_AD82587 == 1)
extern DRV_AUDIO_AMP_TABLE_TYPE GET_AMPLIFIER_ENTRY_NODE(AMPLIFIER_I2S_AD82587);
#endif // #if (ENABLE_MULTI_DIGITAL_AD82587 == 1)

#if (ENABLE_MULTI_ANALOG_DEFAULT == 1)
extern DRV_AUDIO_AMP_TABLE_TYPE GET_AMPLIFIER_ENTRY_NODE(AMPLIFIER_ANALOG_DEFAULT);
#endif // #if (ENABLE_MULTI_ANALOG_DEFAULT == 1)

typedef struct api_audio_amp_table
{
    MS_U32 u32DeviceId; // Audio amplifier device ID.
    DRV_AUDIO_AMP_TABLE_TYPE *pstEntry; // operators of audio amplifier

} API_AUDIO_AMP_TABLE;

// table list of supporting amplifiers
static API_AUDIO_AMP_TABLE _astAmpTable[] =
{
#if (ENABLE_MULTI_DIGITAL_RT9113B == 1)
    API_AUDIO_AMP_TABLE_ENTRY(AMPLIFIER_I2S_RT9113B),
#endif // #if (ENABLE_MULTI_DIGITAL_RT9113B == 1)

#if (ENABLE_MULTI_DIGITAL_TI5707 == 1)
    API_AUDIO_AMP_TABLE_ENTRY(AMPLIFIER_I2S_TI5707),
#endif // #if (ENABLE_MULTI_DIGITAL_TI5707 == 1)

#if (ENABLE_MULTI_DIGITAL_TI5719 == 1)
    API_AUDIO_AMP_TABLE_ENTRY(AMPLIFIER_I2S_TI5719),
#endif // #if (ENABLE_MULTI_DIGITAL_TI5719 == 1)

#if (ENABLE_MULTI_DIGITAL_TI5733 == 1)
    API_AUDIO_AMP_TABLE_ENTRY(AMPLIFIER_I2S_TI5733),
#endif // #if (ENABLE_MULTI_DIGITAL_TI5733 == 1)

#if (ENABLE_MULTI_DIGITAL_AD82010 == 1)
    API_AUDIO_AMP_TABLE_ENTRY(AMPLIFIER_I2S_AD82010),
#endif // #if (ENABLE_MULTI_DIGITAL_AD82010 == 1)

#if (ENABLE_MULTI_DIGITAL_AD82586 == 1)
    API_AUDIO_AMP_TABLE_ENTRY(AMPLIFIER_I2S_AD82586),
#endif // #if (ENABLE_MULTI_DIGITAL_AD82586 == 1)

#if (ENABLE_MULTI_DIGITAL_AD82587 == 1)
    API_AUDIO_AMP_TABLE_ENTRY(AMPLIFIER_I2S_AD82587),
#endif // #if (ENABLE_MULTI_DIGITAL_AD82587 == 1)

#if (ENABLE_MULTI_ANALOG_DEFAULT == 1)
    API_AUDIO_AMP_TABLE_ENTRY(AMPLIFIER_ANALOG_DEFAULT),
#endif // #if (ENABLE_MULTI_ANALOG_DEFAULT == 1)

    {AMPLIFIER_NONE, NULL}, // none for default entry
};

//-------------------------------------------------------------------------------------------------
//  Local Functions
//-------------------------------------------------------------------------------------------------

//-------------------------------------------------------------------------------------------------
//  Static Functions
//-------------------------------------------------------------------------------------------------
static MS_BOOL _MApi_AudioAmp_FindAndUpdateAudioAmpTable(MS_AUDIO_AMP_DEVICE_INFO_t *pstInfo, DRV_AUDIO_AMP_TABLE_TYPE **ppstAmpTable, amp_device_supplemental_info_t *pstAmpDeviceSupplementalInfo)
{
    MS_U8 i = 0;

    *ppstAmpTable = NULL;
    if(AMPLIFIER_NONE == pstInfo->u32DeviceId)
    {
        API_AUDIO_AMP_INFO("%s:pstInfo->u32DeviceId is AMPLIFIER_NONE.\n",
                __FUNCTION__);

        return TRUE;
    }

    for(i = 0; i < AMP_ENTRY_COUNT; i++)
    {
        if(_astAmpTable[i].u32DeviceId == pstInfo->u32DeviceId)
        {
            // Find match amplifier device.
            API_AUDIO_AMP_INFO("%s:Driver of _astAmpTable[%d] for u32DeviceId(%d) is found.\n",
                    __FUNCTION__,
                    i,
                    pstInfo->u32DeviceId);
            *ppstAmpTable = _astAmpTable[i].pstEntry;
            pstAmpDeviceSupplementalInfo->u32DeviceId = pstInfo->u32DeviceId;
            pstAmpDeviceSupplementalInfo->ePort = E_MS_IIC_PORT_NOSUP;
            if(API_AUDIO_AMP_IS_DIGITAL_AUDIO_AMP(pstAmpDeviceSupplementalInfo->u32DeviceId))
            {
                pstAmpDeviceSupplementalInfo->ePort = pstInfo->stIicInfo.ePort;
            }
            break;
        }
    }
    if(i >= AMP_ENTRY_COUNT)
    {
        API_AUDIO_AMP_ERR("%s: There is no matching amplifier device for u32DeviceId(%d).\n",
                __FUNCTION__,
                pstInfo->u32DeviceId);

        return FALSE;
    }

    return TRUE;
}

static MS_BOOL _MApi_AudioAmp_IsAnyAudioAmpTableExisted(void)
{
    MS_U8 i = 0;

    for(i = 0; i < API_AUDIO_AMP_DEVICE_INFO_MAX_NUM; i++)
    {
        if(NULL != _pDRV_AUDIO_AMP_TABLE_TYPE[i])
        {
            break;
        }
    }
    if(API_AUDIO_AMP_DEVICE_INFO_MAX_NUM <= i)
    {
        API_AUDIO_AMP_INFO("%s: There is no matching amplifier device\n", __FUNCTION__);

        return FALSE;
    }

    return TRUE;
}

static MS_BOOL _MApi_AudioAmp_IsNoAudioAmpTableExisted(void)
{
    return !_MApi_AudioAmp_IsAnyAudioAmpTableExisted();
}

static MS_BOOL _MApi_AudioAmp_IsAudioAmpDeviceTypeMatched(MS_U8 u8Index, amp_device_type_e eType)
{
    MS_BOOL bRet = FALSE;

    if(E_AMP_DEVICE_TYPE_DIGITAL == eType)
    {
        bRet = API_AUDIO_AMP_IS_DIGITAL_AUDIO_AMP(_astAmpDeviceSupplementalInfos[u8Index].u32DeviceId);
    }
    else if(E_AMP_DEVICE_TYPE_ANALOG == eType)
    {
        bRet = API_AUDIO_AMP_IS_ANALOG_AUDIO_AMP(_astAmpDeviceSupplementalInfos[u8Index].u32DeviceId);
    }
    else if(API_AUDIO_AMP_DEVICE_TYPE_ALL == eType)
    {
        bRet = API_AUDIO_AMP_IS_DIGITAL_AUDIO_AMP(_astAmpDeviceSupplementalInfos[u8Index].u32DeviceId) | API_AUDIO_AMP_IS_ANALOG_AUDIO_AMP(_astAmpDeviceSupplementalInfos[u8Index].u32DeviceId);
    }
    else
    {
        API_AUDIO_AMP_ERR("eType(%d) is not supported.\n", eType);
    }

    return bRet;
}

#if (ENABLE_API_AUDIO_AMP_SWI2C_HWI2C_SWITCH == 1)
static MS_BOOL _MApi_Convert_2_Disable_HwI2cPort(HWI2C_PORT *peDisableHwI2cPort, MS_IIC_PORT ePort)
{
    MS_BOOL bRet = TRUE;

    if((E_MS_IIC_PORT_0 <= ePort) && (E_MS_IIC_PORT0_7 >= ePort))
    {
        *peDisableHwI2cPort = E_HWI2C_PORT_0;

        return bRet;
    }
    else if((E_MS_IIC_PORT_1 <= ePort) && (E_MS_IIC_PORT1_7 >= ePort))
    {
        *peDisableHwI2cPort = E_HWI2C_PORT_1;

        return bRet;
    }
    else if((E_MS_IIC_PORT_2 <= ePort) && (E_MS_IIC_PORT2_7 >= ePort))
    {
        *peDisableHwI2cPort = E_HWI2C_PORT_2;

        return bRet;
    }
    else if((E_MS_IIC_PORT_3 <= ePort) && (E_MS_IIC_PORT3_7 >= ePort))
    {
        *peDisableHwI2cPort = E_HWI2C_PORT_3;

        return bRet;
    }
    else if((E_MS_IIC_PORT_4 <= ePort) && (E_MS_IIC_PORT4_7 >= ePort))
    {
        *peDisableHwI2cPort = E_HWI2C_PORT_4;

        return bRet;
    }
    else if((E_MS_IIC_PORT_5 <= ePort) && (E_MS_IIC_PORT5_7 >= ePort))
    {
        *peDisableHwI2cPort = E_HWI2C_PORT_5;

        return bRet;
    }

    *peDisableHwI2cPort = E_HWI2C_PORT_NOSUP;
    bRet = FALSE;

    return bRet;
}

static MS_BOOL _MApi_Disable_HwI2c(amp_device_supplemental_info_t *pstAmpDeviceSupplementalInfo)
{
    MS_BOOL bRet = TRUE;
    HWI2C_PORT eDisablePort;

    if(NULL == pstAmpDeviceSupplementalInfo)
    {
        API_AUDIO_AMP_ERR("pstAmpDeviceSupplementalInfo is NULL.\n");
        bRet = FALSE;

        return bRet;
    }

    do
    {
        if(API_AUDIO_AMP_IS_DIGITAL_AUDIO_AMP(pstAmpDeviceSupplementalInfo->u32DeviceId))
        {
            if(!_MApi_Convert_2_Disable_HwI2cPort(&eDisablePort, pstAmpDeviceSupplementalInfo->ePort))
            {
                API_AUDIO_AMP_INFO("_MApi_Convert_2_Disable_Port(...) returns E_HWI2C_PORT_NOSUP.\n");
                // Don't disable it if it is NOT HwI2cPort.

                break;
            }

            if(!MDrv_HWI2C_SelectPort(eDisablePort))
            {
                API_AUDIO_AMP_ERR("MDrv_HWI2C_SelectPort(%d) is failed.\n", eDisablePort);
                bRet = FALSE;

                break;
            }
        }
    } while(0);

    return bRet;
}

static MS_BOOL _MApi_Select_HwI2c(amp_device_supplemental_info_t *pstAmpDeviceSupplementalInfo)
{
    MS_BOOL bRet = TRUE;
    HWI2C_PORT eDisablePort;

    if(NULL == pstAmpDeviceSupplementalInfo)
    {
        API_AUDIO_AMP_ERR("pstAmpDeviceSupplementalInfo is NULL.\n");
        bRet = FALSE;

        return bRet;
    }

    do
    {
        if(API_AUDIO_AMP_IS_DIGITAL_AUDIO_AMP(pstAmpDeviceSupplementalInfo->u32DeviceId))
        {
            if(!_MApi_Convert_2_Disable_HwI2cPort(&eDisablePort, pstAmpDeviceSupplementalInfo->ePort))
            {
                API_AUDIO_AMP_INFO("_MApi_Convert_2_Disable_Port(...) returns E_HWI2C_PORT_NOSUP.\n");
                // Don't select it if it is NOT HwI2cPort.

                break;
            }

            if(!MDrv_HWI2C_SelectPort(pstAmpDeviceSupplementalInfo->ePort))
            {
                API_AUDIO_AMP_ERR("MDrv_HWI2C_SelectPort(...) is failed.\n");
                bRet = FALSE;

                break;
            }
        }

    } while(0);

    return bRet;
}
#else
static MS_BOOL _MApi_Disable_HwI2c(amp_device_supplemental_info_t *pstAmpDeviceSupplementalInfo)
{
    return TRUE;
}

static MS_BOOL _MApi_Select_HwI2c(amp_device_supplemental_info_t *pstAmpDeviceSupplementalInfo)
{
    return TRUE;
}
#endif //#if (ENABLE_API_AUDIO_AMP_SWI2C_HWI2C_SWITCH == 1)

//-------------------------------------------------------------------------------------------------
//  Internal Global Functions
//-------------------------------------------------------------------------------------------------

//-------------------------------------------------------------------------------------------------
//  Global Functions
//-------------------------------------------------------------------------------------------------
MS_BOOL MApi_AudioAmp_FillDeviceInfo(MS_AUDIO_AMP_DEVICE_INFO_t *pstParams)
{
    MS_U8 i = 0;

    if(pstParams == NULL)
    {
        API_AUDIO_AMP_ERR("%s: device info param is NULL\n", __FUNCTION__);

        return FALSE;
    }

    for(i = 0; i < API_AUDIO_AMP_DEVICE_INFO_MAX_NUM; i++)
    {
        // find the specified amplifier and update
        if(!_MApi_AudioAmp_FindAndUpdateAudioAmpTable(&(pstParams[i]), &(_pDRV_AUDIO_AMP_TABLE_TYPE[i]), &(_astAmpDeviceSupplementalInfos[i])))
        {
            API_AUDIO_AMP_ERR("%s: It is failed to _MApi_AudioAmp_FindAndUpdateAudioAmpTable(...) for device(%d).\n",
                    __FUNCTION__,
                    i);

            // Do nothing even though it is failed,
            // because _pDRV_AUDIO_AMP_TABLE_TYPE[i] will be reset.
        }
    }

    for(i = 0; i < API_AUDIO_AMP_DEVICE_INFO_MAX_NUM; i++)
    {
        if(_pDRV_AUDIO_AMP_TABLE_TYPE[i])
        {
            API_AUDIO_AMP_INFO("%s: _pDRV_AUDIO_AMP_TABLE_TYPE[%d]->config \n",
                    __FUNCTION__,
                    i);

            print_amp_device_info(&(pstParams[i]));
            if(!_pDRV_AUDIO_AMP_TABLE_TYPE[i]->config(&(pstParams[i])))
            {
                API_AUDIO_AMP_ERR("%s: It is failed to _pDRV_AUDIO_AMP_TABLE_TYPE[%d]->config(...).\n",
                        __FUNCTION__,
                        i);

                return FALSE;
            }
        }
    }

    _bIsAudioAmpConfigured = TRUE;

    return TRUE;
}

MS_BOOL MApi_AudioAmp_Init(audio_device_init_params_t *pstParams)
{
    MS_BOOL bRet = TRUE;
    MS_U8 i = 0;

    API_AUDIO_AMP_CHECK_CONFIG_RETURN(FALSE);

    // Don't check _bIsAudioAmpInit.
    // It is possible that Audio AMP is powered off in STR suspend, but MApi_AudioAmp_DeInit is not called.

    if(_MApi_AudioAmp_IsNoAudioAmpTableExisted())
    {
        API_AUDIO_AMP_INFO("%s: This chip has no matching amplifier device\n",
                __FUNCTION__);
        _bIsAudioAmpInit = TRUE;

        return TRUE;
    }

    for(i = 0; i < API_AUDIO_AMP_DEVICE_INFO_MAX_NUM; i++)
    {
        _abIsAudioAmpInit[i] = FALSE;

        if(_pDRV_AUDIO_AMP_TABLE_TYPE[i] &&
           _MApi_AudioAmp_IsAudioAmpDeviceTypeMatched(i, pstParams->eType))
        {
            API_AUDIO_AMP_INFO("%s: _pDRV_AUDIO_AMP_TABLE_TYPE[%d]->init \n",
                    __FUNCTION__,
                    i);
            if(_pDRV_AUDIO_AMP_TABLE_TYPE[i]->init())
            {
                _abIsAudioAmpInit[i] = TRUE;
            }
            else
            {
                API_AUDIO_AMP_ERR("%s: It is failed to _pDRV_AUDIO_AMP_TABLE_TYPE[%d]->init(...).\n",
                        __FUNCTION__,
                        i);

                bRet &= FALSE;

            }

            if(!_MApi_Disable_HwI2c(&(_astAmpDeviceSupplementalInfos[i])))
            {
                API_AUDIO_AMP_ERR("_MApi_Disable_HwI2c(&(_astAmpDeviceSupplementalInfos[%d]) is failed.\n", i);
                bRet &= FALSE;
            }
        }
    }

    _bIsAudioAmpInit = TRUE;
    for(i = 0; i < API_AUDIO_AMP_DEVICE_INFO_MAX_NUM; i++)
    {
        if(_pDRV_AUDIO_AMP_TABLE_TYPE[i] &&
           (!_abIsAudioAmpInit[i]))
        {
            _bIsAudioAmpInit = FALSE;
            break;
        }
    }

    return TRUE;
}

MS_BOOL MApi_AudioAmp_DeInit(audio_device_deinit_params_t *pstParams)
{
    MS_BOOL bRet = TRUE;
    MS_U8 i = 0;

    API_AUDIO_AMP_CHECK_CONFIG_RETURN(FALSE);

    if(!_bIsAudioAmpInit)
    {
        API_AUDIO_AMP_INFO("%s: It is not inited yet.\n",
                __FUNCTION__);

        return TRUE;
    }

    if(_MApi_AudioAmp_IsNoAudioAmpTableExisted())
    {
        API_AUDIO_AMP_INFO("%s: This chip has no matching amplifier device\n",
                __FUNCTION__);
        _bIsAudioAmpInit = FALSE;
        for(i = 0; i < API_AUDIO_AMP_DEVICE_INFO_MAX_NUM; i++)
        {
            _abIsAudioAmpInit[i] = FALSE;
        }

        return TRUE;
    }

    for(i = 0; i < API_AUDIO_AMP_DEVICE_INFO_MAX_NUM; i++)
    {
        if(_pDRV_AUDIO_AMP_TABLE_TYPE[i] &&
           _abIsAudioAmpInit[i])
        {
            API_AUDIO_AMP_INFO("%s: _pDRV_AUDIO_AMP_TABLE_TYPE[%d]->deinit \n",
                    __FUNCTION__,
                    i);
            do
            {
                if(!_MApi_Select_HwI2c(&(_astAmpDeviceSupplementalInfos[i])))
                {
                    API_AUDIO_AMP_ERR("_MApi_Select_HwI2c(&(_astAmpDeviceSupplementalInfos[%d])) is failed.\n", i);
                    bRet &= FALSE;

                    break;
                }

                if(_pDRV_AUDIO_AMP_TABLE_TYPE[i]->deinit())
                {
                    _abIsAudioAmpInit[i] = FALSE;
                }
                else
                {
                    API_AUDIO_AMP_ERR("%s: It is failed to _pDRV_AUDIO_AMP_TABLE_TYPE[%d]->deinit(...).\n",
                            __FUNCTION__,
                            i);

                    bRet &= FALSE;
                }

                if(!_MApi_Disable_HwI2c(&(_astAmpDeviceSupplementalInfos[i])))
                {
                    API_AUDIO_AMP_ERR("_MApi_Disable_HwI2c(&(_astAmpDeviceSupplementalInfos[%d]) is failed.\n", i);
                    bRet &= FALSE;

                    break;
                }
            } while(0);
        }
    }

    _bIsAudioAmpInit = FALSE;
    for(i = 0; i < API_AUDIO_AMP_DEVICE_INFO_MAX_NUM; i++)
    {
        if(_pDRV_AUDIO_AMP_TABLE_TYPE[i] &&
           _abIsAudioAmpInit[i])
        {
            _bIsAudioAmpInit = TRUE;
            break;
        }
    }

    return bRet;
}

MS_BOOL MApi_AudioAmp_Mute(audio_device_mute_params_t *pstParams)
{
    MS_BOOL bRet = TRUE;
    MS_U8 i = 0;

    API_AUDIO_AMP_CHECK_CONFIG_RETURN(FALSE);
    API_AUDIO_AMP_CHECK_AMP_TABLE_EXISTED_RETURN(TRUE);

    for(i = 0; i < API_AUDIO_AMP_DEVICE_INFO_MAX_NUM; i++)
    {
        if(_pDRV_AUDIO_AMP_TABLE_TYPE[i] &&
           _MApi_AudioAmp_IsAudioAmpDeviceTypeMatched(i, pstParams->eType))
        {
            API_AUDIO_AMP_INFO("%s: _pDRV_AUDIO_AMP_TABLE_TYPE[%d]->mute \n",
                    __FUNCTION__,
                    i);
            do
            {
                if(!_MApi_Select_HwI2c(&(_astAmpDeviceSupplementalInfos[i])))
                {
                    API_AUDIO_AMP_ERR("_MApi_Select_HwI2c(&(_astAmpDeviceSupplementalInfos[%d])) is failed.\n", i);
                    bRet &= FALSE;

                    break;
                }

                if(!_pDRV_AUDIO_AMP_TABLE_TYPE[i]->mute(pstParams->bMute))
                {
                    API_AUDIO_AMP_ERR("%s: It is failed to _pDRV_AUDIO_AMP_TABLE_TYPE[%d]->mute(...).\n",
                            __FUNCTION__,
                            i);

                    bRet &= FALSE;
                }

                if(!_MApi_Disable_HwI2c(&(_astAmpDeviceSupplementalInfos[i])))
                {
                    API_AUDIO_AMP_ERR("_MApi_Disable_HwI2c(&(_astAmpDeviceSupplementalInfos[%d]) is failed.\n", i);
                    bRet &= FALSE;

                    break;
                }
            } while(0);
        }
    }

    return bRet;
}

MS_BOOL MApi_AudioAmp_SetDebugLevel(API_AUDIO_AMP_DBG_LEVEL u32DbgLevel)
{
    MS_BOOL bRet = TRUE;
    MS_U8 i = 0;

    API_AUDIO_AMP_INFO("%s: u32DbgLevel = 0x%x.\n",
            __FUNCTION__,
            u32DbgLevel);

    _u32ApiAudioAmpDbgLevel = u32DbgLevel;
    for(i = 0; i < API_AUDIO_AMP_DEVICE_INFO_MAX_NUM; i++)
    {
        if(_pDRV_AUDIO_AMP_TABLE_TYPE[i] && _pDRV_AUDIO_AMP_TABLE_TYPE[i]->set_debug_level)
        {
            API_AUDIO_AMP_INFO("%s: _pDRV_AUDIO_AMP_TABLE_TYPE[%d]->set_debug_level \n",
                    __FUNCTION__,
                    i);
            if(!_pDRV_AUDIO_AMP_TABLE_TYPE[i]->set_debug_level(_u32ApiAudioAmpDbgLevel))
            {
                API_AUDIO_AMP_ERR("%s: It is failed to _pDRV_AUDIO_AMP_TABLE_TYPE[%d]->set_debug_level(...).\n",
                        __FUNCTION__,
                        i);

                bRet = FALSE;
            }
        }
    }

    return bRet;
}

MS_BOOL MApi_AudioAmp_GetDebugLevel(API_AUDIO_AMP_DBG_LEVEL *pu32DbgLevel)
{
    MS_BOOL bRet = TRUE;

    if(pu32DbgLevel == NULL)
    {
        API_AUDIO_AMP_ERR("%s: pu32DbgLevel is NULL\n",
                __FUNCTION__);

        return FALSE;
    }

    *pu32DbgLevel = _u32ApiAudioAmpDbgLevel;

    return bRet;
}
