#ifndef _AUDIO_AMP_2_H_
#define _AUDIO_AMP_2_H_

#include "MsCommon.h"
#include "drv_audio_amp.h"

//-------------------------------------------------------------------------------------------------
//  Debug Defines
//-------------------------------------------------------------------------------------------------
#define API_AUDIO_AMP_ENABLE_DBG                        1

//API_AUDIO_AMP_DBG_LEVEL
#define    API_AUDIO_AMP_DBG_LEVEL                      MS_U32
#define    API_AUDIO_AMP_DBG_NONE                       0
#define    API_AUDIO_AMP_DBG_FATAL                      0x10
#define    API_AUDIO_AMP_DBG_ERR                        0x20
#define    API_AUDIO_AMP_DBG_WRN                        0x30
#define    API_AUDIO_AMP_DBG_INFO                       0x40
#define    API_AUDIO_AMP_DBG_TRACE                      0x50
#define    API_AUDIO_AMP_DBG_ALL                        0xF0

#if (API_AUDIO_AMP_ENABLE_DBG == 1)
#define API_AUDIO_AMP_PRINT(fmt, args...)               ({do{printf(fmt, ##args);}while(0);})
#define API_AUDIO_AMP_INFO(fmt, args...)                ({do{if(DBG_LEVEL>=API_AUDIO_AMP_DBG_INFO){printf(ASCII_COLOR_GREEN);printf(fmt, ##args);printf(ASCII_COLOR_END);}}while(0);})
#define API_AUDIO_AMP_WRN(fmt, args...)                 ({do{if(DBG_LEVEL>=API_AUDIO_AMP_DBG_WRN){printf(ASCII_COLOR_YELLOW);printf(fmt, ##args);printf(ASCII_COLOR_END);}}while(0);})
#define API_AUDIO_AMP_ERR(fmt, args...)                 ({do{if(DBG_LEVEL>=API_AUDIO_AMP_DBG_ERR){printf(ASCII_COLOR_RED"%s[%d]: ", __FUNCTION__,__LINE__);printf(fmt, ##args);printf(ASCII_COLOR_END);}}while(0);})
#define API_AUDIO_AMP_ENTER()                           ({do{if(DBG_LEVEL>=API_AUDIO_AMP_DBG_ALL){printf(ASCII_COLOR_PURPLE">>>%s[%d] \n",__FUNCTION__,__LINE__);printf(ASCII_COLOR_END);}}while(0);})
#define API_AUDIO_AMP_EXIT_OK()                         ({do{if(DBG_LEVEL>=API_AUDIO_AMP_DBG_ALL){printf(ASCII_COLOR_PURPLE"<<<%s[%d] \n",__FUNCTION__,__LINE__);printf(ASCII_COLOR_END);}}while(0);})
#define API_AUDIO_AMP_EXIT_ERR(fmt, args...)            ({do{if(DBG_LEVEL>=API_AUDIO_AMP_DBG_ERR){printf(ASCII_COLOR_RED"<<<%s[%d] ",__FUNCTION__,__LINE__);printf(fmt, ##args);printf(ASCII_COLOR_END);}}while(0);})
#else
#define API_AUDIO_AMP_PRINT(fmt, args...)
#define API_AUDIO_AMP_INFO(fmt, args...)
#define API_AUDIO_AMP_WRN(fmt, args...)
#define API_AUDIO_AMP_ERR(fmt, args...)
#define API_AUDIO_AMP_ENTER()
#define API_AUDIO_AMP_EXIT_OK()
#define API_AUDIO_AMP_EXIT_ERR(fmt, args...)
#endif //#if (API_AUDIO_AMP_ENABLE_DBG == 1)

///ASCII color code
#define ASCII_COLOR_BLACK                               "\033[30m"
#define ASCII_COLOR_DARK_RED                            "\033[31m"
#define ASCII_COLOR_DARK_GREEN                          "\033[32m"
#define ASCII_COLOR_BROWN                               "\033[33m"
#define ASCII_COLOR_DARK_BLUE                           "\033[34m"
#define ASCII_COLOR_DARK_PURPLE                         "\033[35m"
#define ASCII_COLOR_DARK_CYAN                           "\033[36m"
#define ASCII_COLOR_GRAY                                "\033[37m"
#define ASCII_COLOR_DARK_GRAY                           "\033[1;30m"
#define ASCII_COLOR_RED                                 "\033[1;31m"
#define ASCII_COLOR_GREEN                               "\033[1;32m"
#define ASCII_COLOR_YELLOW                              "\033[1;33m"
#define ASCII_COLOR_BLUE                                "\033[1;34m"
#define ASCII_COLOR_PURPLE                              "\033[1;35m"
#define ASCII_COLOR_CYAN                                "\033[1;36m"
#define ASCII_COLOR_WHITE                               "\033[1;37m"
#define ASCII_COLOR_INVERSE_BLACK                       "\033[1;7;30m"
#define ASCII_COLOR_INVERSE_RED                         "\033[1;7;31m"
#define ASCII_COLOR_INVERSE_GREEN                       "\033[1;7;32m"
#define ASCII_COLOR_INVERSE_YELLOW                      "\033[1;7;33m"
#define ASCII_COLOR_INVERSE_BLUE                        "\033[1;7;34m"
#define ASCII_COLOR_INVERSE_PURPLE                      "\033[1;7;35m"
#define ASCII_COLOR_INVERSE_CYAN                        "\033[1;7;36m"
#define ASCII_COLOR_INVERSE_WHITE                       "\033[1;7;37m"
#define ASCII_COLOR_END                                 "\033[0m"

//-------------------------------------------------------------------------------------------------
//  Global Defines
//-------------------------------------------------------------------------------------------------
#define API_AUDIO_AMP_DEVICE_TYPE_ALL                   (E_AMP_DEVICE_TYPE_DIGITAL | E_AMP_DEVICE_TYPE_ANALOG)
#define API_AUDIO_AMP_IS_ANALOG_AUDIO_AMP(id)           ((AMPLIFIER_ANALOG_DEFAULT <= (id)) && (AMPLIFIER_NONE != (id)))
#define API_AUDIO_AMP_IS_DIGITAL_AUDIO_AMP(id)          (!API_AUDIO_AMP_IS_ANALOG_AUDIO_AMP(id) && (AMPLIFIER_NONE != (id)))

//-------------------------------------------------------------------------------------------------
//  Global Structures
//-------------------------------------------------------------------------------------------------
// It should be consistent w/ mi_aout_AmpDeviceType_e
typedef enum
{
    E_AMP_DEVICE_TYPE_DIGITAL = 0x01,
    E_AMP_DEVICE_TYPE_ANALOG = 0x02,
} amp_device_type_e;

typedef struct audio_device_init_params_s
{
    amp_device_type_e eType;
} audio_device_init_params_t;

typedef struct audio_device_deinit_params_s
{
    MS_U8 u8Reserved;
} audio_device_deinit_params_t;

typedef struct audio_device_mute_params_s
{
    MS_BOOL bMute;
    amp_device_type_e eType;
} audio_device_mute_params_t;

typedef amp_device_info_t MS_AUDIO_AMP_DEVICE_INFO_t;

//-------------------------------------------------------------------------------------------------
//  Global Functions
//-------------------------------------------------------------------------------------------------
MS_BOOL MApi_AudioAmp_FillDeviceInfo(MS_AUDIO_AMP_DEVICE_INFO_t *pstParams);
MS_BOOL MApi_AudioAmp_Init(audio_device_init_params_t *pstParams);
MS_BOOL MApi_AudioAmp_DeInit(audio_device_deinit_params_t *pstParams);
MS_BOOL MApi_AudioAmp_Mute(audio_device_mute_params_t *pstParams);
MS_BOOL MApi_AudioAmp_SetDebugLevel(API_AUDIO_AMP_DBG_LEVEL u32DbgLevel);
MS_BOOL MApi_AudioAmp_GetDebugLevel(API_AUDIO_AMP_DBG_LEVEL *pu32DbgLevel);

#endif  // #ifndef _AUDIO_AMP_2_H_
