/* SPDX-License-Identifier: GPL-2.0-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2 of the GNU General Public License ("GPLv2 License")
 * or BSD License.
 *
 * GPLv2 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See http://www.gnu.org/licenses/gpl-2.0.html for more details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/
 //wjq20100112>>>
#include <common.h>
#include <Amplifier_AD82088.h>
#include <linux/types.h>
#include <apiSWI2C.h>
#include <MsUboot.h>
#include <drvIIC.h>

#ifndef ARRAY_SIZE
#define ARRAY_SIZE(x)                                   (sizeof(x) / sizeof((x)[0]))
#endif //#ifndef ARRAY_SIZE

#define AD82088_ADDR        0x62
#define AD82088_STATE_CONTROL_3                         (0x02)
#define AD82088_STATE_CONTROL_5                         (0x1a)

#ifndef I2C_BUS_IDX
#define I2C_BUS_IDX         0
#endif

#define AD82088_BUSID     (((MS_U16) I2C_BUS_IDX << 8) | AD82088_ADDR)

/****************************
        amp init regs
****************************/

static MAPI_U8 pu8AmpRegTbl[] =
{
    /*{0x00, */0x04,//##State_Control_1
    /*{0x01, */0x81,//##State_Control_2
    /*{0x02, */0x00,//##State_Control_3
    /*{0x03, */0x04,//##Master_volume_control
    /*{0x04, */0x18,//##Channel_1_volume_control
    /*{0x05, */0x18,//##Channel_2_volume_control
    /*{0x06, */0x18,//##Channel_3_volume_control
    /*{0x07, */0x18,//##Channel_4_volume_control
    /*{0x08, */0x18,//##Channel_5_volume_control
    /*{0x09, */0x18,//##Channel_6_volume_control
    /*{0x0a, */0x10,//##Bass_Tone_Boost_and_Cut
    /*{0x0b, */0x10,//##treble_Tone_Boost_and_Cut
    /*{0x0c, */0x90,//##State_Control_4
    /*{0x0d, */0x00,//##Channel_1_configuration_registers
    /*{0x0e, */0x00,//##Channel_2_configuration_registers
    /*{0x0f, */0x00,//##Channel_3_configuration_registers
    /*{0x10, */0x00,//##Channel_4_configuration_registers
    /*{0x11, */0x00,//##Channel_5_configuration_registers
    /*{0x12, */0x00,//##Channel_6_configuration_registers
    /*{0x13, */0x00,//##Channel_7_configuration_registers
    /*{0x14, */0x00,//##Channel_8_configuration_registers
    /*{0x15, */0x6a,//##DRC1_limiter_attack/release_rate
    /*{0x16, */0x6a,//##DRC2_limiter_attack/release_rate
    /*{0x17, */0x6a,//##DRC3_limiter_attack/release_rate
    /*{0x18, */0x6a,//##DRC4_limiter_attack/release_rate
    /*{0x19, */0x06,//##Error_Delay
    /*{0x1a, */0x32,//##State_Control_5
    /*{0x1b, */0x01,//##HVUV_selection
    /*{0x1c, */0x00,//##State_Control_6
    /*{0x1d, */0x7f,//##Coefficient_RAM_Base_Address
    /*{0x1e, */0x00,//##Top_8-bits_of_coefficients_A1
    /*{0x1f, */0x00,//##Middle_8-bits_of_coefficients_A1
    /*{0x20, */0x00,//##Bottom_8-bits_of_coefficients_A1
    /*{0x21, */0x00,//##Top_8-bits_of_coefficients_A2
    /*{0x22, */0x00,//##Middle_8-bits_of_coefficients_A2
    /*{0x23, */0x00,//##Bottom_8-bits_of_coefficients_A2
    /*{0x24, */0x00,//##Top_8-bits_of_coefficients_B1
    /*{0x25, */0x00,//##Middle_8-bits_of_coefficients_B1
    /*{0x26, */0x00,//##Bottom_8-bits_of_coefficients_B1
    /*{0x27, */0x00,//##Top_8-bits_of_coefficients_B2
    /*{0x28, */0x00,//##Middle_8-bits_of_coefficients_B2
    /*{0x29, */0x00,//##Bottom_8-bits_of_coefficients_B2
    /*{0x2a, */0x40,//##Top_8-bits_of_coefficients_A0
    /*{0x2b, */0x00,//##Middle_8-bits_of_coefficients_A0
    /*{0x2c, */0x00,//##Bottom_8-bits_of_coefficients_A0
    /*{0x2d, */0x00,//##Coefficient_R/W_control
    /*{0x2e, */0x00,//##Protection_Enable/Disable
    /*{0x2f, */0x00,//##Memory_BIST_status
    /*{0x30, */0x00,//##Power_Stage_Status(Read_only)
    /*{0x31, */0x00,//##PWM_Output_Control
    /*{0x32, */0x00,//##Test_Mode_Control_Reg.
    /*{0x33, */0x6d,//##Qua-Ternary/Ternary_Switch_Level
    /*{0x34, */0x00,//##Volume_Fine_tune
    /*{0x35, */0x00,//##Volume_Fine_tune
    /*{0x36, */0x60,//##OC_bypass_&_GVDD_selection
    /*{0x37, */0x52,//##Device_ID_register
    /*{0x38, */0x00,//##RAM1_test_register_address
    /*{0x39, */0x00,//##Top_8-bits_of_RAM1_Data
    /*{0x3a, */0x00,//##Middle_8-bits_of_RAM1_Data
    /*{0x3b, */0x00,//##Bottom_8-bits_of_RAM1_Data
    /*{0x3c, */0x00,//##RAM1_test_r/w_control
    /*{0x3d, */0x00,//##RAM2_test_register_address
    /*{0x3e, */0x00,//##Top_8-bits_of_RAM2_Data
    /*{0x3f, */0x00,//##Middle_8-bits_of_RAM2_Data
    /*{0x40, */0x00,//##Bottom_8-bits_of_RAM2_Data
    /*{0x41, */0x00,//##RAM2_test_r/w_control
    /*{0x42, */0x00,//##Level_Meter_Clear
    /*{0x43, */0x00,//##Power_Meter_Clear
    /*{0x44, */0x00,//##TOP_of_C1_Level_Meter
    /*{0x45, */0x00,//##Middle_of_C1_Level_Meter
    /*{0x46, */0x00,//##Bottom_of_C1_Level_Meter
    /*{0x47, */0x00,//##TOP_of_C2_Level_Meter
    /*{0x48, */0x00,//##Middle_of_C2_Level_Meter
    /*{0x49, */0x00,//##Bottom_of_C2_Level_Meter
    /*{0x4a, */0x00,//##TOP_of_C3_Level_Meter
    /*{0x4b, */0x00,//##Middle_of_C3_Level_Meter
    /*{0x4c, */0x00,//##Bottom_of_C3_Level_Meter
    /*{0x4d, */0x00,//##TOP_of_C4_Level_Meter
    /*{0x4e, */0x00,//##Middle_of_C4_Level_Meter
    /*{0x4f, */0x00,//##Bottom_of_C4_Level_Meter
    /*{0x50, */0x00,//##TOP_of_C5_Level_Meter
    /*{0x51, */0x00,//##Middle_of_C5_Level_Meter
    /*{0x52, */0x00,//##Bottom_of_C5_Level_Meter
    /*{0x53, */0x00,//##TOP_of_C6_Level_Meter
    /*{0x54, */0x00,//##Middle_of_C6_Level_Meter
    /*{0x55, */0x00,//##Bottom_of_C6_Level_Meter
    /*{0x56, */0x00,//##TOP_of_C7_Level_Meter
    /*{0x57, */0x00,//##Middle_of_C7_Level_Meter
    /*{0x58, */0x00,//##Bottom_of_C7_Level_Meter
    /*{0x59, */0x00,//##TOP_of_C8_Level_Meter
    /*{0x5a, */0x00,//##Middle_of_C8_Level_Meter
    /*{0x5b, */0x00,//##Bottom_of_C8_Level_Meter
    /*{0x5c, */0x06,//##I2S_Data_Output_Selection_Register
    /*{0x5d, */0x00,//##Reserve
    /*{0x5e, */0x00,//##Reserve
    /*{0x5f, */0x00,//##Reserve
    /*{0x60, */0x00,//##Reserve
    /*{0x61, */0x00,//##Reserve
    /*{0x62, */0x00,//##Reserve
    /*{0x63, */0x00,//##Reserve
    /*{0x64, */0x00,//##Reserve
    /*{0x65, */0x00,//##Reserve
    /*{0x66, */0x00,//##Reserve
    /*{0x67, */0x00,//##Reserve
    /*{0x68, */0x00,//##Reserve
    /*{0x69, */0x00,//##Reserve
    /*{0x6a, */0x00,//##Reserve
    /*{0x6b, */0x00,//##Reserve
    /*{0x6c, */0x00,//##Reserve
    /*{0x6d, */0x00,//##Reserve
    /*{0x6e, */0x00,//##Reserve
    /*{0x6f, */0x00,//##Reserve
    /*{0x70, */0x00,//##Reserve
    /*{0x71, */0x00,//##Reserve
    /*{0x72, */0x00,//##Reserve
    /*{0x73, */0x00,//##Reserve
    /*{0x74, */0x00,//##Mono_Key_High_Byte
    /*{0x75, */0x00,//##Mono_Key_Low_Byte
    /*{0x76, */0x00,//##Boost_Control
    /*{0x77, */0x07,//##Hi-res_Item
    /*{0x78, */0x40,//##Test_Mode_register
    /*{0x79, */0x62,//##Boost_Strap_OV/UV_Selection
    /*{0x7a, */0x8c,//##OC_Selection_2
    /*{0x7b, */0x55,//##MBIST_User_Program_Top_Byte_Even
    /*{0x7c, */0x55,//##MBIST_User_Program_Middle_Byte_Even
    /*{0x7d, */0x55,//##MBIST_User_Program_Bottom_Byte_Even
    /*{0x7e, */0x55,//##MBIST_User_Program_Top_Byte_Odd
    /*{0x7f, */0x55,//##MBIST_User_Program_Middle_Byte_Odd
    /*{0x80, */0x55,//##MBIST_User_Program_Bottom_Byte_Odd
    /*{0x81, */0x00,//##ERROR_clear_register
    /*{0x82, */0x0c,//##Minimum_duty_test
    /*{0x83, */0x06,//##Reserve
    /*{0x84, */0xfe,//##Reserve
    /*{0x85, */0xc2,//##Reserve
};
static MAPI_U8 _u8AmpRegTblSize = ARRAY_SIZE(pu8AmpRegTbl);

static MAPI_U8 pu32MuteTable[] =
{
    0x7f, /* mute */
    0x00, /* unmute */
};

static inline MS_BOOL __drv_AMP_set_reg(MS_U8 reg_index, MS_U8 reg_data)
{
    MS_BOOL bRet = FALSE;
    bRet = MApi_SWI2C_WriteBytes(
                AD82088_BUSID,     
                1,   
                &reg_index, // Register Intex of AMP
                1,          // Sends One Byte
                &reg_data); // One Byte data to send
    if (!bRet)
    {
        printf("MApi_SWI2C_WriteBytes failed(AMP82088 ! :  0x%x, 0x%x\n",
                reg_index, // Register Intex of AMP
                reg_data); // One Byte data to send
    }
    return bRet;
}

MAPI_BOOL Amplifier_AD82088_SWMute(MAPI_BOOL bMute)
{
	MS_BOOL bRet = TRUE;
	if(bMute)
	{
		bRet &= __drv_AMP_set_reg(AD82088_STATE_CONTROL_3, 0x7f);
	}
	else
	{
		bRet &= __drv_AMP_set_reg(AD82088_STATE_CONTROL_3, 0x00);
	}
	return bRet;
}
MAPI_BOOL Amplifier_AD82088_Mute(MAPI_BOOL bMute)
{
	return TRUE;
}
MAPI_BOOL Amplifier_AD82088_StandBy(MAPI_BOOL bStandby)
{
	if(bStandby)
	{
		Audio_Amplifier_OFF();
    	mdelay(2);
    	Adj_Volume_Off();
	}
	else
	{
		Audio_Amplifier_ON();
    	mdelay(2);
    	Adj_Volume_On();
	}
	return TRUE;
}

MAPI_BOOL Amplifier_AD82088_Reset(MAPI_BOOL bReset)
{
	MS_BOOL bRet = TRUE;
	if(bReset)
	{
		bRet &= __drv_AMP_set_reg(AD82088_STATE_CONTROL_5, 0x12);
	}
	else
	{
		bRet &= __drv_AMP_set_reg(AD82088_STATE_CONTROL_5, 0x32);
	}
	return bRet;
}


MAPI_BOOL Amplifier_AD82088_Init(void)
{
    MAPI_U8 i;
	
    printf("*****Amplifier_AD82088_Init****\n");
	Amplifier_AD82088_StandBy(TRUE);
    mdelay(10);
	Amplifier_AD82088_StandBy(FALSE);
	mdelay(20);
    Amplifier_AD82088_Reset(TRUE);
    mdelay(10);
	Amplifier_AD82088_Reset(FALSE);
	mdelay(20);
	Amplifier_AD82088_SWMute(TRUE);
	
    for(i = 0; i < _u8AmpRegTblSize; i++)
    {
        /* We muted all channels no matter what, so don't mind the value in config file, don't unmute before all is done.*/
        if(i == AD82088_STATE_CONTROL_3 )
            continue;
		
        if(i>=0x0A && i<=0x0F) // Prohibited
            continue;

        __drv_AMP_set_reg(i, pu8AmpRegTbl[i]);

    }
	Amplifier_AD82088_SWMute(FALSE);
    return TRUE;
}

