/* SPDX-License-Identifier: GPL-2.0-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2 of the GNU General Public License ("GPLv2 License")
 * or BSD License.
 *
 * GPLv2 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See http://www.gnu.org/licenses/gpl-2.0.html for more details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/
#include <stdio.h>
#include <common.h>
#include <MsTypes.h>
#include <apiPNL.h>
#include <drvBDMA.h>
#include <MsSystem.h>
#include <MsStr.h>
#include <MsDebug.h>
#include <MsSysUtility.h>
#include <miu/MsDrvMiu.h>
#include <halSIW.h>
#include <ms_utils.h>
#include "MDrvSIW.h"
#include "MsApiDemura.h"
#include <MsApiMspi.h>


//#define DBG_WITHOUT_TCON    1

#undef ALIGN
#define ALIGN(x,a)        __ALIGN_MASK((x),(typeof(x))(a)-1)
#define __ALIGN_MASK(x,mask)    (((x)+(mask))&~(mask))

static SIWBinHeader *pHeader = NULL;

MS_BOOL MApi_SIW_Enable(MS_PHYADDR phy_addr,MS_BOOL bEnable)
{
    //MS_U32 phy_addr;
    //MS_U32 lut_addr;


    //phy_addr = BA2PA( (MS_U32)pHeader );
    //lut_addr = phy_addr + pHeader->u32LayerDataAddr;
    phy_addr += SIW_LUTOffset;

    if (bEnable == TRUE)
    {
        UBOOT_ERROR("phy_addr : 0x%x\n", (uint)phy_addr);
        MDrv_SIW_DEMURA_AutoDownload_Setup(phy_addr, TRUE);
        //MDrv_DEMURA_TurnOn(TRUE);
    }
    else
    {
        MDrv_SIW_DEMURA_AutoDownload_Setup(phy_addr, FALSE);
        //MDrv_DEMURA_TurnOn(FALSE);
    }

    //ADL transmit
    //ADL setting
    MDrv_SIW_EnableClk();
    //MDrv_SIW_Initsetting();////Internal APB setting  //according to setting provided by SiW

    return TRUE;
}



MS_BOOL MApi_SIW_Init(MS_PHYADDR phy_addr,MS_U32 lut_virtaddr)
{
    //MS_U64 bus_addr;
    //MS_U32 lut_virtaddr;
    //MS_U32 lut_phyaddr;
    MS_U32 LutSize = 0, Total_flash_size = 0;
    LutSize = 0x5EEC0;  // 0x5f2bf-0x400 +1
    lut_virtaddr += SIW_LUTOffset;
    if (MDrv_SIW_SetIOMapBase() != TRUE)
    {
        UBOOT_ERROR("MDrv_SIW_SetIOMapBase Failed\n");
        MDrv_SIW_TurnOn(FALSE);
        return FALSE;
    }

    Total_flash_size = 0x400+LutSize+0x6; // Total flash data
    // Set padmux
    MDrv_SIW_Set_SPI_Pad(TRUE);
    if (MDrv_SIW_Set_BDMA(phy_addr, Total_flash_size) != TRUE)
    {
        UBOOT_ERROR("MDrv_SIW_Set_BDMA error\n");
        MDrv_SIW_TurnOn(FALSE);
        MDrv_SIW_Set_SPI_Pad(FALSE);
        return FALSE;
    }
    MDrv_SIW_Set_SPI_Pad(FALSE);

    if (MDrv_SIW_Checksum((MS_U8 *)(lut_virtaddr+0x400), LutSize) != TRUE)
    {
        UBOOT_ERROR("MDrv_SIW_Checksum error\n");
        return FALSE;
    }

    MDrv_SIW_CFG_Panel_Dep((MS_U8 *)lut_virtaddr);
    return TRUE;
}


