/* SPDX-License-Identifier: GPL-2.0-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2 of the GNU General Public License ("GPLv2 License")
 * or BSD License.
 *
 * GPLv2 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See http://www.gnu.org/licenses/gpl-2.0.html for more details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/
#include <stdio.h>
#include <common.h>
#include <MsTypes.h>
#include <drvBDMA.h>
#include <MsSystem.h>
#include <miu/MsDrvMiu.h>
#include <MsDebug.h>
#include <halSIW.h>
#include <drvMMIO.h>
#include <halRegOp.h>
#include "MDrvSIW.h"

void MDrv_SIW_DEMURA_AutoDownload_Setup(MS_PHYADDR phy_addr, MS_BOOL bEnable)
{
    UBOOT_DEBUG("MDrv_SIW_DEMURA_AutoDownload_Setup IN\n");

    if (bEnable == TRUE)
    {
        HAL_SIW_DEMURA_SetDL_BaseAddr(phy_addr);
        //HAL_DEMURA_SetDL_SramIni_Addr(0);

        HAL_SIW_DEMURA_SetDL_Depth(); //HAL_DEMURA_SetDL_Depth(viod)??
        HAL_SIW_DEMURA_SetDL_ReqLenth(); //HAL_DEMURA_SetDL_ReqLenth(void)??
        //HAL_DEMURA_SetDL_TriggerMode(AUTO_DL_ENABLE_MODE); ////[0]:reg_clinetc_en  [1] reg_clinetc_trig_mod
        HAL_SIW_DEMURA_Enable_DL(TRUE); ////[0]:reg_clinetc_en  [1] reg_clinetc_trig_mod
    }
    else
    {
        HAL_SIW_DEMURA_Enable_DL(FALSE);
    }
    UBOOT_DEBUG("OK\n");
}

MS_BOOL MDrv_SIW_SetIOMapBase(void)
{
    MS_VIRT NonPM_Addr = 0;
    MS_PHY  NonPM_Size = 0;

    if (FALSE == MDrv_MMIO_GetBASE(&NonPM_Addr, &NonPM_Size, MS_MODULE_BDMA)) {  // Wait for utopia to fix it
        UBOOT_ERROR("Get IOMAP Base faill!\n");
        return FALSE;
    }
    UBOOT_DEBUG("Get IOMAP ID:%u Base:%lx!\n", MS_MODULE_BDMA, (ulong)NonPM_Addr);
    HAL_DEMURA_SetIOMapBase(NonPM_Addr);

    return TRUE;
}

void MDrv_SIW_CFG_Panel_Dep(MS_U8 *lut_buf)
{
    Hal_SIW_CFG_Panel_Dep(lut_buf);
}

MS_BOOL MDrv_SIW_Checksum(MS_U8 *lut_buf, MS_U32 lut_len)
{
    MS_U16 dump = 0x0000;
    MS_U32 i;
    MS_U16 reg_crc = 0xFFFF;
    //MS_U32 check_start = lut_offset;
    unsigned char *pLut = (unsigned char *)lut_buf;
    MS_U32 check_end   = lut_len;

    while(check_end--)
        {
            reg_crc ^= *pLut;
            for (i=0;i<8;i++)
                {
                    if(reg_crc & 0x01)
                        reg_crc = (reg_crc>>1)^0xA001;
                    else
                        reg_crc = reg_crc >>1;
                }
        }

    dump = reg_crc;
    reg_crc = ((dump<<8) & 0xFF00) | ((dump>>8) & 0x00FF);
    MS_U16 checksum; // calcusum = Compare CRC result with CRC golden ('h5F2C4,'h5F2C5).
    checksum = *(pLut + 0x5F2C4 - 0x400); // high byte
    checksum = (checksum << 8) | (*(pLut + 0x5F2C5 - 0x400)); // high | low byte
    if (reg_crc != checksum)
    {
        UBOOT_ERROR("reg_crc = 0x%x\n", reg_crc);
        UBOOT_ERROR("checksum = 0x%x\n", checksum);
        Hal_SIW_CRC_Done(FALSE);
        return FALSE;
    }

    Hal_SIW_CRC_Done(TRUE);
    return TRUE;

}
void MDrv_SIW_EnableClk(void)
{
    Hal_SIW_EnableClk();
}
void MDrv_SIW_Initsetting(void)
{
    Hal_SIW_Initsetting();
}


/*
MS_BOOL MDrv_FCIC_Set_BDMA(MS_PHYADDR lut_phyAddr, MS_U32 lut_size)
{
    MS_PHYADDR src_addr, dst_addr;
    BDMA_CpyType eCpyType;
    BDMA_Result  bRet;
    EN_MIUID miu_no;

    // MDrv_BDMA_Init(0);  MstarSysInit() has call it !

    miu_no = MsApi_Miu_GetID(lut_phyAddr);

    if (miu_no == E_MIU_ID_0)
    {
        eCpyType = E_BDMA_SDRAM2FCIC;
    }
    else if (miu_no == E_MIU_ID_1)
    {
        eCpyType = E_BDMA_SDRAM12FCIC;
    }
    else
    {
        eCpyType = E_BDMA_SDRAM22FCIC;
    }
    src_addr = lut_phyAddr;

    dst_addr = FCIC_SRAM_START_ADDR;

    UBOOT_DEBUG("src_addr = 0x%x, dst_addr = 0x%x, lut_size = 0x%x, eCpyType = 0x%x\n", \
                (uint)src_addr, (uint)dst_addr, (uint)lut_size, eCpyType);

    bRet = MDrv_BDMA_CopyHnd(src_addr, dst_addr, lut_size, eCpyType, BDMA_OPCFG_DEF);
    if (bRet != E_BDMA_OK)
    {
        UBOOT_ERROR("MDrv_BDMA_CopyHnd Error, ret = %d\n", (int)bRet);
        return FALSE;
    }
    return TRUE;
}
*/

void MDrv_SIW_TurnOn(MS_BOOL bOnOff)
{
    HAL_SIW_TurnOn(bOnOff);
}

void MDrv_SIW_Set_SPI_Pad(MS_BOOL bOnOff)
{
    HAL_SIW_Set_SPI_Pad(bOnOff);
}

void MDrv_SIW_Set_BDMA_Clk(MS_BOOL bOnOff)
{
    HAL_SIW_Set_BDMA_Clk(bOnOff);
}

MS_BOOL MDrv_SIW_Set_BDMA(MS_PHYADDR phy_addr, MS_U32 lut_size)
{
    MS_PHYADDR src_addr, dst_addr;
    //BDMA_CpyType eCpyType;
    BDMA_Result  bRet = TRUE;
    //UNUSED(lut_phyAddr);
    //EN_MIUID miu_no;

    // MDrv_BDMA_Init(0);  MstarSysInit() has call it !
    //miu_no = MsApi_Miu_GetID(lut_phyAddr);
    //src_addr = lut_phyAddr;
    src_addr = 0x00;
    dst_addr = phy_addr; // DRAM dst =from MMAP

    UBOOT_DEBUG("src_addr = 0x%x, dst_addr = 0x%x, lut_size = 0x%x", \
                (uint)src_addr, (uint)dst_addr, (uint)lut_size);

    //printf("Brown test BDMA enum = %d \n", E_BDMA_SPI12DRAM);
    bRet = MDrv_BDMA_CopyHnd (src_addr, dst_addr, lut_size, E_BDMA_SPI12DRAM, 0);
    //bRet = MDrv_BDMA_FlashCopy2Dram(0x00, 0x400, 0x100);
    if (bRet != E_BDMA_OK)
    {
        UBOOT_ERROR("MDrv_BDMA_CopyHnd Error, ret = %d\n", (int)bRet);
        return FALSE;
    }
    return TRUE;
}

