/* SPDX-License-Identifier: GPL-2.0-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2 of the GNU General Public License ("GPLv2 License")
 * or BSD License.
 *
 * GPLv2 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See http://www.gnu.org/licenses/gpl-2.0.html for more details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/
#define FORCE_MY_GPIO_MODE 1
#define _IIC_C_

#include <linux/string.h>
#include "drvIIC.h"
#include "drvGPIO.h"
#include "drvMMIO.h"

#if SWI2C_INUSE
static MS_U8* pu8SwIICPort = NULL;
static MS_U8 u8SwIICCount = 0;
static MS_BOOL bSwIICSetupDone = FALSE;
static SWI2C_ReadMode eSW_Read_Mode_Default = E_SWI2C_READ_MODE_DIRECTION_CHANGE;
#endif
#ifdef ENABLE_IIC_LOCK
static MS_S32 s32IICMutex = -1;
#endif //#ifdef ENABLE_IIC_LOCK
static HWI2C_ReadMode eHW_Read_Mode_Default = E_HWI2C_READ_MODE_DIRECTION_CHANGE;



#ifdef ENABLE_IIC_LOCK
#define MDrv_IIC_MUTEX_TIMEOUT (2000)
#define MDrv_IIC_LOCK()      do{if(s32IICMutex!=-1)MsOS_ObtainMutex(s32IICMutex, MDrv_IIC_MUTEX_TIMEOUT);}while(0);
#define MDrv_IIC_UNLOCK()    do{if(s32IICMutex!=-1)MsOS_ReleaseMutex(s32IICMutex);}while(0);
#else
#define MDrv_IIC_LOCK()
#define MDrv_IIC_UNLOCK()
#endif //#ifdef ENABLE_IIC_LOCK

#if SWI2C_INUSE
static MS_BOOL _SwIICPort_mapping(MS_IIC_PORT ePort, MS_U8* pu8SwI2CBus)
{
    MS_U8 i;

    if(pu8SwIICPort == NULL)
        return FALSE;

    for(i=0;i<u8SwIICCount;i++)
    {
       if(ePort == (*(pu8SwIICPort + i)))
       {
          *pu8SwI2CBus = i;
          return TRUE;
       }
    }

    return FALSE;
}

MS_BOOL MDrv_IIC_SetupSwPort(MS_U8 u8BusCount, MS_IIC_SwBus* pstConfig)
{
    SWI2C_BusCfg stBusCfg[E_MS_IIC_SW_PORT_MAX - E_MS_IIC_SW_PORT_0 + 1];
    MS_IIC_SwBus* pstConfigI = NULL;
    MS_U8 i;

    if(!bSwIICSetupDone)
    {
        if(u8BusCount > (sizeof(stBusCfg)/sizeof(SWI2C_BusCfg)))
        {
           printf("Number of SW I2C Bus Over Limitation\n");
           printf("Max count of SW I2C Bus is %d", (int)(sizeof(stBusCfg)/sizeof(SWI2C_BusCfg)));
           return FALSE;
        }

        if(pu8SwIICPort == NULL)
        {
            pu8SwIICPort = (MS_U8 *)malloc(sizeof(MS_U8) * u8BusCount);
        }

        for(i=0;i<u8BusCount;i++)
        {
            pstConfigI = pstConfig + i;
            if((pstConfigI->ePort >= E_MS_IIC_PORT_NOSUP) ||\
               (pstConfigI->ePort < E_MS_IIC_SW_PORT_0))
            {
                printf("Wrong parameter for SW IIC setup\n");
                return FALSE;
            }

            stBusCfg[i].padSCL = pstConfigI->u16SclPad;
            stBusCfg[i].padSDA = pstConfigI->u16SdaPad;
            stBusCfg[i].defDelay= pstConfigI->u16Delay;
            *(pu8SwIICPort + i) = (MS_U8)pstConfigI->ePort;
            printf("stBusCfg[%d].padSCL = %d\n",i,stBusCfg[i].padSCL);
            printf("stBusCfg[%d].padSDA = %d\n",i,stBusCfg[i].padSDA);
            printf("stBusCfg[%d].defDelay = %d\n",i,stBusCfg[i].defDelay);
            MApi_SWI2C_SetBusReadMode(i,eSW_Read_Mode_Default);
        }

        MApi_SWI2C_Init(stBusCfg, u8BusCount);
        bSwIICSetupDone = TRUE;
        u8SwIICCount = u8BusCount;
    }

    return TRUE;
}
#endif

void MDrv_IIC_InitPort0(MS_IIC_PORT ePort)
{
    MS_BOOL bResult = FALSE;
    HWI2C_UnitCfg hwI2CCfg;
    memset(&hwI2CCfg, 0x00, sizeof(HWI2C_UnitCfg));
#ifdef BD_CUS75_MBQ213A
#if (MS_BOARD_TYPE_SEL == BD_CUS75_MBQ213A) // To improve tuner and demod lock time
    hwI2CCfg.eSpeed = E_HWI2C_HIGH;
#else
    hwI2CCfg.eSpeed = E_HWI2C_VSLOW;//E_HWI2C_NORMAL
#endif
#else
    hwI2CCfg.eSpeed = E_HWI2C_VSLOW;//E_HWI2C_NORMAL
#endif

    hwI2CCfg.sI2CPin.u32Reg = 0;
    hwI2CCfg.sI2CPin.u8BitPos = 0;
    hwI2CCfg.sI2CPin.bEnable = 0;
    hwI2CCfg.ePort = ePort;
    bResult = MDrv_HWI2C_Init(&hwI2CCfg);
    if(bResult)
    {
        printf("MDrv_IIC_InitPort%d: OK\n", (int)ePort);
    }
    else
    {
        printf("MDrv_IIC_InitPort%d: FAIL\n", (int)ePort);
        return;
    }

    MDrv_HWI2C_SelectPort(hwI2CCfg.ePort);
    MDrv_HWI2C_SetClk(hwI2CCfg.eSpeed);
    MDrv_HWI2C_SetReadMode(eHW_Read_Mode_Default);

}

void MDrv_IIC_InitPort1(MS_IIC_PORT ePort)
{
    MS_BOOL bResult;
    HWI2C_UnitCfg hwI2CCfg;
    memset(&hwI2CCfg, 0x00, sizeof(HWI2C_UnitCfg));
#ifdef BD_CUS75_MBQ213A
#if (MS_BOARD_TYPE_SEL == BD_CUS75_MBQ213A) // To improve tuner and demod lock time
    hwI2CCfg.eSpeed = E_HWI2C_HIGH;
#else
    hwI2CCfg.eSpeed = E_HWI2C_VSLOW;//E_HWI2C_NORMAL
#endif
#else
    hwI2CCfg.eSpeed = E_HWI2C_VSLOW;//E_HWI2C_NORMAL
#endif

    hwI2CCfg.sI2CPin.u32Reg = 0;
    hwI2CCfg.sI2CPin.u8BitPos = 0;
    hwI2CCfg.sI2CPin.bEnable = 0;
    hwI2CCfg.ePort = ePort;
    bResult = MDrv_HWI2C_Init(&hwI2CCfg);
    if(bResult)
    {
        printf("MDrv_IIC_InitPort1: OK\n");
    }
    else
    {
        printf("MDrv_IIC_InitPort1: FAIL\n");
        return;
    }

    MDrv_HWI2C_SelectPort1(hwI2CCfg.ePort);
    MDrv_HWI2C_SetClkP1(hwI2CCfg.eSpeed);
    MDrv_HWI2C_SetReadModeP1(eHW_Read_Mode_Default);

}

void MDrv_IIC_Init(MS_IIC_PORT ePort)
{
    printf("MDrv_IIC_Init  PORT  %d \n", ePort);

    if(ePort >= E_MS_IIC_PORT_NOSUP)
    {
        printf("MDrv_IIC_Init: FAIL, Invalid Port number\n");
        return;
    }

#ifdef ENABLE_IIC_LOCK
    if(s32IICMutex == -1)
    {
        s32IICMutex = MsOS_CreateMutex(E_MSOS_FIFO, "Mdrv_IIC_Mutex", MSOS_PROCESS_SHARED);
        if(s32IICMutex == -1)
        {
            printf("MDrv_IIC_Init: FAIL, Create Mutex fail\n");
            return;
        }
    }
#endif //#ifdef ENABLE_IIC_LOCK

    if ((ePort >= E_MS_IIC_SW_PORT_0) && (ePort < E_MS_IIC_PORT_NOSUP))
    {
    #if SWI2C_INUSE
        if(!bSwIICSetupDone)
        {
            printf("MDrv_IIC_Init: FAIL, this is SW I2C port, setup I2C SW port by MDrv_IIC_SetupSwPort\n");
        }
    #else
        printf("MDrv_IIC_Init: FAIL, SW I2C is disabled\n");
    #endif
    }
    else
    {
         MDrv_IIC_InitPort0(ePort);
    }

}



MS_BOOL MDrv_IIC_ReadBytesPort0(MS_U16 u16BusNumSlaveID, MS_U8 ucSubAdr, MS_U8* paddr, MS_U16 ucBufLen, MS_U8* pBuf)
{
    MS_BOOL bRet = FALSE;

    MDrv_IIC_LOCK();
    bRet = MDrv_HWI2C_ReadBytes(u16BusNumSlaveID, (MS_U32)ucSubAdr, paddr, (MS_U32)ucBufLen, pBuf);
    MDrv_IIC_UNLOCK();

    return bRet;
}


MS_BOOL MDrv_IIC_ReadBytesPort1(MS_U16 u16BusNumSlaveID, MS_U8 ucSubAdr, MS_U8* paddr, MS_U16 ucBufLen, MS_U8* pBuf)
{
    MS_BOOL bRet = FALSE;

    MDrv_IIC_LOCK();
    bRet = MDrv_HWI2C_ReadBytesP1(u16BusNumSlaveID, (MS_U32)ucSubAdr, paddr, (MS_U32)ucBufLen, pBuf);
    MDrv_IIC_UNLOCK();

    return bRet;

}

MS_BOOL MDrv_IIC_ReadBytes(MS_IIC_PORT ePort, MS_U16 u16BusNumSlaveID, MS_U8 ucSubAdr, MS_U8* paddr, MS_U16 ucBufLen, MS_U8* pBuf)
{
    MS_U16 u16PortIndex;
    MS_BOOL bRet = FALSE;
#if SWI2C_INUSE
    MS_U8 u8SwI2CBus = 0;
#endif

    MDrv_IIC_LOCK();

    if(ePort < E_MS_IIC_SW_PORT_0)
    {
        //printf("IIC HW mode Read\n");
        u16PortIndex = (ePort/8)<<8;
        u16BusNumSlaveID |= u16PortIndex;
        bRet = MDrv_HWI2C_ReadBytes(u16BusNumSlaveID, (MS_U32)ucSubAdr, paddr, (MS_U32)ucBufLen, pBuf);
    }
    else if(ePort < E_MS_IIC_PORT_NOSUP)
    {
       //printf("IIC SW mode Read\n");
#if SWI2C_INUSE
       if(bSwIICSetupDone)
      {
        if(_SwIICPort_mapping(ePort, &u8SwI2CBus))
        {
            u16PortIndex = (u8SwI2CBus<<8);
            u16BusNumSlaveID |= u16PortIndex;
            bRet = MApi_SWI2C_ReadBytes(u16BusNumSlaveID, ucSubAdr, paddr, ucBufLen, pBuf);
        }
      }
#endif
    }

    MDrv_IIC_UNLOCK();

    return bRet;
}

MS_BOOL MDrv_IIC_ReadWithMode(MS_IIC_PORT ePort, MS_U16 u16BusNumSlaveID, MS_U8 ucSubAdr, MS_U8* paddr, MS_U16 ucBufLen, MS_U8* pBuf, MS_IIC_ReadMode eMode)
{
    MS_U16 u16PortIndex;
    MS_BOOL bRet = TRUE;
#if SWI2C_INUSE
    MS_U8 u8SwI2CBus = 0;
#endif

    MDrv_IIC_LOCK();

    if(ePort < E_MS_IIC_SW_PORT_0)
    {
        //printf("IIC HW mode Read\n");
        u16PortIndex = (ePort/8)<<8;
        u16BusNumSlaveID |= u16PortIndex;
        bRet &= MDrv_HWI2C_SelectPort((HWI2C_PORT)ePort);
        bRet &= MDrv_HWI2C_SetReadMode((HWI2C_ReadMode)eMode);
        bRet &= MDrv_HWI2C_ReadBytes(u16BusNumSlaveID, (MS_U32)ucSubAdr, paddr, (MS_U32)ucBufLen, pBuf);
        MDrv_HWI2C_SetReadMode(eHW_Read_Mode_Default);
    }
    else if(ePort < E_MS_IIC_PORT_NOSUP)
    {
       //printf("IIC SW mode Read\n");
 #if SWI2C_INUSE
       if(bSwIICSetupDone)
      {
        if(_SwIICPort_mapping(ePort, &u8SwI2CBus))
        {
            u16PortIndex = (u8SwI2CBus<<8);
            u16BusNumSlaveID |= u16PortIndex;
            bRet &= MApi_SWI2C_SetBusReadMode(u8SwI2CBus, (SWI2C_ReadMode)eMode);
            bRet &= MApi_SWI2C_ReadBytes(u16BusNumSlaveID, ucSubAdr, paddr, ucBufLen, pBuf);
            bRet &= MApi_SWI2C_SetBusReadMode(u8SwI2CBus, eSW_Read_Mode_Default);
        }
      }
 #endif
    }

    MDrv_IIC_UNLOCK();

    return bRet;
}


MS_BOOL MDrv_IIC_WriteBytesPort0(MS_U16 u16BusNumSlaveID, MS_U8 AddrCnt, MS_U8* pu8addr, MS_U16 u16size, MS_U8* pBuf)
{
    MS_BOOL bRet = FALSE;

    MDrv_IIC_LOCK();
    bRet = MDrv_HWI2C_WriteBytes(u16BusNumSlaveID, (MS_U32)AddrCnt, pu8addr, (MS_U32)u16size, pBuf);
    MDrv_IIC_UNLOCK();

    return bRet;
}

MS_BOOL MDrv_IIC_WriteBytesPort1(MS_U16 u16BusNumSlaveID, MS_U8 AddrCnt, MS_U8* pu8addr, MS_U16 u16size, MS_U8* pBuf)
{
    MS_BOOL bRet = FALSE;

    MDrv_IIC_LOCK();
    bRet = MDrv_HWI2C_WriteBytesP1(u16BusNumSlaveID, (MS_U32)AddrCnt, pu8addr, (MS_U32)u16size, pBuf);
    MDrv_IIC_UNLOCK();

    return bRet;
}


MS_BOOL MDrv_IIC_WriteBytes(MS_IIC_PORT ePort, MS_U16 u16BusNumSlaveID, MS_U8 AddrCnt, MS_U8* pu8addr, MS_U16 u16size, MS_U8* pBuf)
{
    MS_U16 u16PortIndex;
    MS_BOOL bRet = FALSE;
 #if SWI2C_INUSE
    MS_U8 u8SwI2CBus = 0;
 #endif

   MDrv_IIC_LOCK();
    if(ePort < E_MS_IIC_SW_PORT_0)
    {
        u16PortIndex = (ePort/8)<<8;
        u16BusNumSlaveID |= u16PortIndex;
        bRet = MDrv_HWI2C_WriteBytes(u16BusNumSlaveID, (MS_U32)AddrCnt, pu8addr, (MS_U32)u16size, pBuf);
    }
    else if(ePort < E_MS_IIC_PORT_NOSUP)
    {
  #if SWI2C_INUSE
       if(bSwIICSetupDone)
       {
         if(_SwIICPort_mapping(ePort, &u8SwI2CBus))
         {
             u16PortIndex = (u8SwI2CBus<<8);
             u16BusNumSlaveID |= u16PortIndex;
             bRet = MApi_SWI2C_WriteBytes(u16BusNumSlaveID, AddrCnt, pu8addr, u16size, pBuf);
         }
       }
  #endif
    }

   MDrv_IIC_UNLOCK();

    return bRet;
}



MS_BOOL MDrv_IIC_Read(MS_U16 u16BusNumSlaveID, MS_U8* pu8addr, MS_U8 AddrCnt, MS_U8* pBuf,  MS_U16 u16size)
{
    MS_BOOL bRet;

    MDrv_IIC_LOCK();
    bRet = MDrv_HWI2C_ReadBytes(u16BusNumSlaveID, (MS_U32)AddrCnt, pu8addr, (MS_U32)u16size, pBuf);
    MDrv_IIC_UNLOCK();
    return bRet;
}

MS_BOOL MDrv_IIC1_Read(MS_U16 u16BusNumSlaveID, MS_U8* pu8addr, MS_U8 AddrCnt, MS_U8* pBuf,  MS_U16 u16size)
{
    MS_BOOL bRet;

    MDrv_IIC_LOCK();
    bRet = MDrv_HWI2C_ReadBytesP1(u16BusNumSlaveID, (MS_U32)AddrCnt, pu8addr, (MS_U32)u16size, pBuf);
    MDrv_IIC_UNLOCK();
    return bRet;

}

MS_BOOL MDrv_IIC_Write(MS_U16 u16BusNumSlaveID, MS_U8* pu8addr, MS_U8 AddrCnt, MS_U8* pBuf,  MS_U16 u16size)
{
    MS_BOOL bRet;

    bRet = MDrv_IIC_WriteBytesPort0(u16BusNumSlaveID,AddrCnt, pu8addr,u16size, pBuf);

    return bRet;

}

MS_BOOL MDrv_IIC1_Write(MS_U16 u16BusNumSlaveID, MS_U8* pu8addr, MS_U8 AddrCnt, MS_U8* pBuf,  MS_U16 u16size)
{
    MS_BOOL bRet;

    bRet = MDrv_IIC_WriteBytesPort1(u16BusNumSlaveID,AddrCnt, pu8addr,u16size, pBuf);

    return bRet;

}

MS_BOOL MDrv_IIC_SelectCLK(MS_IIC_PORT ePort, HWI2C_CLKSEL eSpeed)
{
    MS_BOOL bRet = FALSE;

    MDrv_IIC_LOCK();
    if((ePort >= E_MS_IIC_PORT_0) && (ePort <= E_MS_IIC_PORT0_7))
    {
        bRet = MDrv_HWI2C_SetClk(eSpeed);
    }
    else if((ePort >= E_MS_IIC_PORT_1) && (ePort <=E_MS_IIC_PORT1_7))
    {
        bRet = MDrv_HWI2C_SetClkP1(eSpeed);
    }
    else
    {
        printf("ePort = %d not support set speed\n",ePort);
    }
    MDrv_IIC_UNLOCK();

    return bRet;
}


void IIC_Stop(void)
{
    printf("IIC_N");
}

MS_BOOL IIC_Start(void)
{
    printf("IIC_N");
    return 0;
}

MS_BOOL IIC_SendByte(MS_U8 u8dat)
{
    printf("IIC_N");
    return 0;
}

#if 0
HWI2C_PORT getI2CPort(MS_U8 drv_frontend_index)
{
    HWI2C_PORT ePort;
    switch(drv_frontend_index)
    {
        case 0:
            ePort = FRONTEND_TUNER_PORT0;
        break;
        case 1:
            ePort = FRONTEND_TUNER_PORT1;
        break;
        case 2:
            ePort = FRONTEND_TUNER_PORT2;
        break;
        case 3:
            ePort = FRONTEND_TUNER_PORT3;
        break;
        default:
            ePort = FRONTEND_TUNER_PORT0;
            break;
    }
    return ePort;
}
#endif

#undef _IIC_C_

