/* SPDX-License-Identifier: GPL-2.0-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2 of the GNU General Public License ("GPLv2 License")
 * or BSD License.
 *
 * GPLv2 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See http://www.gnu.org/licenses/gpl-2.0.html for more details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

#ifndef _IIC_H_
#define _IIC_H_

#include "MsCommon.h"
#include "drvHWI2C.h"

#ifndef SWI2C_INUSE
#define SWI2C_INUSE 1
#endif

#if SWI2C_INUSE
#include "apiSWI2C.h"
#endif

  // Speed Mapping: Base on the Driver loading, maybe you must to use the oscilloscope to measure this Value
  // 250K => SwIicDly =1
  // 200K => SwIicDly =10
  // 150K => SwIicDly =20
  // 100K => SwIicDly =40

#define I2C_SPEED_MAPPING_250K          1
#define I2C_SPEED_MAPPING_200K          10
#define I2C_SPEED_MAPPING_150K          20
#define I2C_SPEED_MAPPING_100K          40



typedef enum {
    E_I2C_BUS_DDCROM,
    E_I2C_BUS_SYS = E_I2C_BUS_DDCROM,
    E_I2C_BUS_DDCA0,
    E_I2C_BUS_DDCD0,
    E_I2C_BUS_DDCD1,
    E_I2C_BUS_DDCD2,
    E_I2C_BUS_USR0,
    E_I2C_BUS_USR1,
    E_I2C_BUS_USR2,
    E_I2C_BUS_USR3,
} E_I2C_BUS_CHANNEL;

typedef enum
{
    E_MS_IIC_PORT_0 = 0, /// port 0_0 //disable port 0
    E_MS_IIC_PORT0_1,    /// port 0_1
    E_MS_IIC_PORT0_2,    /// port 0_2
    E_MS_IIC_PORT0_3,    /// port 0_3
    E_MS_IIC_PORT0_4,    /// port 0_4
    E_MS_IIC_PORT0_5,    /// port 0_5
    E_MS_IIC_PORT0_6,    /// port 0_6
    E_MS_IIC_PORT0_7,    /// port 0_7

    E_MS_IIC_PORT_1 = 8, /// port 1_0 //disable port 1
    E_MS_IIC_PORT1_1,    /// port 1_1
    E_MS_IIC_PORT1_2,    /// port 1_2
    E_MS_IIC_PORT1_3,    /// port 1_3
    E_MS_IIC_PORT1_4,    /// port 1_4
    E_MS_IIC_PORT1_5,    /// port 1_5
    E_MS_IIC_PORT1_6,    /// port 1_6
    E_MS_IIC_PORT1_7,    /// port 1_7

    E_MS_IIC_PORT_2 = 16,/// port 2_0 //disable port 2
    E_MS_IIC_PORT2_1,    /// port 2_1
    E_MS_IIC_PORT2_2,    /// port 2_2
    E_MS_IIC_PORT2_3,    /// port 2_3
    E_MS_IIC_PORT2_4,    /// port 2_4
    E_MS_IIC_PORT2_5,    /// port 2_5
    E_MS_IIC_PORT2_6,    /// port 2_6
    E_MS_IIC_PORT2_7,    /// port 2_7

    E_MS_IIC_PORT_3 = 24,/// port 3_0 //disable port 3
    E_MS_IIC_PORT3_1,    /// port 3_1
    E_MS_IIC_PORT3_2,    /// port 3_2
    E_MS_IIC_PORT3_3,    /// port 3_3
    E_MS_IIC_PORT3_4,    /// port 3_4
    E_MS_IIC_PORT3_5,    /// port 3_5
    E_MS_IIC_PORT3_6,    /// port 3_6
    E_MS_IIC_PORT3_7,    /// port 3_7

    E_MS_IIC_PORT_4 = 32,/// port 4_0 //disable port 4
    E_MS_IIC_PORT4_1,    /// port 4_1
    E_MS_IIC_PORT4_2,    /// port 4_2
    E_MS_IIC_PORT4_3,    /// port 4_3
    E_MS_IIC_PORT4_4,    /// port 4_4
    E_MS_IIC_PORT4_5,    /// port 4_5
    E_MS_IIC_PORT4_6,    /// port 4_6
    E_MS_IIC_PORT4_7,    /// port 4_7

    E_MS_IIC_PORT_5 = 40,/// port 5_0 //disable port 5
    E_MS_IIC_PORT5_1,    /// port 5_1
    E_MS_IIC_PORT5_2,    /// port 5_2
    E_MS_IIC_PORT5_3,    /// port 5_3
    E_MS_IIC_PORT5_4,    /// port 5_4
    E_MS_IIC_PORT5_5,    /// port 5_5
    E_MS_IIC_PORT5_6,    /// port 5_6
    E_MS_IIC_PORT5_7,    /// port 5_7

    E_MS_IIC_SW_PORT_0 = 48,   /// swiic port0
    E_MS_IIC_SW_PORT_1,        /// swiic port1
    E_MS_IIC_SW_PORT_2,        /// swiic port2
    E_MS_IIC_SW_PORT_3,        /// swiic port3
    E_MS_IIC_SW_PORT_4,        /// swiic port4
    E_MS_IIC_SW_PORT_5,        /// swiic port5
    E_MS_IIC_SW_PORT_6,        /// swiic port6
    E_MS_IIC_SW_PORT_7,        /// swiic port7
    E_MS_IIC_SW_PORT_MAX = E_MS_IIC_SW_PORT_7,

    E_MS_IIC_PORT_NOSUP  /// non-support port

} MS_IIC_PORT;

typedef struct
{
    MS_IIC_PORT ePort;
    MS_U16 u16SclPad;
    MS_U16 u16SdaPad;
    MS_U16 u16Delay;
}MS_IIC_SwBus;

typedef enum {
    E_MS_IIC_READ_MODE_DIRECT,                 ///< first transmit slave address + reg address and then start receive the data */
    E_MS_IIC_READ_MODE_DIRECTION_CHANGE,       ///< slave address + reg address in write mode, direction change to read mode, repeat start slave address in read mode, data from device
    E_MS_IIC_READ_MODE_DIRECTION_CHANGE_STOP_START,  ///< slave address + reg address in write mode + stop, direction change to read mode, repeat start slave address in read mode, data from device
    E_MS_IIC_READ_MODE_MAX
} MS_IIC_ReadMode;

void MDrv_IIC_InitPort0(MS_IIC_PORT ePort);
void MDrv_IIC_InitPort1(MS_IIC_PORT ePort);
void MDrv_IIC_Init(MS_IIC_PORT ePort);

#if SWI2C_INUSE
MS_BOOL MDrv_IIC_SetupSwPort(MS_U8 u8BusCount, MS_IIC_SwBus* pstConfig);
#endif
MS_BOOL MDrv_IIC_WriteBytesPort0(MS_U16 u16BusNumSlaveID, MS_U8 AddrCnt, MS_U8* pu8addr, MS_U16 u16size, MS_U8* pBuf);
MS_BOOL MDrv_IIC_WriteBytesPort1(MS_U16 u16BusNumSlaveID, MS_U8 AddrCnt, MS_U8* pu8addr, MS_U16 u16size, MS_U8* pBuf);
MS_BOOL MDrv_IIC_WriteBytes(MS_IIC_PORT ePort,MS_U16 u16BusNumSlaveID, MS_U8 u8addrcount, MS_U8* pu8addr, MS_U16 u16size, MS_U8* pu8data);
MS_BOOL MDrv_IIC_ReadBytesPort0(MS_U16 u16BusNumSlaveID, MS_U8 ucSubAdr, MS_U8* paddr, MS_U16 ucBufLen, MS_U8* pBuf);
MS_BOOL MDrv_IIC_ReadBytesPort1(MS_U16 u16BusNumSlaveID, MS_U8 ucSubAdr, MS_U8* paddr, MS_U16 ucBufLen, MS_U8* pBuf);
MS_BOOL MDrv_IIC_ReadBytes(MS_IIC_PORT ePort,MS_U16 u16BusNumSlaveID, MS_U8 u8AddrNum, MS_U8* paddr, MS_U16 u16size, MS_U8* pu8data);
MS_BOOL MDrv_IIC_SelectCLK(MS_IIC_PORT ePort,HWI2C_CLKSEL eSpeed);
MS_BOOL MDrv_IIC_Read(MS_U16 u16BusNumSlaveID, MS_U8* pu8addr, MS_U8 AddrCnt, MS_U8* pBuf,  MS_U16 u16size);
MS_BOOL MDrv_IIC1_Read(MS_U16 u16BusNumSlaveID, MS_U8* pu8addr, MS_U8 AddrCnt, MS_U8* pBuf,  MS_U16 u16size);
MS_BOOL MDrv_IIC_Write(MS_U16 u16BusNumSlaveID, MS_U8* pu8addr, MS_U8 AddrCnt, MS_U8* pBuf,  MS_U16 u16size);
MS_BOOL MDrv_IIC1_Write(MS_U16 u16BusNumSlaveID, MS_U8* pu8addr, MS_U8 AddrCnt, MS_U8* pBuf,  MS_U16 u16size);
MS_BOOL MDrv_IIC_ReadWithMode(MS_IIC_PORT ePort, MS_U16 u16BusNumSlaveID, MS_U8 ucSubAdr, MS_U8* paddr, MS_U16 ucBufLen, MS_U8* pBuf, MS_IIC_ReadMode eMode);


MS_U8 IIC_GetByte (MS_U16  bAck);
MS_BOOL IIC_SendByte(MS_U8 u8dat);
MS_BOOL IIC_AccessStart(MS_U8 ucSlaveAdr, MS_U8 trans_t);
void IIC_Stop(void);
MS_BOOL IIC_Start(void);

HWI2C_PORT getI2CPort(MS_U8 drv_frontend_index);

#endif // _IIC_H_
