/* SPDX-License-Identifier: GPL-2.0-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2 of the GNU General Public License ("GPLv2 License")
 * or BSD License.
 *
 * GPLv2 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See http://www.gnu.org/licenses/gpl-2.0.html for more details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

//****************************************************
// TCON Drive Chip           : Panasonic_TCON_Driver
// TCON Excel CodeGen Version: 1.05
// TCON Excel SW      Version: 1.00
// TCON Excel update date    : 2013-11-7 16:49
//****************************************************

#ifndef _DRVTCON_TBL_H_
#define _DRVTCON_TBL_H_

#ifdef __cplusplus
extern "C"
{
#endif

#define DRV_TCON_REG(reg)      ((reg>>8)&0xFF), (reg&0xFF)
#define REG_ADDR_SIZE          2
#define REG_MASK_SIZE          1
#define REG_TABLE_END          0xFFFF

#define _PK_L_(bank, addr)     ((MS_U16)(bank << 8) | (addr))
#define _PK_H_(bank, addr)     ((MS_U16)((bank|0x80) << 8) | (addr))

#define  REG_GAMMA_BASE     0x0000
#define  REG_CLK_GEN_BASE     0x0B00
#define  REG_CHIP_TOP_BASE     0x1E00
#define  REG_TCON_BASE     0x3000
#define  REG_LPLL_BASE     0x3100
#define  REG_SC_BASE     0x2F00
#define  REG_MOD_BASE     0x3200

/*!
 *  Tcon init table type.
 */
typedef enum
{
    TCON_TABTYPE_GENERAL,
    TCON_TABTYPE_GPIO,
    TCON_TABTYPE_SCALER,
    TCON_TABTYPE_MOD,
    TCON_TABTYPE_GAMMA,
    TCON_TABTYPE_POWER_SEQUENCE_ON,
    TCON_TABTYPE_POWER_SEQUENCE_OFF,
} E_TCON_TAB_TYPE;

/*!
 *  Tcon power sequence signal type.
 */
typedef enum
{
    TCON_SIGNAL_AUO_T420HVN06_POLL_POLR_CPV_TP1,
    TCON_SIGNAL_AUO_T420HVN06_STV,
    TCON_SIGNAL_AUO_T420HVN06_NUMS,
} E_TCON_AUO_T420HVN06_PWSEQ_SIGNAL_TYPE;

typedef enum
{
    TCON_SIGNAL_POL_YOE_OE2_YCLK_STB,
    TCON_SIGNAL_YDIO1_YDIO2,
    TCON_SIGNAL_NUMS,
} E_TCON_PWSEQ_SIGNAL_TYPE;

//-------------------------------------------------------------------------------------------------
/// @brief structure used for tcon table.
//-------------------------------------------------------------------------------------------------
typedef struct
{
    MS_U8           *pTConInitTab;
    E_TCON_TAB_TYPE eInittype;
    MS_U8           *pTConInit_GPIOTab;
    E_TCON_TAB_TYPE eInit_GPIOtype;
    MS_U8           *pTConInit_SCTab;
    E_TCON_TAB_TYPE eInit_SCtype;
    MS_U8           *pTConInit_MODTab;
    E_TCON_TAB_TYPE eInit_MODtype;
    MS_U8           *pTConGammaTab;
    E_TCON_TAB_TYPE eGammatype;
    MS_U8           *pTConPower_Sequence_OnTab;
    E_TCON_TAB_TYPE ePower_Sequence_Ontype;
    MS_U8           *pTConPower_Sequence_OffTab;
    E_TCON_TAB_TYPE ePower_Sequence_Offtype;
} TCON_TAB_INFO;

/*!
 *  Tcon init type.
 */
typedef enum
{
    TCON_PANEL_AUO_T420HVN06,
    TCON_PANEL_NUMS,
} E_TCON_PANEL_INDEX;

typedef enum
{
    TCON_TAB_Init_ALL,
    TCON_TAB_Init_NUMS
} E_TCON_Init_TYPE;

extern TCON_TAB_INFO TConMAP_Main[TCON_PANEL_NUMS];

/*!
 *  Tcon init gpio type.
 */
typedef enum
{
    TCON_TAB_AUO_T420HVN06_Init_ALL,
    TCON_TAB_AUO_T420HVN06_Init_NUMS
} E_TCON_AUO_T420HVN06_Init_TYPE;

typedef enum
{
    TCON_TAB_Init_GPIO_ALL,
    TCON_TAB_Init_GPIO_NUMS
} E_TCON_Init_GPIO_TYPE;
/*!
 *  Tcon init scaler type.
 */
typedef enum
{
    TCON_TAB_AUO_T420HVN06_Init_GPIO_ALL,
    TCON_TAB_AUO_T420HVN06_Init_GPIO_NUMS
} E_TCON_AUO_T420HVN06_Init_GPIO_TYPE;

typedef enum
{
    TCON_TAB_Init_SC_ALL,
    TCON_TAB_Init_SC_NUMS
} E_TCON_Init_SC_TYPE;

/*!
 *  Tcon init mod type.
 */
typedef enum
{
    TCON_TAB_AUO_T420HVN06_Init_SC_ALL,
    TCON_TAB_AUO_T420HVN06_Init_SC_NUMS
} E_TCON_AUO_T420HVN06_Init_SC_TYPE;

typedef enum
{
    TCON_TAB_Init_MOD_ALL,
    TCON_TAB_Init_MOD_NUMS
} E_TCON_Init_MOD_TYPE;

/*!
 *  Tcon table power on sequence type.
 */
typedef enum
{
    TCON_TAB_AUO_T420HVN06_Init_MOD_ALL,
    TCON_TAB_AUO_T420HVN06_Init_MOD_NUMS
} E_TCON_AUO_T420HVN06_Init_MOD_TYPE;

typedef enum
{
    TCON_TAB_AUO_T420HVN06_Power_Sequence_On_ALL,
    TCON_TAB_AUO_T420HVN06_Power_Sequence_On_Delay_Ready,
    TCON_TAB_AUO_T420HVN06_Power_Sequence_On_Delay_Time,
    TCON_TAB_AUO_T420HVN06_Power_Sequence_On_Signal_Type,
    TCON_TAB_AUO_T420HVN06_Power_Sequence_On_NUMS
} E_TCON_AUO_T420HVN06_Power_Sequence_On_TYPE;

typedef enum
{
    TCON_TAB_Power_Sequence_On_ALL,
    TCON_TAB_Power_Sequence_On_Delay_Ready,
    TCON_TAB_Power_Sequence_On_Delay_Time,
    TCON_TAB_Power_Sequence_On_Signal_Type,
    TCON_TAB_Power_Sequence_On_NUMS
} E_TCON_Power_Sequence_On_TYPE;

/*!
 *  Tcon table power off sequence type.
 */
typedef enum
{
    TCON_TAB_AUO_T420HVN06_Power_Sequence_Off_ALL,
    TCON_TAB_AUO_T420HVN06_Power_Sequence_Off_Delay_Ready,
    TCON_TAB_AUO_T420HVN06_Power_Sequence_Off_Delay_Time,
    TCON_TAB_AUO_T420HVN06_Power_Sequence_Off_Signal_Type,
    TCON_TAB_AUO_T420HVN06_Power_Sequence_Off_NUMS
} E_TCON_AUO_T420HVN06_Power_Sequence_Off_TYPE;

typedef enum
{
    TCON_TAB_Power_Sequence_Off_ALL,
    TCON_TAB_Power_Sequence_Off_Delay_Ready,
    TCON_TAB_Power_Sequence_Off_Delay_Time,
    TCON_TAB_Power_Sequence_Off_Signal_Type,
    TCON_TAB_Power_Sequence_Off_NUMS
} E_TCON_Power_Sequence_Off_TYPE;

//-------------------------------------------------------------------------------------------------
// register define used for TCON table
//-------------------------------------------------------------------------------------------------

#define  REG_GAMMA_00           (REG_GAMMA_BASE + 0x00)
#define  REG_GAMMA_01           (REG_GAMMA_BASE + 0x01)
#define  REG_GAMMA_02           (REG_GAMMA_BASE + 0x02)
#define  REG_GAMMA_03           (REG_GAMMA_BASE + 0x03)
#define  REG_GAMMA_04           (REG_GAMMA_BASE + 0x04)
#define  REG_GAMMA_05           (REG_GAMMA_BASE + 0x05)
#define  REG_GAMMA_06           (REG_GAMMA_BASE + 0x06)
#define  REG_GAMMA_07           (REG_GAMMA_BASE + 0x07)
#define  REG_GAMMA_08           (REG_GAMMA_BASE + 0x08)
#define  REG_GAMMA_09           (REG_GAMMA_BASE + 0x09)
#define  REG_GAMMA_0A           (REG_GAMMA_BASE + 0x0A)
#define  REG_GAMMA_0B           (REG_GAMMA_BASE + 0x0B)
#define  REG_GAMMA_0C           (REG_GAMMA_BASE + 0x0C)
#define  REG_GAMMA_0D           (REG_GAMMA_BASE + 0x0D)
#define  REG_GAMMA_0E           (REG_GAMMA_BASE + 0x0E)
#define  REG_GAMMA_0F           (REG_GAMMA_BASE + 0x0F)
#define  REG_GAMMA_10           (REG_GAMMA_BASE + 0x10)
#define  REG_GAMMA_11           (REG_GAMMA_BASE + 0x11)
#define  REG_GAMMA_12           (REG_GAMMA_BASE + 0x12)
#define  REG_GAMMA_13           (REG_GAMMA_BASE + 0x13)
#define  REG_GAMMA_14           (REG_GAMMA_BASE + 0x14)
#define  REG_GAMMA_15           (REG_GAMMA_BASE + 0x15)
#define  REG_GAMMA_16           (REG_GAMMA_BASE + 0x16)
#define  REG_GAMMA_17           (REG_GAMMA_BASE + 0x17)
#define  REG_GAMMA_18           (REG_GAMMA_BASE + 0x18)
#define  REG_GAMMA_19           (REG_GAMMA_BASE + 0x19)
#define  REG_GAMMA_1A           (REG_GAMMA_BASE + 0x1A)
#define  REG_GAMMA_1B           (REG_GAMMA_BASE + 0x1B)
#define  REG_GAMMA_1C           (REG_GAMMA_BASE + 0x1C)
#define  REG_GAMMA_1D           (REG_GAMMA_BASE + 0x1D)
#define  REG_GAMMA_1E           (REG_GAMMA_BASE + 0x1E)
#define  REG_GAMMA_1F           (REG_GAMMA_BASE + 0x1F)
#define  REG_GAMMA_20           (REG_GAMMA_BASE + 0x20)
#define  REG_GAMMA_21           (REG_GAMMA_BASE + 0x21)
#define  REG_GAMMA_22           (REG_GAMMA_BASE + 0x22)
#define  REG_GAMMA_23           (REG_GAMMA_BASE + 0x23)
#define  REG_GAMMA_24           (REG_GAMMA_BASE + 0x24)
#define  REG_GAMMA_25           (REG_GAMMA_BASE + 0x25)
#define  REG_GAMMA_26           (REG_GAMMA_BASE + 0x26)
#define  REG_GAMMA_27           (REG_GAMMA_BASE + 0x27)
#define  REG_GAMMA_28           (REG_GAMMA_BASE + 0x28)
#define  REG_GAMMA_29           (REG_GAMMA_BASE + 0x29)
#define  REG_GAMMA_2A           (REG_GAMMA_BASE + 0x2A)
#define  REG_GAMMA_2B           (REG_GAMMA_BASE + 0x2B)
#define  REG_GAMMA_2C           (REG_GAMMA_BASE + 0x2C)
#define  REG_GAMMA_2D           (REG_GAMMA_BASE + 0x2D)
#define  REG_GAMMA_2E           (REG_GAMMA_BASE + 0x2E)
#define  REG_GAMMA_2F           (REG_GAMMA_BASE + 0x2F)
#define  REG_GAMMA_30           (REG_GAMMA_BASE + 0x30)
#define  REG_GAMMA_31           (REG_GAMMA_BASE + 0x31)
#define  REG_GAMMA_32           (REG_GAMMA_BASE + 0x32)
#define  REG_GAMMA_33           (REG_GAMMA_BASE + 0x33)
#define  REG_GAMMA_34           (REG_GAMMA_BASE + 0x34)
#define  REG_GAMMA_35           (REG_GAMMA_BASE + 0x35)
#define  REG_GAMMA_36           (REG_GAMMA_BASE + 0x36)
#define  REG_GAMMA_37           (REG_GAMMA_BASE + 0x37)
#define  REG_GAMMA_38           (REG_GAMMA_BASE + 0x38)
#define  REG_GAMMA_39           (REG_GAMMA_BASE + 0x39)
#define  REG_GAMMA_3A           (REG_GAMMA_BASE + 0x3A)
#define  REG_GAMMA_3B           (REG_GAMMA_BASE + 0x3B)
#define  REG_GAMMA_3C           (REG_GAMMA_BASE + 0x3C)
#define  REG_GAMMA_3D           (REG_GAMMA_BASE + 0x3D)
#define  REG_GAMMA_3E           (REG_GAMMA_BASE + 0x3E)
#define  REG_GAMMA_3F           (REG_GAMMA_BASE + 0x3F)

#define  REG_CLK_GEN_00_L       (REG_CLK_GEN_BASE + 0x00)
#define  REG_CLK_GEN_00_H       (REG_CLK_GEN_BASE + 0x01)
#define  REG_CLK_GEN_01_L       (REG_CLK_GEN_BASE + 0x02)
#define  REG_CLK_GEN_01_H       (REG_CLK_GEN_BASE + 0x03)
#define  REG_CLK_GEN_02_L       (REG_CLK_GEN_BASE + 0x04)
#define  REG_CLK_GEN_02_H       (REG_CLK_GEN_BASE + 0x05)
#define  REG_CLK_GEN_03_L       (REG_CLK_GEN_BASE + 0x06)
#define  REG_CLK_GEN_03_H       (REG_CLK_GEN_BASE + 0x07)
#define  REG_CLK_GEN_04_L       (REG_CLK_GEN_BASE + 0x08)
#define  REG_CLK_GEN_04_H       (REG_CLK_GEN_BASE + 0x09)
#define  REG_CLK_GEN_05_L       (REG_CLK_GEN_BASE + 0x0A)
#define  REG_CLK_GEN_05_H       (REG_CLK_GEN_BASE + 0x0B)
#define  REG_CLK_GEN_06_L       (REG_CLK_GEN_BASE + 0x0C)
#define  REG_CLK_GEN_06_H       (REG_CLK_GEN_BASE + 0x0D)
#define  REG_CLK_GEN_07_L       (REG_CLK_GEN_BASE + 0x0E)
#define  REG_CLK_GEN_07_H       (REG_CLK_GEN_BASE + 0x0F)
#define  REG_CLK_GEN_08_L       (REG_CLK_GEN_BASE + 0x10)
#define  REG_CLK_GEN_08_H       (REG_CLK_GEN_BASE + 0x11)
#define  REG_CLK_GEN_09_L       (REG_CLK_GEN_BASE + 0x12)
#define  REG_CLK_GEN_09_H       (REG_CLK_GEN_BASE + 0x13)
#define  REG_CLK_GEN_0A_L       (REG_CLK_GEN_BASE + 0x14)
#define  REG_CLK_GEN_0A_H       (REG_CLK_GEN_BASE + 0x15)
#define  REG_CLK_GEN_0B_L       (REG_CLK_GEN_BASE + 0x16)
#define  REG_CLK_GEN_0B_H       (REG_CLK_GEN_BASE + 0x17)
#define  REG_CLK_GEN_0C_L       (REG_CLK_GEN_BASE + 0x18)
#define  REG_CLK_GEN_0C_H       (REG_CLK_GEN_BASE + 0x19)
#define  REG_CLK_GEN_0D_L       (REG_CLK_GEN_BASE + 0x1A)
#define  REG_CLK_GEN_0D_H       (REG_CLK_GEN_BASE + 0x1B)
#define  REG_CLK_GEN_0E_L       (REG_CLK_GEN_BASE + 0x1C)
#define  REG_CLK_GEN_0E_H       (REG_CLK_GEN_BASE + 0x1D)
#define  REG_CLK_GEN_0F_L       (REG_CLK_GEN_BASE + 0x1E)
#define  REG_CLK_GEN_0F_H       (REG_CLK_GEN_BASE + 0x1F)
#define  REG_CLK_GEN_10_L       (REG_CLK_GEN_BASE + 0x20)
#define  REG_CLK_GEN_10_H       (REG_CLK_GEN_BASE + 0x21)
#define  REG_CLK_GEN_11_L       (REG_CLK_GEN_BASE + 0x22)
#define  REG_CLK_GEN_11_H       (REG_CLK_GEN_BASE + 0x23)
#define  REG_CLK_GEN_12_L       (REG_CLK_GEN_BASE + 0x24)
#define  REG_CLK_GEN_12_H       (REG_CLK_GEN_BASE + 0x25)
#define  REG_CLK_GEN_13_L       (REG_CLK_GEN_BASE + 0x26)
#define  REG_CLK_GEN_13_H       (REG_CLK_GEN_BASE + 0x27)
#define  REG_CLK_GEN_14_L       (REG_CLK_GEN_BASE + 0x28)
#define  REG_CLK_GEN_14_H       (REG_CLK_GEN_BASE + 0x29)
#define  REG_CLK_GEN_15_L       (REG_CLK_GEN_BASE + 0x2A)
#define  REG_CLK_GEN_15_H       (REG_CLK_GEN_BASE + 0x2B)
#define  REG_CLK_GEN_16_L       (REG_CLK_GEN_BASE + 0x2C)
#define  REG_CLK_GEN_16_H       (REG_CLK_GEN_BASE + 0x2D)
#define  REG_CLK_GEN_17_L       (REG_CLK_GEN_BASE + 0x2E)
#define  REG_CLK_GEN_17_H       (REG_CLK_GEN_BASE + 0x2F)
#define  REG_CLK_GEN_18_L       (REG_CLK_GEN_BASE + 0x30)
#define  REG_CLK_GEN_18_H       (REG_CLK_GEN_BASE + 0x31)
#define  REG_CLK_GEN_19_L       (REG_CLK_GEN_BASE + 0x32)
#define  REG_CLK_GEN_19_H       (REG_CLK_GEN_BASE + 0x33)
#define  REG_CLK_GEN_1A_L       (REG_CLK_GEN_BASE + 0x34)
#define  REG_CLK_GEN_1A_H       (REG_CLK_GEN_BASE + 0x35)
#define  REG_CLK_GEN_1B_L       (REG_CLK_GEN_BASE + 0x36)
#define  REG_CLK_GEN_1B_H       (REG_CLK_GEN_BASE + 0x37)
#define  REG_CLK_GEN_1C_L       (REG_CLK_GEN_BASE + 0x38)
#define  REG_CLK_GEN_1C_H       (REG_CLK_GEN_BASE + 0x39)
#define  REG_CLK_GEN_1D_L       (REG_CLK_GEN_BASE + 0x3A)
#define  REG_CLK_GEN_1D_H       (REG_CLK_GEN_BASE + 0x3B)
#define  REG_CLK_GEN_1E_L       (REG_CLK_GEN_BASE + 0x3C)
#define  REG_CLK_GEN_1E_H       (REG_CLK_GEN_BASE + 0x3D)
#define  REG_CLK_GEN_1F_L       (REG_CLK_GEN_BASE + 0x3E)
#define  REG_CLK_GEN_1F_H       (REG_CLK_GEN_BASE + 0x3F)
#define  REG_CLK_GEN_20_L       (REG_CLK_GEN_BASE + 0x40)
#define  REG_CLK_GEN_20_H       (REG_CLK_GEN_BASE + 0x41)
#define  REG_CLK_GEN_21_L       (REG_CLK_GEN_BASE + 0x42)
#define  REG_CLK_GEN_21_H       (REG_CLK_GEN_BASE + 0x43)
#define  REG_CLK_GEN_22_L       (REG_CLK_GEN_BASE + 0x44)
#define  REG_CLK_GEN_22_H       (REG_CLK_GEN_BASE + 0x45)
#define  REG_CLK_GEN_23_L       (REG_CLK_GEN_BASE + 0x46)
#define  REG_CLK_GEN_23_H       (REG_CLK_GEN_BASE + 0x47)
#define  REG_CLK_GEN_24_L       (REG_CLK_GEN_BASE + 0x48)
#define  REG_CLK_GEN_24_H       (REG_CLK_GEN_BASE + 0x49)
#define  REG_CLK_GEN_25_L       (REG_CLK_GEN_BASE + 0x4A)
#define  REG_CLK_GEN_25_H       (REG_CLK_GEN_BASE + 0x4B)
#define  REG_CLK_GEN_26_L       (REG_CLK_GEN_BASE + 0x4C)
#define  REG_CLK_GEN_26_H       (REG_CLK_GEN_BASE + 0x4D)
#define  REG_CLK_GEN_27_L       (REG_CLK_GEN_BASE + 0x4E)
#define  REG_CLK_GEN_27_H       (REG_CLK_GEN_BASE + 0x4F)
#define  REG_CLK_GEN_28_L       (REG_CLK_GEN_BASE + 0x50)
#define  REG_CLK_GEN_28_H       (REG_CLK_GEN_BASE + 0x51)
#define  REG_CLK_GEN_29_L       (REG_CLK_GEN_BASE + 0x52)
#define  REG_CLK_GEN_29_H       (REG_CLK_GEN_BASE + 0x53)
#define  REG_CLK_GEN_2A_L       (REG_CLK_GEN_BASE + 0x54)
#define  REG_CLK_GEN_2A_H       (REG_CLK_GEN_BASE + 0x55)
#define  REG_CLK_GEN_2B_L       (REG_CLK_GEN_BASE + 0x56)
#define  REG_CLK_GEN_2B_H       (REG_CLK_GEN_BASE + 0x57)
#define  REG_CLK_GEN_2C_L       (REG_CLK_GEN_BASE + 0x58)
#define  REG_CLK_GEN_2C_H       (REG_CLK_GEN_BASE + 0x59)
#define  REG_CLK_GEN_2D_L       (REG_CLK_GEN_BASE + 0x5A)
#define  REG_CLK_GEN_2D_H       (REG_CLK_GEN_BASE + 0x5B)
#define  REG_CLK_GEN_2E_L       (REG_CLK_GEN_BASE + 0x5C)
#define  REG_CLK_GEN_2E_H       (REG_CLK_GEN_BASE + 0x5D)
#define  REG_CLK_GEN_2F_L       (REG_CLK_GEN_BASE + 0x5E)
#define  REG_CLK_GEN_2F_H       (REG_CLK_GEN_BASE + 0x5F)
#define  REG_CLK_GEN_30_L       (REG_CLK_GEN_BASE + 0x60)
#define  REG_CLK_GEN_30_H       (REG_CLK_GEN_BASE + 0x61)
#define  REG_CLK_GEN_31_L       (REG_CLK_GEN_BASE + 0x62)
#define  REG_CLK_GEN_31_H       (REG_CLK_GEN_BASE + 0x63)
#define  REG_CLK_GEN_32_L       (REG_CLK_GEN_BASE + 0x64)
#define  REG_CLK_GEN_32_H       (REG_CLK_GEN_BASE + 0x65)
#define  REG_CLK_GEN_33_L       (REG_CLK_GEN_BASE + 0x66)
#define  REG_CLK_GEN_33_H       (REG_CLK_GEN_BASE + 0x67)
#define  REG_CLK_GEN_34_L       (REG_CLK_GEN_BASE + 0x68)
#define  REG_CLK_GEN_34_H       (REG_CLK_GEN_BASE + 0x69)
#define  REG_CLK_GEN_35_L       (REG_CLK_GEN_BASE + 0x6A)
#define  REG_CLK_GEN_35_H       (REG_CLK_GEN_BASE + 0x6B)
#define  REG_CLK_GEN_36_L       (REG_CLK_GEN_BASE + 0x6C)
#define  REG_CLK_GEN_36_H       (REG_CLK_GEN_BASE + 0x6D)
#define  REG_CLK_GEN_37_L       (REG_CLK_GEN_BASE + 0x6E)
#define  REG_CLK_GEN_37_H       (REG_CLK_GEN_BASE + 0x6F)
#define  REG_CLK_GEN_38_L       (REG_CLK_GEN_BASE + 0x70)
#define  REG_CLK_GEN_38_H       (REG_CLK_GEN_BASE + 0x71)
#define  REG_CLK_GEN_39_L       (REG_CLK_GEN_BASE + 0x72)
#define  REG_CLK_GEN_39_H       (REG_CLK_GEN_BASE + 0x73)
#define  REG_CLK_GEN_3A_L       (REG_CLK_GEN_BASE + 0x74)
#define  REG_CLK_GEN_3A_H       (REG_CLK_GEN_BASE + 0x75)
#define  REG_CLK_GEN_3B_L       (REG_CLK_GEN_BASE + 0x76)
#define  REG_CLK_GEN_3B_H       (REG_CLK_GEN_BASE + 0x77)
#define  REG_CLK_GEN_3C_L       (REG_CLK_GEN_BASE + 0x78)
#define  REG_CLK_GEN_3C_H       (REG_CLK_GEN_BASE + 0x79)
#define  REG_CLK_GEN_3D_L       (REG_CLK_GEN_BASE + 0x7A)
#define  REG_CLK_GEN_3D_H       (REG_CLK_GEN_BASE + 0x7B)
#define  REG_CLK_GEN_3E_L       (REG_CLK_GEN_BASE + 0x7C)
#define  REG_CLK_GEN_3E_H       (REG_CLK_GEN_BASE + 0x7D)
#define  REG_CLK_GEN_3F_L       (REG_CLK_GEN_BASE + 0x7E)
#define  REG_CLK_GEN_3F_H       (REG_CLK_GEN_BASE + 0x7F)
#define  REG_CLK_GEN_40_L       (REG_CLK_GEN_BASE + 0x80)
#define  REG_CLK_GEN_40_H       (REG_CLK_GEN_BASE + 0x81)
#define  REG_CLK_GEN_41_L       (REG_CLK_GEN_BASE + 0x82)
#define  REG_CLK_GEN_41_H       (REG_CLK_GEN_BASE + 0x83)
#define  REG_CLK_GEN_42_L       (REG_CLK_GEN_BASE + 0x84)
#define  REG_CLK_GEN_42_H       (REG_CLK_GEN_BASE + 0x85)
#define  REG_CLK_GEN_43_L       (REG_CLK_GEN_BASE + 0x86)
#define  REG_CLK_GEN_43_H       (REG_CLK_GEN_BASE + 0x87)
#define  REG_CLK_GEN_44_L       (REG_CLK_GEN_BASE + 0x88)
#define  REG_CLK_GEN_44_H       (REG_CLK_GEN_BASE + 0x89)
#define  REG_CLK_GEN_45_L       (REG_CLK_GEN_BASE + 0x8A)
#define  REG_CLK_GEN_45_H       (REG_CLK_GEN_BASE + 0x8B)
#define  REG_CLK_GEN_46_L       (REG_CLK_GEN_BASE + 0x8C)
#define  REG_CLK_GEN_46_H       (REG_CLK_GEN_BASE + 0x8D)
#define  REG_CLK_GEN_47_L       (REG_CLK_GEN_BASE + 0x8E)
#define  REG_CLK_GEN_47_H       (REG_CLK_GEN_BASE + 0x8F)
#define  REG_CLK_GEN_48_L       (REG_CLK_GEN_BASE + 0x90)
#define  REG_CLK_GEN_48_H       (REG_CLK_GEN_BASE + 0x91)
#define  REG_CLK_GEN_49_L       (REG_CLK_GEN_BASE + 0x92)
#define  REG_CLK_GEN_49_H       (REG_CLK_GEN_BASE + 0x93)
#define  REG_CLK_GEN_4A_L       (REG_CLK_GEN_BASE + 0x94)
#define  REG_CLK_GEN_4A_H       (REG_CLK_GEN_BASE + 0x95)
#define  REG_CLK_GEN_4B_L       (REG_CLK_GEN_BASE + 0x96)
#define  REG_CLK_GEN_4B_H       (REG_CLK_GEN_BASE + 0x97)
#define  REG_CLK_GEN_4C_L       (REG_CLK_GEN_BASE + 0x98)
#define  REG_CLK_GEN_4C_H       (REG_CLK_GEN_BASE + 0x99)
#define  REG_CLK_GEN_4D_L       (REG_CLK_GEN_BASE + 0x9A)
#define  REG_CLK_GEN_4D_H       (REG_CLK_GEN_BASE + 0x9B)
#define  REG_CLK_GEN_4E_L       (REG_CLK_GEN_BASE + 0x9C)
#define  REG_CLK_GEN_4E_H       (REG_CLK_GEN_BASE + 0x9D)
#define  REG_CLK_GEN_4F_L       (REG_CLK_GEN_BASE + 0x9E)
#define  REG_CLK_GEN_4F_H       (REG_CLK_GEN_BASE + 0x9F)
#define  REG_CLK_GEN_50_L       (REG_CLK_GEN_BASE + 0xA0)
#define  REG_CLK_GEN_50_H       (REG_CLK_GEN_BASE + 0xA1)
#define  REG_CLK_GEN_51_L       (REG_CLK_GEN_BASE + 0xA2)
#define  REG_CLK_GEN_51_H       (REG_CLK_GEN_BASE + 0xA3)
#define  REG_CLK_GEN_52_L       (REG_CLK_GEN_BASE + 0xA4)
#define  REG_CLK_GEN_52_H       (REG_CLK_GEN_BASE + 0xA5)
#define  REG_CLK_GEN_53_L       (REG_CLK_GEN_BASE + 0xA6)
#define  REG_CLK_GEN_53_H       (REG_CLK_GEN_BASE + 0xA7)
#define  REG_CLK_GEN_54_L       (REG_CLK_GEN_BASE + 0xA8)
#define  REG_CLK_GEN_54_H       (REG_CLK_GEN_BASE + 0xA9)
#define  REG_CLK_GEN_55_L       (REG_CLK_GEN_BASE + 0xAA)
#define  REG_CLK_GEN_55_H       (REG_CLK_GEN_BASE + 0xAB)
#define  REG_CLK_GEN_56_L       (REG_CLK_GEN_BASE + 0xAC)
#define  REG_CLK_GEN_56_H       (REG_CLK_GEN_BASE + 0xAD)
#define  REG_CLK_GEN_57_L       (REG_CLK_GEN_BASE + 0xAE)
#define  REG_CLK_GEN_57_H       (REG_CLK_GEN_BASE + 0xAF)
#define  REG_CLK_GEN_58_L       (REG_CLK_GEN_BASE + 0xB0)
#define  REG_CLK_GEN_58_H       (REG_CLK_GEN_BASE + 0xB1)
#define  REG_CLK_GEN_59_L       (REG_CLK_GEN_BASE + 0xB2)
#define  REG_CLK_GEN_59_H       (REG_CLK_GEN_BASE + 0xB3)
#define  REG_CLK_GEN_5A_L       (REG_CLK_GEN_BASE + 0xB4)
#define  REG_CLK_GEN_5A_H       (REG_CLK_GEN_BASE + 0xB5)
#define  REG_CLK_GEN_5B_L       (REG_CLK_GEN_BASE + 0xB6)
#define  REG_CLK_GEN_5B_H       (REG_CLK_GEN_BASE + 0xB7)
#define  REG_CLK_GEN_5C_L       (REG_CLK_GEN_BASE + 0xB8)
#define  REG_CLK_GEN_5C_H       (REG_CLK_GEN_BASE + 0xB9)
#define  REG_CLK_GEN_5D_L       (REG_CLK_GEN_BASE + 0xBA)
#define  REG_CLK_GEN_5D_H       (REG_CLK_GEN_BASE + 0xBB)
#define  REG_CLK_GEN_5E_L       (REG_CLK_GEN_BASE + 0xBC)
#define  REG_CLK_GEN_5E_H       (REG_CLK_GEN_BASE + 0xBD)
#define  REG_CLK_GEN_5F_L       (REG_CLK_GEN_BASE + 0xBE)
#define  REG_CLK_GEN_5F_H       (REG_CLK_GEN_BASE + 0xBF)
#define  REG_CLK_GEN_60_L       (REG_CLK_GEN_BASE + 0xC0)
#define  REG_CLK_GEN_60_H       (REG_CLK_GEN_BASE + 0xC1)
#define  REG_CLK_GEN_61_L       (REG_CLK_GEN_BASE + 0xC2)
#define  REG_CLK_GEN_61_H       (REG_CLK_GEN_BASE + 0xC3)
#define  REG_CLK_GEN_62_L       (REG_CLK_GEN_BASE + 0xC4)
#define  REG_CLK_GEN_62_H       (REG_CLK_GEN_BASE + 0xC5)
#define  REG_CLK_GEN_63_L       (REG_CLK_GEN_BASE + 0xC6)
#define  REG_CLK_GEN_63_H       (REG_CLK_GEN_BASE + 0xC7)
#define  REG_CLK_GEN_64_L       (REG_CLK_GEN_BASE + 0xC8)
#define  REG_CLK_GEN_64_H       (REG_CLK_GEN_BASE + 0xC9)
#define  REG_CLK_GEN_65_L       (REG_CLK_GEN_BASE + 0xCA)
#define  REG_CLK_GEN_65_H       (REG_CLK_GEN_BASE + 0xCB)
#define  REG_CLK_GEN_66_L       (REG_CLK_GEN_BASE + 0xCC)
#define  REG_CLK_GEN_66_H       (REG_CLK_GEN_BASE + 0xCD)
#define  REG_CLK_GEN_67_L       (REG_CLK_GEN_BASE + 0xCE)
#define  REG_CLK_GEN_67_H       (REG_CLK_GEN_BASE + 0xCF)
#define  REG_CLK_GEN_68_L       (REG_CLK_GEN_BASE + 0xD0)
#define  REG_CLK_GEN_68_H       (REG_CLK_GEN_BASE + 0xD1)
#define  REG_CLK_GEN_69_L       (REG_CLK_GEN_BASE + 0xD2)
#define  REG_CLK_GEN_69_H       (REG_CLK_GEN_BASE + 0xD3)
#define  REG_CLK_GEN_6A_L       (REG_CLK_GEN_BASE + 0xD4)
#define  REG_CLK_GEN_6A_H       (REG_CLK_GEN_BASE + 0xD5)
#define  REG_CLK_GEN_6B_L       (REG_CLK_GEN_BASE + 0xD6)
#define  REG_CLK_GEN_6B_H       (REG_CLK_GEN_BASE + 0xD7)
#define  REG_CLK_GEN_6C_L       (REG_CLK_GEN_BASE + 0xD8)
#define  REG_CLK_GEN_6C_H       (REG_CLK_GEN_BASE + 0xD9)
#define  REG_CLK_GEN_6D_L       (REG_CLK_GEN_BASE + 0xDA)
#define  REG_CLK_GEN_6D_H       (REG_CLK_GEN_BASE + 0xDB)
#define  REG_CLK_GEN_6E_L       (REG_CLK_GEN_BASE + 0xDC)
#define  REG_CLK_GEN_6E_H       (REG_CLK_GEN_BASE + 0xDD)
#define  REG_CLK_GEN_6F_L       (REG_CLK_GEN_BASE + 0xDE)
#define  REG_CLK_GEN_6F_H       (REG_CLK_GEN_BASE + 0xDF)
#define  REG_CLK_GEN_70_L       (REG_CLK_GEN_BASE + 0xE0)
#define  REG_CLK_GEN_70_H       (REG_CLK_GEN_BASE + 0xE1)
#define  REG_CLK_GEN_71_L       (REG_CLK_GEN_BASE + 0xE2)
#define  REG_CLK_GEN_71_H       (REG_CLK_GEN_BASE + 0xE3)
#define  REG_CLK_GEN_72_L       (REG_CLK_GEN_BASE + 0xE4)
#define  REG_CLK_GEN_72_H       (REG_CLK_GEN_BASE + 0xE5)
#define  REG_CLK_GEN_73_L       (REG_CLK_GEN_BASE + 0xE6)
#define  REG_CLK_GEN_73_H       (REG_CLK_GEN_BASE + 0xE7)
#define  REG_CLK_GEN_74_L       (REG_CLK_GEN_BASE + 0xE8)
#define  REG_CLK_GEN_74_H       (REG_CLK_GEN_BASE + 0xE9)
#define  REG_CLK_GEN_75_L       (REG_CLK_GEN_BASE + 0xEA)
#define  REG_CLK_GEN_75_H       (REG_CLK_GEN_BASE + 0xEB)
#define  REG_CLK_GEN_76_L       (REG_CLK_GEN_BASE + 0xEC)
#define  REG_CLK_GEN_76_H       (REG_CLK_GEN_BASE + 0xED)
#define  REG_CLK_GEN_77_L       (REG_CLK_GEN_BASE + 0xEE)
#define  REG_CLK_GEN_77_H       (REG_CLK_GEN_BASE + 0xEF)
#define  REG_CLK_GEN_78_L       (REG_CLK_GEN_BASE + 0xF0)
#define  REG_CLK_GEN_78_H       (REG_CLK_GEN_BASE + 0xF1)
#define  REG_CLK_GEN_79_L       (REG_CLK_GEN_BASE + 0xF2)
#define  REG_CLK_GEN_79_H       (REG_CLK_GEN_BASE + 0xF3)
#define  REG_CLK_GEN_7A_L       (REG_CLK_GEN_BASE + 0xF4)
#define  REG_CLK_GEN_7A_H       (REG_CLK_GEN_BASE + 0xF5)
#define  REG_CLK_GEN_7B_L       (REG_CLK_GEN_BASE + 0xF6)
#define  REG_CLK_GEN_7B_H       (REG_CLK_GEN_BASE + 0xF7)
#define  REG_CLK_GEN_7C_L       (REG_CLK_GEN_BASE + 0xF8)
#define  REG_CLK_GEN_7C_H       (REG_CLK_GEN_BASE + 0xF9)
#define  REG_CLK_GEN_7D_L       (REG_CLK_GEN_BASE + 0xFA)
#define  REG_CLK_GEN_7D_H       (REG_CLK_GEN_BASE + 0xFB)
#define  REG_CLK_GEN_7E_L       (REG_CLK_GEN_BASE + 0xFC)
#define  REG_CLK_GEN_7E_H       (REG_CLK_GEN_BASE + 0xFD)
#define  REG_CLK_GEN_7F_L       (REG_CLK_GEN_BASE + 0xFE)
#define  REG_CLK_GEN_7F_H       (REG_CLK_GEN_BASE + 0xFF)

#define  REG_CHIP_TOP_00_L       (REG_CHIP_TOP_BASE + 0x00)
#define  REG_CHIP_TOP_00_H       (REG_CHIP_TOP_BASE + 0x01)
#define  REG_CHIP_TOP_01_L       (REG_CHIP_TOP_BASE + 0x02)
#define  REG_CHIP_TOP_01_H       (REG_CHIP_TOP_BASE + 0x03)
#define  REG_CHIP_TOP_02_L       (REG_CHIP_TOP_BASE + 0x04)
#define  REG_CHIP_TOP_02_H       (REG_CHIP_TOP_BASE + 0x05)
#define  REG_CHIP_TOP_03_L       (REG_CHIP_TOP_BASE + 0x06)
#define  REG_CHIP_TOP_03_H       (REG_CHIP_TOP_BASE + 0x07)
#define  REG_CHIP_TOP_04_L       (REG_CHIP_TOP_BASE + 0x08)
#define  REG_CHIP_TOP_04_H       (REG_CHIP_TOP_BASE + 0x09)
#define  REG_CHIP_TOP_05_L       (REG_CHIP_TOP_BASE + 0x0A)
#define  REG_CHIP_TOP_05_H       (REG_CHIP_TOP_BASE + 0x0B)
#define  REG_CHIP_TOP_06_L       (REG_CHIP_TOP_BASE + 0x0C)
#define  REG_CHIP_TOP_06_H       (REG_CHIP_TOP_BASE + 0x0D)
#define  REG_CHIP_TOP_07_L       (REG_CHIP_TOP_BASE + 0x0E)
#define  REG_CHIP_TOP_07_H       (REG_CHIP_TOP_BASE + 0x0F)
#define  REG_CHIP_TOP_08_L       (REG_CHIP_TOP_BASE + 0x10)
#define  REG_CHIP_TOP_08_H       (REG_CHIP_TOP_BASE + 0x11)
#define  REG_CHIP_TOP_09_L       (REG_CHIP_TOP_BASE + 0x12)
#define  REG_CHIP_TOP_09_H       (REG_CHIP_TOP_BASE + 0x13)
#define  REG_CHIP_TOP_0A_L       (REG_CHIP_TOP_BASE + 0x14)
#define  REG_CHIP_TOP_0A_H       (REG_CHIP_TOP_BASE + 0x15)
#define  REG_CHIP_TOP_0B_L       (REG_CHIP_TOP_BASE + 0x16)
#define  REG_CHIP_TOP_0B_H       (REG_CHIP_TOP_BASE + 0x17)
#define  REG_CHIP_TOP_0C_L       (REG_CHIP_TOP_BASE + 0x18)
#define  REG_CHIP_TOP_0C_H       (REG_CHIP_TOP_BASE + 0x19)
#define  REG_CHIP_TOP_0D_L       (REG_CHIP_TOP_BASE + 0x1A)
#define  REG_CHIP_TOP_0D_H       (REG_CHIP_TOP_BASE + 0x1B)
#define  REG_CHIP_TOP_0E_L       (REG_CHIP_TOP_BASE + 0x1C)
#define  REG_CHIP_TOP_0E_H       (REG_CHIP_TOP_BASE + 0x1D)
#define  REG_CHIP_TOP_0F_L       (REG_CHIP_TOP_BASE + 0x1E)
#define  REG_CHIP_TOP_0F_H       (REG_CHIP_TOP_BASE + 0x1F)
#define  REG_CHIP_TOP_10_L       (REG_CHIP_TOP_BASE + 0x20)
#define  REG_CHIP_TOP_10_H       (REG_CHIP_TOP_BASE + 0x21)
#define  REG_CHIP_TOP_11_L       (REG_CHIP_TOP_BASE + 0x22)
#define  REG_CHIP_TOP_11_H       (REG_CHIP_TOP_BASE + 0x23)
#define  REG_CHIP_TOP_12_L       (REG_CHIP_TOP_BASE + 0x24)
#define  REG_CHIP_TOP_12_H       (REG_CHIP_TOP_BASE + 0x25)
#define  REG_CHIP_TOP_13_L       (REG_CHIP_TOP_BASE + 0x26)
#define  REG_CHIP_TOP_13_H       (REG_CHIP_TOP_BASE + 0x27)
#define  REG_CHIP_TOP_14_L       (REG_CHIP_TOP_BASE + 0x28)
#define  REG_CHIP_TOP_14_H       (REG_CHIP_TOP_BASE + 0x29)
#define  REG_CHIP_TOP_15_L       (REG_CHIP_TOP_BASE + 0x2A)
#define  REG_CHIP_TOP_15_H       (REG_CHIP_TOP_BASE + 0x2B)
#define  REG_CHIP_TOP_16_L       (REG_CHIP_TOP_BASE + 0x2C)
#define  REG_CHIP_TOP_16_H       (REG_CHIP_TOP_BASE + 0x2D)
#define  REG_CHIP_TOP_17_L       (REG_CHIP_TOP_BASE + 0x2E)
#define  REG_CHIP_TOP_17_H       (REG_CHIP_TOP_BASE + 0x2F)
#define  REG_CHIP_TOP_18_L       (REG_CHIP_TOP_BASE + 0x30)
#define  REG_CHIP_TOP_18_H       (REG_CHIP_TOP_BASE + 0x31)
#define  REG_CHIP_TOP_19_L       (REG_CHIP_TOP_BASE + 0x32)
#define  REG_CHIP_TOP_19_H       (REG_CHIP_TOP_BASE + 0x33)
#define  REG_CHIP_TOP_1A_L       (REG_CHIP_TOP_BASE + 0x34)
#define  REG_CHIP_TOP_1A_H       (REG_CHIP_TOP_BASE + 0x35)
#define  REG_CHIP_TOP_1B_L       (REG_CHIP_TOP_BASE + 0x36)
#define  REG_CHIP_TOP_1B_H       (REG_CHIP_TOP_BASE + 0x37)
#define  REG_CHIP_TOP_1C_L       (REG_CHIP_TOP_BASE + 0x38)
#define  REG_CHIP_TOP_1C_H       (REG_CHIP_TOP_BASE + 0x39)
#define  REG_CHIP_TOP_1D_L       (REG_CHIP_TOP_BASE + 0x3A)
#define  REG_CHIP_TOP_1D_H       (REG_CHIP_TOP_BASE + 0x3B)
#define  REG_CHIP_TOP_1E_L       (REG_CHIP_TOP_BASE + 0x3C)
#define  REG_CHIP_TOP_1E_H       (REG_CHIP_TOP_BASE + 0x3D)
#define  REG_CHIP_TOP_1F_L       (REG_CHIP_TOP_BASE + 0x3E)
#define  REG_CHIP_TOP_1F_H       (REG_CHIP_TOP_BASE + 0x3F)
#define  REG_CHIP_TOP_20_L       (REG_CHIP_TOP_BASE + 0x40)
#define  REG_CHIP_TOP_20_H       (REG_CHIP_TOP_BASE + 0x41)
#define  REG_CHIP_TOP_21_L       (REG_CHIP_TOP_BASE + 0x42)
#define  REG_CHIP_TOP_21_H       (REG_CHIP_TOP_BASE + 0x43)
#define  REG_CHIP_TOP_22_L       (REG_CHIP_TOP_BASE + 0x44)
#define  REG_CHIP_TOP_22_H       (REG_CHIP_TOP_BASE + 0x45)
#define  REG_CHIP_TOP_23_L       (REG_CHIP_TOP_BASE + 0x46)
#define  REG_CHIP_TOP_23_H       (REG_CHIP_TOP_BASE + 0x47)
#define  REG_CHIP_TOP_24_L       (REG_CHIP_TOP_BASE + 0x48)
#define  REG_CHIP_TOP_24_H       (REG_CHIP_TOP_BASE + 0x49)
#define  REG_CHIP_TOP_25_L       (REG_CHIP_TOP_BASE + 0x4A)
#define  REG_CHIP_TOP_25_H       (REG_CHIP_TOP_BASE + 0x4B)
#define  REG_CHIP_TOP_26_L       (REG_CHIP_TOP_BASE + 0x4C)
#define  REG_CHIP_TOP_26_H       (REG_CHIP_TOP_BASE + 0x4D)
#define  REG_CHIP_TOP_27_L       (REG_CHIP_TOP_BASE + 0x4E)
#define  REG_CHIP_TOP_27_H       (REG_CHIP_TOP_BASE + 0x4F)
#define  REG_CHIP_TOP_28_L       (REG_CHIP_TOP_BASE + 0x50)
#define  REG_CHIP_TOP_28_H       (REG_CHIP_TOP_BASE + 0x51)
#define  REG_CHIP_TOP_29_L       (REG_CHIP_TOP_BASE + 0x52)
#define  REG_CHIP_TOP_29_H       (REG_CHIP_TOP_BASE + 0x53)
#define  REG_CHIP_TOP_2A_L       (REG_CHIP_TOP_BASE + 0x54)
#define  REG_CHIP_TOP_2A_H       (REG_CHIP_TOP_BASE + 0x55)
#define  REG_CHIP_TOP_2B_L       (REG_CHIP_TOP_BASE + 0x56)
#define  REG_CHIP_TOP_2B_H       (REG_CHIP_TOP_BASE + 0x57)
#define  REG_CHIP_TOP_2C_L       (REG_CHIP_TOP_BASE + 0x58)
#define  REG_CHIP_TOP_2C_H       (REG_CHIP_TOP_BASE + 0x59)
#define  REG_CHIP_TOP_2D_L       (REG_CHIP_TOP_BASE + 0x5A)
#define  REG_CHIP_TOP_2D_H       (REG_CHIP_TOP_BASE + 0x5B)
#define  REG_CHIP_TOP_2E_L       (REG_CHIP_TOP_BASE + 0x5C)
#define  REG_CHIP_TOP_2E_H       (REG_CHIP_TOP_BASE + 0x5D)
#define  REG_CHIP_TOP_2F_L       (REG_CHIP_TOP_BASE + 0x5E)
#define  REG_CHIP_TOP_2F_H       (REG_CHIP_TOP_BASE + 0x5F)
#define  REG_CHIP_TOP_30_L       (REG_CHIP_TOP_BASE + 0x60)
#define  REG_CHIP_TOP_30_H       (REG_CHIP_TOP_BASE + 0x61)
#define  REG_CHIP_TOP_31_L       (REG_CHIP_TOP_BASE + 0x62)
#define  REG_CHIP_TOP_31_H       (REG_CHIP_TOP_BASE + 0x63)
#define  REG_CHIP_TOP_32_L       (REG_CHIP_TOP_BASE + 0x64)
#define  REG_CHIP_TOP_32_H       (REG_CHIP_TOP_BASE + 0x65)
#define  REG_CHIP_TOP_33_L       (REG_CHIP_TOP_BASE + 0x66)
#define  REG_CHIP_TOP_33_H       (REG_CHIP_TOP_BASE + 0x67)
#define  REG_CHIP_TOP_34_L       (REG_CHIP_TOP_BASE + 0x68)
#define  REG_CHIP_TOP_34_H       (REG_CHIP_TOP_BASE + 0x69)
#define  REG_CHIP_TOP_35_L       (REG_CHIP_TOP_BASE + 0x6A)
#define  REG_CHIP_TOP_35_H       (REG_CHIP_TOP_BASE + 0x6B)
#define  REG_CHIP_TOP_36_L       (REG_CHIP_TOP_BASE + 0x6C)
#define  REG_CHIP_TOP_36_H       (REG_CHIP_TOP_BASE + 0x6D)
#define  REG_CHIP_TOP_37_L       (REG_CHIP_TOP_BASE + 0x6E)
#define  REG_CHIP_TOP_37_H       (REG_CHIP_TOP_BASE + 0x6F)
#define  REG_CHIP_TOP_38_L       (REG_CHIP_TOP_BASE + 0x70)
#define  REG_CHIP_TOP_38_H       (REG_CHIP_TOP_BASE + 0x71)
#define  REG_CHIP_TOP_39_L       (REG_CHIP_TOP_BASE + 0x72)
#define  REG_CHIP_TOP_39_H       (REG_CHIP_TOP_BASE + 0x73)
#define  REG_CHIP_TOP_3A_L       (REG_CHIP_TOP_BASE + 0x74)
#define  REG_CHIP_TOP_3A_H       (REG_CHIP_TOP_BASE + 0x75)
#define  REG_CHIP_TOP_3B_L       (REG_CHIP_TOP_BASE + 0x76)
#define  REG_CHIP_TOP_3B_H       (REG_CHIP_TOP_BASE + 0x77)
#define  REG_CHIP_TOP_3C_L       (REG_CHIP_TOP_BASE + 0x78)
#define  REG_CHIP_TOP_3C_H       (REG_CHIP_TOP_BASE + 0x79)
#define  REG_CHIP_TOP_3D_L       (REG_CHIP_TOP_BASE + 0x7A)
#define  REG_CHIP_TOP_3D_H       (REG_CHIP_TOP_BASE + 0x7B)
#define  REG_CHIP_TOP_3E_L       (REG_CHIP_TOP_BASE + 0x7C)
#define  REG_CHIP_TOP_3E_H       (REG_CHIP_TOP_BASE + 0x7D)
#define  REG_CHIP_TOP_3F_L       (REG_CHIP_TOP_BASE + 0x7E)
#define  REG_CHIP_TOP_3F_H       (REG_CHIP_TOP_BASE + 0x7F)
#define  REG_CHIP_TOP_40_L       (REG_CHIP_TOP_BASE + 0x80)
#define  REG_CHIP_TOP_40_H       (REG_CHIP_TOP_BASE + 0x81)
#define  REG_CHIP_TOP_41_L       (REG_CHIP_TOP_BASE + 0x82)
#define  REG_CHIP_TOP_41_H       (REG_CHIP_TOP_BASE + 0x83)
#define  REG_CHIP_TOP_42_L       (REG_CHIP_TOP_BASE + 0x84)
#define  REG_CHIP_TOP_42_H       (REG_CHIP_TOP_BASE + 0x85)
#define  REG_CHIP_TOP_43_L       (REG_CHIP_TOP_BASE + 0x86)
#define  REG_CHIP_TOP_43_H       (REG_CHIP_TOP_BASE + 0x87)
#define  REG_CHIP_TOP_44_L       (REG_CHIP_TOP_BASE + 0x88)
#define  REG_CHIP_TOP_44_H       (REG_CHIP_TOP_BASE + 0x89)
#define  REG_CHIP_TOP_45_L       (REG_CHIP_TOP_BASE + 0x8A)
#define  REG_CHIP_TOP_45_H       (REG_CHIP_TOP_BASE + 0x8B)
#define  REG_CHIP_TOP_46_L       (REG_CHIP_TOP_BASE + 0x8C)
#define  REG_CHIP_TOP_46_H       (REG_CHIP_TOP_BASE + 0x8D)
#define  REG_CHIP_TOP_47_L       (REG_CHIP_TOP_BASE + 0x8E)
#define  REG_CHIP_TOP_47_H       (REG_CHIP_TOP_BASE + 0x8F)
#define  REG_CHIP_TOP_48_L       (REG_CHIP_TOP_BASE + 0x90)
#define  REG_CHIP_TOP_48_H       (REG_CHIP_TOP_BASE + 0x91)
#define  REG_CHIP_TOP_49_L       (REG_CHIP_TOP_BASE + 0x92)
#define  REG_CHIP_TOP_49_H       (REG_CHIP_TOP_BASE + 0x93)
#define  REG_CHIP_TOP_4A_L       (REG_CHIP_TOP_BASE + 0x94)
#define  REG_CHIP_TOP_4A_H       (REG_CHIP_TOP_BASE + 0x95)
#define  REG_CHIP_TOP_4B_L       (REG_CHIP_TOP_BASE + 0x96)
#define  REG_CHIP_TOP_4B_H       (REG_CHIP_TOP_BASE + 0x97)
#define  REG_CHIP_TOP_4C_L       (REG_CHIP_TOP_BASE + 0x98)
#define  REG_CHIP_TOP_4C_H       (REG_CHIP_TOP_BASE + 0x99)
#define  REG_CHIP_TOP_4D_L       (REG_CHIP_TOP_BASE + 0x9A)
#define  REG_CHIP_TOP_4D_H       (REG_CHIP_TOP_BASE + 0x9B)
#define  REG_CHIP_TOP_4E_L       (REG_CHIP_TOP_BASE + 0x9C)
#define  REG_CHIP_TOP_4E_H       (REG_CHIP_TOP_BASE + 0x9D)
#define  REG_CHIP_TOP_4F_L       (REG_CHIP_TOP_BASE + 0x9E)
#define  REG_CHIP_TOP_4F_H       (REG_CHIP_TOP_BASE + 0x9F)
#define  REG_CHIP_TOP_50_L       (REG_CHIP_TOP_BASE + 0xA0)
#define  REG_CHIP_TOP_50_H       (REG_CHIP_TOP_BASE + 0xA1)
#define  REG_CHIP_TOP_51_L       (REG_CHIP_TOP_BASE + 0xA2)
#define  REG_CHIP_TOP_51_H       (REG_CHIP_TOP_BASE + 0xA3)
#define  REG_CHIP_TOP_52_L       (REG_CHIP_TOP_BASE + 0xA4)
#define  REG_CHIP_TOP_52_H       (REG_CHIP_TOP_BASE + 0xA5)
#define  REG_CHIP_TOP_53_L       (REG_CHIP_TOP_BASE + 0xA6)
#define  REG_CHIP_TOP_53_H       (REG_CHIP_TOP_BASE + 0xA7)
#define  REG_CHIP_TOP_54_L       (REG_CHIP_TOP_BASE + 0xA8)
#define  REG_CHIP_TOP_54_H       (REG_CHIP_TOP_BASE + 0xA9)
#define  REG_CHIP_TOP_55_L       (REG_CHIP_TOP_BASE + 0xAA)
#define  REG_CHIP_TOP_55_H       (REG_CHIP_TOP_BASE + 0xAB)
#define  REG_CHIP_TOP_56_L       (REG_CHIP_TOP_BASE + 0xAC)
#define  REG_CHIP_TOP_56_H       (REG_CHIP_TOP_BASE + 0xAD)
#define  REG_CHIP_TOP_57_L       (REG_CHIP_TOP_BASE + 0xAE)
#define  REG_CHIP_TOP_57_H       (REG_CHIP_TOP_BASE + 0xAF)
#define  REG_CHIP_TOP_58_L       (REG_CHIP_TOP_BASE + 0xB0)
#define  REG_CHIP_TOP_58_H       (REG_CHIP_TOP_BASE + 0xB1)
#define  REG_CHIP_TOP_59_L       (REG_CHIP_TOP_BASE + 0xB2)
#define  REG_CHIP_TOP_59_H       (REG_CHIP_TOP_BASE + 0xB3)
#define  REG_CHIP_TOP_5A_L       (REG_CHIP_TOP_BASE + 0xB4)
#define  REG_CHIP_TOP_5A_H       (REG_CHIP_TOP_BASE + 0xB5)
#define  REG_CHIP_TOP_5B_L       (REG_CHIP_TOP_BASE + 0xB6)
#define  REG_CHIP_TOP_5B_H       (REG_CHIP_TOP_BASE + 0xB7)
#define  REG_CHIP_TOP_5C_L       (REG_CHIP_TOP_BASE + 0xB8)
#define  REG_CHIP_TOP_5C_H       (REG_CHIP_TOP_BASE + 0xB9)
#define  REG_CHIP_TOP_5D_L       (REG_CHIP_TOP_BASE + 0xBA)
#define  REG_CHIP_TOP_5D_H       (REG_CHIP_TOP_BASE + 0xBB)
#define  REG_CHIP_TOP_5E_L       (REG_CHIP_TOP_BASE + 0xBC)
#define  REG_CHIP_TOP_5E_H       (REG_CHIP_TOP_BASE + 0xBD)
#define  REG_CHIP_TOP_5F_L       (REG_CHIP_TOP_BASE + 0xBE)
#define  REG_CHIP_TOP_5F_H       (REG_CHIP_TOP_BASE + 0xBF)
#define  REG_CHIP_TOP_60_L       (REG_CHIP_TOP_BASE + 0xC0)
#define  REG_CHIP_TOP_60_H       (REG_CHIP_TOP_BASE + 0xC1)
#define  REG_CHIP_TOP_61_L       (REG_CHIP_TOP_BASE + 0xC2)
#define  REG_CHIP_TOP_61_H       (REG_CHIP_TOP_BASE + 0xC3)
#define  REG_CHIP_TOP_62_L       (REG_CHIP_TOP_BASE + 0xC4)
#define  REG_CHIP_TOP_62_H       (REG_CHIP_TOP_BASE + 0xC5)
#define  REG_CHIP_TOP_63_L       (REG_CHIP_TOP_BASE + 0xC6)
#define  REG_CHIP_TOP_63_H       (REG_CHIP_TOP_BASE + 0xC7)
#define  REG_CHIP_TOP_64_L       (REG_CHIP_TOP_BASE + 0xC8)
#define  REG_CHIP_TOP_64_H       (REG_CHIP_TOP_BASE + 0xC9)
#define  REG_CHIP_TOP_65_L       (REG_CHIP_TOP_BASE + 0xCA)
#define  REG_CHIP_TOP_65_H       (REG_CHIP_TOP_BASE + 0xCB)
#define  REG_CHIP_TOP_66_L       (REG_CHIP_TOP_BASE + 0xCC)
#define  REG_CHIP_TOP_66_H       (REG_CHIP_TOP_BASE + 0xCD)
#define  REG_CHIP_TOP_67_L       (REG_CHIP_TOP_BASE + 0xCE)
#define  REG_CHIP_TOP_67_H       (REG_CHIP_TOP_BASE + 0xCF)
#define  REG_CHIP_TOP_68_L       (REG_CHIP_TOP_BASE + 0xD0)
#define  REG_CHIP_TOP_68_H       (REG_CHIP_TOP_BASE + 0xD1)
#define  REG_CHIP_TOP_69_L       (REG_CHIP_TOP_BASE + 0xD2)
#define  REG_CHIP_TOP_69_H       (REG_CHIP_TOP_BASE + 0xD3)
#define  REG_CHIP_TOP_6A_L       (REG_CHIP_TOP_BASE + 0xD4)
#define  REG_CHIP_TOP_6A_H       (REG_CHIP_TOP_BASE + 0xD5)
#define  REG_CHIP_TOP_6B_L       (REG_CHIP_TOP_BASE + 0xD6)
#define  REG_CHIP_TOP_6B_H       (REG_CHIP_TOP_BASE + 0xD7)
#define  REG_CHIP_TOP_6C_L       (REG_CHIP_TOP_BASE + 0xD8)
#define  REG_CHIP_TOP_6C_H       (REG_CHIP_TOP_BASE + 0xD9)
#define  REG_CHIP_TOP_6D_L       (REG_CHIP_TOP_BASE + 0xDA)
#define  REG_CHIP_TOP_6D_H       (REG_CHIP_TOP_BASE + 0xDB)
#define  REG_CHIP_TOP_6E_L       (REG_CHIP_TOP_BASE + 0xDC)
#define  REG_CHIP_TOP_6E_H       (REG_CHIP_TOP_BASE + 0xDD)
#define  REG_CHIP_TOP_6F_L       (REG_CHIP_TOP_BASE + 0xDE)
#define  REG_CHIP_TOP_6F_H       (REG_CHIP_TOP_BASE + 0xDF)
#define  REG_CHIP_TOP_70_L       (REG_CHIP_TOP_BASE + 0xE0)
#define  REG_CHIP_TOP_70_H       (REG_CHIP_TOP_BASE + 0xE1)
#define  REG_CHIP_TOP_71_L       (REG_CHIP_TOP_BASE + 0xE2)
#define  REG_CHIP_TOP_71_H       (REG_CHIP_TOP_BASE + 0xE3)
#define  REG_CHIP_TOP_72_L       (REG_CHIP_TOP_BASE + 0xE4)
#define  REG_CHIP_TOP_72_H       (REG_CHIP_TOP_BASE + 0xE5)
#define  REG_CHIP_TOP_73_L       (REG_CHIP_TOP_BASE + 0xE6)
#define  REG_CHIP_TOP_73_H       (REG_CHIP_TOP_BASE + 0xE7)
#define  REG_CHIP_TOP_74_L       (REG_CHIP_TOP_BASE + 0xE8)
#define  REG_CHIP_TOP_74_H       (REG_CHIP_TOP_BASE + 0xE9)
#define  REG_CHIP_TOP_75_L       (REG_CHIP_TOP_BASE + 0xEA)
#define  REG_CHIP_TOP_75_H       (REG_CHIP_TOP_BASE + 0xEB)
#define  REG_CHIP_TOP_76_L       (REG_CHIP_TOP_BASE + 0xEC)
#define  REG_CHIP_TOP_76_H       (REG_CHIP_TOP_BASE + 0xED)
#define  REG_CHIP_TOP_77_L       (REG_CHIP_TOP_BASE + 0xEE)
#define  REG_CHIP_TOP_77_H       (REG_CHIP_TOP_BASE + 0xEF)
#define  REG_CHIP_TOP_78_L       (REG_CHIP_TOP_BASE + 0xF0)
#define  REG_CHIP_TOP_78_H       (REG_CHIP_TOP_BASE + 0xF1)
#define  REG_CHIP_TOP_79_L       (REG_CHIP_TOP_BASE + 0xF2)
#define  REG_CHIP_TOP_79_H       (REG_CHIP_TOP_BASE + 0xF3)
#define  REG_CHIP_TOP_7A_L       (REG_CHIP_TOP_BASE + 0xF4)
#define  REG_CHIP_TOP_7A_H       (REG_CHIP_TOP_BASE + 0xF5)
#define  REG_CHIP_TOP_7B_L       (REG_CHIP_TOP_BASE + 0xF6)
#define  REG_CHIP_TOP_7B_H       (REG_CHIP_TOP_BASE + 0xF7)
#define  REG_CHIP_TOP_7C_L       (REG_CHIP_TOP_BASE + 0xF8)
#define  REG_CHIP_TOP_7C_H       (REG_CHIP_TOP_BASE + 0xF9)
#define  REG_CHIP_TOP_7D_L       (REG_CHIP_TOP_BASE + 0xFA)
#define  REG_CHIP_TOP_7D_H       (REG_CHIP_TOP_BASE + 0xFB)
#define  REG_CHIP_TOP_7E_L       (REG_CHIP_TOP_BASE + 0xFC)
#define  REG_CHIP_TOP_7E_H       (REG_CHIP_TOP_BASE + 0xFD)
#define  REG_CHIP_TOP_7F_L       (REG_CHIP_TOP_BASE + 0xFE)
#define  REG_CHIP_TOP_7F_H       (REG_CHIP_TOP_BASE + 0xFF)

#define  REG_TCON_00_L       (REG_TCON_BASE + 0x00)
#define  REG_TCON_00_H       (REG_TCON_BASE + 0x01)
#define  REG_TCON_01_L       (REG_TCON_BASE + 0x02)
#define  REG_TCON_01_H       (REG_TCON_BASE + 0x03)
#define  REG_TCON_02_L       (REG_TCON_BASE + 0x04)
#define  REG_TCON_02_H       (REG_TCON_BASE + 0x05)
#define  REG_TCON_03_L       (REG_TCON_BASE + 0x06)
#define  REG_TCON_03_H       (REG_TCON_BASE + 0x07)
#define  REG_TCON_04_L       (REG_TCON_BASE + 0x08)
#define  REG_TCON_04_H       (REG_TCON_BASE + 0x09)
#define  REG_TCON_05_L       (REG_TCON_BASE + 0x0A)
#define  REG_TCON_05_H       (REG_TCON_BASE + 0x0B)
#define  REG_TCON_06_L       (REG_TCON_BASE + 0x0C)
#define  REG_TCON_06_H       (REG_TCON_BASE + 0x0D)
#define  REG_TCON_07_L       (REG_TCON_BASE + 0x0E)
#define  REG_TCON_07_H       (REG_TCON_BASE + 0x0F)
#define  REG_TCON_08_L       (REG_TCON_BASE + 0x10)
#define  REG_TCON_08_H       (REG_TCON_BASE + 0x11)
#define  REG_TCON_09_L       (REG_TCON_BASE + 0x12)
#define  REG_TCON_09_H       (REG_TCON_BASE + 0x13)
#define  REG_TCON_0A_L       (REG_TCON_BASE + 0x14)
#define  REG_TCON_0A_H       (REG_TCON_BASE + 0x15)
#define  REG_TCON_0B_L       (REG_TCON_BASE + 0x16)
#define  REG_TCON_0B_H       (REG_TCON_BASE + 0x17)
#define  REG_TCON_0C_L       (REG_TCON_BASE + 0x18)
#define  REG_TCON_0C_H       (REG_TCON_BASE + 0x19)
#define  REG_TCON_0D_L       (REG_TCON_BASE + 0x1A)
#define  REG_TCON_0D_H       (REG_TCON_BASE + 0x1B)
#define  REG_TCON_0E_L       (REG_TCON_BASE + 0x1C)
#define  REG_TCON_0E_H       (REG_TCON_BASE + 0x1D)
#define  REG_TCON_0F_L       (REG_TCON_BASE + 0x1E)
#define  REG_TCON_0F_H       (REG_TCON_BASE + 0x1F)
#define  REG_TCON_10_L       (REG_TCON_BASE + 0x20)
#define  REG_TCON_10_H       (REG_TCON_BASE + 0x21)
#define  REG_TCON_11_L       (REG_TCON_BASE + 0x22)
#define  REG_TCON_11_H       (REG_TCON_BASE + 0x23)
#define  REG_TCON_12_L       (REG_TCON_BASE + 0x24)
#define  REG_TCON_12_H       (REG_TCON_BASE + 0x25)
#define  REG_TCON_13_L       (REG_TCON_BASE + 0x26)
#define  REG_TCON_13_H       (REG_TCON_BASE + 0x27)
#define  REG_TCON_14_L       (REG_TCON_BASE + 0x28)
#define  REG_TCON_14_H       (REG_TCON_BASE + 0x29)
#define  REG_TCON_15_L       (REG_TCON_BASE + 0x2A)
#define  REG_TCON_15_H       (REG_TCON_BASE + 0x2B)
#define  REG_TCON_16_L       (REG_TCON_BASE + 0x2C)
#define  REG_TCON_16_H       (REG_TCON_BASE + 0x2D)
#define  REG_TCON_17_L       (REG_TCON_BASE + 0x2E)
#define  REG_TCON_17_H       (REG_TCON_BASE + 0x2F)
#define  REG_TCON_18_L       (REG_TCON_BASE + 0x30)
#define  REG_TCON_18_H       (REG_TCON_BASE + 0x31)
#define  REG_TCON_19_L       (REG_TCON_BASE + 0x32)
#define  REG_TCON_19_H       (REG_TCON_BASE + 0x33)
#define  REG_TCON_1A_L       (REG_TCON_BASE + 0x34)
#define  REG_TCON_1A_H       (REG_TCON_BASE + 0x35)
#define  REG_TCON_1B_L       (REG_TCON_BASE + 0x36)
#define  REG_TCON_1B_H       (REG_TCON_BASE + 0x37)
#define  REG_TCON_1C_L       (REG_TCON_BASE + 0x38)
#define  REG_TCON_1C_H       (REG_TCON_BASE + 0x39)
#define  REG_TCON_1D_L       (REG_TCON_BASE + 0x3A)
#define  REG_TCON_1D_H       (REG_TCON_BASE + 0x3B)
#define  REG_TCON_1E_L       (REG_TCON_BASE + 0x3C)
#define  REG_TCON_1E_H       (REG_TCON_BASE + 0x3D)
#define  REG_TCON_1F_L       (REG_TCON_BASE + 0x3E)
#define  REG_TCON_1F_H       (REG_TCON_BASE + 0x3F)
#define  REG_TCON_20_L       (REG_TCON_BASE + 0x40)
#define  REG_TCON_20_H       (REG_TCON_BASE + 0x41)
#define  REG_TCON_21_L       (REG_TCON_BASE + 0x42)
#define  REG_TCON_21_H       (REG_TCON_BASE + 0x43)
#define  REG_TCON_22_L       (REG_TCON_BASE + 0x44)
#define  REG_TCON_22_H       (REG_TCON_BASE + 0x45)
#define  REG_TCON_23_L       (REG_TCON_BASE + 0x46)
#define  REG_TCON_23_H       (REG_TCON_BASE + 0x47)
#define  REG_TCON_24_L       (REG_TCON_BASE + 0x48)
#define  REG_TCON_24_H       (REG_TCON_BASE + 0x49)
#define  REG_TCON_25_L       (REG_TCON_BASE + 0x4A)
#define  REG_TCON_25_H       (REG_TCON_BASE + 0x4B)
#define  REG_TCON_26_L       (REG_TCON_BASE + 0x4C)
#define  REG_TCON_26_H       (REG_TCON_BASE + 0x4D)
#define  REG_TCON_27_L       (REG_TCON_BASE + 0x4E)
#define  REG_TCON_27_H       (REG_TCON_BASE + 0x4F)
#define  REG_TCON_28_L       (REG_TCON_BASE + 0x50)
#define  REG_TCON_28_H       (REG_TCON_BASE + 0x51)
#define  REG_TCON_29_L       (REG_TCON_BASE + 0x52)
#define  REG_TCON_29_H       (REG_TCON_BASE + 0x53)
#define  REG_TCON_2A_L       (REG_TCON_BASE + 0x54)
#define  REG_TCON_2A_H       (REG_TCON_BASE + 0x55)
#define  REG_TCON_2B_L       (REG_TCON_BASE + 0x56)
#define  REG_TCON_2B_H       (REG_TCON_BASE + 0x57)
#define  REG_TCON_2C_L       (REG_TCON_BASE + 0x58)
#define  REG_TCON_2C_H       (REG_TCON_BASE + 0x59)
#define  REG_TCON_2D_L       (REG_TCON_BASE + 0x5A)
#define  REG_TCON_2D_H       (REG_TCON_BASE + 0x5B)
#define  REG_TCON_2E_L       (REG_TCON_BASE + 0x5C)
#define  REG_TCON_2E_H       (REG_TCON_BASE + 0x5D)
#define  REG_TCON_2F_L       (REG_TCON_BASE + 0x5E)
#define  REG_TCON_2F_H       (REG_TCON_BASE + 0x5F)
#define  REG_TCON_30_L       (REG_TCON_BASE + 0x60)
#define  REG_TCON_30_H       (REG_TCON_BASE + 0x61)
#define  REG_TCON_31_L       (REG_TCON_BASE + 0x62)
#define  REG_TCON_31_H       (REG_TCON_BASE + 0x63)
#define  REG_TCON_32_L       (REG_TCON_BASE + 0x64)
#define  REG_TCON_32_H       (REG_TCON_BASE + 0x65)
#define  REG_TCON_33_L       (REG_TCON_BASE + 0x66)
#define  REG_TCON_33_H       (REG_TCON_BASE + 0x67)
#define  REG_TCON_34_L       (REG_TCON_BASE + 0x68)
#define  REG_TCON_34_H       (REG_TCON_BASE + 0x69)
#define  REG_TCON_35_L       (REG_TCON_BASE + 0x6A)
#define  REG_TCON_35_H       (REG_TCON_BASE + 0x6B)
#define  REG_TCON_36_L       (REG_TCON_BASE + 0x6C)
#define  REG_TCON_36_H       (REG_TCON_BASE + 0x6D)
#define  REG_TCON_37_L       (REG_TCON_BASE + 0x6E)
#define  REG_TCON_37_H       (REG_TCON_BASE + 0x6F)
#define  REG_TCON_38_L       (REG_TCON_BASE + 0x70)
#define  REG_TCON_38_H       (REG_TCON_BASE + 0x71)
#define  REG_TCON_39_L       (REG_TCON_BASE + 0x72)
#define  REG_TCON_39_H       (REG_TCON_BASE + 0x73)
#define  REG_TCON_3A_L       (REG_TCON_BASE + 0x74)
#define  REG_TCON_3A_H       (REG_TCON_BASE + 0x75)
#define  REG_TCON_3B_L       (REG_TCON_BASE + 0x76)
#define  REG_TCON_3B_H       (REG_TCON_BASE + 0x77)
#define  REG_TCON_3C_L       (REG_TCON_BASE + 0x78)
#define  REG_TCON_3C_H       (REG_TCON_BASE + 0x79)
#define  REG_TCON_3D_L       (REG_TCON_BASE + 0x7A)
#define  REG_TCON_3D_H       (REG_TCON_BASE + 0x7B)
#define  REG_TCON_3E_L       (REG_TCON_BASE + 0x7C)
#define  REG_TCON_3E_H       (REG_TCON_BASE + 0x7D)
#define  REG_TCON_3F_L       (REG_TCON_BASE + 0x7E)
#define  REG_TCON_3F_H       (REG_TCON_BASE + 0x7F)
#define  REG_TCON_40_L       (REG_TCON_BASE + 0x80)
#define  REG_TCON_40_H       (REG_TCON_BASE + 0x81)
#define  REG_TCON_41_L       (REG_TCON_BASE + 0x82)
#define  REG_TCON_41_H       (REG_TCON_BASE + 0x83)
#define  REG_TCON_42_L       (REG_TCON_BASE + 0x84)
#define  REG_TCON_42_H       (REG_TCON_BASE + 0x85)
#define  REG_TCON_43_L       (REG_TCON_BASE + 0x86)
#define  REG_TCON_43_H       (REG_TCON_BASE + 0x87)
#define  REG_TCON_44_L       (REG_TCON_BASE + 0x88)
#define  REG_TCON_44_H       (REG_TCON_BASE + 0x89)
#define  REG_TCON_45_L       (REG_TCON_BASE + 0x8A)
#define  REG_TCON_45_H       (REG_TCON_BASE + 0x8B)
#define  REG_TCON_46_L       (REG_TCON_BASE + 0x8C)
#define  REG_TCON_46_H       (REG_TCON_BASE + 0x8D)
#define  REG_TCON_47_L       (REG_TCON_BASE + 0x8E)
#define  REG_TCON_47_H       (REG_TCON_BASE + 0x8F)
#define  REG_TCON_48_L       (REG_TCON_BASE + 0x90)
#define  REG_TCON_48_H       (REG_TCON_BASE + 0x91)
#define  REG_TCON_49_L       (REG_TCON_BASE + 0x92)
#define  REG_TCON_49_H       (REG_TCON_BASE + 0x93)
#define  REG_TCON_4A_L       (REG_TCON_BASE + 0x94)
#define  REG_TCON_4A_H       (REG_TCON_BASE + 0x95)
#define  REG_TCON_4B_L       (REG_TCON_BASE + 0x96)
#define  REG_TCON_4B_H       (REG_TCON_BASE + 0x97)
#define  REG_TCON_4C_L       (REG_TCON_BASE + 0x98)
#define  REG_TCON_4C_H       (REG_TCON_BASE + 0x99)
#define  REG_TCON_4D_L       (REG_TCON_BASE + 0x9A)
#define  REG_TCON_4D_H       (REG_TCON_BASE + 0x9B)
#define  REG_TCON_4E_L       (REG_TCON_BASE + 0x9C)
#define  REG_TCON_4E_H       (REG_TCON_BASE + 0x9D)
#define  REG_TCON_4F_L       (REG_TCON_BASE + 0x9E)
#define  REG_TCON_4F_H       (REG_TCON_BASE + 0x9F)
#define  REG_TCON_50_L       (REG_TCON_BASE + 0xA0)
#define  REG_TCON_50_H       (REG_TCON_BASE + 0xA1)
#define  REG_TCON_51_L       (REG_TCON_BASE + 0xA2)
#define  REG_TCON_51_H       (REG_TCON_BASE + 0xA3)
#define  REG_TCON_52_L       (REG_TCON_BASE + 0xA4)
#define  REG_TCON_52_H       (REG_TCON_BASE + 0xA5)
#define  REG_TCON_53_L       (REG_TCON_BASE + 0xA6)
#define  REG_TCON_53_H       (REG_TCON_BASE + 0xA7)
#define  REG_TCON_54_L       (REG_TCON_BASE + 0xA8)
#define  REG_TCON_54_H       (REG_TCON_BASE + 0xA9)
#define  REG_TCON_55_L       (REG_TCON_BASE + 0xAA)
#define  REG_TCON_55_H       (REG_TCON_BASE + 0xAB)
#define  REG_TCON_56_L       (REG_TCON_BASE + 0xAC)
#define  REG_TCON_56_H       (REG_TCON_BASE + 0xAD)
#define  REG_TCON_57_L       (REG_TCON_BASE + 0xAE)
#define  REG_TCON_57_H       (REG_TCON_BASE + 0xAF)
#define  REG_TCON_58_L       (REG_TCON_BASE + 0xB0)
#define  REG_TCON_58_H       (REG_TCON_BASE + 0xB1)
#define  REG_TCON_59_L       (REG_TCON_BASE + 0xB2)
#define  REG_TCON_59_H       (REG_TCON_BASE + 0xB3)
#define  REG_TCON_5A_L       (REG_TCON_BASE + 0xB4)
#define  REG_TCON_5A_H       (REG_TCON_BASE + 0xB5)
#define  REG_TCON_5B_L       (REG_TCON_BASE + 0xB6)
#define  REG_TCON_5B_H       (REG_TCON_BASE + 0xB7)
#define  REG_TCON_5C_L       (REG_TCON_BASE + 0xB8)
#define  REG_TCON_5C_H       (REG_TCON_BASE + 0xB9)
#define  REG_TCON_5D_L       (REG_TCON_BASE + 0xBA)
#define  REG_TCON_5D_H       (REG_TCON_BASE + 0xBB)
#define  REG_TCON_5E_L       (REG_TCON_BASE + 0xBC)
#define  REG_TCON_5E_H       (REG_TCON_BASE + 0xBD)
#define  REG_TCON_5F_L       (REG_TCON_BASE + 0xBE)
#define  REG_TCON_5F_H       (REG_TCON_BASE + 0xBF)
#define  REG_TCON_60_L       (REG_TCON_BASE + 0xC0)
#define  REG_TCON_60_H       (REG_TCON_BASE + 0xC1)
#define  REG_TCON_61_L       (REG_TCON_BASE + 0xC2)
#define  REG_TCON_61_H       (REG_TCON_BASE + 0xC3)
#define  REG_TCON_62_L       (REG_TCON_BASE + 0xC4)
#define  REG_TCON_62_H       (REG_TCON_BASE + 0xC5)
#define  REG_TCON_63_L       (REG_TCON_BASE + 0xC6)
#define  REG_TCON_63_H       (REG_TCON_BASE + 0xC7)
#define  REG_TCON_64_L       (REG_TCON_BASE + 0xC8)
#define  REG_TCON_64_H       (REG_TCON_BASE + 0xC9)
#define  REG_TCON_65_L       (REG_TCON_BASE + 0xCA)
#define  REG_TCON_65_H       (REG_TCON_BASE + 0xCB)
#define  REG_TCON_66_L       (REG_TCON_BASE + 0xCC)
#define  REG_TCON_66_H       (REG_TCON_BASE + 0xCD)
#define  REG_TCON_67_L       (REG_TCON_BASE + 0xCE)
#define  REG_TCON_67_H       (REG_TCON_BASE + 0xCF)
#define  REG_TCON_68_L       (REG_TCON_BASE + 0xD0)
#define  REG_TCON_68_H       (REG_TCON_BASE + 0xD1)
#define  REG_TCON_69_L       (REG_TCON_BASE + 0xD2)
#define  REG_TCON_69_H       (REG_TCON_BASE + 0xD3)
#define  REG_TCON_6A_L       (REG_TCON_BASE + 0xD4)
#define  REG_TCON_6A_H       (REG_TCON_BASE + 0xD5)
#define  REG_TCON_6B_L       (REG_TCON_BASE + 0xD6)
#define  REG_TCON_6B_H       (REG_TCON_BASE + 0xD7)
#define  REG_TCON_6C_L       (REG_TCON_BASE + 0xD8)
#define  REG_TCON_6C_H       (REG_TCON_BASE + 0xD9)
#define  REG_TCON_6D_L       (REG_TCON_BASE + 0xDA)
#define  REG_TCON_6D_H       (REG_TCON_BASE + 0xDB)
#define  REG_TCON_6E_L       (REG_TCON_BASE + 0xDC)
#define  REG_TCON_6E_H       (REG_TCON_BASE + 0xDD)
#define  REG_TCON_6F_L       (REG_TCON_BASE + 0xDE)
#define  REG_TCON_6F_H       (REG_TCON_BASE + 0xDF)
#define  REG_TCON_70_L       (REG_TCON_BASE + 0xE0)
#define  REG_TCON_70_H       (REG_TCON_BASE + 0xE1)
#define  REG_TCON_71_L       (REG_TCON_BASE + 0xE2)
#define  REG_TCON_71_H       (REG_TCON_BASE + 0xE3)
#define  REG_TCON_72_L       (REG_TCON_BASE + 0xE4)
#define  REG_TCON_72_H       (REG_TCON_BASE + 0xE5)
#define  REG_TCON_73_L       (REG_TCON_BASE + 0xE6)
#define  REG_TCON_73_H       (REG_TCON_BASE + 0xE7)
#define  REG_TCON_74_L       (REG_TCON_BASE + 0xE8)
#define  REG_TCON_74_H       (REG_TCON_BASE + 0xE9)
#define  REG_TCON_75_L       (REG_TCON_BASE + 0xEA)
#define  REG_TCON_75_H       (REG_TCON_BASE + 0xEB)
#define  REG_TCON_76_L       (REG_TCON_BASE + 0xEC)
#define  REG_TCON_76_H       (REG_TCON_BASE + 0xED)
#define  REG_TCON_77_L       (REG_TCON_BASE + 0xEE)
#define  REG_TCON_77_H       (REG_TCON_BASE + 0xEF)
#define  REG_TCON_78_L       (REG_TCON_BASE + 0xF0)
#define  REG_TCON_78_H       (REG_TCON_BASE + 0xF1)
#define  REG_TCON_79_L       (REG_TCON_BASE + 0xF2)
#define  REG_TCON_79_H       (REG_TCON_BASE + 0xF3)
#define  REG_TCON_7A_L       (REG_TCON_BASE + 0xF4)
#define  REG_TCON_7A_H       (REG_TCON_BASE + 0xF5)
#define  REG_TCON_7B_L       (REG_TCON_BASE + 0xF6)
#define  REG_TCON_7B_H       (REG_TCON_BASE + 0xF7)
#define  REG_TCON_7C_L       (REG_TCON_BASE + 0xF8)
#define  REG_TCON_7C_H       (REG_TCON_BASE + 0xF9)
#define  REG_TCON_7D_L       (REG_TCON_BASE + 0xFA)
#define  REG_TCON_7D_H       (REG_TCON_BASE + 0xFB)
#define  REG_TCON_7E_L       (REG_TCON_BASE + 0xFC)
#define  REG_TCON_7E_H       (REG_TCON_BASE + 0xFD)
#define  REG_TCON_7F_L       (REG_TCON_BASE + 0xFE)
#define  REG_TCON_7F_H       (REG_TCON_BASE + 0xFF)

#define  REG_LPLL_00_L       (REG_LPLL_BASE + 0x00)
#define  REG_LPLL_00_H       (REG_LPLL_BASE + 0x01)
#define  REG_LPLL_01_L       (REG_LPLL_BASE + 0x02)
#define  REG_LPLL_01_H       (REG_LPLL_BASE + 0x03)
#define  REG_LPLL_02_L       (REG_LPLL_BASE + 0x04)
#define  REG_LPLL_02_H       (REG_LPLL_BASE + 0x05)
#define  REG_LPLL_03_L       (REG_LPLL_BASE + 0x06)
#define  REG_LPLL_03_H       (REG_LPLL_BASE + 0x07)
#define  REG_LPLL_04_L       (REG_LPLL_BASE + 0x08)
#define  REG_LPLL_04_H       (REG_LPLL_BASE + 0x09)
#define  REG_LPLL_05_L       (REG_LPLL_BASE + 0x0A)
#define  REG_LPLL_05_H       (REG_LPLL_BASE + 0x0B)
#define  REG_LPLL_06_L       (REG_LPLL_BASE + 0x0C)
#define  REG_LPLL_06_H       (REG_LPLL_BASE + 0x0D)
#define  REG_LPLL_07_L       (REG_LPLL_BASE + 0x0E)
#define  REG_LPLL_07_H       (REG_LPLL_BASE + 0x0F)
#define  REG_LPLL_08_L       (REG_LPLL_BASE + 0x10)
#define  REG_LPLL_08_H       (REG_LPLL_BASE + 0x11)
#define  REG_LPLL_09_L       (REG_LPLL_BASE + 0x12)
#define  REG_LPLL_09_H       (REG_LPLL_BASE + 0x13)
#define  REG_LPLL_0A_L       (REG_LPLL_BASE + 0x14)
#define  REG_LPLL_0A_H       (REG_LPLL_BASE + 0x15)
#define  REG_LPLL_0B_L       (REG_LPLL_BASE + 0x16)
#define  REG_LPLL_0B_H       (REG_LPLL_BASE + 0x17)
#define  REG_LPLL_0C_L       (REG_LPLL_BASE + 0x18)
#define  REG_LPLL_0C_H       (REG_LPLL_BASE + 0x19)
#define  REG_LPLL_0D_L       (REG_LPLL_BASE + 0x1A)
#define  REG_LPLL_0D_H       (REG_LPLL_BASE + 0x1B)
#define  REG_LPLL_0E_L       (REG_LPLL_BASE + 0x1C)
#define  REG_LPLL_0E_H       (REG_LPLL_BASE + 0x1D)
#define  REG_LPLL_0F_L       (REG_LPLL_BASE + 0x1E)
#define  REG_LPLL_0F_H       (REG_LPLL_BASE + 0x1F)
#define  REG_LPLL_10_L       (REG_LPLL_BASE + 0x20)
#define  REG_LPLL_10_H       (REG_LPLL_BASE + 0x21)
#define  REG_LPLL_11_L       (REG_LPLL_BASE + 0x22)
#define  REG_LPLL_11_H       (REG_LPLL_BASE + 0x23)
#define  REG_LPLL_12_L       (REG_LPLL_BASE + 0x24)
#define  REG_LPLL_12_H       (REG_LPLL_BASE + 0x25)
#define  REG_LPLL_13_L       (REG_LPLL_BASE + 0x26)
#define  REG_LPLL_13_H       (REG_LPLL_BASE + 0x27)
#define  REG_LPLL_14_L       (REG_LPLL_BASE + 0x28)
#define  REG_LPLL_14_H       (REG_LPLL_BASE + 0x29)
#define  REG_LPLL_15_L       (REG_LPLL_BASE + 0x2A)
#define  REG_LPLL_15_H       (REG_LPLL_BASE + 0x2B)
#define  REG_LPLL_16_L       (REG_LPLL_BASE + 0x2C)
#define  REG_LPLL_16_H       (REG_LPLL_BASE + 0x2D)
#define  REG_LPLL_17_L       (REG_LPLL_BASE + 0x2E)
#define  REG_LPLL_17_H       (REG_LPLL_BASE + 0x2F)
#define  REG_LPLL_18_L       (REG_LPLL_BASE + 0x30)
#define  REG_LPLL_18_H       (REG_LPLL_BASE + 0x31)
#define  REG_LPLL_19_L       (REG_LPLL_BASE + 0x32)
#define  REG_LPLL_19_H       (REG_LPLL_BASE + 0x33)
#define  REG_LPLL_1A_L       (REG_LPLL_BASE + 0x34)
#define  REG_LPLL_1A_H       (REG_LPLL_BASE + 0x35)
#define  REG_LPLL_1B_L       (REG_LPLL_BASE + 0x36)
#define  REG_LPLL_1B_H       (REG_LPLL_BASE + 0x37)
#define  REG_LPLL_1C_L       (REG_LPLL_BASE + 0x38)
#define  REG_LPLL_1C_H       (REG_LPLL_BASE + 0x39)
#define  REG_LPLL_1D_L       (REG_LPLL_BASE + 0x3A)
#define  REG_LPLL_1D_H       (REG_LPLL_BASE + 0x3B)
#define  REG_LPLL_1E_L       (REG_LPLL_BASE + 0x3C)
#define  REG_LPLL_1E_H       (REG_LPLL_BASE + 0x3D)
#define  REG_LPLL_1F_L       (REG_LPLL_BASE + 0x3E)
#define  REG_LPLL_1F_H       (REG_LPLL_BASE + 0x3F)
#define  REG_LPLL_20_L       (REG_LPLL_BASE + 0x40)
#define  REG_LPLL_20_H       (REG_LPLL_BASE + 0x41)
#define  REG_LPLL_21_L       (REG_LPLL_BASE + 0x42)
#define  REG_LPLL_21_H       (REG_LPLL_BASE + 0x43)
#define  REG_LPLL_22_L       (REG_LPLL_BASE + 0x44)
#define  REG_LPLL_22_H       (REG_LPLL_BASE + 0x45)
#define  REG_LPLL_23_L       (REG_LPLL_BASE + 0x46)
#define  REG_LPLL_23_H       (REG_LPLL_BASE + 0x47)
#define  REG_LPLL_24_L       (REG_LPLL_BASE + 0x48)
#define  REG_LPLL_24_H       (REG_LPLL_BASE + 0x49)
#define  REG_LPLL_25_L       (REG_LPLL_BASE + 0x4A)
#define  REG_LPLL_25_H       (REG_LPLL_BASE + 0x4B)
#define  REG_LPLL_26_L       (REG_LPLL_BASE + 0x4C)
#define  REG_LPLL_26_H       (REG_LPLL_BASE + 0x4D)
#define  REG_LPLL_27_L       (REG_LPLL_BASE + 0x4E)
#define  REG_LPLL_27_H       (REG_LPLL_BASE + 0x4F)
#define  REG_LPLL_28_L       (REG_LPLL_BASE + 0x50)
#define  REG_LPLL_28_H       (REG_LPLL_BASE + 0x51)
#define  REG_LPLL_29_L       (REG_LPLL_BASE + 0x52)
#define  REG_LPLL_29_H       (REG_LPLL_BASE + 0x53)
#define  REG_LPLL_2A_L       (REG_LPLL_BASE + 0x54)
#define  REG_LPLL_2A_H       (REG_LPLL_BASE + 0x55)
#define  REG_LPLL_2B_L       (REG_LPLL_BASE + 0x56)
#define  REG_LPLL_2B_H       (REG_LPLL_BASE + 0x57)
#define  REG_LPLL_2C_L       (REG_LPLL_BASE + 0x58)
#define  REG_LPLL_2C_H       (REG_LPLL_BASE + 0x59)
#define  REG_LPLL_2D_L       (REG_LPLL_BASE + 0x5A)
#define  REG_LPLL_2D_H       (REG_LPLL_BASE + 0x5B)
#define  REG_LPLL_2E_L       (REG_LPLL_BASE + 0x5C)
#define  REG_LPLL_2E_H       (REG_LPLL_BASE + 0x5D)
#define  REG_LPLL_2F_L       (REG_LPLL_BASE + 0x5E)
#define  REG_LPLL_2F_H       (REG_LPLL_BASE + 0x5F)
#define  REG_LPLL_30_L       (REG_LPLL_BASE + 0x60)
#define  REG_LPLL_30_H       (REG_LPLL_BASE + 0x61)
#define  REG_LPLL_31_L       (REG_LPLL_BASE + 0x62)
#define  REG_LPLL_31_H       (REG_LPLL_BASE + 0x63)
#define  REG_LPLL_32_L       (REG_LPLL_BASE + 0x64)
#define  REG_LPLL_32_H       (REG_LPLL_BASE + 0x65)
#define  REG_LPLL_33_L       (REG_LPLL_BASE + 0x66)
#define  REG_LPLL_33_H       (REG_LPLL_BASE + 0x67)
#define  REG_LPLL_34_L       (REG_LPLL_BASE + 0x68)
#define  REG_LPLL_34_H       (REG_LPLL_BASE + 0x69)
#define  REG_LPLL_35_L       (REG_LPLL_BASE + 0x6A)
#define  REG_LPLL_35_H       (REG_LPLL_BASE + 0x6B)
#define  REG_LPLL_36_L       (REG_LPLL_BASE + 0x6C)
#define  REG_LPLL_36_H       (REG_LPLL_BASE + 0x6D)
#define  REG_LPLL_37_L       (REG_LPLL_BASE + 0x6E)
#define  REG_LPLL_37_H       (REG_LPLL_BASE + 0x6F)
#define  REG_LPLL_38_L       (REG_LPLL_BASE + 0x70)
#define  REG_LPLL_38_H       (REG_LPLL_BASE + 0x71)
#define  REG_LPLL_39_L       (REG_LPLL_BASE + 0x72)
#define  REG_LPLL_39_H       (REG_LPLL_BASE + 0x73)
#define  REG_LPLL_3A_L       (REG_LPLL_BASE + 0x74)
#define  REG_LPLL_3A_H       (REG_LPLL_BASE + 0x75)
#define  REG_LPLL_3B_L       (REG_LPLL_BASE + 0x76)
#define  REG_LPLL_3B_H       (REG_LPLL_BASE + 0x77)
#define  REG_LPLL_3C_L       (REG_LPLL_BASE + 0x78)
#define  REG_LPLL_3C_H       (REG_LPLL_BASE + 0x79)
#define  REG_LPLL_3D_L       (REG_LPLL_BASE + 0x7A)
#define  REG_LPLL_3D_H       (REG_LPLL_BASE + 0x7B)
#define  REG_LPLL_3E_L       (REG_LPLL_BASE + 0x7C)
#define  REG_LPLL_3E_H       (REG_LPLL_BASE + 0x7D)
#define  REG_LPLL_3F_L       (REG_LPLL_BASE + 0x7E)
#define  REG_LPLL_3F_H       (REG_LPLL_BASE + 0x7F)
#define  REG_LPLL_40_L       (REG_LPLL_BASE + 0x80)
#define  REG_LPLL_40_H       (REG_LPLL_BASE + 0x81)
#define  REG_LPLL_41_L       (REG_LPLL_BASE + 0x82)
#define  REG_LPLL_41_H       (REG_LPLL_BASE + 0x83)
#define  REG_LPLL_42_L       (REG_LPLL_BASE + 0x84)
#define  REG_LPLL_42_H       (REG_LPLL_BASE + 0x85)
#define  REG_LPLL_43_L       (REG_LPLL_BASE + 0x86)
#define  REG_LPLL_43_H       (REG_LPLL_BASE + 0x87)
#define  REG_LPLL_44_L       (REG_LPLL_BASE + 0x88)
#define  REG_LPLL_44_H       (REG_LPLL_BASE + 0x89)
#define  REG_LPLL_45_L       (REG_LPLL_BASE + 0x8A)
#define  REG_LPLL_45_H       (REG_LPLL_BASE + 0x8B)
#define  REG_LPLL_46_L       (REG_LPLL_BASE + 0x8C)
#define  REG_LPLL_46_H       (REG_LPLL_BASE + 0x8D)
#define  REG_LPLL_47_L       (REG_LPLL_BASE + 0x8E)
#define  REG_LPLL_47_H       (REG_LPLL_BASE + 0x8F)
#define  REG_LPLL_48_L       (REG_LPLL_BASE + 0x90)
#define  REG_LPLL_48_H       (REG_LPLL_BASE + 0x91)
#define  REG_LPLL_49_L       (REG_LPLL_BASE + 0x92)
#define  REG_LPLL_49_H       (REG_LPLL_BASE + 0x93)
#define  REG_LPLL_4A_L       (REG_LPLL_BASE + 0x94)
#define  REG_LPLL_4A_H       (REG_LPLL_BASE + 0x95)
#define  REG_LPLL_4B_L       (REG_LPLL_BASE + 0x96)
#define  REG_LPLL_4B_H       (REG_LPLL_BASE + 0x97)
#define  REG_LPLL_4C_L       (REG_LPLL_BASE + 0x98)
#define  REG_LPLL_4C_H       (REG_LPLL_BASE + 0x99)
#define  REG_LPLL_4D_L       (REG_LPLL_BASE + 0x9A)
#define  REG_LPLL_4D_H       (REG_LPLL_BASE + 0x9B)
#define  REG_LPLL_4E_L       (REG_LPLL_BASE + 0x9C)
#define  REG_LPLL_4E_H       (REG_LPLL_BASE + 0x9D)
#define  REG_LPLL_4F_L       (REG_LPLL_BASE + 0x9E)
#define  REG_LPLL_4F_H       (REG_LPLL_BASE + 0x9F)
#define  REG_LPLL_50_L       (REG_LPLL_BASE + 0xA0)
#define  REG_LPLL_50_H       (REG_LPLL_BASE + 0xA1)
#define  REG_LPLL_51_L       (REG_LPLL_BASE + 0xA2)
#define  REG_LPLL_51_H       (REG_LPLL_BASE + 0xA3)
#define  REG_LPLL_52_L       (REG_LPLL_BASE + 0xA4)
#define  REG_LPLL_52_H       (REG_LPLL_BASE + 0xA5)
#define  REG_LPLL_53_L       (REG_LPLL_BASE + 0xA6)
#define  REG_LPLL_53_H       (REG_LPLL_BASE + 0xA7)
#define  REG_LPLL_54_L       (REG_LPLL_BASE + 0xA8)
#define  REG_LPLL_54_H       (REG_LPLL_BASE + 0xA9)
#define  REG_LPLL_55_L       (REG_LPLL_BASE + 0xAA)
#define  REG_LPLL_55_H       (REG_LPLL_BASE + 0xAB)
#define  REG_LPLL_56_L       (REG_LPLL_BASE + 0xAC)
#define  REG_LPLL_56_H       (REG_LPLL_BASE + 0xAD)
#define  REG_LPLL_57_L       (REG_LPLL_BASE + 0xAE)
#define  REG_LPLL_57_H       (REG_LPLL_BASE + 0xAF)
#define  REG_LPLL_58_L       (REG_LPLL_BASE + 0xB0)
#define  REG_LPLL_58_H       (REG_LPLL_BASE + 0xB1)
#define  REG_LPLL_59_L       (REG_LPLL_BASE + 0xB2)
#define  REG_LPLL_59_H       (REG_LPLL_BASE + 0xB3)
#define  REG_LPLL_5A_L       (REG_LPLL_BASE + 0xB4)
#define  REG_LPLL_5A_H       (REG_LPLL_BASE + 0xB5)
#define  REG_LPLL_5B_L       (REG_LPLL_BASE + 0xB6)
#define  REG_LPLL_5B_H       (REG_LPLL_BASE + 0xB7)
#define  REG_LPLL_5C_L       (REG_LPLL_BASE + 0xB8)
#define  REG_LPLL_5C_H       (REG_LPLL_BASE + 0xB9)
#define  REG_LPLL_5D_L       (REG_LPLL_BASE + 0xBA)
#define  REG_LPLL_5D_H       (REG_LPLL_BASE + 0xBB)
#define  REG_LPLL_5E_L       (REG_LPLL_BASE + 0xBC)
#define  REG_LPLL_5E_H       (REG_LPLL_BASE + 0xBD)
#define  REG_LPLL_5F_L       (REG_LPLL_BASE + 0xBE)
#define  REG_LPLL_5F_H       (REG_LPLL_BASE + 0xBF)
#define  REG_LPLL_60_L       (REG_LPLL_BASE + 0xC0)
#define  REG_LPLL_60_H       (REG_LPLL_BASE + 0xC1)
#define  REG_LPLL_61_L       (REG_LPLL_BASE + 0xC2)
#define  REG_LPLL_61_H       (REG_LPLL_BASE + 0xC3)
#define  REG_LPLL_62_L       (REG_LPLL_BASE + 0xC4)
#define  REG_LPLL_62_H       (REG_LPLL_BASE + 0xC5)
#define  REG_LPLL_63_L       (REG_LPLL_BASE + 0xC6)
#define  REG_LPLL_63_H       (REG_LPLL_BASE + 0xC7)
#define  REG_LPLL_64_L       (REG_LPLL_BASE + 0xC8)
#define  REG_LPLL_64_H       (REG_LPLL_BASE + 0xC9)
#define  REG_LPLL_65_L       (REG_LPLL_BASE + 0xCA)
#define  REG_LPLL_65_H       (REG_LPLL_BASE + 0xCB)
#define  REG_LPLL_66_L       (REG_LPLL_BASE + 0xCC)
#define  REG_LPLL_66_H       (REG_LPLL_BASE + 0xCD)
#define  REG_LPLL_67_L       (REG_LPLL_BASE + 0xCE)
#define  REG_LPLL_67_H       (REG_LPLL_BASE + 0xCF)
#define  REG_LPLL_68_L       (REG_LPLL_BASE + 0xD0)
#define  REG_LPLL_68_H       (REG_LPLL_BASE + 0xD1)
#define  REG_LPLL_69_L       (REG_LPLL_BASE + 0xD2)
#define  REG_LPLL_69_H       (REG_LPLL_BASE + 0xD3)
#define  REG_LPLL_6A_L       (REG_LPLL_BASE + 0xD4)
#define  REG_LPLL_6A_H       (REG_LPLL_BASE + 0xD5)
#define  REG_LPLL_6B_L       (REG_LPLL_BASE + 0xD6)
#define  REG_LPLL_6B_H       (REG_LPLL_BASE + 0xD7)
#define  REG_LPLL_6C_L       (REG_LPLL_BASE + 0xD8)
#define  REG_LPLL_6C_H       (REG_LPLL_BASE + 0xD9)
#define  REG_LPLL_6D_L       (REG_LPLL_BASE + 0xDA)
#define  REG_LPLL_6D_H       (REG_LPLL_BASE + 0xDB)
#define  REG_LPLL_6E_L       (REG_LPLL_BASE + 0xDC)
#define  REG_LPLL_6E_H       (REG_LPLL_BASE + 0xDD)
#define  REG_LPLL_6F_L       (REG_LPLL_BASE + 0xDE)
#define  REG_LPLL_6F_H       (REG_LPLL_BASE + 0xDF)
#define  REG_LPLL_70_L       (REG_LPLL_BASE + 0xE0)
#define  REG_LPLL_70_H       (REG_LPLL_BASE + 0xE1)
#define  REG_LPLL_71_L       (REG_LPLL_BASE + 0xE2)
#define  REG_LPLL_71_H       (REG_LPLL_BASE + 0xE3)
#define  REG_LPLL_72_L       (REG_LPLL_BASE + 0xE4)
#define  REG_LPLL_72_H       (REG_LPLL_BASE + 0xE5)
#define  REG_LPLL_73_L       (REG_LPLL_BASE + 0xE6)
#define  REG_LPLL_73_H       (REG_LPLL_BASE + 0xE7)
#define  REG_LPLL_74_L       (REG_LPLL_BASE + 0xE8)
#define  REG_LPLL_74_H       (REG_LPLL_BASE + 0xE9)
#define  REG_LPLL_75_L       (REG_LPLL_BASE + 0xEA)
#define  REG_LPLL_75_H       (REG_LPLL_BASE + 0xEB)
#define  REG_LPLL_76_L       (REG_LPLL_BASE + 0xEC)
#define  REG_LPLL_76_H       (REG_LPLL_BASE + 0xED)
#define  REG_LPLL_77_L       (REG_LPLL_BASE + 0xEE)
#define  REG_LPLL_77_H       (REG_LPLL_BASE + 0xEF)
#define  REG_LPLL_78_L       (REG_LPLL_BASE + 0xF0)
#define  REG_LPLL_78_H       (REG_LPLL_BASE + 0xF1)
#define  REG_LPLL_79_L       (REG_LPLL_BASE + 0xF2)
#define  REG_LPLL_79_H       (REG_LPLL_BASE + 0xF3)
#define  REG_LPLL_7A_L       (REG_LPLL_BASE + 0xF4)
#define  REG_LPLL_7A_H       (REG_LPLL_BASE + 0xF5)
#define  REG_LPLL_7B_L       (REG_LPLL_BASE + 0xF6)
#define  REG_LPLL_7B_H       (REG_LPLL_BASE + 0xF7)
#define  REG_LPLL_7C_L       (REG_LPLL_BASE + 0xF8)
#define  REG_LPLL_7C_H       (REG_LPLL_BASE + 0xF9)
#define  REG_LPLL_7D_L       (REG_LPLL_BASE + 0xFA)
#define  REG_LPLL_7D_H       (REG_LPLL_BASE + 0xFB)
#define  REG_LPLL_7E_L       (REG_LPLL_BASE + 0xFC)
#define  REG_LPLL_7E_H       (REG_LPLL_BASE + 0xFD)
#define  REG_LPLL_7F_L       (REG_LPLL_BASE + 0xFE)
#define  REG_LPLL_7F_H       (REG_LPLL_BASE + 0xFF)

#define  REG_SC_BK10_00_L       _PK_L_(0x10, 0x00)
#define  REG_SC_BK10_00_H       _PK_H_(0x10, 0x00)
#define  REG_SC_BK10_01_L       _PK_L_(0x10, 0x01)
#define  REG_SC_BK10_01_H       _PK_H_(0x10, 0x01)
#define  REG_SC_BK10_02_L       _PK_L_(0x10, 0x02)
#define  REG_SC_BK10_02_H       _PK_H_(0x10, 0x02)
#define  REG_SC_BK10_03_L       _PK_L_(0x10, 0x03)
#define  REG_SC_BK10_03_H       _PK_H_(0x10, 0x03)
#define  REG_SC_BK10_04_L       _PK_L_(0x10, 0x04)
#define  REG_SC_BK10_04_H       _PK_H_(0x10, 0x04)
#define  REG_SC_BK10_05_L       _PK_L_(0x10, 0x05)
#define  REG_SC_BK10_05_H       _PK_H_(0x10, 0x05)
#define  REG_SC_BK10_06_L       _PK_L_(0x10, 0x06)
#define  REG_SC_BK10_06_H       _PK_H_(0x10, 0x06)
#define  REG_SC_BK10_07_L       _PK_L_(0x10, 0x07)
#define  REG_SC_BK10_07_H       _PK_H_(0x10, 0x07)
#define  REG_SC_BK10_08_L       _PK_L_(0x10, 0x08)
#define  REG_SC_BK10_08_H       _PK_H_(0x10, 0x08)
#define  REG_SC_BK10_09_L       _PK_L_(0x10, 0x09)
#define  REG_SC_BK10_09_H       _PK_H_(0x10, 0x09)
#define  REG_SC_BK10_0A_L       _PK_L_(0x10, 0x0A)
#define  REG_SC_BK10_0A_H       _PK_H_(0x10, 0x0A)
#define  REG_SC_BK10_0B_L       _PK_L_(0x10, 0x0B)
#define  REG_SC_BK10_0B_H       _PK_H_(0x10, 0x0B)
#define  REG_SC_BK10_0C_L       _PK_L_(0x10, 0x0C)
#define  REG_SC_BK10_0C_H       _PK_H_(0x10, 0x0C)
#define  REG_SC_BK10_0D_L       _PK_L_(0x10, 0x0D)
#define  REG_SC_BK10_0D_H       _PK_H_(0x10, 0x0D)
#define  REG_SC_BK10_0E_L       _PK_L_(0x10, 0x0E)
#define  REG_SC_BK10_0E_H       _PK_H_(0x10, 0x0E)
#define  REG_SC_BK10_0F_L       _PK_L_(0x10, 0x0F)
#define  REG_SC_BK10_0F_H       _PK_H_(0x10, 0x0F)
#define  REG_SC_BK10_10_L       _PK_L_(0x10, 0x10)
#define  REG_SC_BK10_10_H       _PK_H_(0x10, 0x10)
#define  REG_SC_BK10_11_L       _PK_L_(0x10, 0x11)
#define  REG_SC_BK10_11_H       _PK_H_(0x10, 0x11)
#define  REG_SC_BK10_12_L       _PK_L_(0x10, 0x12)
#define  REG_SC_BK10_12_H       _PK_H_(0x10, 0x12)
#define  REG_SC_BK10_13_L       _PK_L_(0x10, 0x13)
#define  REG_SC_BK10_13_H       _PK_H_(0x10, 0x13)
#define  REG_SC_BK10_14_L       _PK_L_(0x10, 0x14)
#define  REG_SC_BK10_14_H       _PK_H_(0x10, 0x14)
#define  REG_SC_BK10_15_L       _PK_L_(0x10, 0x15)
#define  REG_SC_BK10_15_H       _PK_H_(0x10, 0x15)
#define  REG_SC_BK10_16_L       _PK_L_(0x10, 0x16)
#define  REG_SC_BK10_16_H       _PK_H_(0x10, 0x16)
#define  REG_SC_BK10_17_L       _PK_L_(0x10, 0x17)
#define  REG_SC_BK10_17_H       _PK_H_(0x10, 0x17)
#define  REG_SC_BK10_18_L       _PK_L_(0x10, 0x18)
#define  REG_SC_BK10_18_H       _PK_H_(0x10, 0x18)
#define  REG_SC_BK10_19_L       _PK_L_(0x10, 0x19)
#define  REG_SC_BK10_19_H       _PK_H_(0x10, 0x19)
#define  REG_SC_BK10_1A_L       _PK_L_(0x10, 0x1A)
#define  REG_SC_BK10_1A_H       _PK_H_(0x10, 0x1A)
#define  REG_SC_BK10_1B_L       _PK_L_(0x10, 0x1B)
#define  REG_SC_BK10_1B_H       _PK_H_(0x10, 0x1B)
#define  REG_SC_BK10_1C_L       _PK_L_(0x10, 0x1C)
#define  REG_SC_BK10_1C_H       _PK_H_(0x10, 0x1C)
#define  REG_SC_BK10_1D_L       _PK_L_(0x10, 0x1D)
#define  REG_SC_BK10_1D_H       _PK_H_(0x10, 0x1D)
#define  REG_SC_BK10_1E_L       _PK_L_(0x10, 0x1E)
#define  REG_SC_BK10_1E_H       _PK_H_(0x10, 0x1E)
#define  REG_SC_BK10_1F_L       _PK_L_(0x10, 0x1F)
#define  REG_SC_BK10_1F_H       _PK_H_(0x10, 0x1F)
#define  REG_SC_BK10_20_L       _PK_L_(0x10, 0x20)
#define  REG_SC_BK10_20_H       _PK_H_(0x10, 0x20)
#define  REG_SC_BK10_21_L       _PK_L_(0x10, 0x21)
#define  REG_SC_BK10_21_H       _PK_H_(0x10, 0x21)
#define  REG_SC_BK10_22_L       _PK_L_(0x10, 0x22)
#define  REG_SC_BK10_22_H       _PK_H_(0x10, 0x22)
#define  REG_SC_BK10_23_L       _PK_L_(0x10, 0x23)
#define  REG_SC_BK10_23_H       _PK_H_(0x10, 0x23)
#define  REG_SC_BK10_24_L       _PK_L_(0x10, 0x24)
#define  REG_SC_BK10_24_H       _PK_H_(0x10, 0x24)
#define  REG_SC_BK10_25_L       _PK_L_(0x10, 0x25)
#define  REG_SC_BK10_25_H       _PK_H_(0x10, 0x25)
#define  REG_SC_BK10_26_L       _PK_L_(0x10, 0x26)
#define  REG_SC_BK10_26_H       _PK_H_(0x10, 0x26)
#define  REG_SC_BK10_27_L       _PK_L_(0x10, 0x27)
#define  REG_SC_BK10_27_H       _PK_H_(0x10, 0x27)
#define  REG_SC_BK10_28_L       _PK_L_(0x10, 0x28)
#define  REG_SC_BK10_28_H       _PK_H_(0x10, 0x28)
#define  REG_SC_BK10_29_L       _PK_L_(0x10, 0x29)
#define  REG_SC_BK10_29_H       _PK_H_(0x10, 0x29)
#define  REG_SC_BK10_2A_L       _PK_L_(0x10, 0x2A)
#define  REG_SC_BK10_2A_H       _PK_H_(0x10, 0x2A)
#define  REG_SC_BK10_2B_L       _PK_L_(0x10, 0x2B)
#define  REG_SC_BK10_2B_H       _PK_H_(0x10, 0x2B)
#define  REG_SC_BK10_2C_L       _PK_L_(0x10, 0x2C)
#define  REG_SC_BK10_2C_H       _PK_H_(0x10, 0x2C)
#define  REG_SC_BK10_2D_L       _PK_L_(0x10, 0x2D)
#define  REG_SC_BK10_2D_H       _PK_H_(0x10, 0x2D)
#define  REG_SC_BK10_2E_L       _PK_L_(0x10, 0x2E)
#define  REG_SC_BK10_2E_H       _PK_H_(0x10, 0x2E)
#define  REG_SC_BK10_2F_L       _PK_L_(0x10, 0x2F)
#define  REG_SC_BK10_2F_H       _PK_H_(0x10, 0x2F)
#define  REG_SC_BK10_30_L       _PK_L_(0x10, 0x30)
#define  REG_SC_BK10_30_H       _PK_H_(0x10, 0x30)
#define  REG_SC_BK10_31_L       _PK_L_(0x10, 0x31)
#define  REG_SC_BK10_31_H       _PK_H_(0x10, 0x31)
#define  REG_SC_BK10_32_L       _PK_L_(0x10, 0x32)
#define  REG_SC_BK10_32_H       _PK_H_(0x10, 0x32)
#define  REG_SC_BK10_33_L       _PK_L_(0x10, 0x33)
#define  REG_SC_BK10_33_H       _PK_H_(0x10, 0x33)
#define  REG_SC_BK10_34_L       _PK_L_(0x10, 0x34)
#define  REG_SC_BK10_34_H       _PK_H_(0x10, 0x34)
#define  REG_SC_BK10_35_L       _PK_L_(0x10, 0x35)
#define  REG_SC_BK10_35_H       _PK_H_(0x10, 0x35)
#define  REG_SC_BK10_36_L       _PK_L_(0x10, 0x36)
#define  REG_SC_BK10_36_H       _PK_H_(0x10, 0x36)
#define  REG_SC_BK10_37_L       _PK_L_(0x10, 0x37)
#define  REG_SC_BK10_37_H       _PK_H_(0x10, 0x37)
#define  REG_SC_BK10_38_L       _PK_L_(0x10, 0x38)
#define  REG_SC_BK10_38_H       _PK_H_(0x10, 0x38)
#define  REG_SC_BK10_39_L       _PK_L_(0x10, 0x39)
#define  REG_SC_BK10_39_H       _PK_H_(0x10, 0x39)
#define  REG_SC_BK10_3A_L       _PK_L_(0x10, 0x3A)
#define  REG_SC_BK10_3A_H       _PK_H_(0x10, 0x3A)
#define  REG_SC_BK10_3B_L       _PK_L_(0x10, 0x3B)
#define  REG_SC_BK10_3B_H       _PK_H_(0x10, 0x3B)
#define  REG_SC_BK10_3C_L       _PK_L_(0x10, 0x3C)
#define  REG_SC_BK10_3C_H       _PK_H_(0x10, 0x3C)
#define  REG_SC_BK10_3D_L       _PK_L_(0x10, 0x3D)
#define  REG_SC_BK10_3D_H       _PK_H_(0x10, 0x3D)
#define  REG_SC_BK10_3E_L       _PK_L_(0x10, 0x3E)
#define  REG_SC_BK10_3E_H       _PK_H_(0x10, 0x3E)
#define  REG_SC_BK10_3F_L       _PK_L_(0x10, 0x3F)
#define  REG_SC_BK10_3F_H       _PK_H_(0x10, 0x3F)
#define  REG_SC_BK10_40_L       _PK_L_(0x10, 0x40)
#define  REG_SC_BK10_40_H       _PK_H_(0x10, 0x40)
#define  REG_SC_BK10_41_L       _PK_L_(0x10, 0x41)
#define  REG_SC_BK10_41_H       _PK_H_(0x10, 0x41)
#define  REG_SC_BK10_42_L       _PK_L_(0x10, 0x42)
#define  REG_SC_BK10_42_H       _PK_H_(0x10, 0x42)
#define  REG_SC_BK10_43_L       _PK_L_(0x10, 0x43)
#define  REG_SC_BK10_43_H       _PK_H_(0x10, 0x43)
#define  REG_SC_BK10_44_L       _PK_L_(0x10, 0x44)
#define  REG_SC_BK10_44_H       _PK_H_(0x10, 0x44)
#define  REG_SC_BK10_45_L       _PK_L_(0x10, 0x45)
#define  REG_SC_BK10_45_H       _PK_H_(0x10, 0x45)
#define  REG_SC_BK10_46_L       _PK_L_(0x10, 0x46)
#define  REG_SC_BK10_46_H       _PK_H_(0x10, 0x46)
#define  REG_SC_BK10_47_L       _PK_L_(0x10, 0x47)
#define  REG_SC_BK10_47_H       _PK_H_(0x10, 0x47)
#define  REG_SC_BK10_48_L       _PK_L_(0x10, 0x48)
#define  REG_SC_BK10_48_H       _PK_H_(0x10, 0x48)
#define  REG_SC_BK10_49_L       _PK_L_(0x10, 0x49)
#define  REG_SC_BK10_49_H       _PK_H_(0x10, 0x49)
#define  REG_SC_BK10_4A_L       _PK_L_(0x10, 0x4A)
#define  REG_SC_BK10_4A_H       _PK_H_(0x10, 0x4A)
#define  REG_SC_BK10_4B_L       _PK_L_(0x10, 0x4B)
#define  REG_SC_BK10_4B_H       _PK_H_(0x10, 0x4B)
#define  REG_SC_BK10_4C_L       _PK_L_(0x10, 0x4C)
#define  REG_SC_BK10_4C_H       _PK_H_(0x10, 0x4C)
#define  REG_SC_BK10_4D_L       _PK_L_(0x10, 0x4D)
#define  REG_SC_BK10_4D_H       _PK_H_(0x10, 0x4D)
#define  REG_SC_BK10_4E_L       _PK_L_(0x10, 0x4E)
#define  REG_SC_BK10_4E_H       _PK_H_(0x10, 0x4E)
#define  REG_SC_BK10_4F_L       _PK_L_(0x10, 0x4F)
#define  REG_SC_BK10_4F_H       _PK_H_(0x10, 0x4F)
#define  REG_SC_BK10_50_L       _PK_L_(0x10, 0x50)
#define  REG_SC_BK10_50_H       _PK_H_(0x10, 0x50)
#define  REG_SC_BK10_51_L       _PK_L_(0x10, 0x51)
#define  REG_SC_BK10_51_H       _PK_H_(0x10, 0x51)
#define  REG_SC_BK10_52_L       _PK_L_(0x10, 0x52)
#define  REG_SC_BK10_52_H       _PK_H_(0x10, 0x52)
#define  REG_SC_BK10_53_L       _PK_L_(0x10, 0x53)
#define  REG_SC_BK10_53_H       _PK_H_(0x10, 0x53)
#define  REG_SC_BK10_54_L       _PK_L_(0x10, 0x54)
#define  REG_SC_BK10_54_H       _PK_H_(0x10, 0x54)
#define  REG_SC_BK10_55_L       _PK_L_(0x10, 0x55)
#define  REG_SC_BK10_55_H       _PK_H_(0x10, 0x55)
#define  REG_SC_BK10_56_L       _PK_L_(0x10, 0x56)
#define  REG_SC_BK10_56_H       _PK_H_(0x10, 0x56)
#define  REG_SC_BK10_57_L       _PK_L_(0x10, 0x57)
#define  REG_SC_BK10_57_H       _PK_H_(0x10, 0x57)
#define  REG_SC_BK10_58_L       _PK_L_(0x10, 0x58)
#define  REG_SC_BK10_58_H       _PK_H_(0x10, 0x58)
#define  REG_SC_BK10_59_L       _PK_L_(0x10, 0x59)
#define  REG_SC_BK10_59_H       _PK_H_(0x10, 0x59)
#define  REG_SC_BK10_5A_L       _PK_L_(0x10, 0x5A)
#define  REG_SC_BK10_5A_H       _PK_H_(0x10, 0x5A)
#define  REG_SC_BK10_5B_L       _PK_L_(0x10, 0x5B)
#define  REG_SC_BK10_5B_H       _PK_H_(0x10, 0x5B)
#define  REG_SC_BK10_5C_L       _PK_L_(0x10, 0x5C)
#define  REG_SC_BK10_5C_H       _PK_H_(0x10, 0x5C)
#define  REG_SC_BK10_5D_L       _PK_L_(0x10, 0x5D)
#define  REG_SC_BK10_5D_H       _PK_H_(0x10, 0x5D)
#define  REG_SC_BK10_5E_L       _PK_L_(0x10, 0x5E)
#define  REG_SC_BK10_5E_H       _PK_H_(0x10, 0x5E)
#define  REG_SC_BK10_5F_L       _PK_L_(0x10, 0x5F)
#define  REG_SC_BK10_5F_H       _PK_H_(0x10, 0x5F)
#define  REG_SC_BK10_60_L       _PK_L_(0x10, 0x60)
#define  REG_SC_BK10_60_H       _PK_H_(0x10, 0x60)
#define  REG_SC_BK10_61_L       _PK_L_(0x10, 0x61)
#define  REG_SC_BK10_61_H       _PK_H_(0x10, 0x61)
#define  REG_SC_BK10_62_L       _PK_L_(0x10, 0x62)
#define  REG_SC_BK10_62_H       _PK_H_(0x10, 0x62)
#define  REG_SC_BK10_63_L       _PK_L_(0x10, 0x63)
#define  REG_SC_BK10_63_H       _PK_H_(0x10, 0x63)
#define  REG_SC_BK10_64_L       _PK_L_(0x10, 0x64)
#define  REG_SC_BK10_64_H       _PK_H_(0x10, 0x64)
#define  REG_SC_BK10_65_L       _PK_L_(0x10, 0x65)
#define  REG_SC_BK10_65_H       _PK_H_(0x10, 0x65)
#define  REG_SC_BK10_66_L       _PK_L_(0x10, 0x66)
#define  REG_SC_BK10_66_H       _PK_H_(0x10, 0x66)
#define  REG_SC_BK10_67_L       _PK_L_(0x10, 0x67)
#define  REG_SC_BK10_67_H       _PK_H_(0x10, 0x67)
#define  REG_SC_BK10_68_L       _PK_L_(0x10, 0x68)
#define  REG_SC_BK10_68_H       _PK_H_(0x10, 0x68)
#define  REG_SC_BK10_69_L       _PK_L_(0x10, 0x69)
#define  REG_SC_BK10_69_H       _PK_H_(0x10, 0x69)
#define  REG_SC_BK10_6A_L       _PK_L_(0x10, 0x6A)
#define  REG_SC_BK10_6A_H       _PK_H_(0x10, 0x6A)
#define  REG_SC_BK10_6B_L       _PK_L_(0x10, 0x6B)
#define  REG_SC_BK10_6B_H       _PK_H_(0x10, 0x6B)
#define  REG_SC_BK10_6C_L       _PK_L_(0x10, 0x6C)
#define  REG_SC_BK10_6C_H       _PK_H_(0x10, 0x6C)
#define  REG_SC_BK10_6D_L       _PK_L_(0x10, 0x6D)
#define  REG_SC_BK10_6D_H       _PK_H_(0x10, 0x6D)
#define  REG_SC_BK10_6E_L       _PK_L_(0x10, 0x6E)
#define  REG_SC_BK10_6E_H       _PK_H_(0x10, 0x6E)
#define  REG_SC_BK10_6F_L       _PK_L_(0x10, 0x6F)
#define  REG_SC_BK10_6F_H       _PK_H_(0x10, 0x6F)
#define  REG_SC_BK10_70_L       _PK_L_(0x10, 0x70)
#define  REG_SC_BK10_70_H       _PK_H_(0x10, 0x70)
#define  REG_SC_BK10_71_L       _PK_L_(0x10, 0x71)
#define  REG_SC_BK10_71_H       _PK_H_(0x10, 0x71)
#define  REG_SC_BK10_72_L       _PK_L_(0x10, 0x72)
#define  REG_SC_BK10_72_H       _PK_H_(0x10, 0x72)
#define  REG_SC_BK10_73_L       _PK_L_(0x10, 0x73)
#define  REG_SC_BK10_73_H       _PK_H_(0x10, 0x73)
#define  REG_SC_BK10_74_L       _PK_L_(0x10, 0x74)
#define  REG_SC_BK10_74_H       _PK_H_(0x10, 0x74)
#define  REG_SC_BK10_75_L       _PK_L_(0x10, 0x75)
#define  REG_SC_BK10_75_H       _PK_H_(0x10, 0x75)
#define  REG_SC_BK10_76_L       _PK_L_(0x10, 0x76)
#define  REG_SC_BK10_76_H       _PK_H_(0x10, 0x76)
#define  REG_SC_BK10_77_L       _PK_L_(0x10, 0x77)
#define  REG_SC_BK10_77_H       _PK_H_(0x10, 0x77)
#define  REG_SC_BK10_78_L       _PK_L_(0x10, 0x78)
#define  REG_SC_BK10_78_H       _PK_H_(0x10, 0x78)
#define  REG_SC_BK10_79_L       _PK_L_(0x10, 0x79)
#define  REG_SC_BK10_79_H       _PK_H_(0x10, 0x79)
#define  REG_SC_BK10_7A_L       _PK_L_(0x10, 0x7A)
#define  REG_SC_BK10_7A_H       _PK_H_(0x10, 0x7A)
#define  REG_SC_BK10_7B_L       _PK_L_(0x10, 0x7B)
#define  REG_SC_BK10_7B_H       _PK_H_(0x10, 0x7B)
#define  REG_SC_BK10_7C_L       _PK_L_(0x10, 0x7C)
#define  REG_SC_BK10_7C_H       _PK_H_(0x10, 0x7C)
#define  REG_SC_BK10_7D_L       _PK_L_(0x10, 0x7D)
#define  REG_SC_BK10_7D_H       _PK_H_(0x10, 0x7D)
#define  REG_SC_BK10_7E_L       _PK_L_(0x10, 0x7E)
#define  REG_SC_BK10_7E_H       _PK_H_(0x10, 0x7E)
#define  REG_SC_BK10_7F_L       _PK_L_(0x10, 0x7F)
#define  REG_SC_BK10_7F_H       _PK_H_(0x10, 0x7F)

#define  REG_MOD_BK00_00_L       _PK_L_(0x00, 0x00)
#define  REG_MOD_BK00_00_H       _PK_H_(0x00, 0x00)
#define  REG_MOD_BK00_01_L       _PK_L_(0x00, 0x01)
#define  REG_MOD_BK00_01_H       _PK_H_(0x00, 0x01)
#define  REG_MOD_BK00_02_L       _PK_L_(0x00, 0x02)
#define  REG_MOD_BK00_02_H       _PK_H_(0x00, 0x02)
#define  REG_MOD_BK00_03_L       _PK_L_(0x00, 0x03)
#define  REG_MOD_BK00_03_H       _PK_H_(0x00, 0x03)
#define  REG_MOD_BK00_04_L       _PK_L_(0x00, 0x04)
#define  REG_MOD_BK00_04_H       _PK_H_(0x00, 0x04)
#define  REG_MOD_BK00_05_L       _PK_L_(0x00, 0x05)
#define  REG_MOD_BK00_05_H       _PK_H_(0x00, 0x05)
#define  REG_MOD_BK00_06_L       _PK_L_(0x00, 0x06)
#define  REG_MOD_BK00_06_H       _PK_H_(0x00, 0x06)
#define  REG_MOD_BK00_07_L       _PK_L_(0x00, 0x07)
#define  REG_MOD_BK00_07_H       _PK_H_(0x00, 0x07)
#define  REG_MOD_BK00_08_L       _PK_L_(0x00, 0x08)
#define  REG_MOD_BK00_08_H       _PK_H_(0x00, 0x08)
#define  REG_MOD_BK00_09_L       _PK_L_(0x00, 0x09)
#define  REG_MOD_BK00_09_H       _PK_H_(0x00, 0x09)
#define  REG_MOD_BK00_0A_L       _PK_L_(0x00, 0x0A)
#define  REG_MOD_BK00_0A_H       _PK_H_(0x00, 0x0A)
#define  REG_MOD_BK00_0B_L       _PK_L_(0x00, 0x0B)
#define  REG_MOD_BK00_0B_H       _PK_H_(0x00, 0x0B)
#define  REG_MOD_BK00_0C_L       _PK_L_(0x00, 0x0C)
#define  REG_MOD_BK00_0C_H       _PK_H_(0x00, 0x0C)
#define  REG_MOD_BK00_0D_L       _PK_L_(0x00, 0x0D)
#define  REG_MOD_BK00_0D_H       _PK_H_(0x00, 0x0D)
#define  REG_MOD_BK00_0E_L       _PK_L_(0x00, 0x0E)
#define  REG_MOD_BK00_0E_H       _PK_H_(0x00, 0x0E)
#define  REG_MOD_BK00_0F_L       _PK_L_(0x00, 0x0F)
#define  REG_MOD_BK00_0F_H       _PK_H_(0x00, 0x0F)
#define  REG_MOD_BK00_10_L       _PK_L_(0x00, 0x10)
#define  REG_MOD_BK00_10_H       _PK_H_(0x00, 0x10)
#define  REG_MOD_BK00_11_L       _PK_L_(0x00, 0x11)
#define  REG_MOD_BK00_11_H       _PK_H_(0x00, 0x11)
#define  REG_MOD_BK00_12_L       _PK_L_(0x00, 0x12)
#define  REG_MOD_BK00_12_H       _PK_H_(0x00, 0x12)
#define  REG_MOD_BK00_13_L       _PK_L_(0x00, 0x13)
#define  REG_MOD_BK00_13_H       _PK_H_(0x00, 0x13)
#define  REG_MOD_BK00_14_L       _PK_L_(0x00, 0x14)
#define  REG_MOD_BK00_14_H       _PK_H_(0x00, 0x14)
#define  REG_MOD_BK00_15_L       _PK_L_(0x00, 0x15)
#define  REG_MOD_BK00_15_H       _PK_H_(0x00, 0x15)
#define  REG_MOD_BK00_16_L       _PK_L_(0x00, 0x16)
#define  REG_MOD_BK00_16_H       _PK_H_(0x00, 0x16)
#define  REG_MOD_BK00_17_L       _PK_L_(0x00, 0x17)
#define  REG_MOD_BK00_17_H       _PK_H_(0x00, 0x17)
#define  REG_MOD_BK00_18_L       _PK_L_(0x00, 0x18)
#define  REG_MOD_BK00_18_H       _PK_H_(0x00, 0x18)
#define  REG_MOD_BK00_19_L       _PK_L_(0x00, 0x19)
#define  REG_MOD_BK00_19_H       _PK_H_(0x00, 0x19)
#define  REG_MOD_BK00_1A_L       _PK_L_(0x00, 0x1A)
#define  REG_MOD_BK00_1A_H       _PK_H_(0x00, 0x1A)
#define  REG_MOD_BK00_1B_L       _PK_L_(0x00, 0x1B)
#define  REG_MOD_BK00_1B_H       _PK_H_(0x00, 0x1B)
#define  REG_MOD_BK00_1C_L       _PK_L_(0x00, 0x1C)
#define  REG_MOD_BK00_1C_H       _PK_H_(0x00, 0x1C)
#define  REG_MOD_BK00_1D_L       _PK_L_(0x00, 0x1D)
#define  REG_MOD_BK00_1D_H       _PK_H_(0x00, 0x1D)
#define  REG_MOD_BK00_1E_L       _PK_L_(0x00, 0x1E)
#define  REG_MOD_BK00_1E_H       _PK_H_(0x00, 0x1E)
#define  REG_MOD_BK00_1F_L       _PK_L_(0x00, 0x1F)
#define  REG_MOD_BK00_1F_H       _PK_H_(0x00, 0x1F)
#define  REG_MOD_BK00_20_L       _PK_L_(0x00, 0x20)
#define  REG_MOD_BK00_20_H       _PK_H_(0x00, 0x20)
#define  REG_MOD_BK00_21_L       _PK_L_(0x00, 0x21)
#define  REG_MOD_BK00_21_H       _PK_H_(0x00, 0x21)
#define  REG_MOD_BK00_22_L       _PK_L_(0x00, 0x22)
#define  REG_MOD_BK00_22_H       _PK_H_(0x00, 0x22)
#define  REG_MOD_BK00_23_L       _PK_L_(0x00, 0x23)
#define  REG_MOD_BK00_23_H       _PK_H_(0x00, 0x23)
#define  REG_MOD_BK00_24_L       _PK_L_(0x00, 0x24)
#define  REG_MOD_BK00_24_H       _PK_H_(0x00, 0x24)
#define  REG_MOD_BK00_25_L       _PK_L_(0x00, 0x25)
#define  REG_MOD_BK00_25_H       _PK_H_(0x00, 0x25)
#define  REG_MOD_BK00_26_L       _PK_L_(0x00, 0x26)
#define  REG_MOD_BK00_26_H       _PK_H_(0x00, 0x26)
#define  REG_MOD_BK00_27_L       _PK_L_(0x00, 0x27)
#define  REG_MOD_BK00_27_H       _PK_H_(0x00, 0x27)
#define  REG_MOD_BK00_28_L       _PK_L_(0x00, 0x28)
#define  REG_MOD_BK00_28_H       _PK_H_(0x00, 0x28)
#define  REG_MOD_BK00_29_L       _PK_L_(0x00, 0x29)
#define  REG_MOD_BK00_29_H       _PK_H_(0x00, 0x29)
#define  REG_MOD_BK00_2A_L       _PK_L_(0x00, 0x2A)
#define  REG_MOD_BK00_2A_H       _PK_H_(0x00, 0x2A)
#define  REG_MOD_BK00_2B_L       _PK_L_(0x00, 0x2B)
#define  REG_MOD_BK00_2B_H       _PK_H_(0x00, 0x2B)
#define  REG_MOD_BK00_2C_L       _PK_L_(0x00, 0x2C)
#define  REG_MOD_BK00_2C_H       _PK_H_(0x00, 0x2C)
#define  REG_MOD_BK00_2D_L       _PK_L_(0x00, 0x2D)
#define  REG_MOD_BK00_2D_H       _PK_H_(0x00, 0x2D)
#define  REG_MOD_BK00_2E_L       _PK_L_(0x00, 0x2E)
#define  REG_MOD_BK00_2E_H       _PK_H_(0x00, 0x2E)
#define  REG_MOD_BK00_2F_L       _PK_L_(0x00, 0x2F)
#define  REG_MOD_BK00_2F_H       _PK_H_(0x00, 0x2F)
#define  REG_MOD_BK00_30_L       _PK_L_(0x00, 0x30)
#define  REG_MOD_BK00_30_H       _PK_H_(0x00, 0x30)
#define  REG_MOD_BK00_31_L       _PK_L_(0x00, 0x31)
#define  REG_MOD_BK00_31_H       _PK_H_(0x00, 0x31)
#define  REG_MOD_BK00_32_L       _PK_L_(0x00, 0x32)
#define  REG_MOD_BK00_32_H       _PK_H_(0x00, 0x32)
#define  REG_MOD_BK00_33_L       _PK_L_(0x00, 0x33)
#define  REG_MOD_BK00_33_H       _PK_H_(0x00, 0x33)
#define  REG_MOD_BK00_34_L       _PK_L_(0x00, 0x34)
#define  REG_MOD_BK00_34_H       _PK_H_(0x00, 0x34)
#define  REG_MOD_BK00_35_L       _PK_L_(0x00, 0x35)
#define  REG_MOD_BK00_35_H       _PK_H_(0x00, 0x35)
#define  REG_MOD_BK00_36_L       _PK_L_(0x00, 0x36)
#define  REG_MOD_BK00_36_H       _PK_H_(0x00, 0x36)
#define  REG_MOD_BK00_37_L       _PK_L_(0x00, 0x37)
#define  REG_MOD_BK00_37_H       _PK_H_(0x00, 0x37)
#define  REG_MOD_BK00_38_L       _PK_L_(0x00, 0x38)
#define  REG_MOD_BK00_38_H       _PK_H_(0x00, 0x38)
#define  REG_MOD_BK00_39_L       _PK_L_(0x00, 0x39)
#define  REG_MOD_BK00_39_H       _PK_H_(0x00, 0x39)
#define  REG_MOD_BK00_3A_L       _PK_L_(0x00, 0x3A)
#define  REG_MOD_BK00_3A_H       _PK_H_(0x00, 0x3A)
#define  REG_MOD_BK00_3B_L       _PK_L_(0x00, 0x3B)
#define  REG_MOD_BK00_3B_H       _PK_H_(0x00, 0x3B)
#define  REG_MOD_BK00_3C_L       _PK_L_(0x00, 0x3C)
#define  REG_MOD_BK00_3C_H       _PK_H_(0x00, 0x3C)
#define  REG_MOD_BK00_3D_L       _PK_L_(0x00, 0x3D)
#define  REG_MOD_BK00_3D_H       _PK_H_(0x00, 0x3D)
#define  REG_MOD_BK00_3E_L       _PK_L_(0x00, 0x3E)
#define  REG_MOD_BK00_3E_H       _PK_H_(0x00, 0x3E)
#define  REG_MOD_BK00_3F_L       _PK_L_(0x00, 0x3F)
#define  REG_MOD_BK00_3F_H       _PK_H_(0x00, 0x3F)
#define  REG_MOD_BK00_40_L       _PK_L_(0x00, 0x40)
#define  REG_MOD_BK00_40_H       _PK_H_(0x00, 0x40)
#define  REG_MOD_BK00_41_L       _PK_L_(0x00, 0x41)
#define  REG_MOD_BK00_41_H       _PK_H_(0x00, 0x41)
#define  REG_MOD_BK00_42_L       _PK_L_(0x00, 0x42)
#define  REG_MOD_BK00_42_H       _PK_H_(0x00, 0x42)
#define  REG_MOD_BK00_43_L       _PK_L_(0x00, 0x43)
#define  REG_MOD_BK00_43_H       _PK_H_(0x00, 0x43)
#define  REG_MOD_BK00_44_L       _PK_L_(0x00, 0x44)
#define  REG_MOD_BK00_44_H       _PK_H_(0x00, 0x44)
#define  REG_MOD_BK00_45_L       _PK_L_(0x00, 0x45)
#define  REG_MOD_BK00_45_H       _PK_H_(0x00, 0x45)
#define  REG_MOD_BK00_46_L       _PK_L_(0x00, 0x46)
#define  REG_MOD_BK00_46_H       _PK_H_(0x00, 0x46)
#define  REG_MOD_BK00_47_L       _PK_L_(0x00, 0x47)
#define  REG_MOD_BK00_47_H       _PK_H_(0x00, 0x47)
#define  REG_MOD_BK00_48_L       _PK_L_(0x00, 0x48)
#define  REG_MOD_BK00_48_H       _PK_H_(0x00, 0x48)
#define  REG_MOD_BK00_49_L       _PK_L_(0x00, 0x49)
#define  REG_MOD_BK00_49_H       _PK_H_(0x00, 0x49)
#define  REG_MOD_BK00_4A_L       _PK_L_(0x00, 0x4A)
#define  REG_MOD_BK00_4A_H       _PK_H_(0x00, 0x4A)
#define  REG_MOD_BK00_4B_L       _PK_L_(0x00, 0x4B)
#define  REG_MOD_BK00_4B_H       _PK_H_(0x00, 0x4B)
#define  REG_MOD_BK00_4C_L       _PK_L_(0x00, 0x4C)
#define  REG_MOD_BK00_4C_H       _PK_H_(0x00, 0x4C)
#define  REG_MOD_BK00_4D_L       _PK_L_(0x00, 0x4D)
#define  REG_MOD_BK00_4D_H       _PK_H_(0x00, 0x4D)
#define  REG_MOD_BK00_4E_L       _PK_L_(0x00, 0x4E)
#define  REG_MOD_BK00_4E_H       _PK_H_(0x00, 0x4E)
#define  REG_MOD_BK00_4F_L       _PK_L_(0x00, 0x4F)
#define  REG_MOD_BK00_4F_H       _PK_H_(0x00, 0x4F)
#define  REG_MOD_BK00_50_L       _PK_L_(0x00, 0x50)
#define  REG_MOD_BK00_50_H       _PK_H_(0x00, 0x50)
#define  REG_MOD_BK00_51_L       _PK_L_(0x00, 0x51)
#define  REG_MOD_BK00_51_H       _PK_H_(0x00, 0x51)
#define  REG_MOD_BK00_52_L       _PK_L_(0x00, 0x52)
#define  REG_MOD_BK00_52_H       _PK_H_(0x00, 0x52)
#define  REG_MOD_BK00_53_L       _PK_L_(0x00, 0x53)
#define  REG_MOD_BK00_53_H       _PK_H_(0x00, 0x53)
#define  REG_MOD_BK00_54_L       _PK_L_(0x00, 0x54)
#define  REG_MOD_BK00_54_H       _PK_H_(0x00, 0x54)
#define  REG_MOD_BK00_55_L       _PK_L_(0x00, 0x55)
#define  REG_MOD_BK00_55_H       _PK_H_(0x00, 0x55)
#define  REG_MOD_BK00_56_L       _PK_L_(0x00, 0x56)
#define  REG_MOD_BK00_56_H       _PK_H_(0x00, 0x56)
#define  REG_MOD_BK00_57_L       _PK_L_(0x00, 0x57)
#define  REG_MOD_BK00_57_H       _PK_H_(0x00, 0x57)
#define  REG_MOD_BK00_58_L       _PK_L_(0x00, 0x58)
#define  REG_MOD_BK00_58_H       _PK_H_(0x00, 0x58)
#define  REG_MOD_BK00_59_L       _PK_L_(0x00, 0x59)
#define  REG_MOD_BK00_59_H       _PK_H_(0x00, 0x59)
#define  REG_MOD_BK00_5A_L       _PK_L_(0x00, 0x5A)
#define  REG_MOD_BK00_5A_H       _PK_H_(0x00, 0x5A)
#define  REG_MOD_BK00_5B_L       _PK_L_(0x00, 0x5B)
#define  REG_MOD_BK00_5B_H       _PK_H_(0x00, 0x5B)
#define  REG_MOD_BK00_5C_L       _PK_L_(0x00, 0x5C)
#define  REG_MOD_BK00_5C_H       _PK_H_(0x00, 0x5C)
#define  REG_MOD_BK00_5D_L       _PK_L_(0x00, 0x5D)
#define  REG_MOD_BK00_5D_H       _PK_H_(0x00, 0x5D)
#define  REG_MOD_BK00_5E_L       _PK_L_(0x00, 0x5E)
#define  REG_MOD_BK00_5E_H       _PK_H_(0x00, 0x5E)
#define  REG_MOD_BK00_5F_L       _PK_L_(0x00, 0x5F)
#define  REG_MOD_BK00_5F_H       _PK_H_(0x00, 0x5F)
#define  REG_MOD_BK00_60_L       _PK_L_(0x00, 0x60)
#define  REG_MOD_BK00_60_H       _PK_H_(0x00, 0x60)
#define  REG_MOD_BK00_61_L       _PK_L_(0x00, 0x61)
#define  REG_MOD_BK00_61_H       _PK_H_(0x00, 0x61)
#define  REG_MOD_BK00_62_L       _PK_L_(0x00, 0x62)
#define  REG_MOD_BK00_62_H       _PK_H_(0x00, 0x62)
#define  REG_MOD_BK00_63_L       _PK_L_(0x00, 0x63)
#define  REG_MOD_BK00_63_H       _PK_H_(0x00, 0x63)
#define  REG_MOD_BK00_64_L       _PK_L_(0x00, 0x64)
#define  REG_MOD_BK00_64_H       _PK_H_(0x00, 0x64)
#define  REG_MOD_BK00_65_L       _PK_L_(0x00, 0x65)
#define  REG_MOD_BK00_65_H       _PK_H_(0x00, 0x65)
#define  REG_MOD_BK00_66_L       _PK_L_(0x00, 0x66)
#define  REG_MOD_BK00_66_H       _PK_H_(0x00, 0x66)
#define  REG_MOD_BK00_67_L       _PK_L_(0x00, 0x67)
#define  REG_MOD_BK00_67_H       _PK_H_(0x00, 0x67)
#define  REG_MOD_BK00_68_L       _PK_L_(0x00, 0x68)
#define  REG_MOD_BK00_68_H       _PK_H_(0x00, 0x68)
#define  REG_MOD_BK00_69_L       _PK_L_(0x00, 0x69)
#define  REG_MOD_BK00_69_H       _PK_H_(0x00, 0x69)
#define  REG_MOD_BK00_6A_L       _PK_L_(0x00, 0x6A)
#define  REG_MOD_BK00_6A_H       _PK_H_(0x00, 0x6A)
#define  REG_MOD_BK00_6B_L       _PK_L_(0x00, 0x6B)
#define  REG_MOD_BK00_6B_H       _PK_H_(0x00, 0x6B)
#define  REG_MOD_BK00_6C_L       _PK_L_(0x00, 0x6C)
#define  REG_MOD_BK00_6C_H       _PK_H_(0x00, 0x6C)
#define  REG_MOD_BK00_6D_L       _PK_L_(0x00, 0x6D)
#define  REG_MOD_BK00_6D_H       _PK_H_(0x00, 0x6D)
#define  REG_MOD_BK00_6E_L       _PK_L_(0x00, 0x6E)
#define  REG_MOD_BK00_6E_H       _PK_H_(0x00, 0x6E)
#define  REG_MOD_BK00_6F_L       _PK_L_(0x00, 0x6F)
#define  REG_MOD_BK00_6F_H       _PK_H_(0x00, 0x6F)
#define  REG_MOD_BK00_70_L       _PK_L_(0x00, 0x70)
#define  REG_MOD_BK00_70_H       _PK_H_(0x00, 0x70)
#define  REG_MOD_BK00_71_L       _PK_L_(0x00, 0x71)
#define  REG_MOD_BK00_71_H       _PK_H_(0x00, 0x71)
#define  REG_MOD_BK00_72_L       _PK_L_(0x00, 0x72)
#define  REG_MOD_BK00_72_H       _PK_H_(0x00, 0x72)
#define  REG_MOD_BK00_73_L       _PK_L_(0x00, 0x73)
#define  REG_MOD_BK00_73_H       _PK_H_(0x00, 0x73)
#define  REG_MOD_BK00_74_L       _PK_L_(0x00, 0x74)
#define  REG_MOD_BK00_74_H       _PK_H_(0x00, 0x74)
#define  REG_MOD_BK00_75_L       _PK_L_(0x00, 0x75)
#define  REG_MOD_BK00_75_H       _PK_H_(0x00, 0x75)
#define  REG_MOD_BK00_76_L       _PK_L_(0x00, 0x76)
#define  REG_MOD_BK00_76_H       _PK_H_(0x00, 0x76)
#define  REG_MOD_BK00_77_L       _PK_L_(0x00, 0x77)
#define  REG_MOD_BK00_77_H       _PK_H_(0x00, 0x77)
#define  REG_MOD_BK00_78_L       _PK_L_(0x00, 0x78)
#define  REG_MOD_BK00_78_H       _PK_H_(0x00, 0x78)
#define  REG_MOD_BK00_79_L       _PK_L_(0x00, 0x79)
#define  REG_MOD_BK00_79_H       _PK_H_(0x00, 0x79)
#define  REG_MOD_BK00_7A_L       _PK_L_(0x00, 0x7A)
#define  REG_MOD_BK00_7A_H       _PK_H_(0x00, 0x7A)
#define  REG_MOD_BK00_7B_L       _PK_L_(0x00, 0x7B)
#define  REG_MOD_BK00_7B_H       _PK_H_(0x00, 0x7B)
#define  REG_MOD_BK00_7C_L       _PK_L_(0x00, 0x7C)
#define  REG_MOD_BK00_7C_H       _PK_H_(0x00, 0x7C)
#define  REG_MOD_BK00_7D_L       _PK_L_(0x00, 0x7D)
#define  REG_MOD_BK00_7D_H       _PK_H_(0x00, 0x7D)
#define  REG_MOD_BK00_7E_L       _PK_L_(0x00, 0x7E)
#define  REG_MOD_BK00_7E_H       _PK_H_(0x00, 0x7E)
#define  REG_MOD_BK00_7F_L       _PK_L_(0x00, 0x7F)
#define  REG_MOD_BK00_7F_H       _PK_H_(0x00, 0x7F)

#ifdef __cplusplus
}
#endif

#undef _DRVADCTBL_H_
#endif
