//<MStar Software>
//******************************************************************************
// MStar Software
// Copyright (c) 2010 - 2012 MStar Semiconductor, Inc. All rights reserved.
// All software, firmware and related documentation herein ("MStar Software") are
// intellectual property of MStar Semiconductor, Inc. ("MStar") and protected by
// law, including, but not limited to, copyright law and international treaties.
// Any use, modification, reproduction, retransmission, or republication of all
// or part of MStar Software is expressly prohibited, unless prior written
// permission has been granted by MStar.
//
// By accessing, browsing and/or using MStar Software, you acknowledge that you
// have read, understood, and agree, to be bound by below terms ("Terms") and to
// comply with all applicable laws and regulations:
//
// 1. MStar shall retain any and all right, ownership and interest to MStar
//    Software and any modification/derivatives thereof.
//    No right, ownership, or interest to MStar Software and any
//    modification/derivatives thereof is transferred to you under Terms.
//
// 2. You understand that MStar Software might include, incorporate or be
//    supplied together with third party`s software and the use of MStar
//    Software may require additional licenses from third parties.
//    Therefore, you hereby agree it is your sole responsibility to separately
//    obtain any and all third party right and license necessary for your use of
//    such third party`s software.
//
// 3. MStar Software and any modification/derivatives thereof shall be deemed as
//    MStar`s confidential information and you agree to keep MStar`s
//    confidential information in strictest confidence and not disclose to any
//    third party.
//
// 4. MStar Software is provided on an "AS IS" basis without warranties of any
//    kind. Any warranties are hereby expressly disclaimed by MStar, including
//    without limitation, any warranties of merchantability, non-infringement of
//    intellectual property rights, fitness for a particular purpose, error free
//    and in conformity with any international standard.  You agree to waive any
//    claim against MStar for any loss, damage, cost or expense that you may
//    incur related to your use of MStar Software.
//    In no event shall MStar be liable for any direct, indirect, incidental or
//    consequential damages, including without limitation, lost of profit or
//    revenues, lost or damage of data, and unauthorized system use.
//    You agree that this Section 4 shall still apply without being affected
//    even if MStar Software has been modified by MStar in accordance with your
//    request or instruction for your use, except otherwise agreed by both
//    parties in writing.
//
// 5. If requested, MStar may from time to time provide technical supports or
//    services in relation with MStar Software to you for your use of
//    MStar Software in conjunction with your or your customer`s product
//    ("Services").
//    You understand and agree that, except otherwise agreed by both parties in
//    writing, Services are provided on an "AS IS" basis and the warranty
//    disclaimer set forth in Section 4 above shall apply.
//
// 6. Nothing contained herein shall be construed as by implication, estoppels
//    or otherwise:
//    (a) conferring any license or right to use MStar name, trademark, service
//        mark, symbol or any other identification;
//    (b) obligating MStar or any of its affiliates to furnish any person,
//        including without limitation, you and your customers, any assistance
//        of any kind whatsoever, or any information; or
//    (c) conferring any license or right under any intellectual property right.
//
// 7. These terms shall be governed by and construed in accordance with the laws
//    of Taiwan, R.O.C., excluding its conflict of law rules.
//    Any and all dispute arising out hereof or related hereto shall be finally
//    settled by arbitration referred to the Chinese Arbitration Association,
//    Taipei in accordance with the ROC Arbitration Law and the Arbitration
//    Rules of the Association by three (3) arbitrators appointed in accordance
//    with the said Rules.
//    The place of arbitration shall be in Taipei, Taiwan and the language shall
//    be English.
//    The arbitration award shall be final and binding to both parties.
//
//******************************************************************************
//<MStar Software>

/******************************************************************************/
/*                    Header Files                                            */
/* ****************************************************************************/
#include <linux/string.h>
#include <common.h>
#include <apiSWI2C.h>
#include <MsDebug.h>
#ifdef HAS_HAL_SWI2C_H
#include "hal_swi2c.h"
#endif
#if ENABLE_AMP_VERSION_SEL_2
#include <bootlogo/DbAmpDevice.h>
#include <bootlogo/MsBootLogo.h>
#include <audio/MsDrvAudio.h>
#endif //#if ENABLE_AMP_VERSION_SEL_2

#ifndef UNUSED
#define UNUSED(x) ((x)=(x))
#endif

// TODO: The check of invalid GPIO num is not solid enough.
#define GPIO_NUM_INVALID                                (999)
#define IS_GPIO_NUM_INVALID(gpio_num)                   (GPIO_NUM_INVALID <= (gpio_num))

MS_BOOL g_msI2C_init_ready = 0;
#define E_I2C_BUS_NUM 8

#if (ENABLE_HAL_SWI2C_BUS_CFG == 1)
static SWI2C_BusCfg I2CBusCfg[E_I2C_BUS_NUM]=
{
	//Example for SW I2C => {PAD_DDCR_CK, PAD_DDCR_DA, 60}, //SW_I2C	/SCL_PAD /SDA_PAD /Delay
#if (ENABLE_CUS_BUS_CFG == 1)
	// Bus-0
	{CUS_PAD_SCL_0 , CUS_PAD_SDA_0 , CUS_DELAY_VAL_0},
	// Bus-1
	{CUS_PAD_SCL_1 , CUS_PAD_SDA_1 , CUS_DELAY_VAL_1},
    // Bus-2
	{CUS_PAD_SCL_2 , CUS_PAD_SDA_2 , CUS_DELAY_VAL_2},
	// Bus-3
	{CUS_PAD_SCL_3 , CUS_PAD_SDA_3 , CUS_DELAY_VAL_3},
	// Bus-4
	{CUS_PAD_SCL_4 , CUS_PAD_SDA_4 , CUS_DELAY_VAL_4},
	// Bus-5
	{CUS_PAD_SCL_5 , CUS_PAD_SDA_5 , CUS_DELAY_VAL_5},
	// Bus-6
	{CUS_PAD_SCL_6 , CUS_PAD_SDA_6 , CUS_DELAY_VAL_6},
	// Bus-7
	{CUS_PAD_SCL_7 , CUS_PAD_SDA_7 , CUS_DELAY_VAL_7},
    // Others, add other bus here
#else
	// Bus-0
	{PAD_SCL_0 , PAD_SDA_0 , DELAY_VAL_0},
	// Bus-1
	{PAD_SCL_1 , PAD_SDA_1 , DELAY_VAL_1},
    // Bus-2
	{PAD_SCL_2 , PAD_SDA_2 , DELAY_VAL_2},
	// Bus-3
	{PAD_SCL_3 , PAD_SDA_3 , DELAY_VAL_3},
	// Bus-4
	{PAD_SCL_4 , PAD_SDA_4 , DELAY_VAL_4},
	// Bus-5
	{PAD_SCL_5 , PAD_SDA_5 , DELAY_VAL_5},
	// Bus-6
	{PAD_SCL_6 , PAD_SDA_6 , DELAY_VAL_6},
	// Bus-7
	{PAD_SCL_7 , PAD_SDA_7 , DELAY_VAL_7},
    // Others, add other bus here
#endif
};
#else // #if defined(ENABLE_HAL_SWI2C_BUS_CFG)
static SWI2C_BusCfg I2CBusCfg[E_I2C_BUS_NUM]=
{
	//Example for SW I2C => { 1, PAD_DDCR_CK, PAD_DDCR_DA, 60, }, //SW_I2C	/SCL_PAD /SDA_PAD /Delay
	// Bus-0
	{PAD_DDCR_CK,PAD_DDCR_DA,100}, //IS_SW_I2C	/SCL_PAD /SDA_PAD /Delay
	// Bus-1
	{PAD_TGPIO0, PAD_TGPIO1,100},
    // Bus-2
	{PAD_TGPIO2, PAD_TGPIO3,100},	//IS_SW_I2C  /SCL_PAD /SDA_PAD /Delay
	// Bus-3
	{PAD_TGPIO2, PAD_TGPIO3,100},	//IS_SW_I2C  /SCL_PAD /SDA_PAD /Delay
	// Bus-4
	{PAD_TGPIO2, PAD_TGPIO3,100},	//IS_SW_I2C  /SCL_PAD /SDA_PAD /Delay
	// Bus-5
	{PAD_TGPIO2, PAD_TGPIO3,100},	//IS_SW_I2C  /SCL_PAD /SDA_PAD /Delay
	// Bus-6
	{PAD_TGPIO2, PAD_TGPIO3,100},	//IS_SW_I2C  /SCL_PAD /SDA_PAD /Delay
	// Bus-7
	{PAD_TGPIO2, PAD_TGPIO3,100},	//IS_SW_I2C  /SCL_PAD /SDA_PAD /Delay
    // Others, add other bus here
};
#endif // #if (ENABLE_HAL_SWI2C_BUS_CFG == 1)

#if ENABLE_AMP_VERSION_SEL_2
static MS_BOOL bSwIicCfgWithPortIsPrepared = FALSE;
static MS_IIC_SwBus stSwIicCfgWithPort[E_I2C_BUS_NUM];
#endif //#if ENABLE_AMP_VERSION_SEL_2

void _msI2C_print_cus_i2c_info(void)
{
    int i;

    for(i = 0; i < E_I2C_BUS_NUM; i++)
    {
        UBOOT_DEBUG("I2CBusCfg[%d] padSCL(%u) padSDA(%u)\n", i, (unsigned)I2CBusCfg[i].padSCL, (unsigned)I2CBusCfg[i].padSDA);
    }

    return;
}

#if ENABLE_AMP_VERSION_SEL_2
void _msI2C_print_sw_iic_bus_config(void)
{
    int i;

    for(i = 0; i < E_I2C_BUS_NUM; i++)
    {
        printf("stSwIicCfgWithPort[%d]\n", i);
        printf("ePort = %d\n", stSwIicCfgWithPort[i].ePort);
        printf("u16SclPad = %u\n", (unsigned)stSwIicCfgWithPort[i].u16SclPad);
        printf("u16SdaPad = %u\n", (unsigned)stSwIicCfgWithPort[i].u16SdaPad);
        printf("u16Delay = %u\n", (unsigned)stSwIicCfgWithPort[i].u16Delay);
    }

    return;
}

static int _msI2C_reset_sw_iic_bus_config(void)
{
    int ret = 0;
    int i;

    _msI2C_print_cus_i2c_info();

    memset(stSwIicCfgWithPort, 0, sizeof(stSwIicCfgWithPort));
    for(i = 0; i < E_I2C_BUS_NUM; i++)
    {
        // stConfig[x].eTargetPort = E_MS_IIC_SW_PORT_x;
        stSwIicCfgWithPort[i].ePort = E_MS_IIC_SW_PORT_0 + i;
        stSwIicCfgWithPort[i].u16SclPad = I2CBusCfg[i].padSCL;
        stSwIicCfgWithPort[i].u16SdaPad = I2CBusCfg[i].padSDA;
        stSwIicCfgWithPort[i].u16Delay = I2CBusCfg[i].defDelay;
    }

    return ret;
}

#if (!(CONFIG_LOGO_STORE_IN_MBOOT) && !(CONFIG_LOGO_STORE_NOT_SET) && !(CONFIG_MSTAR_TOOL_ROM_PROGRAM_NAND_BIN))
static int _msI2C_map_swi2c_port_from_db_port(MS_IIC_PORT *pePort, db_iic_port_e eIicPortFromDb)
{
    int ret = -1;

    if((E_DB_IIC_SW_PORT_0 <= eIicPortFromDb) &&
       (E_DB_IIC_PORT_NOSUP > eIicPortFromDb))
    {
        *pePort = eIicPortFromDb - E_DB_IIC_SW_PORT_0 + E_MS_IIC_SW_PORT_0;
        ret = 0;
    }

    return ret;
}

static int _msI2C_map_swi2c_bus_from_port(int *pBus, MS_IIC_PORT ePortFrom)
{
    int ret = -1;
    int i;

    for(i = 0; i < E_I2C_BUS_NUM; i++)
    {
        if(ePortFrom == stSwIicCfgWithPort[i].ePort)
        {
            *pBus = i;
            ret = 0;

            break;
        }
    }

    return ret;
}

static int _msI2C_map_swi2c_bus_from_pad(int *pBus, MS_U16 u16PadClk, MS_U16 u16PadData)
{
    int ret = -1;
    int i;

    for(i = 0; i < E_I2C_BUS_NUM; i++)
    {
        if((stSwIicCfgWithPort[i].u16SclPad == u16PadClk) &&
           (stSwIicCfgWithPort[i].u16SdaPad == u16PadData))
        {
            *pBus = i;
            ret = 0;

            break;
        }
    }

    if(0 != ret)
    {
        UBOOT_DEBUG("%s: %d: u16PadClk(%u), u16PadData(%u) is not found.\n", __func__, __LINE__, (unsigned)u16PadClk, (unsigned)u16PadData);
    }

    return ret;
}

static int _msI2C_swi2c_swap_port(int targetBus, int sourceBus)
{
    int ret = 0;
    MS_IIC_PORT eTmpPort;

    if(targetBus == sourceBus)
    {
        UBOOT_DEBUG("targetBus == sourceBus\n");
        return ret;
    }

    eTmpPort = stSwIicCfgWithPort[targetBus].ePort;
    stSwIicCfgWithPort[targetBus].ePort = stSwIicCfgWithPort[sourceBus].ePort;
    stSwIicCfgWithPort[sourceBus].ePort = eTmpPort;

    return ret;
}

static int _msI2C_swi2c_add_bus_config(int *pBus, SWI2C_BusCfg *pstBusCfg)
{
    int ret = -1;
    int i;

    if(NULL == pBus)
    {
        UBOOT_DEBUG("pBus is NULL.\n");

        return ret;
    }
    if(NULL == pstBusCfg)
    {
        UBOOT_DEBUG("pstBusCfg is NULL.\n");

        return ret;
    }

    for(i = 0; i < E_I2C_BUS_NUM; i++)
    {
        if(IS_GPIO_NUM_INVALID(stSwIicCfgWithPort[i].u16SclPad) || IS_GPIO_NUM_INVALID(stSwIicCfgWithPort[i].u16SdaPad))
        {
            stSwIicCfgWithPort[i].u16SclPad = pstBusCfg->padSCL;
            stSwIicCfgWithPort[i].u16SdaPad = pstBusCfg->padSDA;
            stSwIicCfgWithPort[i].u16Delay = pstBusCfg->defDelay;
            *pBus = i;
            UBOOT_DEBUG("%s: %d: The config is added to slot %d.\n", __func__, __LINE__, *pBus);
            ret = 0;

            break;
        }
    }

    if(0 != ret)
    {
        UBOOT_DEBUG("%s: %d: No empty slot is available.\n", __func__, __LINE__);
    }

    return ret;
}

static int _msI2C_add_iic_bus_config_if_not_existed(db_iic_bus_info_t stIicBusInfos[MAX_IIC_BUS_NUM])
{
    int ret = 0;
    int bus = -1;
    SWI2C_BusCfg stSwI2cBusCfg;
    int i;

    for(i = 0; i < MAX_IIC_BUS_NUM; i++)
    {
        if(E_DB_I2C_TYPE_SW == stIicBusInfos[i].m_eBusType)
        {
            ret = _msI2C_map_swi2c_bus_from_pad(&bus, stIicBusInfos[i].m_stSw.m_u16ClkPad, stIicBusInfos[i].m_stSw.m_u16DataPad);
            if(0 != ret)
            {
                memset(&stSwI2cBusCfg, 0, sizeof(SWI2C_BusCfg));
                stSwI2cBusCfg.padSCL = stIicBusInfos[i].m_stSw.m_u16ClkPad;
                stSwI2cBusCfg.padSDA = stIicBusInfos[i].m_stSw.m_u16DataPad;
                stSwI2cBusCfg.defDelay = stIicBusInfos[i].m_stSw.m_u16Delay;
                ret = _msI2C_swi2c_add_bus_config(&bus, &stSwI2cBusCfg);
                if(0 != ret)
                {
                    UBOOT_DEBUG("_msI2C_swi2c_add_bus_config(...) is failed.\n");
                    break;
                }
            }
        }
    }

    return ret;
}

static int _msI2C_prepare_sw_iic_bus_config(void)
{
    int ret = 0;
    db_iic_bus_info_t stIicBusInfos[MAX_IIC_BUS_NUM];
    int sourceBus = -1;
    int targetBus = -1;
    MS_IIC_PORT eSourcePort = E_MS_IIC_PORT_NOSUP;
    int i;

    if(bSwIicCfgWithPortIsPrepared)
    {
        UBOOT_DEBUG("SwIicCfgWithPort was prepared before.\n");
        return 0;
    }

    if(0 != do_dbtable_init(NULL, 0, 0, NULL))
    {
        UBOOT_DEBUG("do_dbtable_init() is failed.\n");
        return -1;
    }

    if(0 != read_iic_para_from_flash(stIicBusInfos))
    {
        UBOOT_DEBUG("read_iic_para_from_flash() is failed.\n");
        return -1;
    }

    ret = _msI2C_reset_sw_iic_bus_config();
    if(0 != ret)
    {
        UBOOT_DEBUG("_msI2C_reset_sw_iic_bus_config(...) is failed.\n");

        return ret;
    }

    ret = _msI2C_add_iic_bus_config_if_not_existed(stIicBusInfos);
    if(0 != ret)
    {
        UBOOT_DEBUG("_msI2C_add_iic_bus_config_if_not_existed(...) is failed.\n");
        // Abandon the config if it can't be added.
        ret = 0;
    }

    for(i = 0; i < MAX_IIC_BUS_NUM; i++)
    {
        if((E_DB_I2C_TYPE_SW == stIicBusInfos[i].m_eBusType) &&
           (E_DB_IIC_SW_PORT_0 <= stIicBusInfos[i].m_stSw.m_ePort) &&
           (E_DB_IIC_PORT_NOSUP> stIicBusInfos[i].m_stSw.m_ePort))
        {
            eSourcePort = E_MS_IIC_PORT_NOSUP;
            sourceBus = -1;
            targetBus = -1;

            ret = _msI2C_map_swi2c_port_from_db_port(&eSourcePort, stIicBusInfos[i].m_stSw.m_ePort);
            if(0 != ret)
            {
                UBOOT_DEBUG("_msI2C_map_swi2c_port_from_db_port(...) is failed.\n");
                break;
            }
            ret = _msI2C_map_swi2c_bus_from_port(&sourceBus, eSourcePort);
            if(0 != ret)
            {
                UBOOT_DEBUG("_msI2C_map_swi2c_bus_from_port(...) is failed.\n");
                break;
            }

            ret = _msI2C_map_swi2c_bus_from_pad(&targetBus, stIicBusInfos[i].m_stSw.m_u16ClkPad, stIicBusInfos[i].m_stSw.m_u16DataPad);
            if(0 != ret)
            {
                UBOOT_DEBUG("_msI2C_map_swi2c_bus_from_pad(...) is failed.\n");
                // It is possible that BOARD_I2C_SWBUS in board.ini can't be mapped to settings in board.h.
                ret = 0;
                continue;
            }

            ret = _msI2C_swi2c_swap_port(targetBus, sourceBus);
            if(0 != ret)
            {
                UBOOT_DEBUG("_msI2C_swi2c_map_bus_from_pad(...) is failed.\n");
                break;
            }
        }
    }

    if(0 == ret)
    {
        bSwIicCfgWithPortIsPrepared = TRUE;
        _msI2C_print_sw_iic_bus_config();
    }

    return ret;
}
#else
int _msI2C_prepare_sw_iic_bus_config(void)
{
    int ret = 0;

    if(bSwIicCfgWithPortIsPrepared)
    {
        UBOOT_DEBUG("SwIicCfgWithPort was prepared before.\n");
        return 0;
    }

    ret = _msI2C_reset_sw_iic_bus_config();
    if(0 != ret)
    {
        UBOOT_DEBUG("msI2C_reset_sw_iic_bus_config(...) is failed.\n");

        return ret;
    }

    bSwIicCfgWithPortIsPrepared = TRUE;
    _msI2C_print_sw_iic_bus_config();

    return ret;
}
#endif //#if (!(CONFIG_LOGO_STORE_IN_MBOOT) && !(CONFIG_LOGO_STORE_NOT_SET) && !(CONFIG_MSTAR_TOOL_ROM_PROGRAM_NAND_BIN))

#else
int _msI2C_prepare_sw_iic_bus_config(void)
{
    UBOOT_DEBUG("%s is not supported.\n", __func__);

    return -1;
}

#endif //#if ENABLE_AMP_VERSION_SEL_2

int msI2C_init(void)
{
	if(g_msI2C_init_ready == FALSE)
	{
#if ENABLE_AMP_VERSION_SEL_2
	    if(0 != _msI2C_prepare_sw_iic_bus_config())
	    {
            UBOOT_DEBUG("msI2C_prepare_sw_iic_bus_config() is failed.\n");
            return -1;
	    }
	    if(!MDrv_IIC_SetupSwPort(E_I2C_BUS_NUM, stSwIicCfgWithPort))
	    {
	        UBOOT_DEBUG("msI2C_swi2c_init() is failed.\n");
	        return -1;
	    }
#else
		MApi_SWI2C_Init(I2CBusCfg,E_I2C_BUS_NUM);
#endif //#if ENABLE_AMP_VERSION_SEL_2
		g_msI2C_init_ready = TRUE;
	}
	else
	{
		UBOOT_DEBUG("SWI2C already init.\n");
	}
	return 0;
}

#if ENABLE_AMP_VERSION_SEL_2
int msI2C_convert_2_port(MS_IIC_PORT *pePort, SWI2C_BusCfg stI2CBusCfg)
{
    int ret = -1;
    int i;

    if(!bSwIicCfgWithPortIsPrepared)
    {
        UBOOT_DEBUG("SwIicCfgWithPort is NOT prepared yet.\n");
        return -1;
    }

    for(i = 0; i < E_I2C_BUS_NUM; i++)
    {
        if((stI2CBusCfg.padSCL == stSwIicCfgWithPort[i].u16SclPad) &&
           (stI2CBusCfg.padSDA == stSwIicCfgWithPort[i].u16SdaPad))
        {
            *pePort = stSwIicCfgWithPort[i].ePort;

            ret = 0;
            break;
        }
    }

    return ret;
}
#endif //#if ENABLE_AMP_VERSION_SEL_2
