/*
 * (C) Copyright 2002
 * Sysgo Real-Time Solutions, GmbH <www.elinos.com>
 * Marius Groeger <mgroeger@sysgo.de>
 *
 * Copyright (C) 2001  Erik Mouw (J.A.K.Mouw@its.tudelft.nl)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307	 USA
 *
 */

#include <common.h>
#include <command.h>
#include <image.h>
#include <u-boot/zlib.h>
#include <asm/byteorder.h>
#include <fdt.h>
#include <libfdt.h>
#include <fdt_support.h>
#include <MsApiMiu.h>
#include "MsEnvironment.h"
#include <MsDebug.h>
#include <drvSYS.h>
#include <MsOS.h>
#ifdef CONFIG_UFBL_FEATURE_IDME
#include <MsUtility.h>
#include <idme/CusIDME.h>
#include <idme/idme.h>
#endif

#if defined(CONFIG_ARMV7_NONSEC) || defined(CONFIG_ARMV7_VIRT)
#include <asm/armv7.h>
#endif

#if defined(CONFIG_ENABLE_MIU_SSC) && defined(CONFIG_MSTAR_BD_MST148B_10NQAT_18115_M7322_VESTEL_MB170)
#define MIU_SSC_MODULATION    25
#define MIU_SSC_DEVIATION     10
#define E_MIU_0               0
#define E_MIU_1               1
#define MIU0_SSC_ENABLE       TRUE
#define MIU1_SSC_ENABLE       TRUE
#endif

DECLARE_GLOBAL_DATA_PTR;

#if defined (CONFIG_SETUP_MEMORY_TAGS) || \
    defined (CONFIG_CMDLINE_TAG) || \
    defined (CONFIG_INITRD_TAG) || \
    defined (CONFIG_SERIAL_TAG) || \
    defined (CONFIG_REVISION_TAG)
static void setup_start_tag (bd_t *bd);

# ifdef CONFIG_SETUP_MEMORY_TAGS
static void setup_memory_tags (bd_t *bd);
# endif
static void setup_commandline_tag (bd_t *bd, char *commandline);

# ifdef CONFIG_INITRD_TAG
static void setup_initrd_tag (bd_t *bd, ulong initrd_start,
			      ulong initrd_end);
# endif
static void setup_end_tag (bd_t *bd);

static struct tag *params;
#endif /* CONFIG_SETUP_MEMORY_TAGS || CONFIG_CMDLINE_TAG || CONFIG_INITRD_TAG */

static ulong get_sp(void);
#if defined(CONFIG_OF_LIBFDT) && !defined(CONFIG_UFBL_FEATURE_IDME)
static int bootm_linux_fdt(int machid, bootm_headers_t *images);
#endif

void arch_lmb_reserve(struct lmb *lmb)
{
	ulong sp;

	/*
	 * Booting a (Linux) kernel image
	 *
	 * Allocate space for command line and board info - the
	 * address should be as high as possible within the reach of
	 * the kernel (see CONFIG_SYS_BOOTMAPSZ settings), but in unused
	 * memory, which means far enough below the current stack
	 * pointer.
	 */
	sp = get_sp();
	debug("## Current stack ends at 0x%08lx ", sp);

	/* adjust sp by 1K to be safe */
	sp -= 1024;
	lmb_reserve(lmb, sp,
		    gd->bd->bi_dram[0].start + gd->bd->bi_dram[0].size - sp);
}

#if 0
// MStar start
#define BOOTTIME_SBOOT_STR "BOOTTIME_SBOOT"
#define BOOTTIME_UBOOT_STR "BOOTTIME_UBOOT"
#include <stdlib.h>
#include <malloc.h>
#include "MsEnvironment.h"

unsigned long G_MS_BOOTTIME_SBOOT=1; // global variable for storing the boot time used in sboot (ms)
unsigned long G_MS_BOOTTIME_UBOOT=1; // global variable for storing the boot time used in sboot (ms)

static void _boottime_set_to_env(void)
{
    //extern unsigned long G_MS_BOOTTIME_SBOOT; // global variable for storing the boot time used in sboot (ms)
    //extern unsigned long G_MS_BOOTTIME_UBOOT; // global variable for storing the boot time used in sboot (ms)
    extern int snprintf(char *str, size_t size, const char *fmt, ...);
    const char *strBootArg = "bootargs";
    char *strEnv = NULL;
    char *strPrevEnv = NULL;
    const int u32EnvSize = 32;

    {
        unsigned int PiuTick = *(volatile unsigned int *)(0x1f006090);
        PiuTick |= (*(volatile unsigned int *)(0x1f006094)) << 16;
        unsigned int PiuTime = PiuTick / 12000;
        G_MS_BOOTTIME_UBOOT = (unsigned long)PiuTime;
    }

    strEnv = malloc(u32EnvSize);
    if(strEnv  != NULL)
    {
        strPrevEnv = getenv((char*)strBootArg);
        if(strPrevEnv != NULL)
        {
            memset(strEnv , 0, u32EnvSize);
            snprintf(strEnv , u32EnvSize-1, "%s=%lu", BOOTTIME_SBOOT_STR, G_MS_BOOTTIME_SBOOT);
            if(0 != set_bootargs_cfg((char*)strBootArg, strEnv, 1))
            {
                printf("%s: Error: set_bootargs_cfg failed at %d\n", __func__, __LINE__);
            }

            //NOTE: getenv again because if has been updated
            strPrevEnv = getenv((char*)strBootArg);
            if(strPrevEnv != NULL)
            {
                memset(strEnv , 0,u32EnvSize);
                snprintf(strEnv , u32EnvSize-1, "%s=%lu", BOOTTIME_UBOOT_STR, G_MS_BOOTTIME_UBOOT);
                if(0 != set_bootargs_cfg((char*)strBootArg, strEnv, 1))
                {
                    printf("%s: Error: set_bootargs_cfg failed at %d\n", __func__, __LINE__);
                }
            }
        }

        free(strEnv );
    }
}
// MStar end
#endif
static void announce_and_cleanup(void)
{
    char* s = NULL;
    char* t = NULL;

    /* checkpoint for autotest boottime, plz dont remove it */
    unsigned int PiuTick = *(volatile unsigned int *)(0x1f006090);
    PiuTick |= (*(volatile unsigned int *)(0x1f006094)) << 16;
    unsigned int PiuTime = PiuTick / 12000;
    printf("[AT][MB][start kr][%u]\n", PiuTime);

	open_console_log();
	printf("\nStarting kernel ...\n\n");

    #ifdef STR_FOR_AGATE
	    *(volatile unsigned int*)(0x1f206700) = 0xdfdf;
	#else
        MsOS_DisableAllInterrupts();
    #endif

    s = getenv("sync_miuprot");
    t = getenv("KERNEL_PROTECT");

    if(!(strcmp(s, "2")))
    {
        MsApi_MiuProtect();
    }
    else if ((!(strcmp(s, "1"))) || (t != NULL))
    {
        MsApi_kernelProtect();
        s = getenv("kernelProtectBist");
        if (s != NULL)
        {
            MsApi_kernelProtectBist();
        }
    }

#ifdef CONFIG_USB_DEVICE
	{
		extern void udc_disconnect(void);
		udc_disconnect();
	}
#endif
	cleanup_before_linux();
}

#if defined(CONFIG_ARMV7_NONSEC) || defined(CONFIG_ARMV7_VIRT)
static void do_nonsec_virt_switch(void)
{
#if defined(CONFIG_ARMV7_NONSEC) || defined(CONFIG_ARMV7_VIRT)
	if (armv7_switch_nonsec() == 0)
#ifdef CONFIG_ARMV7_VIRT
	if (armv7_switch_hyp() == 0)
		debug("entered HYP mode\n");
#else
	debug("entered non-secure state\n");
#endif
#endif
}
#endif

/*
 *	AMP Nuttx flow
 *	OPTEE flow
 *	ARM FW flow
 */
#include <MsTrustZone.h>
#ifdef CONFIG_ARMv7_OPTEE
typedef void (*optee_entry)(int zero, int arch, uint params, unsigned long kernel_entry);
#endif

extern unsigned int optee_real_va_addr;
#ifdef CONFIG_ARMv8_ARM_TRUSTED_FIRMWARE
extern unsigned int armfw_real_va_addr;
#endif

#if defined(CONFIG_ARMv8_ARM_TRUSTED_FIRMWARE)
void ARM_FW_Enter_64(unsigned long cmd_ptr, ulong ramdisk_start, ulong ramdisk_len)
{
#if !defined(CONFIG_MSTAR_NS_UBOOT) && !defined(CONFIG_OPTEE_TEELOADER) //SMP ARMv8 optee flow with 64bit kernel
	mstar_boot_prameters_t bootp;
	unsigned long Fireware_entry = armfw_real_va_addr;//CONFIG_MIU1_BUSADDR + 0x00023000;//0xC0000000
	UBOOT_DEBUG("armfw_real_va_addr : %x \n",armfw_real_va_addr);
	UBOOT_DEBUG("optee_real_va_addr : %x \n",optee_real_va_addr);
	UBOOT_DEBUG("\033[0;32;31m %s %d  ARMv8 Uboot for 64bit kernel Enter ARM FW\033[m\n",__func__,__LINE__);
	UBOOT_DEBUG("\033[0;32;31m %s %d %x %x %x\033[m\n",__func__,__LINE__,ramdisk_start,ramdisk_len,cmd_ptr);

	bootp.BIN0_entry_point = optee_real_va_addr;//CONFIG_MIU1_BUSADDR + 0x02100000;
	bootp.BIN0_RW64 = MODE_RW_32; //not use now
	bootp.BIN0_bootargs = 0;
	bootp.BIN1_entry_point = 0x20280000;
	bootp.BIN1_RW64 = MODE_RW_64;
	bootp.BIN1_r2 = cmd_ptr;
	bootp.BIN1_r3 = ramdisk_start;
	bootp.BIN1_r4 = ramdisk_len;
	bootp.BIN2_RW64 = MODE_RW_64;
	UBOOT_DEBUG("\033[0;32;31m %s %d %llx %llx\033[m\n",__func__,__LINE__,bootp.BIN0_entry_point,bootp.BIN1_entry_point);
	UBOOT_DEBUG("\033[0;32;31m %s %d %llx %llx %llx\033[m\n",__func__,__LINE__,bootp.BIN1_r2,bootp.BIN1_r3,bootp.BIN1_r4);
	cleanup_before_linux();
	__asm__ __volatile__(
		"ldr  r1,[%0]  \n\t"
		"mov  r0,%1  \n\t"
		"smc #0  \n\t"
		:
		:"r"(&Fireware_entry),"r"(&bootp)
		:"r0","r1"
	);
#elif defined(CONFIG_MSTAR_NS_UBOOT) || defined(CONFIG_OPTEE_TEELOADER)//SMP ARMv8 optee flow with 64bit kernel for NS-Uboot
	UBOOT_DEBUG("armfw_real_va_addr : %x \n",armfw_real_va_addr);
	UBOOT_DEBUG("optee_real_va_addr : %x \n",optee_real_va_addr);
	UBOOT_DEBUG("\033[0;32;31m %s %d  ARMv8 NS-Uboot for 64bit kernel\033[m\n",__func__,__LINE__);
	UBOOT_DEBUG("\033[0;32;31m %s %d %x %x %x\033[m\n",__func__,__LINE__,ramdisk_start,ramdisk_len,cmd_ptr);

	__asm__ __volatile__(
	"ldr  r0, =0x8400000F\n\t"
	"ldr  r3, [%0]  \n\t"
	"ldr  r4, [%1]  \n\t"
	"ldr  r2, [%2]  \n\t"
	"smc #0         \n\t"
	:
	: "r"(&ramdisk_start),"r"(&ramdisk_len),"r"(&cmd_ptr)
	: "r0","r3","r4","r2"
	);
#endif
}

void ARM_FW_Enter_32(int machid,char* bi_boot_params)
{
	mstar_boot_prameters_t bootp;
	printf("armfw_real_va_addr : %x \n",armfw_real_va_addr);
	unsigned long Fireware_entry = armfw_real_va_addr;//CONFIG_MIU1_BUSADDR + 0x00023000;//0xC0000000
	printf("optee_real_va_addr : %x \n",optee_real_va_addr);
	bootp.BIN0_entry_point = optee_real_va_addr;//CONFIG_MIU1_BUSADDR + 0x02100000;
	bootp.BIN0_RW64 = MODE_RW_32; //not use now
	bootp.BIN0_bootargs = 0;
	bootp.BIN1_entry_point = 0x20208000;
	bootp.BIN1_RW64 = MODE_RW_32;
	printf("\033[0;32;31m %s %d %llx %llx\033[m\n",__func__,__LINE__,bootp.BIN0_entry_point,bootp.BIN1_entry_point);
	bootp.BIN1_r0 = 0;
	bootp.BIN1_r1 = machid;
	bootp.BIN1_r2 = bi_boot_params;
	bootp.BIN2_RW64 = MODE_RW_32;
	cleanup_before_linux();
	__asm__ __volatile__(
		"ldr  r1,[%0]  \n\t"
		"mov  r0,%1  \n\t"
		"smc #0  \n\t"
		:
		:"r"(&Fireware_entry),"r"(&bootp)
		:"r0","r1"
	);
}
#endif

#if defined(CONFIG_ENABLE_MIU_SSC) && defined(CONFIG_MSTAR_BD_MST148B_10NQAT_18115_M7322_VESTEL_MB170)
static void do_MIU_SSC(void)
{
	printf("[%s]:IN\n",__func__);
	MDrv_MIU_SetSscValue(E_MIU_0,(MS_U16)MIU_SSC_MODULATION, (MS_U16)MIU_SSC_DEVIATION,(MS_BOOL)MIU0_SSC_ENABLE);
	MDrv_MIU_SetSscValue(E_MIU_1,(MS_U16)MIU_SSC_MODULATION, (MS_U16)MIU_SSC_DEVIATION,(MS_BOOL)MIU1_SSC_ENABLE);
	//MDrv_MIU_SetSscValue(E_MIU_2,(MS_U16)MIU_SSC_MODULATION, (MS_U16)MIU_SSC_DEVIATION,(MS_BOOL)MIU0_SSC_ENABLE);
	printf("[%s]:OK\n",__func__);
}
#endif

int do_bootm_linux(int flag, int argc, char *argv[], bootm_headers_t *images)
{
	bd_t	*bd = gd->bd;
	char	*s;
	int	machid = bd->bi_arch_number;
	void	(*kernel_entry)(int zero, int arch, uint params);
  // To get optee status
  extern unsigned long gOpteeFailFlag;

#ifdef CONFIG_CMDLINE_TAG
	char *commandline = getenv ("bootargs");
#endif

	if ((flag != 0) && (flag != BOOTM_STATE_OS_GO))
		return 1;

	s = getenv ("machid");
	if (s) {
		machid = simple_strtoul (s, NULL, 16);
		printf ("Using machid 0x%x from environment\n", machid);
	}

	show_boot_progress (15);

#if defined(CONFIG_OF_LIBFDT) && !defined(CONFIG_UFBL_FEATURE_IDME)
	if (images->ft_len)
		return bootm_linux_fdt(machid, images);
#endif

	kernel_entry = (void (*)(int, int, uint))images->ep;

	debug ("## Transferring control to Linux (at address %08lx) ...\n",
	       (ulong) kernel_entry);

//MStar start
    //_boottime_set_to_env();
#ifdef CONFIG_CMDLINE_TAG

#ifdef CONFIG_UFBL_FEATURE_IDME
#define ENV_RECOVER_BUFFER "recovery_fbaddr"
#define CMD_BUF 128
  
    int ret=0;
    MS_PHYADDR addr=0;
    char buf[CMD_BUF]="\0";
    char* new_commandline;
    int bNeedToSaveEnv = 0;

    #if 0
    // delete root and androidboot.selinux conf from bootargs
    run_command("dc root=", 0);
    run_command("dc androidboot.selinux=", 0);

    commandline = getenv ("bootargs"); // NOTE: should get agaig, becuase the env has been updated
    UBOOT_DEBUG("bootarg=%s \n",commandline);

    /* remove params that should not in env variables, for security reason */

    char* new_commandline = str_replace(commandline, "androidboot.unlocked_kernel=true", " ");
    if (new_commandline) {
        setenv("bootargs", new_commandline);
        bNeedToSaveEnv = 1;
        commandline = getenv("bootargs");
        free(new_commandline);
    }
    new_commandline = str_replace(commandline, "androidboot.veritymode=disabled", " ");
    if (new_commandline) {
        setenv("bootargs", new_commandline);
        bNeedToSaveEnv = 1;
        commandline = getenv("bootargs");
        free(new_commandline);
    }
    new_commandline = str_replace(commandline, "androidboot.prod=0", " ");
    if (new_commandline) {
        setenv("bootargs", new_commandline);
        bNeedToSaveEnv = 1;
        commandline = getenv("bootargs");
        free(new_commandline);
    }
    new_commandline = str_replace(commandline, "androidboot.selinux=permissive", " ");
    if (new_commandline) {
        setenv("bootargs", new_commandline);
        bNeedToSaveEnv = 1;
        commandline = getenv("bootargs");
        free(new_commandline);
    }
    new_commandline = str_replace(commandline, "loglevel=7", " ");
    if (new_commandline) {
        setenv("bootargs", new_commandline);
        bNeedToSaveEnv = 1;
        commandline = getenv("bootargs");
        free(new_commandline);
    }
    #endif
    //Recovery Buffer
    ret=get_map_addr_from_env(E_RECOVERY_BUFFER, NO_DEFAULT_MMAP_VALUE, &addr);
    if(ret!=0)
    {
        UBOOT_DEBUG("[%s] No %s information\n", __FUNCTION__ ,"E_MMAP_ID_RECOVERY_BUFFER_ADR");
    }
    else
    {
        snprintf(buf, CMD_BUF,"%s=0x%llx", ENV_RECOVER_BUFFER, (MS_U64)addr);
        if(isBootToRecovery())
        {
            UBOOT_DEBUG("Add buf=[%s] into bootargs\n", buf);
            env_add_bootargs(ENV_RECOVER_BUFFER, buf);
            commandline = getenv("bootargs");
        }
        else
        {
            new_commandline = str_replace(commandline, buf, " ");
            if (new_commandline) {
                setenv("bootargs", new_commandline);
                bNeedToSaveEnv = 1;
                commandline = getenv("bootargs");
                free(new_commandline);
            }
        }
    }
    if (bNeedToSaveEnv)
        saveenv();
#endif //CONFIG_UFBL_FEATURE_IDME
#endif //CONFIG_CMDLINE_TAG
    commandline = getenv ("bootargs"); // NOTE: should get agaig, becuase the env has been updated
    UBOOT_DEBUG("bootarg=%s \n",commandline);

//MStar end
#if defined (CONFIG_SETUP_MEMORY_TAGS) || \
    defined (CONFIG_CMDLINE_TAG) || \
    defined (CONFIG_INITRD_TAG) || \
    defined (CONFIG_SERIAL_TAG) || \
    defined (CONFIG_REVISION_TAG)
	setup_start_tag (bd);
#ifdef CONFIG_SERIAL_TAG
	setup_serial_tag (&params);
#endif
#ifdef CONFIG_REVISION_TAG
	setup_revision_tag (&params);
#endif
#ifdef CONFIG_SETUP_MEMORY_TAGS
	setup_memory_tags (bd);
#endif
#ifdef CONFIG_CMDLINE_TAG
	setup_commandline_tag (bd, commandline);
#endif
#ifdef CONFIG_INITRD_TAG
	if (images->rd_start && images->rd_end)
		setup_initrd_tag (bd, images->rd_start, images->rd_end);
#endif
	setup_end_tag(bd);
#endif

    char *get_dto_debug = getenv("dto_debug");
    if(get_dto_debug != NULL)
    {
        if(strncmp(get_dto_debug, "TRUE", 4)==0)
        {
            run_command("fdt addr 0x23000000", 0);
            run_command("fdt print", 0);
        }
    }

	announce_and_cleanup();

#if defined(CONFIG_ENABLE_MIU_SSC) && defined(CONFIG_MSTAR_BD_MST148B_10NQAT_18115_M7322_VESTEL_MB170)
    do_MIU_SSC(); //enable MIU0, MIU1 SSC
#endif

#if defined (CONFIG_ARMv8_64BIT_KERNEL) //Kernel 64bit
    //there is no atags pointer within 64 bit kernel, so we need to pass bootargs to kernel
    char* __cmdline = bd->bi_boot_params;
    //cmd_ptr is used to save pointer of command line
    unsigned long cmd_ptr = (unsigned long)bd->bi_boot_params;
    memset(__cmdline,0, strlen(commandline)+1);
    strncpy(__cmdline,commandline,strlen(commandline));
    printf("set 64 bit mode \n");
    extern ulong ramdisk_start,ramdisk_len;
#if defined(CONFIG_UFBL_FEATURE_IDME)
    char* dev_flags_str = NULL;
    unsigned long dev_flags = 0;
    if ((dev_flags_str = getenv("dev_flags")) != NULL)
            dev_flags = simple_strtoul(dev_flags_str, NULL, 16);

    if ((dev_flags & DEV_FLAGS_SELINUX_FORCE_ENFORCING) == DEV_FLAGS_SELINUX_FORCE_ENFORCING)
    {
        printf("force selinux enforcing mode\n");
        char* enforcing_cmdline = str_replace(__cmdline, "selinux=permissive", "selinux=enforcing");
        if (enforcing_cmdline)
        {
                int length = strlen(enforcing_cmdline);
                strncpy(__cmdline, enforcing_cmdline, length);
                __cmdline[length] = '\0';
                free(enforcing_cmdline);
        }
    } else if (((dev_flags & DEV_FLAGS_SELINUX_FORCE_PERMISSIVE) == DEV_FLAGS_SELINUX_FORCE_PERMISSIVE)
        && !is_lockdown())
    {
        printf("force selinux permissive mode\n");
        char* permissive_cmdline = str_replace(__cmdline, "selinux=enforcing", "selinux=permissive");
        if (permissive_cmdline)
        {
                int length = strlen(permissive_cmdline);
                strncpy(__cmdline, permissive_cmdline, length);
                __cmdline[length] = '\0';
                free(permissive_cmdline);
        }
    }
#endif

#if defined(CONFIG_ARMv8_ARM_TRUSTED_FIRMWARE) && !defined(CONFIG_SECARM_OPTEE)
  if(gOpteeFailFlag == 0)
  {
    ARM_FW_Enter_64(cmd_ptr,ramdisk_start,ramdisk_len);
	}
	else
	{
		//Write gOpteeFailFlag flag back to 0x103382 since this bank may be overwritten by other APP
		Write2Byte(0x103382,0xEEFF);
    __asm__ __volatile__(
    #if (CONFIG_GIC_PATCH == 1)
      "ldr     r3, =0x16001080\n\t"
      "ldr     r2, [r3]\n\t"
      "ldr     r4, =(0x3 << 30)\n\t"
      "orr     r2, r2, r4\n\t"
      "str     r2, [r3]\n\t"
      "ldr     r3, =0x16001000\n\t"
      "mov     r2, #3\n\t"
      "str     r2, [r3]\n\t"
      "ldr     r3, =0x16002000\n\t"
      "mov     r2, #3\n\t"
      "str     r2, [r3]\n\t"
      "ldr     r3, =0x16002004\n\t"
      "mov     r2, #0xf8\n\t"
      "str     r2, [r3]\n\t"
    #endif
      "ldr  r3, [%0]  \n\t"
      "ldr  r4, [%1]  \n\t"
      "ldr  r2, [%2]  \n\t"
      :
      : "r"(&ramdisk_start),"r"(&ramdisk_len),"r"(&cmd_ptr)
      : "r3","r4","r2"
      );
      __asm__ __volatile__("smc #0": );
	}
#else //ARMv8 Normal boot flow with 64bit kernel
	__asm__ __volatile__(
#if (CONFIG_GIC_PATCH == 1)
	"ldr     r3, =0x16001080\n\t"
	"ldr     r2, [r3]\n\t"
	"ldr     r4, =(0x3 << 30)\n\t"
	"orr     r2, r2, r4\n\t"
	"str     r2, [r3]\n\t"
	"ldr     r3, =0x16001000\n\t"
	"mov     r2, #3\n\t"
	"str     r2, [r3]\n\t"
	"ldr     r3, =0x16002000\n\t"
	"mov     r2, #3\n\t"
	"str     r2, [r3]\n\t"
	"ldr     r3, =0x16002004\n\t"
	"mov     r2, #0xf8\n\t"
	"str     r2, [r3]\n\t"
#endif
	"ldr  r3, [%0]  \n\t"
	"ldr  r4, [%1]  \n\t"
	"ldr  r2, [%2]  \n\t"
	:
	: "r"(&ramdisk_start),"r"(&ramdisk_len),"r"(&cmd_ptr)
	: "r3","r4","r2"
	);

#if defined(CONFIG_MSTAR_BOOT_WITH_CPU_EL2_STATE)
    // 0xe1400070 = hvc #0
	__asm__ __volatile__(
	".word 0xe1400070\n":
	);
#else
	__asm__ __volatile__("smc #0": );
#endif
#endif

#else //Kernel 32bit
#if defined(CONFIG_ARMV7_NONSEC) || defined(CONFIG_ARMV7_VIRT) //AMP Nuttx flow
	do_nonsec_virt_switch();
	kernel_entry(0, machid, bd->bi_boot_params);
#elif defined(CONFIG_ARMv7_OPTEE) && !defined(CONFIG_MSTAR_NS_UBOOT) && !defined(CONFIG_OPTEE_TEELOADER)//SMP ARMv8 optee flow with 32bit kernel
	optee_entry entry;
	entry = (optee_entry) optee_real_va_addr; //here have to modify by tee.bin
	entry(0, machid, bd->bi_boot_params,(unsigned long)kernel_entry); //r0 r1 r2 r3=kernel_entry
	asm volatile( //because optee return to mboot will entry sm_set_nsec_ret_vals so we have to move back
		"mov r3,r2\n\t"
		"mov r2,r1\n\t"
		"mov r1,r0\n\t"
		"ldr r0,=0x0\n\t"
		"mov pc,r3\n\t":::
	);
#elif defined(CONFIG_ARMv8_ARM_TRUSTED_FIRMWARE) && !defined(CONFIG_MSTAR_NS_UBOOT) && !defined(CONFIG_OPTEE_TEELOADER) //SMP ARMv8 optee flow with 32bit kernel
	ARM_FW_Enter_32(machid, bd->bi_boot_params);
#else // ARMv8 and ARMv7 Normal boot flow with 32bit kernel & NS_UBOOT
	kernel_entry(0, machid, bd->bi_boot_params);
	/* does not return */
#endif
#endif
	return 1;
}

#if defined(CONFIG_OF_LIBFDT) && !defined(CONFIG_UFBL_FEATURE_IDME)
static int fixup_memory_node(void *blob)
{
	bd_t	*bd = gd->bd;
	int bank;
	u64 start[CONFIG_NR_DRAM_BANKS];
	u64 size[CONFIG_NR_DRAM_BANKS];

	for (bank = 0; bank < CONFIG_NR_DRAM_BANKS; bank++) {
		start[bank] = bd->bi_dram[bank].start;
		size[bank] = bd->bi_dram[bank].size;
	}

	return fdt_fixup_memory_banks(blob, start, size, CONFIG_NR_DRAM_BANKS);
}

static int fdt_add_all_memory_node(void *blob)
{
    char* s = NULL;
    int i = 0;
    u64 start[CONFIG_NR_DRAM_BANKS];
    u64 size[CONFIG_NR_DRAM_BANKS];
    int bank = 0;
    char tmp_buf = {0};
    char nodep[100] = {0};		/* new node to add */
    int  nodeoffset;	/* node offset from libfdt */
    int  ret;		/* return value */
    int err = 0;
    s = getenv("MAP_TYPE");
    if( s == NULL)
    {
        for(i=CONFIG_NR_DRAM_NODES; i > 0; i--)
        {
            if(i==1)
               strcpy(tmp_buf, "E_LX_MEM_ADR");
            else
                sprintf(tmp_buf, "E_LX_MEM%d_ADR", i);

            printf("Get %s\n", tmp_buf);

            s = getenv(tmp_buf);
            if( s == NULL)
            {
                continue;
            }

            start[bank] = CONFIG_SYS_MIU0_BUS + simple_strtoull (s, NULL, 16);

            if(i==1)
                strcpy(tmp_buf, "E_LX_MEM_LEN");
            else
                sprintf(tmp_buf, "E_LX_MEM%d_LEN", i);

            printf("Get %s\n", tmp_buf);

            s = getenv(tmp_buf);
            if( s == NULL)
            {
                continue;
            }

            size[bank] = simple_strtoull (s, NULL, 16);

            sprintf(nodep, "memory@%8X", start[bank]);
            nodeoffset = fdt_path_offset (blob, "/");
            if (nodeoffset < 0) {
                /*
                 * Not found or something else bad happened.
                 */
                printf ("libfdt fdt_path_offset() returned %s\n",
                    fdt_strerror(nodeoffset));
                return 1;
            }

            err = fdt_add_subnode(blob, 0, nodep);
            if (err < 0) {
                printf ("libfdt fdt_add_subnode(): %s\n",
                    fdt_strerror(err));
                return 1;
            }

            fdt_add_new_memory_banks(blob, start, size, CONFIG_NR_DRAM_BANKS, nodep);
        }
    }
    else
    {
		if( strcmp(s, "MI") == 0 )
	    {
	    	for(i=CONFIG_NR_DRAM_NODES; i > 0; i--)
	    	{
		    	sprintf(tmp_buf, "MI_KERNEL_POOL%d_ADR", i);
		        s = getenv(tmp_buf);
		        if( s == NULL)
		            continue;

		        start[bank] = CONFIG_SYS_MIU0_BUS + simple_strtoull (s, NULL, 16);

		        sprintf(tmp_buf, "MI_KERNEL_POOL%d_LEN", i);
		        s = getenv(tmp_buf);
		        if( s == NULL)
		        	continue;

		        size[bank] = simple_strtoull (s, NULL, 16);

		        sprintf(nodep, "memory@%8X", start[bank]);
		        nodeoffset = fdt_path_offset (blob, "/");
				if (nodeoffset < 0) {
					/*
					 * Not found or something else bad happened.
					 */
					printf ("libfdt fdt_path_offset() returned %s\n",
						fdt_strerror(nodeoffset));
					return 1;
				}

				err = fdt_add_subnode(blob, 0, nodep);
				if (err < 0) {
					printf ("libfdt fdt_add_subnode(): %s\n",
						fdt_strerror(err));
					return 1;
				}

				fdt_add_new_memory_banks(blob, start, size, CONFIG_NR_DRAM_BANKS, nodep);
	    	}
	    }
    }
    return 0;
}

static int set_seailno_to_dts()
{
    int ret = 0;
    uint16_t did[8] = {0};
    char did_buf[32] = {0};
    char cmd_buf[128] = {0};
    memset(did, 0, 8);
    memset(did_buf, 0, 32);

    char *pEnv = NULL;
    pEnv = getenv("serialno");
    if (pEnv != NULL)
    {
        int did_size = strlen(pEnv) > 31 ? 31 : strlen(pEnv);
        memcpy(did_buf, pEnv, did_size);
    }
    else
    {
        ret = MDrv_SYS_GetEfuseDid(&did);
        if(ret != 1)
        {
            printf("MDrv_SYS_GetEfuseDid fail\n");
            return -1;
        }

        snprintf(did_buf, 10, "%02x%02x%02x%02x%02x", did[0], did[1], did[2], did[3], did[4]);
	}

    memset(cmd_buf, 0, 128);
    snprintf(cmd_buf, 128, "fdt addr 0x%x", CONFIG_DTB_ADDR);
    run_command(cmd_buf, 0);

    memset(cmd_buf, 0, 128);
    snprintf(cmd_buf, 128, "fdt set /firmware/android/ serialno %s", did_buf);
    run_command(cmd_buf, 0);
    return 0;
}

static int bootm_linux_fdt(int machid, bootm_headers_t *images)
{
	ulong rd_len;
	void (*kernel_entry)(int zero, int dt_machid, void *dtblob);
	ulong of_size = images->ft_len;
	char **of_flat_tree = &images->ft_addr;
	ulong *initrd_start = &images->initrd_start;
	ulong *initrd_end = &images->initrd_end;
	struct lmb *lmb = &images->lmb;
	int ret;
	extern unsigned long gOpteeFailFlag;
	extern ulong ramdisk_start,ramdisk_len;


	kernel_entry = (void (*)(int, int, void *))images->ep;

	boot_fdt_add_mem_rsv_regions(lmb, *of_flat_tree);

	rd_len = images->rd_end - images->rd_start;

#if 0
	ret = boot_ramdisk_high(lmb, images->rd_start, rd_len,
				initrd_start, initrd_end);
	if (ret)
		return ret;
#endif

    #ifdef CONFIG_SUPPORT_DTO
    ret = boot_ufdt_overlay(of_flat_tree, &of_size);
    if (ret)
    {
        printf("## %s: Merge DTBO to main DTB fail!\n",__FUNCTION__);
        //return ret;
    }
    #endif

	ret = boot_relocate_fdt(lmb, of_flat_tree, &of_size);
	if (ret)
		return ret;

	debug("## Transferring control to Linux (at address %08lx) ...\n",
	       (ulong) kernel_entry);

	fdt_chosen(*of_flat_tree, 1);

#if 0
	fixup_memory_node(*of_flat_tree);
#endif

        ret = fdt_add_all_memory_node(*of_flat_tree);
        if (ret)
        {                                         
            printf("## %s: fdt_add_all_memory_node fail!\n",__FUNCTION__);
            //return ret;
        }

	fdt_initrd(*of_flat_tree, ramdisk_start, ramdisk_start +ramdisk_len, 1);

	// copy to CONFIG_DTB_ADDR
	memcpy(CONFIG_DTB_ADDR, *of_flat_tree, 0x100000);

	uint32_t new_dtb_addr = CONFIG_DTB_ADDR;
	//ramdisk_start = CONFIG_RAMDISK_ADDR;

    ret = set_seailno_to_dts();
    if (ret)
    {
        printf("No serialno updated in dtb!\n");
    }

    char *get_dto_debug = getenv("dto_debug");
    if(get_dto_debug != NULL)
    {
        if(strncmp(get_dto_debug, "TRUE", 4)==0)
        {
            run_command("fdt addr 0x23000000", 0);
            run_command("fdt print", 0);
        }
    }

    announce_and_cleanup();

#if defined (CONFIG_ARMv8_64BIT_KERNEL) //Kernel 64bit
    //there is no atags pointer within 64 bit kernel, so we need to pass bootargs to kernel
    printf("set 64 bit mode \n");
#if defined(CONFIG_ARMv8_ARM_TRUSTED_FIRMWARE) && !defined(CONFIG_SECARM_OPTEE)
  if(gOpteeFailFlag == 0)
  {
    ARM_FW_Enter_64(new_dtb_addr,ramdisk_start,ramdisk_len);
	}
	else
	{
		//Write gOpteeFailFlag flag back to 0x103382 since this bank may be overwritten by other APP
		Write2Byte(0x103382,0xEEFF);
	    __asm__ __volatile__(
	    #if (CONFIG_GIC_PATCH == 1)
	      "ldr     r3, =0x16001080\n\t"
	      "ldr     r2, [r3]\n\t"
	      "ldr     r4, =(0x3 << 30)\n\t"
	      "orr     r2, r2, r4\n\t"
	      "str     r2, [r3]\n\t"
	      "ldr     r3, =0x16001000\n\t"
	      "mov     r2, #3\n\t"
	      "str     r2, [r3]\n\t"
	      "ldr     r3, =0x16002000\n\t"
	      "mov     r2, #3\n\t"
	      "str     r2, [r3]\n\t"
	      "ldr     r3, =0x16002004\n\t"
	      "mov     r2, #0xf8\n\t"
	      "str     r2, [r3]\n\t"
	    #endif
	      "ldr  r3, [%0]  \n\t"
	      "ldr  r4, [%1]  \n\t"
	      "ldr  r2, [%2]  \n\t"
	      :
	      : "r"(&ramdisk_start),"r"(&ramdisk_len),"r"(of_flat_tree)
	      : "r3","r4","r2"
	      );
	      __asm__ __volatile__("smc #0": );
	}
#else //ARMv8 Normal boot flow with 64bit kernel
	__asm__ __volatile__(
#if (CONFIG_GIC_PATCH == 1)
	"ldr     r3, =0x16001080\n\t"
	"ldr     r2, [r3]\n\t"
	"ldr     r4, =(0x3 << 30)\n\t"
	"orr     r2, r2, r4\n\t"
	"str     r2, [r3]\n\t"
	"ldr     r3, =0x16001000\n\t"
	"mov     r2, #3\n\t"
	"str     r2, [r3]\n\t"
	"ldr     r3, =0x16002000\n\t"
	"mov     r2, #3\n\t"
	"str     r2, [r3]\n\t"
	"ldr     r3, =0x16002004\n\t"
	"mov     r2, #0xf8\n\t"
	"str     r2, [r3]\n\t"
#endif
	"ldr  r3, [%0]  \n\t"
	"ldr  r4, [%1]  \n\t"
	"ldr  r2, [%2]  \n\t"
	:
	: "r"(&ramdisk_start),"r"(&ramdisk_len),"r"(of_flat_tree)
	: "r3","r4","r2"
	);
	__asm__ __volatile__("smc #0": );
#endif
#else //Kernel 32bit
    //while(1);
	kernel_entry(0, machid, &new_dtb_addr);
#endif

	/* does not return */

	return 1;
}
#endif

#if defined (CONFIG_SETUP_MEMORY_TAGS) || \
    defined (CONFIG_CMDLINE_TAG) || \
    defined (CONFIG_INITRD_TAG) || \
    defined (CONFIG_SERIAL_TAG) || \
    defined (CONFIG_REVISION_TAG)
static void setup_start_tag (bd_t *bd)
{
	params = (struct tag *) bd->bi_boot_params;

	params->hdr.tag = ATAG_CORE;
	params->hdr.size = tag_size (tag_core);

	params->u.core.flags = 0;
	params->u.core.pagesize = 0;
	params->u.core.rootdev = 0;

	params = tag_next (params);
}


#ifdef CONFIG_SETUP_MEMORY_TAGS
static void setup_memory_tags (bd_t *bd)
{
	int i;

	for (i = 0; i < CONFIG_NR_DRAM_BANKS; i++) {
		params->hdr.tag = ATAG_MEM;
		params->hdr.size = tag_size (tag_mem32);

		params->u.mem.start = bd->bi_dram[i].start;
		params->u.mem.size = bd->bi_dram[i].size;

		params = tag_next (params);
	}
}
#endif /* CONFIG_SETUP_MEMORY_TAGS */


static void setup_commandline_tag (bd_t *bd, char *commandline)
{
	char *p;

	if (!commandline)
		return;

	/* eat leading white space */
	for (p = commandline; *p == ' '; p++);

	/* skip non-existent command lines so the kernel will still
	 * use its default command line.
	 */
	if (*p == '\0')
		return;

	params->hdr.tag = ATAG_CMDLINE;
	params->hdr.size =
		(sizeof (struct tag_header) + strlen (p) + 1 + 4) >> 2;

	strcpy (params->u.cmdline.cmdline, p);

	params = tag_next (params);
}


#ifdef CONFIG_INITRD_TAG
static void setup_initrd_tag (bd_t *bd, ulong initrd_start, ulong initrd_end)
{
	/* an ATAG_INITRD node tells the kernel where the compressed
	 * ramdisk can be found. ATAG_RDIMG is a better name, actually.
	 */
	params->hdr.tag = ATAG_INITRD2;
	params->hdr.size = tag_size (tag_initrd);

	params->u.initrd.start = initrd_start;
	params->u.initrd.size = initrd_end - initrd_start;

	params = tag_next (params);
}
#endif /* CONFIG_INITRD_TAG */

#ifdef CONFIG_SERIAL_TAG
void setup_serial_tag (struct tag **tmp)
{
	struct tag *params = *tmp;
	struct tag_serialnr serialnr;
	void get_board_serial(struct tag_serialnr *serialnr);

	get_board_serial(&serialnr);
	params->hdr.tag = ATAG_SERIAL;
	params->hdr.size = tag_size (tag_serialnr);
	params->u.serialnr.low = serialnr.low;
	params->u.serialnr.high= serialnr.high;
	params = tag_next (params);
	*tmp = params;
}
#endif

#ifdef CONFIG_REVISION_TAG
void setup_revision_tag(struct tag **in_params)
{
	u32 rev = 0;
	u32 get_board_rev(void);

	rev = get_board_rev();
	params->hdr.tag = ATAG_REVISION;
	params->hdr.size = tag_size (tag_revision);
	params->u.revision.rev = rev;
	params = tag_next (params);
}
#endif  /* CONFIG_REVISION_TAG */

static void setup_end_tag (bd_t *bd)
{
	params->hdr.tag = ATAG_NONE;
	params->hdr.size = 0;
}
#endif /* CONFIG_SETUP_MEMORY_TAGS || CONFIG_CMDLINE_TAG || CONFIG_INITRD_TAG */

static ulong get_sp(void)
{
	ulong ret;

	asm("mov %0, sp" : "=r"(ret) : );
	return ret;
}
