#include <config.h>
#include <common.h>
#include <errno.h>
#include <malloc.h>
#include <command.h>
#include <net.h>
#include <image-sparse.h>
#include <fb_mmc.h>
#include <MsMmc.h>
#include <MsRawIO.h>
#include <mmc.h>
#if defined(CONFIG_ANDROID_AVB_ENABLE)
#include "avb_ops_uboot.h"
#endif

#define U32 unsigned int
#define FASTBOOT_VERSION       "0.4"
#define FASTBOOT_RESPONSE_LEN  65    //64 defined bytes+'\0'
#define kDefaultPort           5554
#define kDefaultIP             "192.168.1.101"
#define kProduct               "UNDEFINED"
#define kSerialNo              "1234567890"
#define CMD_BUF                128

static unsigned short kProtocolVersion=1;

#if defined(CONFIG_FASTBOOT_GOOGLE)
#define kHeaderSize 4
#else
// fastboot udp packet header for amazon
#define kHeaderSize 8
#endif

static int critical_lock = 1;

// header fields
typedef enum {
    kIndexId = 0,
    kIndexFlags = 1,
    kIndexSeqH = 2,
    kIndexSeqL = 3,
    kIndexNumHH = 4,
    kIndexNumHL = 5,
    kIndexNumLH = 6,
    kIndexNumLL = 7
} Index;
// packet type id
typedef enum {
    kIdError = 0,
    kIdDeviceQuery = 1,
    kIdInitialization = 2,
    kIdFastboot = 3
} Id;
// continuation flag
typedef enum {
    kFlagNone = 0,
    kFlagContinuation = 1
} Flag;


//// fastboot command status
// status type
typedef enum {
    kStatusError = 0,
    kStatusInfo  = 1,
    kStatusOkay  = 2,
    kStatusData  = 3
} StatusCode;
// status structure
typedef struct fb_status_t {
   StatusCode status_code;
   char       message[FASTBOOT_RESPONSE_LEN];
} fb_status_t;
// current status
static fb_status_t fb_status;

//// download status
static int fb_download_flag = 0;
typedef struct fb_download_t {
    U32 total;
    U32 received;
    U32 max_num;
    U32 last_size;
} fb_download_t;
// download status
static fb_download_t fb_download;
// for download progress
static int pkt_count = 0;


//// ip & port
static IPaddr_t fb_remote_ip;
static int      fb_remote_port;
static int      fb_udp_our_port = kDefaultPort;


struct fastboot_var {
    struct fastboot_var *next;
    char name[64];
    char value[64];
    int (*handle)(const char *arg,const char *data,int data_length);
};

// buffer for sending response back
char fb_packet[FASTBOOT_RESPONSE_LEN];

// expected next packet sequence
unsigned short fb_expected_seq = 0;

// RAM address to hold the downloaded image
U32 download_addr = 0x20200000;

// for reboot command
static int fb_need_reset = 0;

#if defined(CONFIG_FASTBOOT_GOOGLE)
// receive flag bitmap
#define RECV_BITMAP_SIZE (32*1024) //can support 16*1024*8 packet
#else
#define RECV_BITMAP_SIZE (16*1024) //can support 16*1024*8 packet
#endif

#if defined(CONFIG_FASTBOOT_GOOGLE)
#define INVALID_NUM   (0x00000000)
#else
#define INVALID_NUM   (0xFFFFFFFF)
#endif

#define BARRIER_NUM   1000
static unsigned char *receive_bitmap = NULL;

// MAX_DEFINED_SIZE is 200MB, MAX_DOWNLOAD_SIZE needs to less than MAX_DEFINED_SIZE
// We use MAX_DEFINED_SIZE to get download address
// and use MAX_DOWNLOAD_SIZE to control fastboot.exe transmission size
#define MAX_DEFINED_SIZE  (200*1024*1024)  // 200M
#define MAX_DOWNLOAD_SIZE (100*1024*1024)  // 100M

#if defined(CONFIG_FASTBOOT_GOOGLE)
// payload size 1024+4
#define PAYLOAD_SIZE 1028
#else
#define PAYLOAD_SIZE 1400
#endif

// sparse image
struct fb_mmc_sparse {
    struct blk_desc *dev_desc;
};

U32 g_packet_num = 0;

U32 g_first_data_packet_received = 0;
U32 g_first_data_packet_seq = 0;
U32 g_chunking_packet_seq = 0;
U32 g_index_count = 0;

static const unsigned char wipedata_bin[] = {
    0x62, 0x6f, 0x6f, 0x74, 0x2d, 0x72, 0x65, 0x63, 0x6f, 0x76, 0x65, 0x72,
    0x79, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x72, 0x65, 0x63, 0x6f, 0x76, 0x65, 0x72, 0x79,
    0x0a, 0x2d, 0x2d, 0x77, 0x69, 0x70, 0x65, 0x5f, 0x64, 0x61, 0x74, 0x61,
    0x0a
};

static unsigned short max_packet_len = PAYLOAD_SIZE-kHeaderSize;

#ifdef CONFIG_ANDROID_SUPPORT_AB_UPDATE
static int cb_setActive(const char *data, unsigned int data_length);
static int do_getvar_slotSuffixes(const char *arg, const char *data, unsigned int data_length);
static int do_getvar_hasSlot(const char *arg, const char *data, unsigned int data_length);
static int do_getvar_currentSlot(const char *arg, const char *data, unsigned int data_length);
static int do_getvar_slotSuccessful(const char *arg, const char *data, unsigned int data_length);
static int do_getvar_slotUnbootable(const char *arg, const char *data, unsigned int data_length);
static int do_getvar_slotRetryCount(const char *arg, const char *data, unsigned int data_length);
#endif

struct fastboot_var *varlist;
#ifdef CONFIG_USB_FASTBOOT_ENABLE
static void fastboot_publish(char *name, char *value, void (*handle)(const char,const char,int))
{
}
#else
void fastboot_publish(char *name, char *value, void (*handle)(const char,const char,int))
{
  struct fastboot_var *var;
  var = malloc(sizeof(*var));
  memset(var, 0, sizeof(*var));
  if (var) {
    strncpy(var->name,name, sizeof(var->name)-1);
    strncpy(var->value,value, sizeof(var->value)-1);
    var->handle = handle;
    var->next = varlist;
    varlist = var;
  }
}
#endif
//// Utilities

// utility to check if s2 starts with s1
static int strcmp_l1(const char *s1, const char *s2)
{
    if (!s1 || !s2)
        return -1;
    return strncmp(s1, s2, strlen(s1));
}

// Extracts a big-endian uint16_t from a byte array.
unsigned short extract_uint16(const uchar* bytes)
{
    return ((unsigned short)(bytes[0]) << 8) | bytes[1];
}

// Put a big-endian uint16_t to a byte array.
void put_uint16(uchar* bytes, unsigned short value)
{
    bytes[0] = value >> 8;
    bytes[1] = value;
}

// Extracts a big-endian uint32_t from a byte array.
U32 extract_uint32(const uchar* bytes)
{
    return ((U32)(bytes[0]) << 24) | ((U32)(bytes[1]) << 16) |
           ((U32)(bytes[2]) << 8) | bytes[3];
}

// Put a big-endian uint32_t to a byte array.
void put_uint32(uchar* bytes, U32 value)
{
    bytes[0] = value >> 24;
    bytes[1] = value >> 16;
    bytes[2] = value >> 8;
    bytes[3] = value;
}

void header_set(uchar* header, uchar id, unsigned short sequence,
                Flag flag, U32 packetnum)
{
    header[kIndexId] = id;
    header[kIndexFlags] = flag;
    header[kIndexSeqH] = sequence >> 8;
    header[kIndexSeqL] = sequence;
    header[kIndexNumHH] = packetnum >> 24;
    header[kIndexNumHL] = packetnum >> 16;
    header[kIndexNumLH] = packetnum >> 8;
    header[kIndexNumLL] = packetnum;
}

void set_receive_bitmap(int index)
{
   receive_bitmap[index/8] |= 1 << index % 8;
}

int get_receive_bitmap(int index)
{
   if ((receive_bitmap[index/8] & (1 << index%8)) != 0)  {
      return 1;
   } else {
      return 0;
   }
}

int first_unset_bit(void)
{
   int index = 0;
   int bit_index = 0;
   while ((index < RECV_BITMAP_SIZE)&&(receive_bitmap[index] == 0xFF)) {
      index++;
   }
   if (index < RECV_BITMAP_SIZE) {
      while ((bit_index < 8) &&
             ((receive_bitmap[index] & (1 << bit_index)) != 0)) {
         bit_index ++;
      }
   }
   return index * 8 + bit_index;
}

extern char *get_script_next_line(char **line_buf_ptr);

int util_run_script(char* buffer, unsigned buffer_len)
{
    char* script_buf = 0;
    char* next_line = 0;
    int BeAbort = 1;
    char* tmp = 0;

    // Set commmand abort or not when cmd error.
    tmp = getenv ("CmdAbort");
    if (tmp != NULL)
    {
        BeAbort = (int)simple_strtol(tmp, NULL, 10);
        printf("BeAbort=%d\n",BeAbort);
    }

    script_buf = buffer;
    script_buf[buffer_len]='\n'; script_buf[buffer_len+1]='%';
    while ((next_line = get_script_next_line(&script_buf)) != 0)
    {
        printf("\n>> %s \n", next_line);
        if(-1 == run_command(next_line, 0))
        {
            printf("command error!\n");
            if(BeAbort == 1)
            {
               return -1;
            }
        }
    }
    return 0;
}

static void get_download_addr()
{
    // Just for initialize, these values may be change after calculate
    U32 download_start_address = 0x20200000;
    U32 download_end_address = 0x20200000;

    const U32 MIU_OFFSET = 0x20000000;
    U32 mboot_address = (CONFIG_DRAM_SIZE_MB_RELOCATE*1024*1024) + MIU_OFFSET;

    // check if the mboot address < 256MB, ex. 137MB
    // use 256MB~512MB
    if( mboot_address < (MAX_DEFINED_SIZE + MIU_OFFSET) )
    {
        download_start_address = 0x10000000+MIU_OFFSET;
        download_end_address = download_start_address + MAX_DOWNLOAD_SIZE - 1;
    }
    else
    {
        // if the mboot address > 256MB, ex. 305MB
        // use 2MB~256MB
        download_start_address = 0x200000+MIU_OFFSET;
        download_end_address = download_start_address + MAX_DOWNLOAD_SIZE - 1;
    }

    // set address into global download_addr var.
    download_addr = download_start_address;
}

static void fb_init(void)
{
    fb_download_flag = 0;
    fb_download.total = 0;
    fb_download.received = 0;
    fb_download.max_num = 0;
    fb_download.last_size = 0;
    fb_need_reset = 0;
    fb_status.status_code = kStatusOkay;
    memset(fb_status.message, 0, FASTBOOT_RESPONSE_LEN);
    memset(fb_packet, 0, FASTBOOT_RESPONSE_LEN);

    get_download_addr();

    //Client Variables for getvar:%s
    fastboot_publish("version", FASTBOOT_VERSION, NULL);
    fastboot_publish("serialno", kSerialNo, NULL);
    fastboot_publish("version-bootloader", "1.1", NULL);
    fastboot_publish("version-baseband", "2.0",NULL);
    #ifdef MODEL_NAME
    fastboot_publish("product", MODEL_NAME, NULL);
    #else
    fastboot_publish("product", kProduct, NULL);
    #endif
    fastboot_publish("secure", "no", NULL);
    fastboot_publish("unlocked", "yes", NULL);
    fastboot_publish("max-download-size", "0x6400000",NULL); //100MB.
    #ifdef CONFIG_ANDROID_SUPPORT_AB_UPDATE
    fastboot_publish("slot-count", "2",NULL);
    fastboot_publish("slot-suffixes", "",do_getvar_slotSuffixes);
    fastboot_publish("has-slot:", "",do_getvar_hasSlot);
    fastboot_publish("current-slot", "",do_getvar_currentSlot);
    fastboot_publish("slot-successful:", "",do_getvar_slotSuccessful);
    fastboot_publish("slot-unbootable:", "",do_getvar_slotUnbootable);
    fastboot_publish("slot-retry-count:", "",do_getvar_slotRetryCount);
    #endif
}

// utility to generate response based on fb_status to generate the response
void fb_gen_resp(char *response)
{
    char* temp_response;
    temp_response = response + kHeaderSize;
    switch (fb_status.status_code) {
        case kStatusError:
        strncpy(temp_response, "FAIL\0", 5);
        break;
        case kStatusInfo:
        strncpy(temp_response, "INFO\0", 5);
        break;
        case kStatusData:
        strncpy(temp_response, "DATA\0", 5);
        break;
        default:
        strncpy(temp_response, "OKAY\0", 5);
        break;
    }
    strncat(temp_response, fb_status.message, FASTBOOT_RESPONSE_LEN - 4 - 1);
}

void fb_send(char* buffer, unsigned buffer_len)
{
    memcpy((void*)(NetTxPacket + NetEthHdrSize() + IP_HDR_SIZE), buffer, buffer_len);

    // have to use the remote port that we get from udp handler, it is changing
    NetSendUDPPacket(NetServerEther, fb_remote_ip, fb_remote_port,
                    fb_udp_our_port, buffer_len);
}

static void TrimLeft(char *pDest, const char *pSrc, int len)
{
    const char *t = pSrc;
    while (*(t++) == ' ');
    strncpy(pDest, t - 1, len-1);
}


#ifdef  CONFIG_ANDROID_SECURE_BOOT_CLASS_B
static void EraseUserData(void)
{
    int ret1 = 0;
    unsigned int sz = sizeof(wipedata_bin) / sizeof(wipedata_bin[0]);

    raw_io_config_push();
    ret1 = raw_io_config(FLASH_RAW_DATA_TARGET, "misc", UNNECESSARY_VOLUME);

    if(ret1 != 0)
    {
        printf("raw_io_config setting fail!\n");
    }

    printf("begin erase userdata.\n");
    ret1 = raw_write((const char *)wipedata_bin, 0, sz);
    if(ret1 != 0)
    {
        printf("raw_read misc fail!\n");
    }

    raw_io_config_pop();
    run_command("mmc erase.p userdata", 0);
    printf("end erase userdata.\n");
}
#endif //CONFIG_ANDROID_SECURE_BOOT_CLASS_B

#ifdef CONFIG_ANDROID_SUPPORT_AB_UPDATE
static int cb_setActive(const char *data, unsigned int data_length)
{
    unsigned int slotNumber = 0;

    if(*data != 'a' && *data != 'b')
    {
        strncpy(fb_status.message, "Unsupported slot!!", sizeof(fb_status.message)-1);
        fastboot_fail("Unsupported slot!!");
        return -1;
    }

    slotNumber = *data -'a';

    printf("%s:%d:%s: fastboot set active of slot number %d.\n",__FILE__,__LINE__,__FUNCTION__, slotNumber);
    if(ab_setActiveSlot(slotNumber) == 0) //0 means set OK.
    {
        strncpy(fb_status.message, "Done!!", sizeof(fb_status.message)-1);
        fastboot_okay("Done!!");
        return 0;
    }
    else
    {
        strncpy(fb_status.message, "Unknown error!!", sizeof(fb_status.message)-1);
        fastboot_fail("Unknown error!!");
        return -1;
    }
}

static int do_getvar_slotSuffixes(const char *arg, const char *data, unsigned int data_length)
{
    char slotSuffix[4] = {0};

    if(!strcmp(data, "all"))
    {
        printf("%s:%d:%s: Unsupport parameter: all\n",__FILE__,__LINE__,__FUNCTION__);
        strncpy(fb_status.message, "Unsupport parameter: all", sizeof(fb_status.message)-1);
        return -1;
    }

    if(ab_appendSlotSuffix(slotSuffix,""))
    {
        strncpy(fb_status.message, slotSuffix, sizeof(fb_status.message)-1);
        fastboot_okay(slotSuffix);
        return 0;
    }
    else
    {
        strncpy(fb_status.message, "Unknown error!!", sizeof(fb_status.message)-1);
        fastboot_fail("Unknown error!!");
        return -1;
    }
}

static int do_getvar_hasSlot(const char *arg, const char *data, unsigned int data_length)
{
    char partition_a[64] = {0};
    char partition_b[64] = {0};
    int pnum;
    struct mtd_device *dev;
    struct part_info *part;

    if(!strcmp(data, "all"))
    {
        printf("%s:%d:%s: Unsupport parameter: all\n",__FILE__,__LINE__,__FUNCTION__);
        strncpy(fb_status.message, "Unsupport parameter: all", sizeof(fb_status.message)-1);
        return -1;
    }

    memcpy(partition_a, data, data_length);
    strncat(partition_a, "_a", sizeof(partition_a)-strlen(partition_a)-1);
    memcpy(partition_b, data, data_length);
    strncat(partition_b, "_b", sizeof(partition_b)-strlen(partition_b)-1);
    if(avb_get_part_number(partition_a ,&pnum) == 0 && avb_get_part_number(partition_b ,&pnum) == 0)
    {
      strncpy(fb_status.message, "yes", sizeof(fb_status.message)-1);
      fastboot_okay("yes");
    }
    else
    {
      strncpy(fb_status.message, "no", sizeof(fb_status.message)-1);
      fastboot_okay("no");
    }
    return 0;
}

static int do_getvar_currentSlot(const char *arg, const char *data, unsigned int data_length)
{
    char str[32] = {0};
    int currentSlot = 0;

    if(!strcmp(data, "all"))
    {
        printf("%s:%d:%s: Unsupport parameter: all\n",__FILE__,__LINE__,__FUNCTION__);
        strncpy(fb_status.message, "Unsupport parameter: all", sizeof(fb_status.message)-1);
        return -1;
    }

    currentSlot = ab_getCurrentSlotIndex();
    if(currentSlot >= 0)
    {
        snprintf(str, sizeof(str), "%c", currentSlot+'a');
        strncpy(fb_status.message, str, sizeof(fb_status.message)-1);
        fastboot_okay(str);
        return 0;
    }
    else
    {
        strncpy(fb_status.message, "Unknown error!!", sizeof(fb_status.message)-1);
        fastboot_fail("Unknown error!!");
        return -1;
    }
}

static int do_getvar_slotSuccessful(const char *arg, const char *data, unsigned int data_length)
{
    int slotNumber = 0, isSuccessful = -1;

    if(!strcmp(data, "all"))
    {
        printf("%s:%d:%s: Unsupport parameter: all\n",__FILE__,__LINE__,__FUNCTION__);
        strncpy(fb_status.message, "Unsupport parameter: all", sizeof(fb_status.message)-1);
        return -1;
    }

    if(*data != 'a' && *data != 'b')
    {
        strncpy(fb_status.message, "Unsupported slot!!", sizeof(fb_status.message)-1);
        fastboot_fail("Unsupported slot!!");
        return -1;
    }

    slotNumber = *data - 'a';
    isSuccessful = ab_getSlotSuccessful(slotNumber);
    if(isSuccessful >= 0)
    {
        if(isSuccessful)
        {
            strncpy(fb_status.message, "yes", sizeof(fb_status.message)-1);
            fastboot_okay("yes");
        }
        else
        {
            strncpy(fb_status.message, "no", sizeof(fb_status.message)-1);
            fastboot_okay("no");
        }
        return 0;
    }
    else
    {
        strncpy(fb_status.message, "Unknown error!!", sizeof(fb_status.message)-1);
        fastboot_fail("Unknown error!!");
        return -1;
    }
}

static int do_getvar_slotUnbootable(const char *arg, const char *data, unsigned int data_length)
{
    int slotNumber = 0, isUnbootable = -1;

    if(!strcmp(data, "all"))
    {
        printf("%s:%d:%s: Unsupport parameter: all\n",__FILE__,__LINE__,__FUNCTION__);
        strncpy(fb_status.message, "Unsupport parameter: all", sizeof(fb_status.message)-1);
        return -1;
    }

    if(*data != 'a' && *data != 'b')
    {
        strncpy(fb_status.message, "Unsupported slot!!", sizeof(fb_status.message)-1);
        fastboot_fail("Unsupported slot!!");
        return -1;
    }

    slotNumber = *data - 'a';
    isUnbootable = ab_getSlotUnbootable(slotNumber);
    if(isUnbootable >= 0)
    {
        if(isUnbootable)
        {
            strncpy(fb_status.message, "yes", sizeof(fb_status.message)-1);
            fastboot_okay("yes");
        }
        else
        {
            strncpy(fb_status.message, "no", sizeof(fb_status.message)-1);
            fastboot_okay("no");
        }
        return 0;
    }
    else
    {
        strncpy(fb_status.message, "Unknown error!!", sizeof(fb_status.message)-1);
        fastboot_fail("Unknown error!!");
        return -1;
    }
}

static int do_getvar_slotRetryCount(const char *arg, const char *data, unsigned int data_length)
{
    char str[32] = {0};
    int retryCount = 0;
    int slotNumber = 0;

    if(!strcmp(data, "all"))
    {
        printf("%s:%d:%s: Unsupport parameter: all\n",__FILE__,__LINE__,__FUNCTION__);
        strncpy(fb_status.message, "Unsupport parameter: all", sizeof(fb_status.message)-1);
        return -1;
    }

    if(*data != 'a' && *data != 'b')
    {
        strncpy(fb_status.message, "Unsupported slot!!", sizeof(fb_status.message)-1);
        fastboot_fail("Unsupported slot!!");
        return -1;
    }

    slotNumber = *data - 'a';
    retryCount = ab_getSlotRetryCount(slotNumber);
    if(retryCount >= 0)
    {
        snprintf(str, sizeof(str), "%d", retryCount);
        strncpy(fb_status.message, str, sizeof(fb_status.message)-1);
        fastboot_okay(str);
        return 0;
    }
    else
    {
        strncpy(fb_status.message, "Unknown error!!", sizeof(fb_status.message)-1);
        fastboot_fail("Unknown error!!");
        return -1;
    }
}
#endif // CONFIG_ANDROID_SUPPORT_AB_UPDATE

#if defined(CONFIG_FASTBOOT_GOOGLE)
// fastboot functions
// send an ACK, for all fastboot packet, an ACK is needed
static void fb_udp_ack(uchar *pkt, unsigned len, U32 packet_num)
{
    memcpy(fb_packet, pkt, kHeaderSize);

    put_uint16((uchar*)fb_packet+kIndexSeqH, packet_num);

    // need to clear continuation flag
    fb_packet[kIndexFlags] = kFlagNone;
    fb_send(fb_packet, kHeaderSize);
}

// send an error packet
static void fb_udp_error(void)
{
    int str_len = 0;
    header_set((uchar*)fb_packet, 0, 0, 0, INVALID_NUM);
    str_len = (strlen("packet Id is invalid")+1 < (sizeof(fb_packet)-kHeaderSize)) ? (strlen("packet Id is invalid")+1) : (sizeof(fb_packet)-kHeaderSize-1);
    strncpy(fb_packet+kHeaderSize, "packet Id is invalid", str_len);
    fb_send(fb_packet, kHeaderSize+strlen(fb_packet+kHeaderSize));
}

// reboot command handler
static void cb_reboot(char *buffer, unsigned buffer_len)
{
    fb_need_reset = 1;
}

// reboot-bootloader command handler
static void cb_reboot_bootloader(char *buffer, unsigned buffer_len)
{
    run_command("setenv reboot-bootloader 1", 0);
    run_command("saveenv", 0);
    fb_need_reset = 1;
}

// getvar command handler
int cb_getvar(char *buffer, unsigned buffer_len)
{
  struct fastboot_var *var;
  char response[64];
  printf("cb_getvar, var[%s].\n", buffer);
  int ret = 0;

  // clear the status
  fb_status.status_code = kStatusOkay;
  memset(fb_status.message, 0, FASTBOOT_RESPONSE_LEN);

  if(!strcmp(buffer, "all")){
    for (var = varlist; var; var = var->next){
      if(var->handle)
      {
        ret = var->handle(var->name, buffer, buffer_len);
      }
      else
      {
        snprintf(response, 64, "\t%s: %s", var->name, var->value);
        //fastboot_info(response);
        strncpy(fb_status.message, response, sizeof(fb_status.message)-1);
      }
    }

    fb_udp_ack(buffer, buffer_len, g_packet_num);
    fastboot_okay("Done!!");
    return 0;
  }

  for (var = varlist; var; var = var->next) {
    if (!strncmp(var->name, buffer,strlen(var->name))) {
      if(var->handle)
      {
        ret = var->handle(var->name, buffer + strlen(var->name), buffer_len - strlen(var->name));
        if(ret != 0)
          fb_status.status_code = kStatusError;
      }
      else
      {
        strncpy(fb_status.message, var->value, sizeof(fb_status.message)-1);
        fastboot_okay(var->value);
      }
      fb_udp_ack(buffer, buffer_len, g_packet_num);
      return 0;
    }
  }

  fb_status.status_code = kStatusError;
  printf("invalid var[%s].\n", buffer);
  strncpy(fb_status.message, "Variable not specified", sizeof(fb_status.message)-1);
  fastboot_fail("invalid var");
  fb_udp_ack(buffer, buffer_len, g_packet_num);
  return -1;
}

int is_device_unlocked()
{
#if defined(CONFIG_ANDROID_AVB_ENABLE)
    // Assume default state is lock state
    uint16_t devicestate = 1;
    extern int read_device_state(uint16_t *devicestate);
    read_device_state(&devicestate);

    if( devicestate == 1 )
        return 0;
    else if( devicestate == 0 )
        return 1;
    else
    {
        // unknown device state, return lock state
        printf("unknown device state: %d\n", devicestate);
        return 0;
    }
#else
    return 1;
#endif
}

static void cb_download_start(char *buffer, unsigned buffer_len)
{
    // simple_strtoul takes the last valid chars, not the first
    U32 download_size = simple_strtoul(buffer, 0, 16);

    printf("Starting download of %d bytes\n", download_size);

    // init download progress count
    pkt_count = 0;

    if (0 == download_size)
    {
       fb_status.status_code = kStatusError;
       strncpy(fb_status.message, "data invalid size", sizeof(fb_status.message)-1);
    } else if (download_size > MAX_DOWNLOAD_SIZE) {
       download_size = 0;
       fb_status.status_code = kStatusError;
       strncpy(fb_status.message, "data too large", sizeof(fb_status.message)-1);
    }
    else
    {
       fb_status.status_code = kStatusData;
       snprintf(fb_status.message, sizeof(fb_status.message), "%08x", download_size);
       fb_download_flag = 1;
       fb_download.total = download_size;
       fb_download.received = 0;
       fb_download.last_size = download_size % max_packet_len;
       fb_download.max_num = download_size/max_packet_len;
       if (fb_download.last_size == 0) {
          fb_download.last_size = max_packet_len;
       } else {
          fb_download.max_num++;
       }

       g_first_data_packet_seq = 0;
       g_first_data_packet_received = 0;
       g_index_count = 0;
       g_chunking_packet_seq = 0;

       memset(receive_bitmap, 0, RECV_BITMAP_SIZE);

       // put a 0 at the end of the buffer
       memset((char*)download_addr+fb_download.total, 0, 1);
       fb_udp_ack(buffer, buffer_len, g_packet_num);
       return;
    }
    fb_udp_ack(buffer, buffer_len, INVALID_NUM);
}

static void cb_download_cont(char *buffer, unsigned buffer_len)
{
    // started download, but not complete, change the status
    if (fb_status.status_code == kStatusData) {
       fb_status.status_code = kStatusError;
       strncpy(fb_status.message, "download incomplete", sizeof(fb_status.message)-1);
    }

    U32 packet_num = (int)extract_uint16((const uchar*)buffer+kIndexSeqH);

    // it should be a full package, otherwise ignore
    if (buffer_len != kHeaderSize + max_packet_len) {
       printf("[%d]FB download_cont, invalid packet size, num:%u, size:%d\n", __LINE__,
              packet_num, buffer_len);
       return;
    }

    fb_expected_seq = g_packet_num + 1;
    fb_udp_ack((uchar*)buffer, buffer_len, (U32)packet_num);

    // already received this packet
    if (get_receive_bitmap(g_chunking_packet_seq)) {
       printf("FB download_cont, packet %u is already received\n", g_chunking_packet_seq);
       return;
    }

    g_packet_num=packet_num;
    set_receive_bitmap(g_chunking_packet_seq);
    g_chunking_packet_seq++;

    // download progress indicator
    if (pkt_count == 8000) // start a new line every 8000 packets
    {
        printf("\n");
        pkt_count = 0;
    }
    if (!(pkt_count%100)) printf("#"); // print a # every 100 packets
        pkt_count++;

    memcpy((char*)download_addr+max_packet_len*g_index_count,
           buffer+kHeaderSize, max_packet_len);
    fb_download.received += max_packet_len;
    g_index_count++;
}

static void cb_download_end(char *buffer, unsigned buffer_len)
{
    U32 packet_num = (int)extract_uint16((const uchar*)buffer+kIndexSeqH);

    //printf("[%d]packet_num=%u\n", __LINE__, packet_num);

    int is_received = get_receive_bitmap(g_chunking_packet_seq);

    //printf("buffer_len=%d\n", buffer_len);
    //printf("kHeaderSize=%d\n", kHeaderSize);
    //printf("fb_download.max_num=%d\n", fb_download.max_num);
    //printf("fb_download.last_size=%d\n", fb_download.last_size);
    //printf("max_packet_len=%d\n", max_packet_len);
    //printf("[%d]packet_num=%u\n", __LINE__, packet_num);

    if ( ((g_chunking_packet_seq == fb_download.max_num-1) &&
          (buffer_len-kHeaderSize != fb_download.last_size)) ||
         ((g_chunking_packet_seq != fb_download.max_num-1) &&
          (buffer_len-kHeaderSize != max_packet_len)) ) {
        printf("[%d]FB download_end, invalid packet size, num:%u, size:%d\n", __LINE__,
              packet_num, buffer_len-kHeaderSize);
        printf("[%d]g_chunking_packet_seq:%u\n", __LINE__,
              g_chunking_packet_seq);
       return;  // invalid packet
    }

    if (is_received) { // already received this packet, print a warning
       printf("FB download_end, packet %u is already received\n", packet_num);
    } else {
       set_receive_bitmap(g_chunking_packet_seq);
       // use download_addr to keep the downloaded data
       memcpy((char*)download_addr+g_index_count*max_packet_len,
              buffer+kHeaderSize, buffer_len-kHeaderSize);
       fb_download.received += buffer_len-kHeaderSize;
    }

    //fb_expected_seq++;
    fb_expected_seq = g_packet_num + 1;
    if (fb_download.received == fb_download.total)
    {
        fb_status.status_code = kStatusOkay;
        memset(fb_status.message, 0, FASTBOOT_RESPONSE_LEN);

        // download done, only need to send a packet ACK,
        // not a full resposne as in the protocol description
        fb_udp_ack((uchar*)buffer, buffer_len, packet_num);

        fb_download_flag = 0;  // clear download session flag
        printf("\nEnd download, %d of %d bytes\n",
               fb_download.received, fb_download.total);
    }
    else
    {
        printf("fb_download.received=%d, fb_download.total=%d\n", fb_download.received, fb_download.total);
        U32 first_missing = (U32)first_unset_bit();
        printf("[%d] g_first_data_packet_seq=%u\n", __LINE__, g_first_data_packet_seq);
        printf("Getting missing packet, num: %u\n", first_missing);
        fb_udp_ack((uchar*)buffer, buffer_len, first_missing);
    }
}

static void cb_flash(char *buffer, unsigned buffer_len)
{
    char tmpCmd[128];
    char partition_name[64];
    memset(tmpCmd, 0, 128);
    memset(partition_name, 0, 64);
    static char last_part_name[CMD_BUF]={0};

#ifdef CONFIG_ANDROID_SECURE_BOOT

    int dev_is_unlock = is_device_unlocked();
    // dev_is_unlock, 1 is unlock, 0 is lock
    if( dev_is_unlock == 0 )
    {
        printf("device locked, forbid to write flash.\n");
        fastboot_fail("device locked, forbid to write flash.");
        fb_status.status_code = kStatusError;
        return -1;
    }
#endif//CONFIG_ANDROID_SECURE_BOOT

    if(critical_lock == 1 &&
      ( !strcmp_l1("rom_emmc_boot", buffer) || !strcmp_l1("MBOOT", buffer) ))
    {
        printf("critical locked, forbid to write flash.\n");
        fastboot_fail("critical locked, forbid to write flash.");
        fb_status.status_code = kStatusError;
        return -1;
    }

    //printf("buffer=%s\n",buffer);
    //printf("last_part_name=%s\n",last_part_name);
    if (!buffer_len)
    {
        fb_status.status_code = kStatusError;
        strncpy(fb_status.message, "missing partition name", sizeof(fb_status.message)-1);
        return;
    }

    snprintf(partition_name, buffer_len+1, buffer);

    if (!strcmp_l1("dl_addr", buffer))
    {
        printf("current downloader_add=0x%08x\n",download_addr);
        download_addr = simple_strtoul((char*)download_addr, NULL, 16);
        printf("set downloader_add=0x%08x\n",download_addr);
    }
    else if (!strcmp_l1("secure_info_boot", buffer))
    {
        snprintf(tmpCmd, 128, "store_secure_info bootSign 0x%08x", download_addr);
        printf("\n%s\n", tmpCmd);
        run_command(tmpCmd, 0);
    }
    else if (!strcmp_l1("secure_info_recovery", buffer))
    {
        snprintf(tmpCmd, 128, "store_secure_info recoverySign 0x%08x", download_addr);
        printf("\n%s\n", tmpCmd);
        run_command(tmpCmd, 0);
    }
    else if (!strcmp_l1("secure_info_tee", buffer))
    {
        snprintf(tmpCmd, 128, "store_secure_info teeSign 0x%08x", download_addr);
        printf("\n%s\n", tmpCmd);
        run_command(tmpCmd, 0);
    }
    else if (!strcmp_l1("nuttx_config", buffer))
    {
        snprintf(tmpCmd, 128, "store_nuttx_config NuttxConfig 0x%08x", download_addr);
        printf("\n%s\n", tmpCmd);
        run_command(tmpCmd, 0);
    }
    else if (!strcmp_l1("rom_emmc_boot", buffer))
    {
        snprintf(tmpCmd, 128, "mmc write.boot 1 0x%08x 0 0x%08x",
                download_addr, fb_download.received);
        printf("\n%s\n", tmpCmd);
        run_command(tmpCmd, 0);
    }
    else if (!strcmp_l1("MBOOT", buffer))
    {
#ifdef CONFIG_SPI
        snprintf(tmpCmd, 128, "spi wrc 0x%08x 0 0x%08x",
               download_addr, fb_download.received);
        printf("\n%s\n", tmpCmd);
        run_command(tmpCmd, 0);
#else
        fb_mmc_flash_write("MBOOT", download_addr, fb_download.total);
#endif
    }
    else if (!strcmp_l1("mscript", buffer))
    {
        util_run_script((char*)download_addr, fb_download.received);
    }
    // flash Image
    else if (getenv("flash_lzo")!=NULL)
    {
        if (strcmp(buffer,last_part_name)!=0)
        {
            snprintf(tmpCmd, 128, "mmc unlzo 0x%08x 0x%08x %s 1",
                    download_addr, fb_download.received, buffer);
        }
        else
        {
            snprintf(tmpCmd, 128, "mmc unlzo.cont 0x%08x 0x%08x %s 1",
                    download_addr, fb_download.received, buffer);
        }
    }
    else
    {
       fb_mmc_flash_write(partition_name, download_addr, fb_download.total);
    }

    memset(last_part_name,0,sizeof(last_part_name));
    strncpy(last_part_name, buffer, buffer_len);

    fb_udp_ack(buffer, buffer_len, g_packet_num);
    return;
}

static void cb_flashing(char* buffer, unsigned buffer_len)
{
    int ret = 0;
    char TempData[20] = {0};
    char dev_state[20] = {0};
    TrimLeft(TempData, buffer, sizeof(TempData));

#ifdef  CONFIG_ANDROID_SECURE_BOOT_CLASS_B
    // dev_is_unlock, 1 is unlock, 0 is lock
    int dev_is_unlock = is_device_unlocked();

    // printf("TempData=%s\n", TempData);
    if (strcmp_l1("get_unlock_ability", TempData) == 0)
    {
        if( dev_is_unlock == 0 )
            strncpy(dev_state, "lock", sizeof(dev_state)-1);
        else
            strncpy(dev_state, "unlock", sizeof(dev_state)-1);

        printf("%s\n", dev_state);
    }
    else if (strcmp_l1("lock_critical", TempData) == 0)
    {
        critical_lock = 1;
    }
    else if (strcmp_l1("unlock_critical", TempData) == 0)
    {
        printf("flashing bootloader-related partitions has its own risk\n");
        critical_lock = 0;
    }
    else if (strcmp_l1("lock", TempData) == 0)
    {
        // skip dev_is_unlock==0, device state not change
        if( dev_is_unlock == 1 )
        {
            printf("wipe user data\n");
            EraseUserData();
            // write lock (1)
            write_device_state(1);
        }
    }
    else if (strcmp_l1("unlock", TempData) == 0)
    {
        // skip dev_is_unlock==1, device state not change
        if( dev_is_unlock == 0 )
        {
            printf("wipe user data\n");
            EraseUserData();
            // write unlock (0)
            write_device_state(0);
        }
    }
    else
    {
        ret = -1;
    }

    if (ret == 0)
        fastboot_okay("Done!!");
    else
        fastboot_fail("invalid command");
#else
    ret = -1;
    fastboot_fail("invalid command");
#endif//CONFIG_ANDROID_SECURE_BOOT_CLASS_B
    
    return ret;
}

static void cb_erase(char* buffer, unsigned buffer_len)
{
    char tmpCmd[128] = {0};
    memset(tmpCmd, 0, 128);
    snprintf(tmpCmd, 128, "mmc erase.p %s", buffer);
    run_command(tmpCmd, 0);
}

struct cmd_dispatch_info {
    char *cmd;
    void (*cb)(char* buffer, unsigned buffer_len);
};

// download command queries:  "getvar:partition-type:boot" and
//                             getvar:max-download-size
// need to implement this two
static const struct cmd_dispatch_info cmd_dispatch_info[] =
{
    {
        .cmd = "reboot-bootloader",
        .cb = cb_reboot_bootloader,
    },
    {
        .cmd = "reboot",
        .cb = cb_reboot,
    },
    {
        .cmd = "getvar:",
        .cb = cb_getvar,
    },
    {
        .cmd = "download:",
        .cb = cb_download_start,
    },
    {
        .cmd = "flash:",
        .cb = cb_flash,
    },
    {
        .cmd = "erase:",
        .cb = cb_erase,
    },
    {
        .cmd = "flashing",
        .cb = cb_flashing,
    },
#ifdef CONFIG_ANDROID_SUPPORT_AB_UPDATE
    {
        .cmd = "set_active:",
        .cb = cb_setActive,
    },
#endif
};

static void fb_handler_command(char* buffer, unsigned buffer_len)
{
    char cmdbuf[1024];   // assuming max command length
    void (*func_cb)(char* buffer, unsigned buffer_len) = NULL;

    fb_status.status_code = kStatusOkay;
    memset(fb_status.message, 0, FASTBOOT_RESPONSE_LEN);

    if (buffer_len >= 1024)
    {
        fb_status.status_code = kStatusError;
        printf("command too long: %d\n", buffer_len);
        strncpy(fb_status.message, "command too long", sizeof(fb_status.message)-1);
        return;
    }

    // put the command in the cmdbuf and add a '\0'
    strncpy(cmdbuf, buffer, buffer_len);
    cmdbuf[buffer_len] = 0;
    printf("\ncommand: %s, length: %d\n", cmdbuf, buffer_len);

    int i;
    for (i = 0; i < ARRAY_SIZE(cmd_dispatch_info); i++)
    {
        if (!strcmp_l1(cmd_dispatch_info[i].cmd, cmdbuf))
        {
            func_cb = cmd_dispatch_info[i].cb;
            break;
        }
    }

    if (!func_cb)
    {
        fb_status.status_code = kStatusError;
        printf("unknown command: %s", cmdbuf);
        strncpy(fb_status.message, "unknown command", sizeof(fb_status.message)-1);
    }
    else
    {
        int cmdLen = strlen(cmd_dispatch_info[i].cmd);
        func_cb(cmdbuf+cmdLen, buffer_len-cmdLen);
    }
}


//************************ UDP layer ************************

int isContinued(Flag pktFlag, unsigned short pktSeq)
{
  if(pktFlag)
    return 1;

  if(fb_download.received==0)
  {
    int count = fb_download.total/(PAYLOAD_SIZE-kHeaderSize);
    if( (fb_download.total%(PAYLOAD_SIZE-kHeaderSize) != 0) && (fb_download.total >= (PAYLOAD_SIZE-kHeaderSize)) )
      count++;

    if(count > 0)
      return 1;
  }

  if((fb_download.total-fb_download.received)>(PAYLOAD_SIZE-kHeaderSize))
    return 1;

  return 0;
}

static void fb_udp_handler(uchar *pkt, unsigned dest, IPaddr_t sip, unsigned src,
                           unsigned len)
{
    // only handle the packet for us (fastboot)
    if (dest == fb_udp_our_port)
    {
        // need to keep the remote ip and port, needed when sending data back
        fb_remote_ip = sip;
        fb_remote_port = src;

        // incompleted packet
        if (len < kHeaderSize)
            return;

        // read the packet header
        Id             pktId   = pkt[kIndexId];
        Flag           pktFlag = pkt[kIndexFlags];
        unsigned short pktSeq  = extract_uint16(pkt+kIndexSeqH);

        g_packet_num = pktSeq;

        // clear the packet for response
        memset(fb_packet, 0, FASTBOOT_RESPONSE_LEN);
        switch(pktId) {
            case kIdDeviceQuery:
            memcpy(fb_packet, pkt, kHeaderSize);
            put_uint32((uchar*)(fb_packet+kIndexNumHH), INVALID_NUM);
            put_uint16((uchar*)(fb_packet+kHeaderSize), fb_expected_seq);
            fb_send(fb_packet, kHeaderSize+2);
            break;

            case kIdInitialization:
            // seq # is not correct, ignore
            if (pktSeq != fb_expected_seq)
            {
               return;
            }

            if (len < kHeaderSize + 4)
            {
                fb_udp_error();
            }
            else
            {
                unsigned short host_version = extract_uint16(pkt+kHeaderSize);
                max_packet_len = extract_uint16(pkt+kHeaderSize+2);

                printf("\n**************************************\n");
                printf("host version: %d, host max packet: %d\n",
                      host_version, max_packet_len);

                // PKTSIZE doesn't seem work, 1400 seems a good number
                //if (max_packet_len > (PKTSIZE - NetEthHdrSize() - IP_HDR_SIZE)) {
                //   max_packet_len = PKTSIZE - NetEthHdrSize() - IP_HDR_SIZE;
                //}
                if (max_packet_len > PAYLOAD_SIZE)
                {
                    max_packet_len = PAYLOAD_SIZE;
                }
                max_packet_len -= kHeaderSize;

                fb_expected_seq = g_packet_num + 1;
                memcpy(fb_packet, pkt, kHeaderSize);
                put_uint32((uchar*)(fb_packet+kIndexNumHH), INVALID_NUM);
                put_uint16((uchar*)(fb_packet+kHeaderSize), kProtocolVersion);
                put_uint16((uchar*)(fb_packet+kHeaderSize+2), max_packet_len+kHeaderSize);
                fb_send(fb_packet, kHeaderSize+4);

                // init status, prepare for fastboot commands
                fb_init();
            }
            break;

            case kIdFastboot:
            if (pktSeq != fb_expected_seq) // repeat packet
            {
                fb_udp_ack((uchar*)pkt, len, (U32)pktSeq);
                return;
            }
            if (len == kHeaderSize) // empty packet for get status
            {
                //printf("Getting an empty packet for status\n");
                fb_expected_seq = g_packet_num + 1;
                memcpy(fb_packet, pkt, kHeaderSize);
                fb_gen_resp(fb_packet);
                printf("response: %s\n", fb_packet+kHeaderSize);
                fb_send(fb_packet, kHeaderSize+strlen(fb_packet+kHeaderSize));
                // need to wait for the status packet before resetting
                if (fb_need_reset)
                {
                    printf("Start to reset\n");
                    mdelay(500);
                    run_command("reset", 0);
                }
            }
            else
            {
                if (fb_download_flag)  // in a download session
                {
                    if (isContinued(pktFlag, pktSeq))
                        cb_download_cont((char*)pkt, len);
                    else
                        cb_download_end((char*)pkt, len);
                }
                else  // other fastboot commands
                {
                    fb_expected_seq = g_packet_num + 1;
                    // send ack first
                    g_first_data_packet_seq = fb_expected_seq;
                    fb_udp_ack(pkt, len, INVALID_NUM);
                    fb_handler_command((char*)(pkt+kHeaderSize), len-kHeaderSize);
                }
            }
            break;

            case kIdError:  // fall through
            default:
            // send back a error mesage
            fb_udp_error();
            break;
        }
    }
}

#else // CONFIG_FASTBOOT_GOOGLE
// fastboot function for amazon
// send an ACK, for all fastboot packet, an ACK is needed
static void fb_udp_ack(uchar *pkt, unsigned len, U32 packet_num)
{
    memcpy(fb_packet, pkt, kHeaderSize);

    put_uint32((uchar*)fb_packet+kIndexNumHH, packet_num);

    // need to clear continuation flag
    fb_packet[kIndexFlags] = kFlagNone;
    fb_send(fb_packet, kHeaderSize);
}

// send an error packet
static void fb_udp_error(void)
{
    header_set((uchar*)fb_packet, 0, 0, 0, INVALID_NUM);
    strncpy(fb_packet+kHeaderSize, "packet Id is invalid", sizeof(fb_packet)-kHeaderSize-1);
    fb_send(fb_packet, kHeaderSize+strlen(fb_packet+kHeaderSize));
}

// reboot command handler
static void cb_reboot(char *buffer, unsigned buffer_len)
{
    fb_need_reset = 1;
}

// reboot-bootloader command handler
static void cb_reboot_bootloader(char *buffer, unsigned buffer_len)
{
    run_command("setenv reboot-bootloader 1", 0);
    run_command("saveenv", 0);
    fb_need_reset = 1;
}

// getvar command handler
static void cb_getvar(char *buffer, unsigned buffer_len)
{
    // clear the status
    fb_status.status_code = kStatusOkay;
    memset(fb_status.message, 0, FASTBOOT_RESPONSE_LEN);

    if (buffer_len)
    {
        if (!strcmp_l1("version", buffer))
        {
            strncpy(fb_status.message, FASTBOOT_VERSION, sizeof(fb_status.message)-1);
        }
        else if (!strcmp_l1("product", buffer))
        {
            strncpy(fb_status.message, kProduct, sizeof(fb_status.message)-1);
        }
        else if (!strcmp_l1("serialno", buffer))
        {
            strncpy(fb_status.message, kSerialNo, sizeof(fb_status.message)-1);
        }
        else if (!strcmp_l1("max-download-size", buffer))
        {
            snprintf(fb_status.message, sizeof(fb_status.message), "0x%08x", MAX_DOWNLOAD_SIZE);
        }
//        else if (!strcmp_l1("bootloader-version", buffer))
//        {
//            // U_BOOT_VERSION undefined
//            strncpy(fb_status.message, U_BOOT_VERSION, sizeof(fb_status.message)-1);
//        }
        else if (!strcmp_l1("partition-type:", buffer))
        {
            snprintf(fb_status.message, sizeof(fb_status.message), "Unknown");
        }
        else
        {
            fb_status.status_code = kStatusError;
            strncpy(fb_status.message, "Variable not implemented", sizeof(fb_status.message)-1);
        }
    }
    else
    {
        fb_status.status_code = kStatusError;
        strncpy(fb_status.message, "Variable not specified", sizeof(fb_status.message)-1);
    }
}

static void cb_download_start(char *buffer, unsigned buffer_len)
{
    // simple_strtoul takes the last valid chars, not the first
    U32 download_size = simple_strtoul(buffer, 0, 16);

    printf("Starting download of %d bytes\n", download_size);

    // init download progress count
    pkt_count = 0;

    if (0 == download_size)
    {
       fb_status.status_code = kStatusError;
       strncpy(fb_status.message, "data invalid size", sizeof(fb_status.message)-1);
    } else if (download_size > MAX_DOWNLOAD_SIZE) {
       download_size = 0;
       fb_status.status_code = kStatusError;
       strncpy(fb_status.message, "data too large", sizeof(fb_status.message)-1);
    }
    else
    {
       fb_status.status_code = kStatusData;
       snprintf(fb_status.message, sizeof(fb_status.message), "%08x", download_size);
       fb_download_flag = 1;
       fb_download.total = download_size;
       fb_download.received = 0;
       fb_download.last_size = download_size % max_packet_len;
       fb_download.max_num = download_size/max_packet_len;
       if (fb_download.last_size == 0) {
          fb_download.last_size = max_packet_len;
       } else {
          fb_download.max_num++;
       }
       memset(receive_bitmap, 0, RECV_BITMAP_SIZE);

       // put a 0 at the end of the buffer
       memset((char*)download_addr+fb_download.total, 0, 1);
    }
}

static void cb_download_cont(char *buffer, unsigned buffer_len)
{
    // started download, but not complete, change the status
    if (fb_status.status_code == kStatusData) {
       fb_status.status_code = kStatusError;
       strncpy(fb_status.message, "download incomplete", sizeof(fb_status.message)-1);
    }

    int packet_num = (int)extract_uint32((const uchar*)buffer+kIndexNumHH);

    // it should be a full package, otherwise ignore
    if (buffer_len != kHeaderSize + max_packet_len) {
       printf("FB download_cont, invalid packet size, num:%d, size:%d\n",
              packet_num, buffer_len);
       return;
    }

    // send an ack for every BARRIER_NUM packet
    if (!(packet_num % BARRIER_NUM)) {
       fb_expected_seq++;
       fb_udp_ack((uchar*)buffer, buffer_len, (U32)packet_num);
    }

    // already received this packet
    if (get_receive_bitmap(packet_num)) {
       //printf("FB download_cont, packet %d is already received\n", packet_num);
       debug("FB download_cont, packet %d is already received\n", packet_num);
       return;
    }

    set_receive_bitmap(packet_num);

    // download progress indicator
    if (pkt_count == 8000) // start a new line every 8000 packets
    {
        printf("\n");
        pkt_count = 0;
    }
    if (!(pkt_count%100)) printf("#"); // print a # every 100 packets
    pkt_count++;

    memcpy((char*)download_addr+max_packet_len*packet_num,
           buffer+kHeaderSize, max_packet_len);
    fb_download.received += max_packet_len;
}

static void cb_download_end(char *buffer, unsigned buffer_len)
{
    int packet_num = (int)extract_uint32((const uchar*)buffer+kIndexNumHH);
    int is_received = get_receive_bitmap(packet_num);

    if ( ((packet_num == fb_download.max_num-1) &&
          (buffer_len-kHeaderSize != fb_download.last_size)) ||
         ((packet_num != fb_download.max_num-1) &&
          (buffer_len-kHeaderSize != max_packet_len)) ) {
       printf("FB download_end, invalid packet size, num:%d, size:%d\n",
              packet_num, buffer_len-kHeaderSize);
       return;  // invalid packet
    }

    if (is_received) { // already received this packet, print a warning
       printf("FB download_end, packet %d is already received\n", packet_num);
    } else {
       set_receive_bitmap(packet_num);
       // use download_addr to keep the downloaded data
       memcpy((char*)download_addr+packet_num*max_packet_len,
              buffer+kHeaderSize, buffer_len-kHeaderSize);
       fb_download.received += buffer_len-kHeaderSize;
    }

    fb_expected_seq++;
    if (fb_download.received == fb_download.total)
    {
        fb_status.status_code = kStatusOkay;
        memset(fb_status.message, 0, FASTBOOT_RESPONSE_LEN);

        // download done, only need to send a packet ACK,
        // not a full resposne as in the protocol description
        fb_udp_ack((uchar*)buffer, buffer_len, INVALID_NUM);

        fb_download_flag = 0;  // clear download session flag
        printf("\nEnd download, %d of %d bytes\n",
               fb_download.received, fb_download.total);
    }
    else
    {
        U32 first_missing = (U32)first_unset_bit();
        //printf("Getting missing packet, num: %d\n", first_missing);
        fb_udp_ack((uchar*)buffer, buffer_len, first_missing);
    }
}

static void cb_flash(char *buffer, unsigned buffer_len)
{
    char tmpCmd[128];
    char partition_name[64];
    memset(tmpCmd, 0, 128);
    memset(partition_name, 0, 64);
    static char last_part_name[CMD_BUF]={0};

    //printf("buffer=%s\n",buffer);
    //printf("last_part_name=%s\n",last_part_name);
    if (!buffer_len)
    {
        fb_status.status_code = kStatusError;
        strncpy(fb_status.message, "missing partition name", sizeof(fb_status.message)-1);
        return;
    }

    snprintf(partition_name, buffer_len+1, buffer);

    if (!strcmp_l1("dl_addr", buffer))
    {
        printf("current downloader_add=0x%08x\n",download_addr);
        download_addr = simple_strtoul((char*)download_addr, NULL, 16);
        printf("set downloader_add=0x%08x\n",download_addr);
        return;
    }


    if (!strcmp_l1("secure_info_boot", buffer))
    {
        snprintf(tmpCmd, sizeof(tmpCmd), "store_secure_info bootSign 0x%08x", download_addr);
        printf("\n%s\n", tmpCmd);
        run_command(tmpCmd, 0);
        return;
    }

    if (!strcmp_l1("secure_info_recovery", buffer))
    {
        snprintf(tmpCmd, sizeof(tmpCmd), "store_secure_info recoverySign 0x%08x", download_addr);
        printf("\n%s\n", tmpCmd);
        run_command(tmpCmd, 0);
        return;
    }

    if (!strcmp_l1("secure_info_tee", buffer))
    {
        snprintf(tmpCmd, sizeof(tmpCmd), "store_secure_info teeSign 0x%08x", download_addr);
        printf("\n%s\n", tmpCmd);
        run_command(tmpCmd, 0);
        return;
    }

    if (!strcmp_l1("nuttx_config", buffer))
    {
        snprintf(tmpCmd, sizeof(tmpCmd), "store_nuttx_config NuttxConfig 0x%08x", download_addr);
        printf("\n%s\n", tmpCmd);
        run_command(tmpCmd, 0);
        return;
    }

    if (!strcmp_l1("rom_emmc_boot", buffer))
    {
        snprintf(tmpCmd, sizeof(tmpCmd), "mmc write.boot 1 0x%08x 0 0x%08x",
                download_addr, fb_download.received);
        printf("\n%s\n", tmpCmd);
        run_command(tmpCmd, 0);
        return;
    }

    if (!strcmp_l1("MBOOT", buffer))
    {
#ifdef CONFIG_SPI
        snprintf(tmpCmd, sizeof(tmpCmd), "spi wrc 0x%08x 0 0x%08x",
               download_addr, fb_download.received);
        printf("\n%s\n", tmpCmd);
        run_command(tmpCmd, 0);
#else
        fb_mmc_flash_write("MBOOT", download_addr, fb_download.total);
#endif
        return;
    }

    if (!strcmp_l1("mscript", buffer))
    {
        util_run_script((char*)download_addr, fb_download.received);
        return;
    }

    // flash Image
    if (getenv("flash_lzo")!=NULL)
    {
        if (strcmp(buffer,last_part_name)!=0)
        {
            snprintf(tmpCmd, sizeof(tmpCmd), "mmc unlzo 0x%08x 0x%08x %s 1",
                    download_addr, fb_download.received, buffer);
        }
        else
        {
            snprintf(tmpCmd, sizeof(tmpCmd), "mmc unlzo.cont 0x%08x 0x%08x %s 1",
                    download_addr, fb_download.received, buffer);
        }
    }
    else
    {
       fb_mmc_flash_write(partition_name, download_addr, fb_download.total);
       //snprintf(tmpCmd, sizeof(tmpCmd), "mmc write.p 0x%08x %s 0x%08x 1",
       //        download_addr, buffer, fb_download.received);
    }

    /*if (strlen(tmpCmd))
    {
        printf("\n%s\n", tmpCmd);
        run_command(tmpCmd, 0);
    }
    else
    {
        fb_status.status_code = kStatusError;
        strncpy(fb_status.message, "Unknown partition", sizeof(fb_status.message)-1);
    }*/

    memset(last_part_name,0,sizeof(last_part_name));
    strncpy(last_part_name, buffer, buffer_len);
}

static void cb_erase(char* buffer, unsigned buffer_len)
{
    char tmpCmd[128];

    memset(tmpCmd, 0, 128);
    snprintf(tmpCmd, sizeof(tmpCmd), "mmc erase.p %s", buffer);
    run_command(tmpCmd, 0);
}

struct cmd_dispatch_info {
    char *cmd;
    void (*cb)(char* buffer, unsigned buffer_len);
};

// download command queries:  "getvar:partition-type:boot" and
//                             getvar:max-download-size
// need to implement this two
static const struct cmd_dispatch_info cmd_dispatch_info[] =
{
    {
        .cmd = "reboot-bootloader",
        .cb = cb_reboot_bootloader,
    }, {
        .cmd = "reboot",
        .cb = cb_reboot,
    }, {
        .cmd = "getvar:",
        .cb = cb_getvar,
    }, {
        .cmd = "download:",
        .cb = cb_download_start,
    }, {
        .cmd = "flash:",
        .cb = cb_flash,
    }, {
        .cmd = "erase:",
        .cb = cb_erase,
    },
};

static void fb_handler_command(char* buffer, unsigned buffer_len)
{
    char cmdbuf[1024];   // assuming max command length
    void (*func_cb)(char* buffer, unsigned buffer_len) = NULL;

    fb_status.status_code = kStatusOkay;
    memset(fb_status.message, 0, FASTBOOT_RESPONSE_LEN);

    if (buffer_len >= 1024)
    {
        fb_status.status_code = kStatusError;
        printf("command too long: %d\n", buffer_len);
        strncpy(fb_status.message, "command too long", sizeof(fb_status.message)-1);
        return;
    }

    // put the command in the cmdbuf and add a '\0'
    strncpy(cmdbuf, buffer, buffer_len);
    cmdbuf[buffer_len] = 0;
    printf("\ncommand: %s, length: %d\n", cmdbuf, buffer_len);

    int i;
    for (i = 0; i < ARRAY_SIZE(cmd_dispatch_info); i++)
    {
        if (!strcmp_l1(cmd_dispatch_info[i].cmd, cmdbuf))
        {
            func_cb = cmd_dispatch_info[i].cb;
            break;
        }
    }

    if (!func_cb)
    {
        fb_status.status_code = kStatusError;
        printf("unknown command: %s", cmdbuf);
        strncpy(fb_status.message, "unknown command", sizeof(fb_status.message)-1);
    }
    else
    {
        int cmdLen = strlen(cmd_dispatch_info[i].cmd);
        func_cb(cmdbuf+cmdLen, buffer_len-cmdLen);
    }
}


//************************ UDP layer ************************

static void fb_udp_handler(uchar *pkt, unsigned dest, IPaddr_t sip, unsigned src,
                           unsigned len)
{
    // only handle the packet for us (fastboot)
    if (dest == fb_udp_our_port)
    {
        // need to keep the remote ip and port, needed when sending data back
        fb_remote_ip = sip;
        fb_remote_port = src;

        // incompleted packet
        if (len < kHeaderSize)
            return;

        // read the packet header
        Id             pktId   = pkt[kIndexId];
        Flag           pktFlag = pkt[kIndexFlags];
        unsigned short pktSeq  = extract_uint16(pkt+kIndexSeqH);

        // clear the packet for response
        memset(fb_packet, 0, FASTBOOT_RESPONSE_LEN);
        switch(pktId) {
            case kIdDeviceQuery:
            memcpy(fb_packet, pkt, kHeaderSize);
            put_uint32((uchar*)(fb_packet+kIndexNumHH), INVALID_NUM);
            put_uint16((uchar*)(fb_packet+kHeaderSize), fb_expected_seq);
            fb_send(fb_packet, kHeaderSize+2);
            break;

            case kIdInitialization:
            // seq # is not correct, ignore
            if (pktSeq != fb_expected_seq)
            {
               return;
            }

            if (len < kHeaderSize + 4)
            {
                fb_udp_error();
            }
            else
            {
                unsigned short host_version = extract_uint16(pkt+kHeaderSize);
                max_packet_len = extract_uint16(pkt+kHeaderSize+2);

                printf("\n**************************************\n");
                printf("host version: %d, host max packet: %d\n",
                      host_version, max_packet_len);

                // PKTSIZE doesn't seem work, 1400 seems a good number
                //if (max_packet_len > (PKTSIZE - NetEthHdrSize() - IP_HDR_SIZE)) {
                //   max_packet_len = PKTSIZE - NetEthHdrSize() - IP_HDR_SIZE;
                //}
                if (max_packet_len > PAYLOAD_SIZE)
                {
                    max_packet_len = PAYLOAD_SIZE;
                }
                max_packet_len -= kHeaderSize;

                fb_expected_seq++;
                memcpy(fb_packet, pkt, kHeaderSize);
                put_uint32((uchar*)(fb_packet+kIndexNumHH), INVALID_NUM);
                put_uint16((uchar*)(fb_packet+kHeaderSize), kProtocolVersion);
                put_uint16((uchar*)(fb_packet+kHeaderSize+2), max_packet_len+kHeaderSize);
                fb_send(fb_packet, kHeaderSize+4);

                // init status, prepare for fastboot commands
                fb_init();
            }
            break;

            case kIdFastboot:
            if (pktSeq != fb_expected_seq) // repeat packet
            {
               return;
            }
            if (len == kHeaderSize) // empty packet for get status
            {
                //printf("Getting an empty packet for status\n");
                fb_expected_seq++;
                memcpy(fb_packet, pkt, kHeaderSize);
                put_uint32((uchar*)(fb_packet+kIndexNumHH), INVALID_NUM);
                fb_gen_resp(fb_packet);
                printf("response: %s\n", fb_packet+kHeaderSize);
                fb_send(fb_packet, kHeaderSize+strlen(fb_packet+kHeaderSize));

                // need to wait for the status packet before resetting
                if (fb_need_reset)
                {
                    printf("Start to reset\n");
                    mdelay(500);
                    run_command("reset", 0);
                }
            }
            else
            {
                if (fb_download_flag)  // in a download session
                {
                    if (pktFlag)
                        cb_download_cont((char*)pkt, len);
                    else
                        cb_download_end((char*)pkt, len);
                }
                else  // other fastboot commands
                {
                    fb_expected_seq++;

                    // send ack first
                    fb_udp_ack(pkt, len, INVALID_NUM);
                    fb_handler_command((char*)(pkt+kHeaderSize), len-kHeaderSize);
                }
            }
            break;

            case kIdError:  // fall through
            default:
            // send back a error mesage
            fb_udp_error();
            break;
        }
    }
}

#endif // CONFIG_FASTBOOT_GOOGLE

void fb_udp_start(void)
{
    // allocate memory for receive flag bitmap upfront
    if (!receive_bitmap) {
       receive_bitmap = (unsigned char*)calloc(1, RECV_BITMAP_SIZE);
       if (!receive_bitmap) {
          printf("\nfastboot - out of memory!\n");
          return;
       }
    }

    char *s;
    if ((s = getenv("fastboot_port")) != NULL) {
       fb_udp_our_port = (int)simple_strtoul(s, NULL, 10);
       printf("fb_udp_our_port=%d\n", fb_udp_our_port);
    }

    char sIPStr[32];

    ip_to_string(NetOurIP, sIPStr);
    printf("Using %s device\n", eth_get_name());
    printf("Listening for fastboot transfer on %s\n", sIPStr);

    fb_init();

    NetSetHandler(fb_udp_handler);
}
