/** @file gcpu_if.c
 *  GCPU module api function.
 */

//---------------------------------------------------------------------------
// Include files
//---------------------------------------------------------------------------
#include "drvAESDMA.h"
#include "gcpu_if.h"

#define GCPU_RSA_SW 0
#define MDRV_SHA    0
#define MI_CIPHER_WORKING_BUFFER_NUMBER                     (100)

unsigned char GCPU_SHA256_Init(UPTR u4Handle, unsigned char fgIOMMU)
{
	static unsigned char _init = 0;

    GCPU_LoaderInit(0);
#if !MDRV_SHA
	DRVAESDMA_RESULT eAesRet = DRVAESDMA_OK;
	GCPU_SHA_HDL_T *params = (GCPU_SHA_HDL_T*)u4Handle;

	params->u4WorkingSize = MI_CIPHER_HASH_BLOCK_SIZE_SHA256 * MI_CIPHER_WORKING_BUFFER_NUMBER;
	params->au1WorkingBuffer = (unsigned char *)x_mem_calloc(1, params->u4WorkingSize);

	if (!params->au1WorkingBuffer)
		return FALSE;

	params->fgFirstPacket = TRUE;
#endif

    return TRUE;
}

static const char *eStage_str(int stage)
{
    switch(stage) {
    case E_DRVAESDMA_HASH_STAGE_ONCE:
        return "E_DRVAESDMA_HASH_STAGE_ONCE";
    case E_DRVAESDMA_HASH_STAGE_FIRST:
        return "E_DRVAESDMA_HASH_STAGE_FIRST";
    case E_DRVAESDMA_HASH_STAGE_LAST:
        return "E_DRVAESDMA_HASH_STAGE_LAST";
    case E_DRVAESDMA_HASH_STAGE_UPDATE:
        return "E_DRVAESDMA_HASH_STAGE_UPDATE";
    default:
        break;
    }
    return "(Unknown eStage)";
}

unsigned char GCPU_SHA256_Update(UPTR u4Handle, unsigned char *pu1Buf, unsigned int u4Size, unsigned char fgLastPacket)
{
	DRVAESDMA_RESULT eAesRet = DRVAESDMA_OK;
	DrvAESDMA_HASH_STAGE eStage;
	GCPU_SHA_HDL_T *params = (GCPU_SHA_HDL_T*)u4Handle;

#if MDRV_SHA
    DRVAESDMA_RESULT ret;
    //printf("MDrv_SHA_Calculate("\
            "\n    E_DRVAESDMA_SHA256,  // DrvAESDMA_SHAMode eMode"\
            "\n    0x%08x,    // MS_PHY phyPAInBuf = \"%s\""\
            "\n    %-10d,    // MS_U32 u32Size"\
            "\n    0x%08x,    // MS_PHY phyPAOutBuf\n",\
            MsOS_VA2PA(pu1Buf), pu1Buf, u4Size, MsOS_VA2PA(params->au1Hash));
    ret = MDrv_SHA_Calculate(E_DRVAESDMA_SHA256,
            MsOS_VA2PA(pu1Buf), u4Size, MsOS_VA2PA(params->au1Hash));
    //printf("ret = %d\n", ret);
    return ret;
#else
	if (!params || !params->au1WorkingBuffer)
		return FALSE;

	if (params->fgFirstPacket) {
		if (fgLastPacket) {
			eStage = E_DRVAESDMA_HASH_STAGE_ONCE;
		} else {
			eStage = E_DRVAESDMA_HASH_STAGE_FIRST;
		}
	} else {
		if (fgLastPacket) {
			eStage = E_DRVAESDMA_HASH_STAGE_LAST;
		} else {
			eStage = E_DRVAESDMA_HASH_STAGE_UPDATE;
		}
	}

    unsigned char *in_buf = pu1Buf;
    if ((unsigned int)in_buf & 0xf) {
        in_buf = x_mem_align(16, u4Size);
        memcpy(in_buf, pu1Buf, u4Size);
        //printf("[WARNING] input buffer is not 16 byte aligned: 0x%08x, "\\
                "use bounce buffer 0x08x\n", pu1Buf, in_buf);
    }
    //printf("MDrv_SHA_Accumulation(%s, // DrvAESDMA_HASH_STAGE eStage"\
            "\n    E_DRVAESDMA_SHA256,     // DrvAESDMA_SHAMode eMode"\
            "\n    0x%08x,             // MS_PHY phySrcBufAddr (VA: 0x%08x)"\
            "\n    %-10d,             // MS_U32 u32SrcSize"\
            "\n    0x%08x,             // MS_PHY phyWorkingBufAddr"\
            "\n    %-10d,             // MS_U32 u32WorkingSize"\
            "\n    0x%08x);            // MS_U16* pu16ShaVal\n",\
            eStage_str(eStage), MsOS_VA2PA(in_buf), in_buf, u4Size,\
            MsOS_VA2PA(params->au1WorkingBuffer), params->u4WorkingSize,\
            params->au1Hash);
	        eAesRet = MDrv_SHA_Accumulation(eStage,
			E_DRVAESDMA_SHA256,
			(MS_PHY)MsOS_VA2PA(in_buf),
			u4Size,
			(MS_PHY)MsOS_VA2PA(params->au1WorkingBuffer),
			params->u4WorkingSize,
			(MS_U16*)params->au1Hash);
    if (in_buf != pu1Buf) {
        x_mem_free(in_buf);
    }
	if (eAesRet != DRVAESDMA_OK) {
		return FALSE;
	}
	params->fgFirstPacket = FALSE;
	return TRUE;
#endif
}

static void  _MI_CIPHER_Revert(unsigned char* pu8Buf, unsigned int u32Size)
{
    int i = 0;
    unsigned char u8Temp;
    if(pu8Buf == NULL)
        return;

    for(i = 0; i < u32Size / 2; i++)
    {
        u8Temp = pu8Buf[i];
        pu8Buf[i] = pu8Buf[(u32Size - 1) - i];
        pu8Buf[(u32Size - 1) - i] = u8Temp;
    }
    return;
}

unsigned char GCPU_SHA256_Final(UPTR u4Handle, unsigned char au1Digest[32])
{
	GCPU_SHA_HDL_T *params = (GCPU_SHA_HDL_T*)u4Handle;

	if (!params)
		return FALSE;

    _MI_CIPHER_Revert(params->au1Hash, 32);
	memcpy(au1Digest, params->au1Hash, 32);
	if (params->au1WorkingBuffer) {
		x_mem_free(params->au1WorkingBuffer);
		params->au1WorkingBuffer = NULL;
	}
	return TRUE;
}

unsigned char GCPU_LoaderInit(unsigned int u4Mode)
{
    static unsigned char init = 0;
    if (!init) {
        init = 1;
#if !MDRV_SHA
		if(MDrv_AESDMA_Init(0, 0, 1) != DRVAESDMA_OK)
		{
			printf("Error: MDrv_AESDMA_Init() failed\n");
			return FALSE;
		}
#else
        Mt_AESDMA_Init();
#endif
    }
    return TRUE;
}

static void _reverse_copy(void *dest, const void *src, int size)
{
    int i, j;

    for (i=0, j=size-1; i<size; i++, j--)
        ((unsigned char*)dest)[i] = ((unsigned char*)src)[j];
}


