/*
 * This is from the Android Project,
 * bootloader/legacy/include/boot/bootimg.h
 *
 * Copyright (C) 2008 The Android Open Source Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef _ANDROID_IMAGE_H_
#define _ANDROID_IMAGE_H_

#define ANDR_BOOT_MAGIC "ANDROID!"
#define ANDR_BOOT_MAGIC_SIZE 8
#define ANDR_BOOT_NAME_SIZE 16
#define ANDR_BOOT_ARGS_SIZE 512
#define ANDR_BOOT_EXTRA_ARGS_SIZE 1024

typedef struct andr_img_hdr_legacy andr_img_hdr_legacy;

struct andr_img_hdr_legacy {
	u8 magic[ANDR_BOOT_MAGIC_SIZE];

	u32 kernel_size;	/* size in bytes */
	u32 kernel_addr;	/* physical load addr */

	u32 ramdisk_size;	/* size in bytes */
	u32 ramdisk_addr;	/* physical load addr */

	u32 second_size;	/* size in bytes */
	u32 second_addr;	/* physical load addr */

	u32 tags_addr;		/* physical addr for kernel tags */
	u32 page_size;		/* flash page size we assume */
	u32 dt_size;		/* device tree in bytes */
	u32 unused[2];		/* future expansion: should be 0 */
	u32 os_version;         /* operating system version and security patch level */

	u8 name[ANDR_BOOT_NAME_SIZE]; /* asciiz product name */

	u8 cmdline[ANDR_BOOT_ARGS_SIZE];

	u32 id[8]; /* timestamp / checksum / sha1 / etc */
};

/*
 * +-----------------+
 * | boot header     | 1 page
 * +-----------------+
 * | kernel          | n pages
 * +-----------------+
 * | ramdisk         | m pages
 * +-----------------+
 * | second stage    | o pages
 * +-----------------+
 *
 * n = (kernel_size + page_size - 1) / page_size
 * m = (ramdisk_size + page_size - 1) / page_size
 * o = (second_size + page_size - 1) / page_size
 *
 * 0. all entities are page_size aligned in flash
 * 1. kernel and ramdisk are required (size != 0)
 * 2. second is optional (second_size == 0 -> no second)
 * 3. load each element (kernel, ramdisk, second) at
 *    the specified physical address (kernel_addr, etc)
 * 4. prepare tags at tag_addr.  kernel_args[] is
 *    appended to the kernel commandline in the tags.
 * 5. r0 = 0, r1 = MACHINE_TYPE, r2 = tags_addr
 * 6. if second_size != 0: jump to second_addr
 *    else: jump to kernel_addr
 */

typedef struct andr_img_hdr andr_img_hdr;

struct andr_img_hdr {
    u8 magic[ANDR_BOOT_MAGIC_SIZE];

    u32 kernel_size;    /* size in bytes */
    u32 kernel_addr;    /* physical load addr */

    u32 ramdisk_size;   /* size in bytes */
    u32 ramdisk_addr;   /* physical load addr */

    u32 second_size;    /* size in bytes */
    u32 second_addr;    /* physical load addr */

    u32 tags_addr;      /* physical addr for kernel tags */
    u32 page_size;      /* flash page size we assume */
    u32 header_version; /* future expansion: should be 0 */
    u32 os_version;     /* operating system version and security patch level */

    u8 name[ANDR_BOOT_NAME_SIZE]; /* asciiz product name */

    u8 cmdline[ANDR_BOOT_ARGS_SIZE];

    u32 id[8]; /* timestamp / checksum / sha1 / etc */

    u8 extra_cmdline[ANDR_BOOT_EXTRA_ARGS_SIZE];

    // boot_img_hdr_v1
    u32 recovery_dtbo_size;
    u64 recovery_dtbo_offset;
    u32 header_size;

    // boot_img_hdr_v2
    u32 dtb_size; /* size in bytes for DTB image */
    u32 dtb_addr; /* physical load address for DTB image */
}__attribute__((packed));

/*
 * DTBO image header
 */
#define DT_TABLE_MAGIC 0xd7b7ab1e

struct dt_table_header {
    u32 magic;              //DT_TABLE_MAGIC: 0xd7b7able
    u32 total_size;         // includes dt_table_header + all dt_table_entry and all dtb/dtbo
    u32 header_size;        // sizeof(dt_table_header)

    u32 dt_entry_size;      // sizeof(dt_table_entry)
    u32 dt_entry_count;     // number of dt_table_entry
    u32 dt_entries_offset;  // offset to the first dt_table_entry

    u32 page_size;          // flash page size we assume
    u32 version;            // must be zero in android P.
};

/*
 * DTBO Entry header
 */
struct dt_table_entry {
    u32 dt_size;
    u32 dt_offset;      // offset from head of dt_table_header

    u32 id;             // optional, must be zero if unused
    u32 rev;            // optional, must be zero if unused
    u32 custom[4];      // optional, must be zero if unused
};
#endif