#include <stdio.h>
//#include <string.h>
#include <stdint.h>
#include <stdbool.h>
#include "sha256.h"

#ifndef AVB_VERSION_H_
#define AVB_VERSION_H_

#ifdef __cplusplus
extern "C" {
#endif
//#define PRINTDBGMSG

extern void sha256_block_data_order (uint32_t *ctx, const void *in, size_t num);

// SHA-256 initial hash value
const uint32_t H_0[8] = {
	0x6a09e667,
	0xbb67ae85,
	0x3c6ef372,
	0xa54ff53a,
	0x510e527f,
	0x9b05688c,
	0x1f83d9ab,
	0x5be0cd19,
};


void sha256_print_hash(uint32_t *H, const char *title) {
	printf("-----------------------------------------------------------------------------------------\n");
	printf("| %-85s |\n", title);
	printf("-----------------------------------------------------------------------------------------\n");
	printf("|   H[0]   |   H[1]   |   H[2]   |   H[3]   |   H[4]   |   H[5]   |   H[6]   |   H[7]   |\n");
	printf("| %08x | %08x | %08x | %08x | %08x | %08x | %08x | %08x |\n", H[0], H[1], H[2], H[3], H[4], H[5], H[6], H[7]);
	printf("-----------------------------------------------------------------------------------------\n");
}


// initialize hash value
#if 0
void sha256_init(uint32_t *H) {
	H[0] = 0x6a09e667;
	H[1] = 0xbb67ae85;
	H[2] = 0x3c6ef372;
	H[3] = 0xa54ff53a;
	H[4] = 0x510e527f;
	H[5] = 0x9b05688c;
	H[6] = 0x1f83d9ab;
	H[7] = 0x5be0cd19;
}

int main(int argc, char **argv) {
	unsigned int i;

	// check arguments
	if (argc != 2) {
		printf("Usage: %s <input file>\n", argv[0]);
		return -1;
	}

	// open input file
	FILE *fp = fopen(argv[1], "r");
	if (!fp) {
		printf("Error opening file '%s' for reading.\n", argv[1]);
		return -1;
	}

	// initialize hash value
	uint32_t H[8];
	memcpy(H, H_0, 8*4);

	// read file and calculate hash
	uint64_t bits = 0;
	unsigned char buffer[64];
	size_t len;
	while (len = fread(buffer, 1, sizeof(buffer), fp)) {
		bits += len * 8;

		// preserve value of len by breaking on end of file (or error)
		if (len < 64) {
			break;
		}
		sha256_block_data_order(H, buffer,1);
	}

	// add padding
	if (len < 56) {
		// padd current block to 56 byte
		buffer[len] = 0x80;
		i = len + 1;
	} else {
		// fill up current block and update hash
		buffer[len] = 0x80;
		for (i = len + 1; i < 64; i++) {
			buffer[i] = 0x00;
		}

		sha256_block_data_order(H, buffer,1);

		// add (almost) one block of zero bytes
		i = 0;
	}
	for (; i < 56; i++) {
		buffer[i] = 0x00;
	}

	// add message length in bits in big endian
	for (i = 0; i < 8; i++) {
		buffer[63 - i] = bits >> (i * 8);
	}

	sha256_block_data_order(H, buffer,1);

	// print hash
	sha256_print_hash(H, "Final Hash");

	// convert hash to char array (in correct order)
	for (i = 0; i < 8; i++) {
		buffer[i*4 + 0] = H[i] >> 24;
		buffer[i*4 + 1] = H[i] >> 16;
		buffer[i*4 + 2] = H[i] >>  8;
		buffer[i*4 + 3] = H[i];
	}

	// print hash
	printf("Hash:\t");
	for (i = 0; i < 32; i++) {
		printf("%02x", buffer[i]);
	}
	printf("\n");

	return 0;
}
#endif

unsigned char buffer[64*2];
uint64_t bits = 0;
uint32_t H[8];
size_t len = 0;
void sha256_calc_init()
{
  bits = 0;
  len = 0;
  // initialize hash value
  memcpy(H, H_0, 8*4);
}

void sha256_calc_UpdateData(const unsigned char* data,size_t dataLen) {
  unsigned int i=0;
  const unsigned char* dataIN = data;
  size_t remLen=64-len;
  size_t numOfSlice=0;
  size_t remOfSlice=0;
  if(dataLen+len>=64)
  {
      if(dataLen>=64)
      {
          numOfSlice= (len==0)? dataLen/64 : (dataLen-remLen)/64;
          remOfSlice= (len==0)? dataLen%64 : (dataLen-remLen)%64;
      }
      else//dataLen is less than 64, there's no slice calculation
      {
          numOfSlice=0;
          remOfSlice=dataLen-remLen;
      }

  }
  else//no calculate this time
  {
        numOfSlice=0;
        remOfSlice=dataLen+len;
  }

#ifdef PRINTDBGMSG
  printf("dataLen=%d,len=%d,calSize=%d\n",dataLen,len,dataLen+len);
  printf("numOfSlice=%d,remOfSlice=%d\n",numOfSlice,remOfSlice);
#endif

  if(dataLen+len < 64)//Data slice is less than a block, save it to buff for next calulation
  {
      memcpy(&buffer[len],dataIN,remOfSlice);
      len = remOfSlice;
      return;
  }
  //Calculate the first block with previous data slice (buffer+data of data)
  if(len)
  {
      memcpy(buffer+len,dataIN,64-len);
      bits += 64 * 8;
      sha256_block_data_order(H, buffer,1);
      dataIN+=len;
  }

  //Calculate data blocks slice by slice
	for(i=0;i<numOfSlice;i++)
	{
	    bits += 64 * 8;
	    sha256_block_data_order(H, dataIN,1);
	    dataIN += 64;
	}

	memcpy(buffer,dataIN,64);
	//Rest data less than 64 bytes
	len = remOfSlice;

#ifdef PRINTDBGMSG
	printf("len=%d\n",len);
#endif
}

void* sha256_calc_FinalCalc(void* digest)
{
	 unsigned int i;

   bits += len * 8;
#ifdef PRINTDBGMSG
   printf("%lld bytes processed\n",bits/8);
#endif
	// add padding
	if (len < 56) {
		// padd current block to 56 byte
		buffer[len] = 0x80;
		i = len + 1;
	} else {
		// fill up current block and update hash
		buffer[len] = 0x80;
		for (i = len + 1; i < 64; i++) {
			buffer[i] = 0x00;
		}

		sha256_block_data_order(H, buffer,1);

		// add (almost) one block of zero bytes
		i = 0;
	}
	for (; i < 56; i++) {
		buffer[i] = 0x00;
	}

	// add message length in bits in big endian
	for (i = 0; i < 8; i++) {
		buffer[63 - i] = bits >> (i * 8);
	}

	sha256_block_data_order(H, buffer,1);

	// print hash
	//sha256_print_hash(H, "Final Hash");

	// convert hash to char array (in correct order)
	for (i = 0; i < 8; i++) {
		buffer[i*4 + 0] = H[i] >> 24;
		buffer[i*4 + 1] = H[i] >> 16;
		buffer[i*4 + 2] = H[i] >>  8;
		buffer[i*4 + 3] = H[i];
	}
	if(digest!=0)
  {
      memcpy(digest,buffer,32);
  }

#ifdef PRINTDBGMSG
	// print hash
	printf("Hash:\t");
	for (i = 0; i < 32; i++) {
		printf("%02x", buffer[i]);
	}
	printf("\n");
#endif
  return digest;
}

#ifdef __cplusplus
}
#endif

#endif /* AVB_SHA_H_ */